@@ -272,7 +272,7 @@ QVariant KateCompletionModel::data( const QModelIndex & index, int role ) const
     switch (role) {
       case Qt::DisplayRole:
           //We return the group-header for all columns, ExpandingDelegate will paint them properly over the whole space
-          return ' ' + g->title;
+          return QString(' ' + g->title);
         break;
 
       case Qt::FontRole:
@@ -895,14 +895,14 @@ QModelIndex KateCompletionModel::mapFromSource( const QModelIndex & sourceIndex
   foreach (Group* g, m_rowTable) {
     int row = g->rowOf(modelRowPair(sourceIndex));
     if (row != -1)
-      return index(row, sourceIndex.column(), QModelIndex());
+      return index(row, sourceIndex.column(), indexForGroup(g));
   }
 
   // Copied from above
   foreach (Group* g, m_emptyGroups) {
     int row = g->rowOf(modelRowPair(sourceIndex));
     if (row != -1)
-      return index(row, sourceIndex.column(), QModelIndex());
+      return index(row, sourceIndex.column(), indexForGroup(g));
   }
 
   return QModelIndex();
@@ -917,7 +917,7 @@ void KateCompletionModel::setCurrentCompletion( KTextEditor::CodeCompletionModel
     m_currentMatch[model] = completion;
     return;
   }
-
+  
   changeTypes changeType = Change;
 
   if (m_currentMatch[model].length() > completion.length() && m_currentMatch[model].startsWith(completion, m_matchCaseSensitivity)) {
@@ -957,6 +957,65 @@ void KateCompletionModel::setCurrentCompletion( KTextEditor::CodeCompletionModel
   emit contentGeometryChanged();
 }
 
+QString KateCompletionModel::commonPrefixInternal(const QString &forcePrefix) const
+{
+  QString commonPrefix;   // isNull() = true
+  
+  QList< Group* > groups = m_rowTable; 
+  groups += m_ungrouped;
+  
+  foreach (Group* g, groups) {
+    foreach(const Item& item, g->filtered)
+    {
+      uint startPos = m_currentMatch[item.sourceRow().first].length();
+      const QString candidate = item.name().mid(startPos);
+      
+      if(!candidate.startsWith(forcePrefix))
+        continue;
+      
+      if(commonPrefix.isNull()) {
+        commonPrefix = candidate;
+        
+        //Replace QString::null prefix with QString(""), so we won't initialize it again
+        if(commonPrefix.isNull())
+          commonPrefix = QString("");  // isEmpty() = true, isNull() = false
+      }else{
+        commonPrefix = commonPrefix.left(candidate.length());
+        
+        for(int a = 0; a < commonPrefix.length(); ++a) {
+          if(commonPrefix[a] != candidate[a]) {
+            commonPrefix = commonPrefix.left(a);
+            break;
+          }
+        }
+      }
+    }
+  }
+  
+  return commonPrefix;
+}
+
+QString KateCompletionModel::commonPrefix(QModelIndex selectedIndex) const
+{
+  QString commonPrefix = commonPrefixInternal(QString());
+  
+  if(commonPrefix.isEmpty() && selectedIndex.isValid()) {
+    Group* g = m_ungrouped;
+    if(hasGroups())
+      g = groupOfParent(selectedIndex);
+    
+    if(g && selectedIndex.row() < g->filtered.size())
+    {
+      //Follow the path of the selected item, finding the next non-empty common prefix
+      Item item = g->filtered[selectedIndex.row()];
+      int matchLength = m_currentMatch[item.sourceRow().first].length();
+      commonPrefix = commonPrefixInternal(item.name().mid(matchLength).left(1));
+    }
+  }
+  
+  return commonPrefix;
+}
+
 bool KateCompletionModel::changeCompletions( Group * g, changeTypes changeType )
 {
   bool notifyModel = true;
@@ -1644,6 +1703,15 @@ bool KateCompletionModel::Item::filter( )
   return matchFilters;
 }
 
+uint KateCompletionModel::filteredItemCount() const
+{
+  uint ret = 0;
+  foreach(Group* group, m_rowTable)
+    ret += group->filtered.size();
+  
+  return ret;
+}
+
 bool KateCompletionModel::shouldMatchHideCompletionList() const {
 //   return m_haveExactMatch;
   ///@todo Make this faster
