@@ -4,6 +4,8 @@
    Copyright (C) 2001 Joseph Wenninger <jowenn@kde.org>
    Copyright (C) 2006 Dominik Haumann <dhdev@gmx.de>
    Copyright (C) 2007 Mirko Stocker <me@misto.ch>
+   Copyright (C) 2009 Michel Ludwig <michel.ludwig@kdemail.net>
+   Copyright (C) 2009 Erlend Hamberg <ehamberg@gmail.com>
 
    Based on work of:
      Copyright (C) 1999 Jochen Wilhelmy <digisnap@cs.tu-berlin.de>
@@ -32,12 +34,15 @@
 #include "kateconfig.h"
 #include "katedocument.h"
 #include "kateglobal.h"
+#include "kateviglobal.h"
+#include "katevikeyparser.h"
 #include "kateschema.h"
 #include "katesyntaxdocument.h"
 #include "katemodeconfigpage.h"
 #include "kateview.h"
 #include "katepartpluginmanager.h"
 #include "kpluginselector.h"
+#include "spellcheck/spellcheck.h"
 
 // auto generated ui files
 #include "ui_modonhdwidget.h"
@@ -49,6 +54,7 @@
 #include "ui_opensaveconfigwidget.h"
 #include "ui_opensaveconfigadvwidget.h"
 #include "ui_viinputmodeconfigwidget.h"
+#include "ui_spellcheckconfigwidget.h"
 
 #include <ktexteditor/plugin.h>
 
@@ -84,8 +90,8 @@
 #include <kvbox.h>
 #include <kactioncollection.h>
 #include <kplugininfo.h>
-
 #include <ktabwidget.h>
+
 //#include <knewstuff/knewstuff.h>
 #include <QtGui/QCheckBox>
 #include <QtGui/QComboBox>
@@ -99,7 +105,6 @@
 #include <QtGui/QPainter>
 #include <QtGui/QRadioButton>
 #include <QtGui/QSlider>
-#include <QtGui/QSpinBox>
 #include <QtCore/QStringList>
 #include <QtGui/QTabWidget>
 #include <QtCore/QTextCodec>
@@ -226,6 +231,7 @@ void KateIndentConfigTab::reload ()
 {
   uint configFlags = KateDocumentConfig::global()->configFlags();
 
+  ui->sbIndentWidth->setSuffix(ki18np(" character", " characters"));
   ui->sbIndentWidth->setValue(KateDocumentConfig::global()->indentationWidth());
   ui->chkKeepExtraSpaces->setChecked(configFlags & KateDocumentConfig::cfKeepExtraSpaces);
   ui->chkIndentPaste->setChecked(configFlags & KateDocumentConfig::cfIndentPastedText);
@@ -260,6 +266,8 @@ KateCompletionConfigTab::KateCompletionConfigTab(QWidget *parent)
   //
 
   connect(ui->chkAutoCompletionEnabled, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->gbWordCompletion, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->minimalWordLength, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
   layout->addWidget(newWidget);
   setLayout(layout);
@@ -284,12 +292,16 @@ void KateCompletionConfigTab::apply ()
 
   KateViewConfig::global()->configStart ();
   KateViewConfig::global()->setAutomaticCompletionInvocation (ui->chkAutoCompletionEnabled->isChecked());
+  KateViewConfig::global()->setWordCompletion (ui->gbWordCompletion->isChecked());
+  KateViewConfig::global()->setWordCompletionMinimalWordLength (ui->minimalWordLength->value());
   KateViewConfig::global()->configEnd ();
 }
 
 void KateCompletionConfigTab::reload ()
 {
   ui->chkAutoCompletionEnabled->setChecked( KateViewConfig::global()->automaticCompletionInvocation () );
+  ui->gbWordCompletion->setChecked( KateViewConfig::global()->wordCompletion () );
+  ui->minimalWordLength->setValue (KateViewConfig::global()->wordCompletionMinimalWordLength ());
 }
 //END KateCompletionConfigTab
 
@@ -316,6 +328,9 @@ KateViInputModeConfigTab::KateViInputModeConfigTab(QWidget *parent)
   connect(ui->chkViInputModeDefault, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->chkViCommandsOverride, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->chkViStatusBarHide, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->tblNormalModeMappings, SIGNAL(cellChanged(int, int)), this, SLOT(slotChanged()));
+  connect(ui->btnAddNewNormal, SIGNAL(clicked()), this, SLOT(addNewNormalModeMappingRow()));
+  connect(ui->btnRemoveSelectedNormal, SIGNAL(clicked()), this, SLOT(removeSelectedNormalMappingRow()));
 
   layout->addWidget(newWidget);
   setLayout(layout);
@@ -342,6 +357,15 @@ void KateViInputModeConfigTab::apply ()
   KateViewConfig::global()->setViInputMode (ui->chkViInputModeDefault->isChecked());
   KateViewConfig::global()->setViInputModeStealKeys (ui->chkViCommandsOverride->isChecked());
   KateViewConfig::global()->setViInputModeHideStatusBar (ui->chkViStatusBarHide->isChecked());
+  KateGlobal::self()->viInputModeGlobal()->clearMappings( NormalMode );
+  for ( int i = 0; i < ui->tblNormalModeMappings->rowCount(); i++ ) {
+    QTableWidgetItem* from = ui->tblNormalModeMappings->item( i, 0 );
+    QTableWidgetItem* to = ui->tblNormalModeMappings->item( i, 1 );
+
+    if ( from && to ) {
+      KateGlobal::self()->viInputModeGlobal()->addMapping( NormalMode, from->text(), to->text() );
+    }
+  }
   KateViewConfig::global()->configEnd ();
 }
 
@@ -350,9 +374,109 @@ void KateViInputModeConfigTab::reload ()
   ui->chkViInputModeDefault->setChecked( KateViewConfig::global()->viInputMode () );
   ui->chkViCommandsOverride->setChecked( KateViewConfig::global()->viInputModeStealKeys () );
   ui->chkViStatusBarHide->setChecked( KateViewConfig::global()->viInputModeHideStatusBar () );
+
+  ui->chkViCommandsOverride->setEnabled(ui->chkViInputModeDefault->isChecked());
+  ui->chkViStatusBarHide->setEnabled(ui->chkViInputModeDefault->isChecked());
+
+  QStringList l = KateGlobal::self()->viInputModeGlobal()->getMappings( NormalMode );
+  ui->tblNormalModeMappings->setRowCount( l.size() );
+
+  // make the two columns fill the entire table width
+  ui->tblNormalModeMappings->setColumnWidth( 0, ui->tblNormalModeMappings->width()/3 );
+  ui->tblNormalModeMappings->horizontalHeader()->setStretchLastSection( true );
+
+  int i = 0;
+  foreach( const QString &f, l ) {
+    QTableWidgetItem *from
+      = new QTableWidgetItem( KateViKeyParser::getInstance()->decodeKeySequence( f ) );
+    QString s = KateGlobal::self()->viInputModeGlobal()->getMapping( NormalMode, f );
+    QTableWidgetItem *to =
+      new QTableWidgetItem( KateViKeyParser::getInstance()->decodeKeySequence( s ) );
+
+    ui->tblNormalModeMappings->setItem(i, 0, from);
+    ui->tblNormalModeMappings->setItem(i++, 1, to);
+  }
+}
+
+void KateViInputModeConfigTab::addNewNormalModeMappingRow()
+{
+  int rows = ui->tblNormalModeMappings->rowCount();
+  ui->tblNormalModeMappings->insertRow( rows );
+  ui->tblNormalModeMappings->setCurrentCell( rows, 0 );
+  ui->tblNormalModeMappings->editItem( ui->tblNormalModeMappings->currentItem() );
+}
+
+void KateViInputModeConfigTab::removeSelectedNormalMappingRow()
+{
+  QList<QTableWidgetSelectionRange> l = ui->tblNormalModeMappings->selectedRanges();
+
+  foreach( const QTableWidgetSelectionRange &range, l ) {
+    for ( int i = 0; i < range.bottomRow()-range.topRow()+1; i++ ) {
+      ui->tblNormalModeMappings->removeRow( range.topRow() );
+    }
+  }
 }
 //END KateViInputModeConfigTab
 
+//BEGIN KateSpellCheckConfigTab
+KateSpellCheckConfigTab::KateSpellCheckConfigTab(QWidget *parent)
+  : KateConfigPage(parent)
+{
+  // This will let us have more separation between this page and
+  // the KTabWidget edge (ereslibre)
+  QVBoxLayout *layout = new QVBoxLayout;
+  QWidget *newWidget = new QWidget(this);
+
+  ui = new Ui::SpellCheckConfigWidget();
+  ui->setupUi(newWidget);
+
+  // What's This? help can be found in the ui file
+  reload();
+
+  //
+  // after initial reload, connect the stuff for the changed () signal
+
+  m_sonnetConfigWidget = new Sonnet::ConfigWidget(KGlobal::config().data(), this);
+  connect(m_sonnetConfigWidget, SIGNAL(configChanged()), this, SLOT(slotChanged()));
+  layout->addWidget(m_sonnetConfigWidget);
+
+  layout->addWidget(newWidget);
+  setLayout(layout);
+}
+
+KateSpellCheckConfigTab::~KateSpellCheckConfigTab()
+{
+  delete ui;
+}
+
+void KateSpellCheckConfigTab::showWhatsThis(const QString& text)
+{
+  QWhatsThis::showText(QCursor::pos(), text);
+}
+
+void KateSpellCheckConfigTab::apply()
+{
+// nothing changed, no need to apply stuff
+  if (!hasChanged()) {
+    return;
+  }
+  m_changed = false;
+
+  KateDocumentConfig::global()->configStart();
+  m_sonnetConfigWidget->save();
+  KateDocumentConfig::global()->configEnd();
+  const QList<KateDocument*>& kateDocumentList = KateGlobal::self()->kateDocuments();
+  for (QList<KateDocument*>::const_iterator it = kateDocumentList.begin();
+       it != kateDocumentList.end(); ++it) {
+    (*it)->refreshOnTheFlyCheck();
+  }
+}
+
+void KateSpellCheckConfigTab::reload()
+{
+  // does nothing
+}
+//END KateSpellCheckConfigTab
 
 //BEGIN KateSelectConfigTab
 KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
@@ -363,23 +487,9 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
   QVBoxLayout *layout = new QVBoxLayout;
   QWidget *newWidget = new QWidget(this);
 
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
   ui = new Ui::CursorConfigWidget();
   ui->setupUi( newWidget );
 
-  ui->chkSmartHome->setChecked(configFlags & KateDocumentConfig::cfSmartHome);
-  connect(ui->chkSmartHome, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->chkWrapCursor->setChecked(configFlags & KateDocumentConfig::cfWrapCursor);
-  connect(ui->chkWrapCursor, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->chkPagingMovesCursor->setChecked(KateDocumentConfig::global()->pageUpDownMovesCursor());
-  connect(ui->chkPagingMovesCursor, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->sbAutoCenterCursor->setValue(KateViewConfig::global()->autoCenterLines());
-  connect(ui->sbAutoCenterCursor, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-
   // What's This? Help is in the ui-files
 
   reload ();
@@ -390,6 +500,10 @@ KateSelectConfigTab::KateSelectConfigTab(QWidget *parent)
 
   connect(ui->rbNormal, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect(ui->rbPersistent, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkSmartHome, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkWrapCursor, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkPagingMovesCursor, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->sbAutoCenterCursor, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
 
   layout->addWidget(newWidget);
   setLayout(layout);
@@ -433,95 +547,52 @@ void KateSelectConfigTab::reload ()
 {
   ui->rbNormal->setChecked( ! KateViewConfig::global()->persistentSelection() );
   ui->rbPersistent->setChecked( KateViewConfig::global()->persistentSelection() );
+
+  const uint configFlags = KateDocumentConfig::global()->configFlags();
+
+  ui->chkSmartHome->setChecked(configFlags & KateDocumentConfig::cfSmartHome);
+  ui->chkWrapCursor->setChecked(configFlags & KateDocumentConfig::cfWrapCursor);
+  ui->chkPagingMovesCursor->setChecked(KateDocumentConfig::global()->pageUpDownMovesCursor());
+  ui->sbAutoCenterCursor->setValue(KateViewConfig::global()->autoCenterLines());
 }
 //END KateSelectConfigTab
 
-//BEGIN KateEditConfigTab
-KateEditConfigTab::KateEditConfigTab(QWidget *parent)
+//BEGIN KateEditGeneralConfigTab
+KateEditGeneralConfigTab::KateEditGeneralConfigTab(QWidget *parent)
   : KateConfigPage(parent)
-  , selectConfigTab(new KateSelectConfigTab(this))
-  , indentConfigTab(new KateIndentConfigTab(this))
-  , completionConfigTab (new KateCompletionConfigTab(this))
-  , viInputModeConfigTab(new KateViInputModeConfigTab(this))
 {
-  // FIXME: Is really needed to move all this code below to another class,
-  // since it is another tab itself on the config dialog. This means we should
-  // initialize, add and work with as we do with selectConfigTab and
-  // indentConfigTab (ereslibre)
   QVBoxLayout *layout = new QVBoxLayout;
-  layout->setMargin(0);
-  KTabWidget *tabWidget = new KTabWidget(this);
-  uint configFlags = KateDocumentConfig::global()->configFlags();
-
-  QWidget *tmpWidget = new QWidget(tabWidget);
-  QVBoxLayout *internalLayout = new QVBoxLayout;
-  QWidget *newWidget = new QWidget(tabWidget);
+  QWidget *newWidget = new QWidget(this);
   ui = new Ui::EditConfigWidget();
   ui->setupUi( newWidget );
 
-  ui->chkReplaceTabs->setChecked( configFlags & KateDocumentConfig::cfReplaceTabsDyn );
+  reload();
+
   connect( ui->chkReplaceTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
-
-  ui->chkShowTabs->setChecked( configFlags & KateDocumentConfig::cfShowTabs );
   connect(ui->chkShowTabs, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->chkShowSpaces->setChecked( configFlags & KateDocumentConfig::cfShowSpaces );
   connect(ui->chkShowSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->sbTabWidth->setValue( KateDocumentConfig::global()->tabWidth() );
   connect(ui->sbTabWidth, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-
-
-  ui->chkStaticWordWrap->setChecked(KateDocumentConfig::global()->wordWrap());
   connect(ui->chkStaticWordWrap, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->chkShowStaticWordWrapMarker->setChecked( KateRendererConfig::global()->wordWrapMarker() );
   connect(ui->chkShowStaticWordWrapMarker, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
-
-  ui->sbWordWrap->setValue( KateDocumentConfig::global()->wordWrapAt() );
   connect(ui->sbWordWrap, SIGNAL(valueChanged(int)), this, SLOT(slotChanged()));
-
-
-  ui->chkRemoveTrailingSpaces->setChecked( configFlags & KateDocumentConfig::cfRemoveTrailingDyn );
   connect( ui->chkRemoveTrailingSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()) );
-
-  ui->chkAutoBrackets->setChecked( configFlags & KateDocumentConfig::cfAutoBrackets );
   connect(ui->chkAutoBrackets, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkSmartCopyCut, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
+  connect(ui->chkScrollPastEnd, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
 
-  // What is this? help is in the ui-file
+  // "What's this?" help is in the ui-file
 
-  internalLayout->addWidget(newWidget);
-  tmpWidget->setLayout(internalLayout);
-
-  // add all tabs
-  tabWidget->insertTab(0, tmpWidget, i18n("General"));
-  tabWidget->insertTab(1, selectConfigTab, i18n("Cursor & Selection"));
-  tabWidget->insertTab(2, indentConfigTab, i18n("Indentation"));
-  tabWidget->insertTab(3, completionConfigTab, i18n("Auto Completion"));
-  tabWidget->insertTab(4, viInputModeConfigTab, i18n("Vi Input Mode"));
-
-  connect(selectConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
-  connect(indentConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
-  connect(completionConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
-  connect(viInputModeConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
-
-  layout->addWidget(tabWidget);
+  layout->addWidget(newWidget);
   setLayout(layout);
 }
 
-KateEditConfigTab::~KateEditConfigTab()
+KateEditGeneralConfigTab::~KateEditGeneralConfigTab()
 {
   delete ui;
 }
 
-void KateEditConfigTab::apply ()
+void KateEditGeneralConfigTab::apply ()
 {
-  // try to update the rest of tabs
-  selectConfigTab->apply();
-  indentConfigTab->apply();
-  completionConfigTab->apply();
-  viInputModeConfigTab->apply();
-
   // nothing changed, no need to apply stuff
   if (!hasChanged())
     return;
@@ -553,31 +624,108 @@ void KateEditConfigTab::apply ()
   KateRendererConfig::global()->setWordWrapMarker (ui->chkShowStaticWordWrapMarker->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
+  KateViewConfig::global()->setSmartCopyCut(ui->chkSmartCopyCut->isChecked());
+  KateViewConfig::global()->setScrollPastEnd(ui->chkScrollPastEnd->isChecked());
   KateViewConfig::global()->configEnd ();
 }
 
+void KateEditGeneralConfigTab::reload ()
+{
+  uint configFlags = KateDocumentConfig::global()->configFlags();
+
+  ui->chkReplaceTabs->setChecked( configFlags & KateDocumentConfig::cfReplaceTabsDyn );
+  ui->chkShowTabs->setChecked( configFlags & KateDocumentConfig::cfShowTabs );
+  ui->chkShowSpaces->setChecked( configFlags & KateDocumentConfig::cfShowSpaces );
+  ui->sbTabWidth->setSuffix(ki18np(" character", " characters"));
+  ui->sbTabWidth->setValue( KateDocumentConfig::global()->tabWidth() );
+  ui->chkStaticWordWrap->setChecked(KateDocumentConfig::global()->wordWrap());
+  ui->chkShowStaticWordWrapMarker->setChecked( KateRendererConfig::global()->wordWrapMarker() );
+  ui->sbWordWrap->setSuffix(ki18np(" character", " characters"));
+  ui->sbWordWrap->setValue( KateDocumentConfig::global()->wordWrapAt() );
+  ui->chkRemoveTrailingSpaces->setChecked( configFlags & KateDocumentConfig::cfRemoveTrailingDyn );
+  ui->chkAutoBrackets->setChecked( configFlags & KateDocumentConfig::cfAutoBrackets );
+  ui->chkSmartCopyCut->setChecked( KateViewConfig::global()->smartCopyCut() );
+  ui->chkScrollPastEnd->setChecked( KateViewConfig::global()->scrollPastEnd() );
+}
+//END KateEditGeneralConfigTab
+
+
+//BEGIN KateEditConfigTab
+KateEditConfigTab::KateEditConfigTab(QWidget *parent)
+  : KateConfigPage(parent)
+  , editConfigTab(new KateEditGeneralConfigTab(this))
+  , selectConfigTab(new KateSelectConfigTab(this))
+  , indentConfigTab(new KateIndentConfigTab(this))
+  , completionConfigTab (new KateCompletionConfigTab(this))
+  , viInputModeConfigTab(new KateViInputModeConfigTab(this))
+  , spellCheckConfigTab(new KateSpellCheckConfigTab(this))
+{
+  QVBoxLayout *layout = new QVBoxLayout;
+  layout->setMargin(0);
+  KTabWidget *tabWidget = new KTabWidget(this);
+
+  // add all tabs
+  tabWidget->insertTab(0, editConfigTab, i18n("General"));
+  tabWidget->insertTab(1, selectConfigTab, i18n("Cursor && Selection"));
+  tabWidget->insertTab(2, indentConfigTab, i18n("Indentation"));
+  tabWidget->insertTab(3, completionConfigTab, i18n("Auto Completion"));
+  tabWidget->insertTab(4, viInputModeConfigTab, i18n("Vi Input Mode"));
+  tabWidget->insertTab(5, spellCheckConfigTab, i18n("Spellcheck"));
+
+  connect(editConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+  connect(selectConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+  connect(indentConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+  connect(completionConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+  connect(viInputModeConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+  connect(spellCheckConfigTab, SIGNAL(changed()), this, SLOT(slotChanged()));
+
+  layout->addWidget(tabWidget);
+  setLayout(layout);
+}
+
+KateEditConfigTab::~KateEditConfigTab()
+{
+}
+
+void KateEditConfigTab::apply ()
+{
+  // try to update the rest of tabs
+  editConfigTab->apply();
+  selectConfigTab->apply();
+  indentConfigTab->apply();
+  completionConfigTab->apply();
+  viInputModeConfigTab->apply();
+  spellCheckConfigTab->apply();
+}
+
 void KateEditConfigTab::reload ()
 {
+  editConfigTab->reload();
   selectConfigTab->reload();
   indentConfigTab->reload();
   completionConfigTab->reload();
   viInputModeConfigTab->reload();
+  spellCheckConfigTab->reload();
 }
 
 void KateEditConfigTab::reset ()
 {
+  editConfigTab->reset();
   selectConfigTab->reset();
   indentConfigTab->reset();
   completionConfigTab->reset();
   viInputModeConfigTab->reset();
+  spellCheckConfigTab->reset();
 }
 
 void KateEditConfigTab::defaults ()
 {
+  editConfigTab->defaults();
   selectConfigTab->defaults();
   indentConfigTab->defaults();
   completionConfigTab->defaults();
   viInputModeConfigTab->defaults();
+  spellCheckConfigTab->defaults();
 }
 //END KateEditConfigTab
 
@@ -595,9 +743,10 @@ KateViewDefaultsConfig::KateViewDefaultsConfig(QWidget *parent)
   ui->cmbDynamicWordWrapIndicator->addItem( i18n("Follow Line Numbers") );
   ui->cmbDynamicWordWrapIndicator->addItem( i18n("Always On") );
 
-  ui->chkShowIndentationLines->setChecked(KateRendererConfig::global()->showIndentationLines());
-  ui->chkShowWholeBracketExpression->setChecked(KateRendererConfig::global()->showWholeBracketExpression());
-
+  // hide power user mode if activated anyway
+  if (!KateGlobal::self()->simpleMode ())
+    ui->chkDeveloperMode->hide ();
+    
   // What's This? help is in the ui-file
 
   reload();
@@ -647,7 +796,17 @@ void KateViewDefaultsConfig::apply ()
   KateRendererConfig::global()->setShowIndentationLines(ui->chkShowIndentationLines->isChecked());
   KateRendererConfig::global()->setShowWholeBracketExpression(ui->chkShowWholeBracketExpression->isChecked());
 
-  KateDocumentConfig::global()->setAllowSimpleMode (!ui->chkDeveloperMode->isChecked());
+  // warn user that he needs restart the application
+  if (!ui->chkDeveloperMode->isChecked() != KateDocumentConfig::global()->allowSimpleMode())
+  {
+    // inform...
+    KMessageBox::information(
+                this,
+                i18n("Changing the power user mode affects only newly opened / created documents. In KWrite a restart is recommended."),
+                i18n("Power user mode changed"));
+
+    KateDocumentConfig::global()->setAllowSimpleMode (!ui->chkDeveloperMode->isChecked());
+  }
 
   KateRendererConfig::global()->configEnd ();
   KateViewConfig::global()->configEnd ();
@@ -709,6 +868,7 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   connect( ui->cmbEncodingDetection, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->cmbEOL, SIGNAL(activated(int)), this, SLOT(slotChanged()));
   connect( ui->chkDetectEOL, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
+  connect( ui->chkEnableBOM, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( ui->chkRemoveTrailingSpaces, SIGNAL(toggled(bool)), this, SLOT(slotChanged()));
   connect( uiadv->chkBackupLocalFiles, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
   connect( uiadv->chkBackupRemoteFiles, SIGNAL( toggled(bool) ), this, SLOT( slotChanged() ) );
@@ -724,7 +884,7 @@ KateSaveConfigTab::KateSaveConfigTab( QWidget *parent )
   // add all tabs
   tabWidget->insertTab(0, tmpWidget, i18n("General"));
   tabWidget->insertTab(1, tmpWidget2, i18n("Advanced"));
-  tabWidget->insertTab(2, modeConfigPage, i18n("Modes & Filetypes"));
+  tabWidget->insertTab(2, modeConfigPage, i18n("Modes && Filetypes"));
 
   connect(modeConfigPage, SIGNAL(changed()), this, SLOT(slotChanged()));
 
@@ -782,6 +942,7 @@ void KateSaveConfigTab::apply()
 
   KateDocumentConfig::global()->setEol(ui->cmbEOL->currentIndex());
   KateDocumentConfig::global()->setAllowEolDetection(ui->chkDetectEOL->isChecked());
+  KateDocumentConfig::global()->setBom(ui->chkEnableBOM->isChecked());
 
   KateDocumentConfig::global()->configEnd ();
 }
@@ -836,7 +997,8 @@ void KateSaveConfigTab::reload()
   // eol
   ui->cmbEOL->setCurrentIndex(KateDocumentConfig::global()->eol());
   ui->chkDetectEOL->setChecked(KateDocumentConfig::global()->allowEolDetection());
-
+  ui->chkEnableBOM->setChecked(KateDocumentConfig::global()->bom());
+  
   const uint configFlags = KateDocumentConfig::global()->configFlags();
   ui->chkRemoveTrailingSpaces->setChecked(configFlags & KateDocumentConfig::cfRemoveSpaces);
   uiadv->sbConfigFileSearchDepth->setValue(KateDocumentConfig::global()->searchDirConfigDepth());
@@ -909,7 +1071,7 @@ KatePartPluginConfigPage::KatePartPluginConfigPage (QWidget *parent)
 
   // add all tabs
   tabWidget->insertTab(0, tmpWidget, i18n("Plugins"));
-  tabWidget->insertTab(1, scriptConfigPage, i18n("Scripts"));
+//  tabWidget->insertTab(1, scriptConfigPage, i18n("Scripts"));
 
   generalLayout->addWidget(tabWidget);
   setLayout(generalLayout);
@@ -1011,7 +1173,7 @@ KateHlDownloadDialog::KateHlDownloadDialog(QWidget *parent, const char *name, bo
 
   KVBox* vbox = new KVBox(this);
   setMainWidget(vbox);
-  vbox->setSpacing(spacingHint());
+  vbox->setSpacing(-1);
   new QLabel(i18n("Select the syntax highlighting files you want to update:"), vbox);
   list = new QTreeWidget(vbox);
   list->setColumnCount(4);
@@ -1043,6 +1205,8 @@ void KateHlDownloadDialog::listDataReceived(KIO::Job *, const QByteArray &data)
   if (!transferJob || transferJob->isErrorPage())
   {
     enableButton( User1, false );
+    if (data.size()==0)
+      KMessageBox::error(this,i18n("The list of highlightings could not be found on / retrieved from the server"));
     return;
   }
 
@@ -1128,8 +1292,7 @@ KateGotoBar::KateGotoBar(KateView* view, QWidget *parent)
 {
   QHBoxLayout *topLayout = new QHBoxLayout( centralWidget() );
   topLayout->setMargin(0);
-  //topLayout->setSpacing(spacingHint());
-  gotoRange = new QSpinBox(centralWidget());
+  gotoRange = new KIntSpinBox(centralWidget());
 
   QLabel *label = new QLabel(i18n("&Go to line:"), centralWidget() );
   label->setBuddy(gotoRange);
@@ -1181,6 +1344,65 @@ void KateGotoBar::gotoLine()
 }
 //END KateGotoBar
 
+//BEGIN KateDictionaryBar
+KateDictionaryBar::KateDictionaryBar(KateView* view, QWidget *parent)
+  : KateViewBarWidget( true, view, parent )
+{
+
+  QHBoxLayout *topLayout = new QHBoxLayout(centralWidget());
+  topLayout->setMargin(0);
+  //topLayout->setSpacing(spacingHint());
+  m_dictionaryComboBox = new Sonnet::DictionaryComboBox(centralWidget());
+  connect(m_dictionaryComboBox, SIGNAL(dictionaryChanged(const QString&)),
+          this, SLOT(dictionaryChanged(const QString&)));
+  connect(view->doc(), SIGNAL(defaultDictionaryChanged(KateDocument*)),
+          this, SLOT(updateData()));
+  QLabel *label = new QLabel(i18n("Dictionary:"), centralWidget());
+  label->setBuddy(m_dictionaryComboBox);
+
+  topLayout->addWidget(label);
+  topLayout->addWidget(m_dictionaryComboBox, 1);
+  topLayout->setStretchFactor(m_dictionaryComboBox, 0);
+  topLayout->addStretch();
+}
+
+KateDictionaryBar::~KateDictionaryBar()
+{
+}
+
+void KateDictionaryBar::updateData()
+{
+  if (!view()) {
+    return;
+  }
+
+  KateDocument *document = view()->doc();
+  QString dictionary = document->defaultDictionary();
+  if(dictionary.isEmpty()) {
+    dictionary = Sonnet::Speller().defaultLanguage();
+  }
+  m_dictionaryComboBox->setCurrentByDictionary(dictionary);
+}
+
+void KateDictionaryBar::dictionaryChanged(const QString& dictionary)
+{
+  KateView *kateView = view();
+  if (!kateView) {
+    return;
+  }
+
+  KTextEditor::Range selection = kateView->selectionRange();
+  if(selection.isValid() && !selection.isEmpty()) {
+    kateView->doc()->setDictionary(dictionary, selection);
+  }
+  else {
+    kateView->doc()->setDefaultDictionary(dictionary);
+  }
+}
+
+//END KateGotoBar
+
+
 //BEGIN KateModOnHdPrompt
 KateModOnHdPrompt::KateModOnHdPrompt( KateDocument *doc,
                                       KTextEditor::ModificationInterface::ModifiedOnDiskReason modtype,
@@ -1265,7 +1487,7 @@ void KateModOnHdPrompt::slotDiff()
   m_proc = new KProcess( this );
   m_proc->setOutputChannelMode( KProcess::MergedChannels );
   *m_proc << "diff" << QString(ui->chkIgnoreWhiteSpaces->isChecked() ? "-ub" : "-u")
-     << "-" <<  m_doc->url().path();
+     << "-" <<  m_doc->url().toLocalFile();
   connect( m_proc, SIGNAL(readyRead()), this, SLOT(slotDataAvailable()) );
   connect( m_proc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(slotPDone()) );
 
