@@ -223,6 +223,7 @@ KateSchemaConfigColorTab::KateSchemaConfigColorTab()
   connect( ui->tmarker   , SIGNAL( changed( const QColor& ) ), SIGNAL( changed() ) );
   connect( ui->linenumber, SIGNAL( changed( const QColor& ) ), SIGNAL( changed() ) );
   connect( ui->markers   , SIGNAL( changed( const QColor& ) ), SLOT( slotMarkerColorChanged( const QColor& ) ) );
+  connect( ui->spellingmistakeline, SIGNAL( changed( const QColor& ) ), SIGNAL( changed() ) );
 }
 
 KateSchemaConfigColorTab::~KateSchemaConfigColorTab()
@@ -243,6 +244,7 @@ void KateSchemaConfigColorTab::schemaChanged ( int newSchema )
     m_schemas[ m_schema ].iconborder = ui->iconborder->color();
     m_schemas[ m_schema ].tmarker = ui->tmarker->color();
     m_schemas[ m_schema ].linenumber = ui->linenumber->color();
+    m_schemas[ m_schema ].spellingmistakeline = ui->spellingmistakeline->color();
   }
 
   if ( newSchema == m_schema ) return;
@@ -271,6 +273,7 @@ void KateSchemaConfigColorTab::schemaChanged ( int newSchema )
     QColor tmp5( KColorUtils::shade( tmp0, bgLuma > 0.7 ? -0.35 : 0.3 ) );
     QColor tmp6( schemeWindow.background().color() );
     QColor tmp7( schemeWindow.foreground().color() );
+    QColor tmp8( Qt::red );
 
     // same std colors like in KateDocument::markColor
     QVector <QColor> mark(KTextEditor::MarkInterface::reservedMarkersCount());
@@ -294,6 +297,7 @@ void KateSchemaConfigColorTab::schemaChanged ( int newSchema )
     c.tmarker = config.readEntry("Color Tab Marker", tmp5);
     c.iconborder = config.readEntry("Color Icon Bar", tmp6);
     c.linenumber = config.readEntry("Color Line Number", tmp7);
+    c.spellingmistakeline = config.readEntry("Color Spelling Mistake Line", tmp8);
 
     for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
       c.markerColors[i] =  config.readEntry( QString("Color MarkType%1").arg(i+1), mark[i] );
@@ -314,6 +318,7 @@ void KateSchemaConfigColorTab::schemaChanged ( int newSchema )
   ui->tmarker->setColor(  m_schemas [ newSchema ].tmarker );
   ui->iconborder->setColor(  m_schemas [ newSchema ].iconborder );
   ui->linenumber->setColor(  m_schemas [ newSchema ].linenumber );
+  ui->spellingmistakeline->setColor(  m_schemas [ newSchema ].spellingmistakeline );
 
   // map from 0..reservedMarkersCount()-1 - the same index as in markInterface
   for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
@@ -356,6 +361,7 @@ void KateSchemaConfigColorTab::apply ()
     config.writeEntry("Color Tab Marker", c.tmarker);
     config.writeEntry("Color Icon Bar", c.iconborder);
     config.writeEntry("Color Line Number", c.linenumber);
+    config.writeEntry("Color Spelling Mistake Line", c.spellingmistakeline);
 
     for (int i = 0; i < KTextEditor::MarkInterface::reservedMarkersCount(); i++)
     {
@@ -531,7 +537,7 @@ void KateSchemaConfigFontColorTab::apply ()
 //END FontColorConfig
 
 //BEGIN KateSchemaConfigHighlightTab -- 'Highlighting Text Styles' tab
-KateSchemaConfigHighlightTab::KateSchemaConfigHighlightTab(KateSchemaConfigFontColorTab *page, uint hl )
+KateSchemaConfigHighlightTab::KateSchemaConfigHighlightTab(KateSchemaConfigFontColorTab *page)
 {
   m_defaults = page;
 
@@ -539,14 +545,12 @@ KateSchemaConfigHighlightTab::KateSchemaConfigHighlightTab(KateSchemaConfigFontC
   m_hl = 0;
 
   QVBoxLayout *layout = new QVBoxLayout(this);
-  layout->setMargin(0);
-  layout->setSpacing(KDialog::spacingHint());
 
   // hl chooser
   KHBox *hbHl = new KHBox( this );
   layout->addWidget (hbHl);
 
-  hbHl->setSpacing( KDialog::spacingHint() );
+  hbHl->setSpacing( -1 );
   QLabel *lHl = new QLabel( i18n("H&ighlight:"), hbHl );
   hlCombo = new KComboBox( hbHl );
   hlCombo->setEditable( false );
@@ -567,6 +571,16 @@ KateSchemaConfigHighlightTab::KateSchemaConfigHighlightTab(KateSchemaConfigFontC
   connect(m_styles, SIGNAL(changed()), this, SIGNAL(changed()));
   layout->addWidget (m_styles, 999);
 
+  // get current highlighting from the host application
+  int hl = 0;
+  KTextEditor::MdiContainer *iface = qobject_cast<KTextEditor::MdiContainer*>(KateGlobal::self()->container());
+  if (iface) {
+    KateView *kv = qobject_cast<KateView*>(iface->activeView());
+    if (kv) {
+      const QString hlName = kv->doc()->highlight()->name();
+      hl = KateHlManager::self()->nameFind(hlName);
+    }
+  }
   hlCombo->setCurrentIndex ( hl );
   hlChanged ( hl );
 
@@ -693,17 +707,16 @@ void KateSchemaConfigHighlightTab::apply ()
 //END KateSchemaConfigHighlightTab
 
 //BEGIN KateSchemaConfigPage -- Main dialog page
-KateSchemaConfigPage::KateSchemaConfigPage( QWidget *parent, KateDocument * )
+KateSchemaConfigPage::KateSchemaConfigPage( QWidget *parent)
   : KateConfigPage( parent ),
     m_lastSchema (-1)
 {
   QVBoxLayout *layout = new QVBoxLayout(this);
   layout->setMargin(0);
-  layout->setSpacing(KDialog::spacingHint());
 
   KHBox *hbHl = new KHBox( this );
   layout->addWidget(hbHl);
-  hbHl->setSpacing( KDialog::spacingHint() );
+  hbHl->setSpacing( -1 );
   QLabel *lHl = new QLabel( i18n("&Schema:"), hbHl );
   schemaCombo = new KComboBox( hbHl );
   schemaCombo->setEditable( false );
@@ -717,11 +730,11 @@ KateSchemaConfigPage::KateSchemaConfigPage( QWidget *parent, KateDocument * )
   btndel = new QPushButton( i18n("&Delete"), hbHl );
   connect( btndel, SIGNAL(clicked()), this, SLOT(deleteSchema()) );
 
+  qobject_cast<QBoxLayout *>(hbHl->layout())->addStretch();
+
   m_tabWidget = new KTabWidget ( this );
   layout->addWidget (m_tabWidget);
 
-  connect (m_tabWidget, SIGNAL(currentChanged(int)), this, SLOT (newCurrentPage(int)));
-
   m_colorTab = new KateSchemaConfigColorTab();
   m_tabWidget->addTab (m_colorTab, i18n("Colors"));
   connect(m_colorTab, SIGNAL(changed()), SLOT(slotChanged()));
@@ -734,13 +747,15 @@ KateSchemaConfigPage::KateSchemaConfigPage( QWidget *parent, KateDocument * )
   m_tabWidget->addTab (m_fontColorTab, i18n("Normal Text Styles"));
   connect(m_fontColorTab, SIGNAL(changed()), SLOT(slotChanged()));
 
-  m_highlightTab = new KateSchemaConfigHighlightTab(m_fontColorTab, 0 );
+  m_highlightTab = new KateSchemaConfigHighlightTab(m_fontColorTab);
   m_tabWidget->addTab(m_highlightTab, i18n("Highlighting Text Styles"));
   connect(m_highlightTab, SIGNAL(changed()), SLOT(slotChanged()));
 
+  connect (m_tabWidget, SIGNAL(currentChanged(int)), this, SLOT (newCurrentPage(int)));
+
   hbHl = new KHBox( this );
   layout->addWidget (hbHl);
-  hbHl->setSpacing( KDialog::spacingHint() );
+  hbHl->setSpacing( -1 );
   lHl = new QLabel( i18n("&Default schema for %1:", KGlobal::mainComponent().aboutData()->programName ()), hbHl );
   defaultSchemaCombo = new KComboBox( hbHl );
   defaultSchemaCombo->setEditable( false );
