@@ -1,16 +1,14 @@
 /*
     Copyright (C) 2002-2003 Arash Bijanzadeh  and FarsiKDE Project <www.farsikde.org>
     Contact: Arash Bijanzadeh <a.bijanzadeh@linuxiran.org>
-    Copyright (c) 2007 John Layt <john@layt.net>
+    Copyright (c) 2007, 2008, 2009 John Layt <john@layt.net>
 
-    This program is part of FarsiKDE
-
-    FarsiKDE is free software; you can redistribute it and/or
+    This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
     version 2 of the License, or (at your option) any later version.
 
-    FarsiKDE is distributed in the hope that it will be useful,
+    This library is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     Library General Public License for more details.
@@ -19,229 +17,60 @@
     along with this library; see the file COPYING.LIB.  If not, write to
     the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
     Boston, MA 02110-1301, USA.
- */
+*/
 
+/*
+    This is an implementation of the artithmetic Persian calendar using the
+    Birashk algorithm with adjustments to always be correct in the period 1244
+    and 1530 (1865 to 2152 Gregorian).
 
-#include "kcalendarsystemjalali.h"
+    In future this will be replaced with the correct astronomical calendar.
+*/
+
+#include "kcalendarsystemjalali_p.h"
 
 #include <QtCore/QDate>
-#include <QtCore/QCharRef>
-#include <math.h>
 
-#include <kglobal.h>
-#include <klocale.h>
-#include <kdebug.h>
-#include <stdio.h>
-
-
-//===========================================================================
-//  This section holds the old Jalali <=> jd <=> Gregorian conversion code
-//  Delete once conversion to new code complete and fully tested
-//===========================================================================
-
-static const int  gMonthDay[2][13] =
+class KCalendarSystemJalaliPrivate
+{
+public:
+    KCalendarSystemJalaliPrivate( KCalendarSystemJalali *q ): q( q )
     {
-        {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
-        {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
-    };
+    }
 
-static const    int     jMonthDay[2][13] =
+    ~KCalendarSystemJalaliPrivate()
     {
-        {0, 31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29},
-        {0, 31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 30},
-    };
-
-typedef struct
-{
-    int day;
-    int mon;
-    int year;
-} SDATE;
-
-// converting funcs from
-
-static int Ceil( float number )
-{
-    int ret;
-
-    if( number > 0 ) {
-        number += 0.5;
     }
-    ret = ( int ) number;
 
-    return ret;
-}
+    KCalendarSystem *q;
 
-static long jalali_jdn( int year, int month, int day )
+    int daysInMonth( int year, int month ) const;
+};
+
+int KCalendarSystemJalaliPrivate::daysInMonth( int year, int month ) const
 {
-    const long PERSIAN_EPOCH = 1948321; /* The JDN of 1 Farvardin 1*/
-    int epbase;
-    long epyear;
-    long mdays;
-    long jdn;
-
-    epbase = year - 474;
-    epyear = 474 + ( epbase % 2820 );
-    if ( month <= 7 ) {
-        mdays = ( month - 1 ) * 31;
+    if ( month <= 6 ) {
+        return 31;
+    } else if ( month <= 11 ) {
+        return 30;
     } else {
-        mdays = ( month - 1 ) * 30 + 6;
-    }
-    jdn = day + mdays ;
-    jdn += ( ( ( epyear * 682 ) - 110 ) / 2816 ) ;
-    jdn += ( epyear - 1 ) * 365;
-    jdn += ( epbase / 2820 ) * 1029983 ;
-    jdn += ( PERSIAN_EPOCH - 1 );
-
-    return jdn;
-}
-
-
-static SDATE jdn_jalali( long jdn )
-{
-    static SDATE ret;
-    int day, month, year;
-    int iYear, iMonth, iDay;
-    int depoch;
-    int cycle;
-    int cyear;
-    int ycycle;
-    int aux1, aux2;
-    int yday;
-
-    day = 1;
-    month = 1;
-    year = 475;
-    depoch = jdn - jalali_jdn( year, month, day );
-    cycle = ( int ) ( depoch / 1029983 );
-    cyear = depoch % 1029983;
-    if( cyear == 1029982 ) {
-        ycycle = 2820;
-    } else {
-        aux1 = cyear / 366;
-        aux2 = cyear % 366;
-        ycycle = ( ( ( 2134 * aux1 ) + ( 2816 * aux2 ) + 2815 ) / 1028522 ) + aux1 + 1;
-    }
-    iYear = ycycle + ( 2820 * cycle ) + 474;
-    if ( iYear <= 0 ) {
-        iYear = iYear - 1;
-    }
-    year = iYear;
-    yday = ( jdn - jalali_jdn( year, month, day ) ) + 1;
-    if( yday <= 186 ) {
-        iMonth = Ceil( ( yday - 1 ) / 31 );
-    } else {
-        iMonth = Ceil( ( yday - 7 ) / 30 );
-    }
-    iMonth++;
-    month = iMonth;
-    iDay = ( jdn - jalali_jdn( year, month, day ) ) + 1;
-    ret.day = iDay;
-    ret.mon = iMonth;
-    ret.year = iYear;
-
-    return ret;
-}
-
-static long civil_jdn( int year, int month, int day )
-{
-    long jdn = ( ( 1461 * ( year + 4800 + ( ( month - 14 ) / 12 ) ) ) / 4 )
-               + ( ( 367 * ( month - 2 - 12 * ( ( ( month - 14 ) / 12 ) ) ) ) / 12 )
-               - ( ( 3 * ( ( ( year + 4900 + ( ( month - 14 ) / 12 ) ) / 100 ) ) ) / 4 )
-               + day - 32075;
-
-    return jdn;
-}
-
-static SDATE jdn_civil( long jdn )
-{
-    long l, n, i, j;
-    static SDATE ret;
-    int iday, imonth, iyear;
-
-    l = jdn + 68569;
-    n = ( ( 4 * l ) / 146097 );
-    l = l - ( ( 146097 * n + 3 ) / 4 );
-    i = ( ( 4000 * ( l + 1 ) ) / 1461001 );
-    l = l - ( ( 1461 * i ) / 4 ) + 31;
-    j = ( ( 80 * l ) / 2447 );
-    iday = l - ( ( 2447 * j ) / 80 );
-    l = ( j / 11 );
-    imonth = j + 2 - 12 * l;
-    iyear = 100 * ( n - 49 ) + i + l;
-    ret.day = iday;
-    ret.mon = imonth;
-    ret.year = iyear;
-
-    return ( ret );
-}
-
-static SDATE *jalaliToGregorian( int y, int m, int d )
-{
-    static SDATE sd;
-
-    long jday = jalali_jdn( y, m, d );
-    sd = jdn_civil( jday );
-
-    return ( &sd );
-}
-
-static SDATE *gregorianToJalali( int y, int m, int d )
-{
-    static SDATE sd;
-
-    long   jdn = civil_jdn( y, m, d );//QDate::gregorianToJulian(y, m, d);
-    sd = jdn_jalali( jdn );
-
-    return( &sd );
-}
-
-static void gregorianToJalali( const QDate &date, int *pYear, int *pMonth, int *pDay )
-{
-    SDATE * sd;
-
-    sd = gregorianToJalali( date.year(), date.month(), date.day() );
-    if ( pYear ) {
-        * pYear = sd->year;
-    }
-    if ( pMonth ) {
-        * pMonth = sd->mon;
-    }
-    if ( pDay ) {
-        * pDay = sd->day;
+        if ( q->isLeapYear( year ) ) {
+            return 30;
+        } else {
+            return 29;
+        }
     }
 }
 
-// End of converting functions
-
-static int isJalaliLeap( int year )
-{
-    int     tmp;
-
-    tmp = year % 33;
-    if ( tmp == 1 || tmp == 5 || tmp == 9 || tmp == 13 || tmp == 17 || tmp == 22 || tmp == 26 || tmp == 30 ) {
-        return 1;
-    } else {
-        return 0;
-    }
-}
-
-static int hndays( int m, int y )
-{
-    return jMonthDay[isJalaliLeap( y )][m];
-}
-
-
-//===========================================================================
-
-
 KCalendarSystemJalali::KCalendarSystemJalali( const KLocale * locale )
-                      : KCalendarSystem( locale ), d( 0 )
+                      : KCalendarSystem( locale ),
+                        d( new KCalendarSystemJalaliPrivate( this ) )
 {
 }
 
 KCalendarSystemJalali::~KCalendarSystemJalali()
 {
+    delete d;
 }
 
 QString KCalendarSystemJalali::calendarType() const
@@ -257,32 +86,30 @@ QDate KCalendarSystemJalali::epoch() const
 
 QDate KCalendarSystemJalali::earliestValidDate() const
 {
-    // 19 March 622 in the Julian calendar
-    return KCalendarSystem::earliestValidDate();
+    // Using the Birashk formula which is accurate in period AP 1244 to 1530 (AD 1865 to 2152)
+    // 1244-01-01 Jalali 1865-03-21 Gregorian
+    return QDate::fromJulianDay( 2402317 );
 }
 
 QDate KCalendarSystemJalali::latestValidDate() const
 {
-    // Set to last day of year 9999 until confirm date formats & widets support > 9999
-    // Last day of Jalali year 9999 is 9999-12-29
-    // Which in Gregorian is 10621-03-17
-    // Which is jd xxxx FIXME Find out jd and use that instead
-    // Can't call setDate() as it creates circular reference!
-    return QDate( 10621, 3, 17 );
+    // Using the Birashk formula which is accurate in period AP 1244 to 1530 (AD 1865 to 2152)
+    // 1530-12-29 Jalali 2152-03-19 Gregorian
+    return QDate::fromJulianDay( 2507140 );
 }
 
-bool KCalendarSystemJalali::isValid( int y, int month, int day ) const
+bool KCalendarSystemJalali::isValid( int year, int month, int day ) const
 {
-    // taken from setYMD below, adapted to use new methods
-    if ( y < year( earliestValidDate() ) || y > year( latestValidDate() ) ) {
+    // Using the Birashk formula which is accurate in period AP 1244 to 1530 (AD 1865 to 2152)
+    if ( year < 1244 || year > 1530 ) {
         return false;
     }
 
-    if ( month < 1 || month > 12 ) {  // Fix me!
+    if ( month < 1 || month > 12 ) {
         return false;
     }
 
-    if ( day < 1 || day > hndays( month, y ) ) {
+    if ( day < 1 || day > d->daysInMonth( year, month ) ) {
         return false;
     }
 
@@ -299,120 +126,46 @@ bool KCalendarSystemJalali::setDate( QDate &date, int year, int month, int day )
     return KCalendarSystem::setDate( date, year, month, day );
 }
 
-//MAY BE BUGGY
-bool KCalendarSystemJalali::setYMD( QDate &date, int y, int m, int d ) const
+// Deprecated
+bool KCalendarSystemJalali::setYMD( QDate &date, int year, int month, int day ) const
 {
-    // range checks
-    // Removed deleted minValidYear and maxValidYear methods
-    // Still use minimum of 1753 gregorain for now due to QDate using Julian calendar before then
-    // Later change to following once new methods validated
-    // if ( y < year( earliestValidDate() ) || y > year( latestValidDate() ) )
-    if ( y < year( QDate( 1753, 1, 1 ) ) || y > 9999 ) {
-        return false;
-    }
-
-    if ( m < 1 || m > 12 ) {
-        return false;
-    }
-
-    if ( d < 1 || d > hndays( m, y ) ) {
-        return false;
-    }
-
-    SDATE  *gd = jalaliToGregorian( y, m, d );
-
-    return date.setYMD( gd->year, gd->mon, gd->day );
+    return KCalendarSystem::setYMD( date, year, month, day );
 }
 
 int KCalendarSystemJalali::year( const QDate &date ) const
 {
-    kDebug( 5400 ) << "Jalali year...";
-    int y;
-
-    gregorianToJalali( date, &y, 0, 0 );
-
-    return y;
+    return KCalendarSystem::year( date );
 }
 
 int KCalendarSystemJalali::month ( const QDate& date ) const
 
 {
-    kDebug( 5400 ) << "Jalali month...";
-    int m;
-
-    gregorianToJalali( date, 0 , &m, 0 );
-
-    return m;
+    return KCalendarSystem::month( date );
 }
 
 int KCalendarSystemJalali::day( const QDate &date ) const
 {
-    kDebug( 5400 ) << "Jalali day...";
-    int d;
-
-    gregorianToJalali( date, 0, 0, &d );
-
-    return d;
+    return KCalendarSystem::day( date );
 }
 
-//Does not cope when destination month has fewer days than current month
 QDate KCalendarSystemJalali::addYears( const QDate &date, int nyears ) const
 {
-    QDate result = date;
-
-    int y = year( date ) + nyears;
-    int m = month( date );
-
-    //Quick hack to support destination month with fewer days than current month
-    if ( setYMD( result, y, m, 1 ) ) {
-        setYMD( result, y, m, qMin( day( date ), daysInMonth( result ) ) );
-    }
-
-    return result;
-
-/*  Use this once new jd formulas are verified
     return KCalendarSystem::addYears( date, nyears );
-*/
 }
 
 QDate KCalendarSystemJalali::addMonths( const QDate &date, int nmonths ) const
 {
-    QDate result = date;
-    int m = month( date );
-    int y = year( date );
-
-    if ( nmonths < 0 ) {
-        m += 12;
-        y -= 1;
-    }
-
-    --m; // this only works if we start counting at zero
-    m += nmonths;
-    y += m / 12;
-    m %= 12;
-    ++m;
-
-    //Quick hack to support destination month with fewer days than current month
-    if ( setYMD( result, y, m, 1 ) ) {
-        setYMD( result, y, m, qMin( day( date ), daysInMonth( result ) ) );
-    }
-
-    return result;
-
-/*  Use this once new jd formulas are verified
     return KCalendarSystem::addMonths( date, nmonths );
-*/
 }
 
 QDate KCalendarSystemJalali::addDays( const QDate &date, int ndays ) const
 {
-    return date.addDays( ndays );
+    return KCalendarSystem::addDays( date, ndays );
 }
 
 int KCalendarSystemJalali::monthsInYear( const QDate &date ) const
 {
     Q_UNUSED( date )
-
     return 12;
 }
 
@@ -422,29 +175,30 @@ int KCalendarSystemJalali::weeksInYear( const QDate &date ) const
 }
 
 int KCalendarSystemJalali::weeksInYear( int year ) const
-
 {
-    Q_UNUSED( year );
-// couldn't understand it!
-    return 52;
+    return KCalendarSystem::weeksInYear( year );
 }
 
 int KCalendarSystemJalali::daysInYear( const QDate &date ) const
 {
-    Q_UNUSED( date );
-    int result;
-//SDATE *sd = gregorianToJalali(year(date),month(date),day(date));
-//if (isJalaliLeap(sd->year))
-    result = 366;
-//else
-// result=365;
-    return result;
+    if ( !isValid( date ) ) {
+        return -1;
+    }
+
+    if ( isLeapYear( date ) ) {
+        return 366;
+    } else {
+        return 365;
+    }
 }
 
 int KCalendarSystemJalali::daysInMonth( const QDate &date ) const
 {
-    SDATE * sd = gregorianToJalali( date.year(), date.month(), date.day() );
-    return hndays( sd->mon, sd->year );
+    if ( !isValid( date ) ) {
+        return -1;
+    }
+
+    return d->daysInMonth( year( date ), month( date ) );
 }
 
 int KCalendarSystemJalali::daysInWeek( const QDate &date ) const
@@ -452,69 +206,41 @@ int KCalendarSystemJalali::daysInWeek( const QDate &date ) const
     return KCalendarSystem::daysInWeek( date );
 }
 
-//NOT TESTED YET
 int KCalendarSystemJalali::dayOfYear( const QDate &date ) const
 {
-    QDate first;
-    setYMD( first, year( date ), 1, 1 );
-
-    return first.daysTo( date ) + 1;
+    return KCalendarSystem::dayOfYear( date );
 }
 
 int KCalendarSystemJalali::dayOfWeek( const QDate &date ) const
 {
-    //same same I think?!
-    return date.dayOfWeek();
-
+    return KCalendarSystem::dayOfWeek( date );
 }
 
 int KCalendarSystemJalali::weekNumber( const QDate &date, int *yearNum ) const
 {
-    QDate firstDayWeek1, lastDayOfYear;
-    int y = year( date );
-    int week;
-    int weekDay1, dayOfWeek1InYear;
-
-    // let's guess 1st day of 1st week
-    setYMD( firstDayWeek1, y, 1, 1 );
-    weekDay1 = dayOfWeek( firstDayWeek1 );
-
-    // iso 8601: week 1  is the first containing thursday and week starts on
-    // monday
-    if ( weekDay1 > 4 /*Thursday*/ )
-        firstDayWeek1 = addDays( firstDayWeek1 , 7 - weekDay1 + 1 ); // next monday
-
-    dayOfWeek1InYear = dayOfYear( firstDayWeek1 );
-
-    // if our date in prev year's week
-    if ( dayOfYear( date ) < dayOfWeek1InYear ) {
-        if ( yearNum ) {
-            * yearNum = y - 1;
-        }
-        return weeksInYear( y - 1 );
-    }
-    // let's check if its last week belongs to next year
-    setYMD( lastDayOfYear, y, 12, hndays( 12, y ) );
-    // if our date is in last week && 1st week in next year has thursday
-    if ( ( dayOfYear( date ) >= daysInYear( date ) - dayOfWeek( lastDayOfYear ) + 1 )
-            && dayOfWeek( lastDayOfYear ) < 4 ) {
-        if ( yearNum ) {
-            * yearNum = y + 1;
-        }
-        week = 1;
-    } else {
-        week = firstDayWeek1.daysTo( date ) / 7 + 1;
-    }
-
-    return week;
+    return KCalendarSystem::weekNumber( date, yearNum );
 }
 
 bool KCalendarSystemJalali::isLeapYear( int year ) const
 {
-    // from isJalaliLeap above
-    int     tmp;
-    tmp = year % 33;
-    if ( tmp == 1 || tmp == 5 || tmp == 9 || tmp == 13 || tmp == 17 || tmp == 22 || tmp == 26 || tmp == 30 ) {
+    // From formilab Public Domain code http://www.fourmilab.ch/documents/calendar/
+    // Use Birashk algorithm as it matches the to/from jd code below
+
+    // Birashk algorithm is incorrect in two years in period AP 1244 to 1531,
+    // 1403/1404 and 1436/1437, and so catch them here first
+    if ( year == 1403 || year == 1436 ) {
+        return true;
+    } else if ( year == 1404 || year == 1437 ) {
+        return false;
+    }
+
+    if ( year >= 0 ) {
+        year = year - 474;
+    } else {
+        year = year - 473;
+    }
+
+    if ( ( ( ( ( ( year % 2820 ) + 474 ) + 38 ) * 682 ) % 2816 ) < 682 ) {
         return true;
     } else {
         return false;
@@ -523,7 +249,7 @@ bool KCalendarSystemJalali::isLeapYear( int year ) const
 
 bool KCalendarSystemJalali::isLeapYear( const QDate &date ) const
 {
-    return QDate::isLeapYear( year( date ) );
+    return KCalendarSystem::isLeapYear( date );
 }
 
 QString KCalendarSystemJalali::monthName( int month, int year, MonthNameFormat format )  const
@@ -790,71 +516,112 @@ bool KCalendarSystemJalali::isProleptic() const
 
 bool KCalendarSystemJalali::julianDayToDate( int jd, int &year, int &month, int &day ) const
 {
-    // from jdn_jalali above.  Fix me!
-    if ( jd >= earliestValidDate().toJulianDay() && jd <= latestValidDate().toJulianDay() ) {
-        int y, m, d;
-        int iYear, iMonth, iDay;
-        int depoch;
-        int cycle;
-        int cyear;
-        int ycycle;
-        int aux1, aux2;
-        int yday;
-        d = 1;
-        m = 1;
-        y = 475;
-        depoch = jd - jalali_jdn( y, m, d );
-        cycle = ( int ) ( depoch / 1029983 );
-        cyear = depoch % 1029983;
-        if ( cyear == 1029982 ) {
-            ycycle = 2820;
-        } else {
-            aux1 = cyear / 366;
-            aux2 = cyear % 366;
-            ycycle = ( ( ( 2134 * aux1 ) + ( 2816 * aux2 ) + 2815 ) / 1028522 ) + aux1 + 1;
-        }
-        iYear = ycycle + ( 2820 * cycle ) + 474;
-        if ( iYear <= 0 ) {
-            iYear = iYear - 1;
-        }
-        y = iYear;
-        yday = ( jd - jalali_jdn( y, m, d ) ) + 1;
-        if( yday <= 186 ) {
-            iMonth = Ceil( ( yday - 1 ) / 31 );
-        } else {
-            iMonth = Ceil( ( yday - 7 ) / 30 );
-        }
-        iMonth++;
-        m = iMonth;
-        iDay = ( jd - jalali_jdn( y, m, d ) ) + 1;
-        day = iDay;
-        month = iMonth;
-        year = iYear;
+    // Birashk algorithm is incorrect in two years in period AP 1244 to 1531.
+    // This results in a leap day being added to the end of 1404 instead of 1403
+    // and to the end of 1437 instead of 1436.  Check for these dates first and
+    // return accordingly.  Relies on later use of dateToJulianDay() to correctly
+    // calculate firstDayOfYear in 1404 and 1437, so no other adjustments needed.
+    if ( jd == 2460755 ) {
+        year = 1403;
+        month = 12;
+        day = 30;
         return true;
     }
-    return false;
+    if ( jd == 2472808 ) {
+        year = 1436;
+        month = 12;
+        day = 30;
+        return true;
+    }
+
+    // From original KDE3 code, source unknown?  Unable to contact author or committer to confirm
+    // Matches Fermilab code, EMACS and D&R so check for PD source, likely Birashk's book
+
+    int jdCycleStart;
+    int daysSinceCycleStart;
+    int cycle;
+    int dayInCycle;
+    int yearInCycle;
+    dateToJulianDay( 475, 1, 1, jdCycleStart );
+    daysSinceCycleStart = jd - jdCycleStart;
+    cycle = daysSinceCycleStart / 1029983;
+    dayInCycle = daysSinceCycleStart % 1029983;
+    if ( dayInCycle == 1029982 ) {
+        yearInCycle = 2820;
+    } else {
+        int aux1 = dayInCycle / 366;
+        int aux2 = dayInCycle % 366;
+        yearInCycle = ( ( ( 2134 * aux1 ) + ( 2816 * aux2 ) + 2815 ) / 1028522 ) + aux1 + 1;
+    }
+    year = yearInCycle + ( 2820 * cycle ) + 474;
+    if ( year <= 0 ) {
+        year = year - 1;
+    }
+
+    int firstDayOfYear;
+    dateToJulianDay( year, 1, 1, firstDayOfYear );
+    int dayinYear = jd - firstDayOfYear + 1;
+    if( dayinYear <= 186 ) {
+        month = ( ( dayinYear - 1 ) / 31 ) + 1;
+        day = dayinYear - ( ( month - 1 ) * 31 );
+    } else {
+        month = ( ( dayinYear - 7 ) / 30 ) + 1;
+        day = dayinYear - ( ( month - 1 ) * 30 ) - 6;
+    }
+
+    return true;
 }
 
 bool KCalendarSystemJalali::dateToJulianDay( int year, int month, int day, int &jd ) const
 {
-    // From jalali_jdn above.  Fix me!
-    if ( isValid( year, month, day ) ) {
-        int epbase;
-        long epyear;
-        long mdays;
-        epbase = year - 474;
-        epyear = 474 + ( epbase % 2820 );
-        if ( month <= 7 ) {
-            mdays = ( month - 1 ) * 31;
-        } else {
-            mdays = ( month - 1 ) * 30 + 6;
-        }
-        jd = day + mdays;
-        jd += ( ( ( epyear * 682 ) - 110 ) / 2816 ) ;
-        jd += ( epyear - 1 ) * 365;
-        jd += ( epbase / 2820 ) * 1029983 ;
-        jd += ( epoch().toJulianDay() - 1 );
+
+    // Birashk algorithm is incorrect in two years in period AP 1244 to 1531.
+    // This results in a leap day being added to the end of 1404 instead of 1403
+    // and to the end of 1437 instead of 1436.  Thus all dates in 1404 and 1437
+    // are off by 1 JD.  Check for these dates first and adjust accordingly.
+    if ( year == 1403 && month == 12 && day == 30 ) {
+        jd = 2460755;
         return true;
     }
-    return false;
+    if ( year == 1436 && month == 12 && day == 30 ) {
+        jd = 2472808;
+        return true;
+    }
+    if ( year == 1404 || year == 1437 ) {
+        if ( month < 12 && day + 1 > d->daysInMonth( year, month ) ) {
+            day = 1;
+            month = month + 1;
+        } else {
+            day = day + 1;
+        }
+    }
+
+    // From original KDE3 code, source unknown?  Unable to contact author or committer to confirm
+    // Matches Fermilab code, EMACS and D&R so check for PD source, likely Birashk's book
+    int epbase;
+    long epyear;
+    long monthDays;
+
+    if ( year >= 0 ) {
+        epbase = year - 474;
+    } else {
+        epbase = year - 473;
+    }
+
+    epyear = 474 + ( epbase % 2820 );
+
+    if ( month <= 7 ) {
+        monthDays = ( month - 1 ) * 31;
+    } else {
+        monthDays = ( ( month - 1 ) * 30 ) + 6;
+    }
+
+    jd = ( epoch().toJulianDay() - 1 ) +          // days before epoch
+         ( epyear - 1 ) * 365 +                   // normal days in previous years
+         ( ( ( epyear * 682 ) - 110 ) / 2816 ) +  // leap days in previous years
+         ( epbase / 2820 ) * 1029983 +
+         monthDays +                              // days in previous months this year
+         day;                                     // days in this month
+
+    return true;
 }
