@@ -29,9 +29,17 @@ class KDebugTest : public QObject
 
 private Q_SLOTS:
     void initTestCase();
+    void cleanupTestCase();
+
     void testDebugToFile();
     void testDisableArea();
-    void cleanupTestCase();
+    void testDynamicArea();
+    void testDisabledDynamicArea();
+    void testHasNullOutput();
+    void testMultipleThreads();
+
+private:
+    void compareLines(const QList<QByteArray>& expectedLines, const char* fileName = "kdebug.dbg");
 };
 
 #endif
