@@ -20,6 +20,7 @@
 #include "klocaletest.h"
 #include "qtest_kde.h"
 
+#include "kdebug.h"
 #include "klocale.h"
 #include "kglobal.h"
 #include "kcalendarsystem.h"
@@ -34,7 +35,6 @@
 void
 KLocaleTest::initTestCase()
 {
-    KGlobal::setLocale(new KLocale(QString(), QLatin1String("en_US"), QLatin1String("C")));
     KGlobal::locale()->setThousandsSeparator(QLatin1String(","));
 }
 
@@ -43,6 +43,7 @@ KLocaleTest::formatNumbers()
 {
 	KLocale locale(*KGlobal::locale());
 
+    KGlobal::locale()->setDecimalPlaces(2);
 	QCOMPARE(locale.formatNumber(70), QString("70.00"));
 	QCOMPARE(locale.formatNumber(70, 0), QString("70"));
 	QCOMPARE(locale.formatNumber(70.2), QString("70.20"));
@@ -57,6 +58,7 @@ KLocaleTest::formatNumberStrings()
 {
 	KLocale locale(*KGlobal::locale());
 
+    KGlobal::locale()->setDecimalPlaces(2);
 	QCOMPARE(locale.formatNumber("70"), QString("70.00"));
 	QCOMPARE(locale.formatNumber("70", true, 2), QString("70.00"));
 	QCOMPARE(locale.formatNumber("70", true, 0), QString("70"));
@@ -152,7 +154,6 @@ KLocaleTest::readDate()
 	QString full("dddd dd MMMM yyyy");
 	bool ok=false;
 	QDate date;
-
 	date.setYMD( 2002, 5, 3 );
 	QCOMPARE(locale.readDate(date.toString(small)), date);
 	QCOMPARE(locale.readDate(date.toString(full)), date);
@@ -180,20 +181,21 @@ KLocaleTest::readDate()
 void
 KLocaleTest::formatTime()
 {
-	KLocale locale(*KGlobal::locale());
-	QTime time(0,22,33);
+    KLocale locale(*KGlobal::locale());
+    QTime time(0,22,33);
 
-	locale.setTimeFormat("%H:%M %p");
-	QCOMPARE(locale.formatTime(time, true, false), QString("00:22 am"));
-	QCOMPARE(locale.formatTime(time, true, true), QString("00:22"));
+    locale.setTimeFormat("%H:%M %p");
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeWithoutSeconds), QString("00:22 am"));
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeWithoutSeconds | KLocale::TimeDuration),
+             QString("00:22"));
 
-	locale.setTimeFormat("%H:%M:%S %p");
-	QCOMPARE(locale.formatTime(time, true, false), QString("00:22:33 am"));
-	QCOMPARE(locale.formatTime(time, true, true), QString("00:22:33"));
+    locale.setTimeFormat("%H:%M:%S %p");
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeDefault), QString("00:22:33 am"));
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeDuration), QString("00:22:33"));
 
-	locale.setTimeFormat("%l : %M : %S %p"); // #164813
-	QCOMPARE(locale.formatTime(time, true), QString("12 : 22 : 33 am"));
-	QCOMPARE(locale.formatTime(time, false), QString("12 : 22 am"));
+    locale.setTimeFormat("%l : %M : %S %p"); // #164813
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeDefault), QString("12 : 22 : 33 am"));
+    QCOMPARE(locale.formatLocaleTime(time, KLocale::TimeWithoutSeconds), QString("12 : 22 am"));
 }
 
 void
@@ -298,20 +300,30 @@ KLocaleTest::formatDateTime()
 void
 KLocaleTest::readTime()
 {
-	KLocale locale(*KGlobal::locale());
-	bool ok = false;
+    KLocale locale(*KGlobal::locale());
+    bool ok = false;
 
-	QCOMPARE(locale.readTime("11:22:33", &ok), QTime(11,22,33));
-	QVERIFY(ok);
+    QCOMPARE(locale.readTime("11:22:33", &ok), QTime(11,22,33));
+    QVERIFY(ok);
 
-	QCOMPARE(locale.readTime("11:22", &ok), QTime(11,22,0));
-	QVERIFY(ok);
+    QCOMPARE(locale.readTime("11:22", &ok), QTime(11,22,0));
+    QVERIFY(ok);
 
-	locale.readTime("11:22:33", KLocale::WithoutSeconds, &ok);
-	QVERIFY(!ok);
+    locale.readLocaleTime("11:22:33", &ok, KLocale::TimeWithoutSeconds);
+    QVERIFY(!ok);
 
-	QCOMPARE(locale.readTime("11:22", KLocale::WithoutSeconds, &ok), QTime(11, 22, 0));
-	QVERIFY(ok);
+    QCOMPARE(locale.readLocaleTime("11:22", &ok, KLocale::TimeWithoutSeconds), QTime(11, 22, 0));
+    QVERIFY(ok);
+
+    // strict processing of a time string with a missing space
+    locale.setTimeFormat("%I:%M:%S %p");
+    QString timeString = QString("11:13:55%1").arg(i18n("am"));
+    locale.readLocaleTime(timeString, &ok, KLocale::TimeDefault, KLocale::ProcessStrict);
+    QVERIFY(!ok);
+
+    // lax processing of the same time
+    QCOMPARE(locale.readLocaleTime(timeString, &ok, KLocale::TimeDefault), QTime(11, 13, 55));
+    QVERIFY(ok);
 }
 
 void
@@ -337,6 +349,30 @@ KLocaleTest::prettyFormatDuration()
 	QCOMPARE(locale.prettyFormatDuration(3610000), QString("1 hour"));
 }
 
+void
+KLocaleTest::formatByteSize()
+{
+    KLocale locale(*KGlobal::locale());
+
+    QCOMPARE(locale.formatByteSize(0), QString("0 B"));
+    QCOMPARE(locale.formatByteSize(50), QString("50 B"));
+    QCOMPARE(locale.formatByteSize(500), QString("500 B"));
+    QCOMPARE(locale.formatByteSize(5000), QString("4.9 KiB"));
+    QCOMPARE(locale.formatByteSize(50000), QString("48.8 KiB"));
+    QCOMPARE(locale.formatByteSize(500000), QString("488.3 KiB"));
+    QCOMPARE(locale.formatByteSize(5000000), QString("4.8 MiB"));
+    QCOMPARE(locale.formatByteSize(50000000), QString("47.7 MiB"));
+    QCOMPARE(locale.formatByteSize(500000000), QString("476.8 MiB"));
+#if LONG_IS_64_BITS
+    QCOMPARE(locale.formatByteSize(5000000000), QString("4.7 GiB"));
+    QCOMPARE(locale.formatByteSize(50000000000), QString("46.6 GiB"));
+    QCOMPARE(locale.formatByteSize(500000000000), QString("465.7 GiB"));
+    QCOMPARE(locale.formatByteSize(5000000000000), QString("4.5 TiB"));
+    QCOMPARE(locale.formatByteSize(50000000000000), QString("45.5 TiB"));
+    QCOMPARE(locale.formatByteSize(500000000000000), QString("454.7 TiB"));
+#endif
+}
+
 void
 KLocaleTest::bug95511()
 {
@@ -457,4 +493,53 @@ KLocaleTest::removeAcceleratorMarker()
 	         QString("Foo & Bar"));
 }
 
+void
+KLocaleTest::formatByteSize2()
+{
+    KLocale locale(*KGlobal::locale());
+
+    // IEC is also default
+    locale.setBinaryUnitDialect(KLocale::IECBinaryDialect);
+    QCOMPARE(locale.binaryUnitDialect(), KLocale::IECBinaryDialect);
+
+    QCOMPARE(locale.formatByteSize(1024.0), QString("1.0 KiB"));
+    QCOMPARE(locale.formatByteSize(1023.0), QString("1,023 B"));
+    QCOMPARE(locale.formatByteSize(1163000.0), QString("1.1 MiB")); // 1.2 metric
+
+    locale.setBinaryUnitDialect(KLocale::JEDECBinaryDialect);
+    QCOMPARE(locale.binaryUnitDialect(), KLocale::JEDECBinaryDialect);
+
+    QCOMPARE(locale.formatByteSize(1024.0), QString("1.0 KB"));
+    QCOMPARE(locale.formatByteSize(1023.0), QString("1,023 B"));
+    QCOMPARE(locale.formatByteSize(1163000.0), QString("1.1 MB"));
+
+    locale.setBinaryUnitDialect(KLocale::MetricBinaryDialect);
+    QCOMPARE(locale.binaryUnitDialect(), KLocale::MetricBinaryDialect);
+
+    QCOMPARE(locale.formatByteSize(1024.0), QString("1.0 kB"));
+    QCOMPARE(locale.formatByteSize(1023.0), QString("1.0 kB"));
+    QCOMPARE(locale.formatByteSize(1163000.0), QString("1.2 MB"));
+
+    // Ensure specific dialects work, still on metric by default
+    QCOMPARE(locale.formatByteSize(1024.0, 1, KLocale::JEDECBinaryDialect), QString("1.0 KB"));
+    QCOMPARE(locale.formatByteSize(2097152.0, 1, KLocale::IECBinaryDialect), QString("2.0 MiB"));
+
+    locale.setBinaryUnitDialect(KLocale::JEDECBinaryDialect);
+    QCOMPARE(locale.binaryUnitDialect(), KLocale::JEDECBinaryDialect);
+    QCOMPARE(locale.formatByteSize(2097152.0, 1, KLocale::MetricBinaryDialect), QString("2.1 MB"));
+
+    // Ensure all units are represented (still in JEDEC mode)
+    QCOMPARE(locale.formatByteSize(2.0e9, 1, KLocale::MetricBinaryDialect), QString("2.0 GB"));
+    QCOMPARE(locale.formatByteSize(3.2e12, 1, KLocale::MetricBinaryDialect), QString("3.2 TB"));
+    QCOMPARE(locale.formatByteSize(4.1e15, 1, KLocale::MetricBinaryDialect), QString("4.1 PB"));
+    QCOMPARE(locale.formatByteSize(6.7e18, 2, KLocale::MetricBinaryDialect), QString("6.70 EB"));
+    QCOMPARE(locale.formatByteSize(5.6e20, 2, KLocale::MetricBinaryDialect), QString("560.00 EB"));
+    QCOMPARE(locale.formatByteSize(2.3e22, 2, KLocale::MetricBinaryDialect), QString("23.00 ZB"));
+    QCOMPARE(locale.formatByteSize(1.0e27, 1, KLocale::MetricBinaryDialect), QString("1,000.0 YB"));
+
+    // Spattering of specific units
+    QCOMPARE(locale.formatByteSize(823000, 3, KLocale::IECBinaryDialect, KLocale::UnitMegaByte), QString("0.785 MiB"));
+    QCOMPARE(locale.formatByteSize(1234034.0, 4, KLocale::JEDECBinaryDialect, KLocale::UnitByte), QString("1,234,034 B"));
+}
+
 QTEST_KDEMAIN_CORE(KLocaleTest)
