@@ -1,9 +1,11 @@
 /*
- *  Copyright (C) 2005 David Faure   <faure@kde.org>
+ *  Copyright (C) 2005-2009 David Faure   <faure@kde.org>
  *
- *  This library is free software; you can redistribute it and/or
- *  modify it under the terms of the GNU Library General Public
- *  License version 2 as published by the Free Software Foundation;
+ *  This library is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License or ( at
+ *  your option ) version 3 or, at the discretion of KDE e.V. ( which shall
+ *  act as a proxy as in section 14 of the GPLv3 ), any later version.
  *
  *  This library is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,8 +21,8 @@
 #include <config.h>
 #include <kdefakes.h>
 #include "kmimetypetest.h"
+#include <kde_file.h>
 #include <kprocess.h>
-#include "kmimetypetest.moc"
 #include <kmimetype.h>
 #include <ksycoca.h>
 #include <kglobal.h>
@@ -36,10 +38,18 @@
 #include <kservicetypetrader.h>
 #include <kmimetypefactory.h>
 #include <ktemporaryfile.h>
+#include <ktempdir.h>
 #include <kdesktopfile.h>
 
 void KMimeTypeTest::initTestCase()
 {
+    // Clean up local xdg dir in case of leftover mimetype definitions
+    const QString xdgDir = QString::fromLocal8Bit(getenv("XDG_DATA_HOME"));
+    if (!xdgDir.isEmpty()) {
+        KTempDir::removeDir(xdgDir);
+        // No need to run update-mime-database here, the dir is entirely gone.
+    }
+
     // Create fake text/plain part with a higher initial preference than katepart.
     const QString fakePart = KStandardDirs::locateLocal("services", "faketextpart.desktop");
     bool mustUpdateKSycoca = false;
@@ -123,6 +133,10 @@ void KMimeTypeTest::testByName()
 
     KMimeType::Ptr bzip2 = KMimeType::mimeType("application/x-bzip2", KMimeType::ResolveAliases);
     QVERIFY( bzip2 );
+
+    KMimeType::Ptr defaultMime = KMimeType::mimeType("application/octet-stream");
+    QVERIFY(defaultMime);
+    QVERIFY(defaultMime->isDefault());
 }
 
 void KMimeTypeTest::testIcons()
@@ -139,9 +153,7 @@ void KMimeTypeTest::testIcons()
         tmp.setAutoRemove( true );
         KUrl url( tmp.name() );
         checkIcon( url, "inode-directory" ); // was folder_locked, but we don't have that anymore - TODO
-#ifndef Q_WS_WIN
-        chmod( QFile::encodeName( tmp.name() ), 0500 ); // so we can 'rm -rf' it
-#endif
+        KDE::chmod( tmp.name(), 0500 ); // so we can 'rm -rf' it
     }
 }
 
@@ -156,20 +168,23 @@ void KMimeTypeTest::testFindByPathUsingFileName_data()
 
     QTest::newRow("text") << "textfile.txt" << "text/plain";
     QTest::newRow("case-insensitive search") << "textfile.TxT" << "text/plain";
+
     QTest::newRow("case-sensitive uppercase match") << "textfile.C" << "text/x-c++src";
     QTest::newRow("case-sensitive lowercase match") << "textfile.c" << "text/x-csrc";
+    QTest::newRow("case-sensitive long-extension match") << "foo.PS.gz" << "application/x-gzpostscript";
+    QTest::newRow("case-sensitive-only match") << "core" << "application/x-core";
+    QTest::newRow("case-sensitive-only match") << "Core" << "application/octet-stream"; // #198477
+
     QTest::newRow("desktop file") << "foo.desktop" << "application/x-desktop";
     QTest::newRow("old kdelnk file is x-desktop too") << "foo.kdelnk" << "application/x-desktop";
     QTest::newRow("double-extension file") << "foo.tar.bz2" << "application/x-bzip-compressed-tar";
     QTest::newRow("single-extension file") << "foo.bz2" << "application/x-bzip";
-//    QTest::newRow(".doc should assume msword") << "somefile.doc" << "application/msword";
+    QTest::newRow(".doc should assume msword") << "somefile.doc" << "application/msword"; // #204139
     QTest::newRow("glob that uses [] syntax, 1") << "Makefile" << "text/x-makefile";
     QTest::newRow("glob that uses [] syntax, 2") << "makefile" << "text/x-makefile";
     QTest::newRow("glob that ends with *, no extension") << "README" << "text/x-readme";
     QTest::newRow("glob that ends with *, extension") << "README.foo" << "text/x-readme";
     QTest::newRow("glob that ends with *, also matches *.txt. Higher weight wins.") << "README.txt" << "text/plain";
-    // This test has undefined results if we can't use sniffing to sort out whether it's word or text/plain.
-    //QTest::newRow("glob that ends with *, also matches *.doc, which is either word or text/plain.") << "README.doc" << "text/plain";
     QTest::newRow("glob that ends with *, also matches *.nfo. Higher weight wins.") << "README.nfo" << "text/x-nfo";
     // fdo bug 15436, needs shared-mime-info >= 0.40 (and this tests the globs2-parsing code).
     QTest::newRow("glob that ends with *, also matches *.pdf. *.pdf has higher weight") << "README.pdf" << "application/pdf";
@@ -178,9 +193,13 @@ void KMimeTypeTest::testFindByPathUsingFileName_data()
     QTest::newRow("doesn't exist but has known extension") << "IDontExist.txt" << "text/plain";
 
     // Can't use KIconLoader since this is a "without GUI" test.
-    QString fh = KStandardDirs::locate( "icon", "oxygen/22x22/places/folder.png" );
-    QVERIFY( !fh.isEmpty() ); // if the file doesn't exist, please fix the above to point to an existing icon
-    QTest::newRow("png image") << fh << "image/png";
+    if (!KStandardDirs::locate("icon", "oxygen/").isEmpty()) {
+        QString fh = KStandardDirs::locate("icon", "oxygen/22x22/places/folder.png");
+        QVERIFY(!fh.isEmpty()); // if the file doesn't exist, please fix the above to point to an existing icon
+        QTest::newRow("png image") << fh << "image/png";
+    } else {
+        kWarning() << "oxygen icon theme not found";
+    }
 
     QString exePath = KStandardDirs::findExe( "kioexec" );
     if ( exePath.isEmpty() )
@@ -375,14 +394,14 @@ void KMimeTypeTest::testFindByContent_data()
     QTest::addColumn<QString>("expectedMimeType");
     QTest::newRow("simple text") << QByteArray("Hello world") << "text/plain";
     QTest::newRow("html: <html>") << QByteArray("<html>foo</html>") << "text/html";
-#if 0 // currently buggy, xml and html have conflicting magic rules, discussed on xdg list.
-      // Solution: making the magic rules for xml priority 40
+
+    // fixed in smi-0.30, xml magic has prio 40
     QTest::newRow("html: comment+<html>") << QByteArray("<!--foo--><html>foo</html>") << "text/html";
-#endif
-#if 0 // https://bugs.freedesktop.org/show_bug.cgi?id=11259
+    // https://bugs.freedesktop.org/show_bug.cgi?id=11259, fixed in smi-0.22
     QTest::newRow("html: <script>") << QByteArray("<script>foo</script>") << "text/html";
-#endif
+
     QTest::newRow("pdf") << QByteArray("%PDF-") << "application/pdf";
+    QTest::newRow("no mimetype known") << QByteArray("\261\032\341\265") << "application/octet-stream";
 
     QByteArray mswordData = "\320\317\021\340\241\261\032\341";
     // same as \xD0\xCF\x11\xE0 \xA1\xB1\x1A\xE1
@@ -451,7 +470,7 @@ void KMimeTypeTest::testAlias()
 
     const KMimeType::Ptr canonical = KMimeType::mimeType( "application/xml" );
     QVERIFY( canonical );
-    KMimeType::Ptr alias = KMimeType::mimeType( "text/xml" );
+    KMimeType::Ptr alias = KMimeType::mimeType("text/xml", KMimeType::DontResolveAlias);
     QVERIFY( !alias );
     alias = KMimeType::mimeType( "text/xml", KMimeType::ResolveAliases );
     QVERIFY( alias );
@@ -513,8 +532,8 @@ void KMimeTypeTest::testMimeTypeParent()
     // Must be least-specific last, i.e. breadth first.
     QCOMPARE(allShellParents.last(), QString("application/octet-stream"));
 
-    // Check that text/mrml knows that it inherits from text/plain (implicitly)
-    const KMimeType::Ptr mrml = KMimeType::mimeType("text/mrml");
+    // Check that text/x-mrml knows that it inherits from text/plain (implicitly)
+    const KMimeType::Ptr mrml = KMimeType::mimeType("text/x-mrml");
     if (!mrml)
         QSKIP("kdelibs not installed", SkipAll);
     QVERIFY(mrml->is("text/plain"));
@@ -534,8 +553,7 @@ void KMimeTypeTest::testMimeTypeInheritancePerformance()
     KMimeType::Ptr mime = KMimeType::mimeType("text/x-chdr");
     QVERIFY(mime);
     QTime dt; dt.start();
-    const int numIterations = 2000;
-    for (int i = 0; i < numIterations; ++i) {
+    QBENCHMARK {
         QString match;
         foreach (const QString& mt, mimeTypes) {
             if (mime->is(mt)) {
@@ -546,7 +564,13 @@ void KMimeTypeTest::testMimeTypeInheritancePerformance()
         }
         QCOMPARE(match, QString("text/plain"));
     }
-    qDebug() << "Calling is()" << mimeTypes.count() * numIterations << "times took" << dt.elapsed() << "msecs";
+    // Results on David's machine (April 2009):
+    // With the KMimeType::is() code that loaded every parent KMimeType:
+    // 3.5 msec / 7,000,000 ticks / 5,021,498 instr. loads per iteration
+    // After the QHash for parent mimetypes in ksycoca, removing the need to load full mimetypes:
+    // 0.57 msec / 1,115,000 ticks / 938,356 instr. loads per iteration
+    // After converting the QMap for aliases into a QHash too:
+    // 0.48 msec / 960,000 ticks / 791,404 instr. loads per iteration
 }
 
 // Helper method for all the trader tests
@@ -668,16 +692,24 @@ void KMimeTypeTest::testPatterns_data()
 {
     QTest::addColumn<QString>("mimeType");
     QTest::addColumn<QString>("patterns");
-    QTest::newRow("mimetype with a single pattern") << "application/pdf" << "*.pdf";
-    QTest::newRow("mimetype with multiple patterns") << "application/x-kpresenter" << "*.kpr;*.kpt";
-    QTest::newRow("mimetype with multiple patterns, *.doc added by kde") << "text/plain" << "*.asc;*.txt;*.doc;*,v";
-    QTest::newRow("mimetype with no patterns") << "application/pkcs7-mime" << QString();
+    QTest::addColumn<QString>("mainExtension");
+    QTest::newRow("mimetype with a single pattern") << "application/pdf" << "*.pdf" << ".pdf";
+    QTest::newRow("mimetype with multiple patterns") << "application/x-kpresenter" << "*.kpr;*.kpt" << ".kpr";
+    if (KMimeType::sharedMimeInfoVersion() > KDE_MAKE_VERSION(0, 60, 0)) {
+        QTest::newRow("mimetype with many patterns") << "application/vnd.wordperfect" << "*.wp;*.wp4;*.wp5;*.wp6;*.wpd;*.wpp" << ".wp";
+    }
+    QTest::newRow("oasis text mimetype") << "application/vnd.oasis.opendocument.text" << "*.odt" << ".odt";
+    QTest::newRow("oasis presentation mimetype") << "application/vnd.oasis.opendocument.presentation" << "*.odp" << ".odp";
+    QTest::newRow("mimetype with multiple patterns, *.doc added by kde") << "text/plain" << "*.asc;*.txt;*.doc;*,v" << ".txt";
+    QTest::newRow("mimetype with uncommon pattern") << "application/x-kcachegrind" << "callgrind.out*;cachegrind.out*" << QString();
+    QTest::newRow("mimetype with no patterns") << "application/pkcs7-mime" << QString() << QString();
 }
 
 void KMimeTypeTest::testPatterns()
 {
     QFETCH(QString, mimeType);
     QFETCH(QString, patterns);
+    QFETCH(QString, mainExtension);
     KMimeType::Ptr mime = KMimeType::mimeType( mimeType );
     QVERIFY(mime);
     // Sort both lists; order is unreliable since shared-mime-info uses hashes internally.
@@ -690,6 +722,8 @@ void KMimeTypeTest::testPatterns()
         mimePatterns.append("*,v");
     mimePatterns.sort();
     QCOMPARE(mimePatterns.join(";"), expectedPatterns.join(";"));
+
+    QCOMPARE(mime->mainExtension(), mainExtension);
 }
 
 void KMimeTypeTest::testExtractKnownExtension_data()
@@ -809,9 +843,7 @@ void KMimeTypeTest::testParseMagicFile()
     QVERIFY(testBuffer.open(QIODevice::ReadOnly));
     QString found;
     QByteArray beginning;
-    for ( QList<KMimeMagicRule>::const_iterator it = m_rules.begin(), end = m_rules.end();
-          it != end; ++it ) {
-        const KMimeMagicRule& rule = *it;
+    Q_FOREACH(const KMimeMagicRule& rule, m_rules) {
         if (rule.match(&testBuffer, beginning)) {
             found = rule.mimetype();
             break;
@@ -820,3 +852,5 @@ void KMimeTypeTest::testParseMagicFile()
     QCOMPARE(found, expected);
     testBuffer.close();
 }
+
+#include "kmimetypetest.moc"
