@@ -65,6 +65,7 @@
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kseparator.h>
+#include <kstandarddirs.h>
 #include <kcolorcollection.h>
 
 #include "kcolormimedata.h"
@@ -435,7 +436,7 @@ public:
 
 KColorPatch::KColorPatch(QWidget *parent) : QFrame(parent), d(new KColorPatchPrivate(this))
 {
-    setFrameStyle(QFrame::Panel | QFrame::Sunken);
+    setFrameStyle(QFrame::StyledPanel | QFrame::Sunken);
     setAcceptDrops(true);
     setMinimumSize(12, 12);
 }
@@ -445,6 +446,11 @@ KColorPatch::~KColorPatch()
     delete d;
 }
 
+QColor KColorPatch::color() const
+{
+    return d->color;
+}
+
 void KColorPatch::setColor(const QColor &col)
 {
     d->color.setRgb(col.rgb());
@@ -573,16 +579,31 @@ static const char * const *namedColorFilePath(void)
 {
     //
     // 2000-02-05 Espen Sand.
-    // Add missing filepaths here. Make sure the last entry is 0!
+    // Add missing filepaths here. Make sure the last entry is 0, 0!
+    //
+    // 2009-06-16 Pino Toscano
+    //
+    // You can specify either absolute paths or relative locations
+    // wrt KStandardDirs resources. In either way, there should be two
+    // "strings" for each path.
+    // - absolute path: specify it directly, then add 0 as second item
+    //   * example: "/usr/share/X11/rgb.txt", 0,
+    // - KStandardDirs location: specify the filename as first item,
+    //   then add the resource as second
+    //   * example: "kdeui/rgb.txt", "data",
     //
     static const char * const path[] = {
+#ifdef Q_WS_X11
 #ifdef X11_RGBFILE
-        X11_RGBFILE,
+        X11_RGBFILE, 0,
 #endif
-        "/usr/share/X11/rgb.txt",
-        "/usr/X11R6/lib/X11/rgb.txt",
-        "/usr/openwin/lib/X11/rgb.txt", // for Solaris.
-        0
+        "/usr/share/X11/rgb.txt", 0,
+        "/usr/X11R6/lib/X11/rgb.txt", 0,
+        "/usr/openwin/lib/X11/rgb.txt", 0, // for Solaris.
+#else /* systems without X11 */
+        "kdeui/rgb.txt", "data",
+#endif
+        0, 0
     };
     return path;
 }
@@ -604,8 +625,18 @@ KColorTable::readNamedColor(void)
     //
 
     const char * const *path = namedColorFilePath();
-    for (int i = 0; path[i]; ++i) {
-        QFile paletteFile(path[i]);
+    for (int i = 0; path[i]; i += 2) {
+        QString file;
+        if (path[i + 1]) {
+            file = KStandardDirs::locate(path[i + 1], QString::fromLatin1(path[i]));
+            if (file.isEmpty()) {
+                continue;
+            }
+        } else {
+            file = QString::fromLatin1(path[i]);
+        }
+
+        QFile paletteFile(file);
         if (!paletteFile.open(QIODevice::ReadOnly)) {
             continue;
         }
@@ -661,16 +692,27 @@ void
 KColorTable::KColorTablePrivate::slotShowNamedColorReadError(void)
 {
     if (mNamedColorList->count() == 0) {
-        QString msg = i18n(""
-                           "Unable to read X11 RGB color strings. The following "
-                           "file location(s) were examined:\n");
+        QString pathMsg;
+        int pathCount = 0;
 
         const char * const *path = namedColorFilePath();
-        for (int i = 0; path[i]; ++i) {
-            msg += path[i];
-            msg += '\n';
+        for (int i = 0; path[i]; i += 2, ++pathCount) {
+            if (path[i + 1]) {
+                pathMsg += QLatin1String(path[i + 1]) + ", " + QString::fromLatin1(path[i]);
+            } else {
+                pathMsg += QLatin1String(path[i]);
+            }
+            pathMsg += '\n';
         }
-        KMessageBox::sorry(q, msg);
+
+        QString finalMsg  = i18ncp("%1 is the number of paths, %2 is the list of paths (with newlines between them)",
+                                   "Unable to read X11 RGB color strings. The following "
+                                   "file location was examined:\n%2",
+                                   "Unable to read X11 RGB color strings. The following "
+                                   "file locations were examined:\n%2",
+                                   pathCount, pathMsg );
+
+        KMessageBox::sorry(q, finalMsg);
     }
 }
 
@@ -970,7 +1012,6 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
 
     QGridLayout *tl_layout = new QGridLayout(page);
     tl_layout->setMargin(0);
-    tl_layout->setSpacing(spacingHint());
     d->tl_layout = tl_layout;
     tl_layout->addItem(new QSpacerItem(spacingHint()*2, 0), 0, 1);
 
@@ -989,7 +1030,7 @@ KColorDialog::KColorDialog(QWidget *parent, bool modal)
     l_left->addLayout(l_ltop);
 
     // a little space between
-    l_left->addSpacing(10);
+    l_left->addSpacing(10); // FIXME: remove hardcoded values
 
     QGridLayout *l_lbot = new QGridLayout();
     l_left->addLayout(l_lbot);
@@ -1370,8 +1411,16 @@ KColorDialog::readSettings()
     KConfigGroup group(KGlobal::config(), "Colors");
 
     QString collectionName = group.readEntry("CurrentPalette");
-    if (collectionName.isEmpty())
+    if (collectionName.isEmpty()) {
         collectionName = i18nc("palette name", colorCollectionName[fortyColorIndex].m_displayName);
+    } else {
+        for (int i = 0; colorCollectionName[i].m_fileName; ++i) {
+            if (collectionName == colorCollectionName[i].m_displayName) {
+                collectionName = i18nc("palette name", colorCollectionName[i].m_displayName);
+                break;
+            }
+        }
+    }
 
     d->table->setColors(collectionName);
 }
@@ -1385,7 +1434,14 @@ KColorDialog::KColorDialogPrivate::slotWriteSettings()
     if (!group.hasDefault("CurrentPalette") && table->name() == originalPalette) {
         group.revertToDefault("CurrentPalette");
     } else {
-        group.writeEntry("CurrentPalette", table->name()); //Shouldn't here the unstranslated name be saved ??
+        QString collectionName(table->name());
+        for (int i = 0; colorCollectionName[i].m_fileName; ++i) {
+            if (collectionName == i18nc("palette name", colorCollectionName[i].m_displayName)) {
+                collectionName = colorCollectionName[i].m_displayName;
+                break;
+            }
+        }
+        group.writeEntry("CurrentPalette", collectionName); //Make sure the untranslated name is saved, assuming there is one
     }
 }
 
@@ -1809,4 +1865,3 @@ void KColorDialog::KColorDialogPrivate::setHsvEdit(const QColor &col)
 
 #include "kcolordialog.moc"
 #include "kcolordialog_p.moc"
-//#endif
