@@ -53,7 +53,7 @@
 class KBugReportPrivate {
 public:
     KBugReportPrivate(KBugReport *q): q(q) {}
-  
+
     void _k_slotConfigureEmail();
     void _k_slotSetFrom();
     void _k_appChanged(int);
@@ -62,7 +62,7 @@ public:
     KBugReport *q;
     QProcess * m_process;
     const KAboutData * m_aboutData;
-    
+
     KTextEdit * m_lineedit;
     KLineEdit * m_subject;
     QLabel * m_from;
@@ -70,7 +70,7 @@ public:
     QString m_strVersion;
     QGroupBox * m_bgSeverity;
     QPushButton * m_configureEmail;
-  
+
     KComboBox *appcombo;
     QString lastError;
     QString kde_version;
@@ -199,7 +199,7 @@ KBugReport::KBugReport( QWidget * _parent, bool modal, const KAboutData *aboutDa
   if (d->m_aboutData)
       d->m_strVersion = d->m_aboutData->version();
   else
-      d->m_strVersion = i18n("no version set (programmer error!)");
+      d->m_strVersion = i18n("no version set (programmer error)");
   d->kde_version = QString::fromLatin1( KDE_VERSION_STRING );
   d->kde_version += ", " + QString::fromLatin1( KDE_DISTRIBUTION_TEXT );
   if ( !d->submitBugWeb )
@@ -273,6 +273,8 @@ KBugReport::KBugReport( QWidget * _parent, bool modal, const KAboutData *aboutDa
     d->m_lineedit->setMinimumHeight( 180 ); // make it big
     d->m_lineedit->setWordWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
     d->m_lineedit->setLineWrapMode(QTextEdit::WidgetWidth);
+    d->m_lineedit->setCheckSpellingEnabled(true);
+    d->m_lineedit->setSpellCheckingLanguage("en");
     lay->addWidget( d->m_lineedit, 10 /*stretch*/ );
 
     d->_k_slotSetFrom();
@@ -290,6 +292,8 @@ KBugReport::KBugReport( QWidget * _parent, bool modal, const KAboutData *aboutDa
     lay->addWidget( label );
     lay->addSpacing(10);
 
+    d->appcombo->setFocus();
+
     d->_k_updateUrl();
 
     setButtonText(Ok, i18n("&Launch Bug Report Wizard"));
@@ -337,15 +341,21 @@ void KBugReportPrivate::_k_appChanged(int i)
         appName = appName.left( index );
     kDebug() << "appName " << appName;
 
-    if (appname == appName && m_aboutData)
+    QString strDisplayVersion; //Version string to show in the UI
+    if (appname == appName && m_aboutData) {
         m_strVersion = m_aboutData->version();
-    else
-        m_strVersion = i18nc("unknown program name", "unknown");
+        strDisplayVersion = m_strVersion;
+    } else {
+        m_strVersion = QLatin1String("unknown"); //English string to put in the bug report
+        strDisplayVersion = i18nc("unknown program name", "unknown");
+    }
 
-    if ( !submitBugWeb )
+    if ( !submitBugWeb ) {
         m_strVersion += ' ' + kde_version;
+        strDisplayVersion += ' ' + kde_version;
+    }
 
-    m_version->setText(m_strVersion);
+    m_version->setText(strDisplayVersion);
     if ( submitBugWeb )
         _k_updateUrl();
 }
@@ -416,23 +426,23 @@ void KBugReport::accept()
         case 0: // critical
             if ( KMessageBox::questionYesNo( this, i18n(
                 "<p>You chose the severity <b>Critical</b>. "
-                "Please note that this severity is intended only for bugs that</p>"
+                "Please note that this severity is intended only for bugs that:</p>"
                 "<ul><li>break unrelated software on the system (or the whole system)</li>"
                 "<li>cause serious data loss</li>"
                 "<li>introduce a security hole on the system where the affected package is installed</li></ul>\n"
                 "<p>Does the bug you are reporting cause any of the above damage? "
-                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
+                "If it does not, please select a lower severity. Thank you.</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
                 return;
             break;
         case 1: // grave
             if ( KMessageBox::questionYesNo( this, i18n(
                 "<p>You chose the severity <b>Grave</b>. "
-                "Please note that this severity is intended only for bugs that</p>"
+                "Please note that this severity is intended only for bugs that:</p>"
                 "<ul><li>make the package in question unusable or mostly so</li>"
                 "<li>cause data loss</li>"
                 "<li>introduce a security hole allowing access to the accounts of users who use the affected package</li></ul>\n"
                 "<p>Does the bug you are reporting cause any of the above damage? "
-                "If it does not, please select a lower severity. Thank you!</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
+                "If it does not, please select a lower severity. Thank you.</p>" ),QString(),KStandardGuiItem::cont(),KStandardGuiItem::cancel() ) == KMessageBox::No )
                 return;
             break;
         default:
@@ -441,7 +451,7 @@ void KBugReport::accept()
     if( !sendBugReport() )
     {
         QString msg = i18n("Unable to send the bug report.\n"
-                           "Please submit a bug report manually...\n"
+                           "Please submit a bug report manually....\n"
                            "See http://bugs.kde.org/ for instructions.");
         KMessageBox::error(this, msg + "\n\n" + d->lastError);
         return;
@@ -524,19 +534,23 @@ bool KBugReport::sendBugReport()
       command = KStandardDirs::findExe( QString::fromLatin1("ksendbugmail") );
 
   QProcess proc;
-  proc.start( command, QStringList() << " --subject " << d->m_subject->text() << " --recipient " << recipient );
+  QStringList args;
+  args << "--subject" << d->m_subject->text() << "--recipient" << recipient;
+  proc.start( command, args );
+  //kDebug() << command << args;
   if (!proc.waitForStarted())
   {
     kError() << "Unable to open a pipe to " << command << endl;
     return false;
   }
-  proc.write( text().toAscii() );
+  proc.write( text().toUtf8() );
+  proc.closeWriteChannel();
 
   proc.waitForFinished();
   kDebug() << "kbugreport: sendbugmail exit, status " << proc.exitStatus() << " code " << proc.exitCode();
 
   QByteArray line;
-  if (proc.exitStatus() == QProcess::NormalExit && proc.exitCode() == 1) { // XXX too restrictive?
+  if (proc.exitStatus() == QProcess::NormalExit && proc.exitCode() != 0) {
       // XXX not stderr?
       while (!proc.atEnd())
           line = proc.readLine();
