@@ -46,6 +46,7 @@
 #include "kdeprintdialog.h"
 #include "kglobalaccel.h"
 #include "kmessagebox.h"
+#include "kshortcut.h"
 #include "kaboutdata.h"
 
 //---------------------------------------------------------------------
@@ -123,7 +124,7 @@ void KShortcutsEditor::addCollection(KActionCollection *collection, const QStrin
         }
         // Yes it happens. Some apps don't set the programName.
         if (displayTitle.isEmpty()) {
-            displayTitle = "Unknown";
+            displayTitle = i18n("Unknown");
         }
     }
 
@@ -166,26 +167,35 @@ void KShortcutsEditor::addCollection(KActionCollection *collection, const QStrin
 }
 
 
+void KShortcutsEditor::clearConfiguration()
+{
+    d->clearConfiguration();
+}
+
+
 void KShortcutsEditor::importConfiguration( KConfig *config)
 {
-    if (d->actionTypes & KShortcutsEditor::GlobalAction) {
-        QString groupName = "Global Shortcuts";
-        KConfigGroup group( config, groupName );
-        foreach (KActionCollection* collection, d->actionCollections) {
-            collection->importGlobalShortcuts( &group );
-        }
-    }
-    if (d->actionTypes & ~KShortcutsEditor::GlobalAction) {
-        QString groupName = "Shortcuts";
-        KConfigGroup group( config, groupName );
-        foreach (KActionCollection* collection, d->actionCollections) {
-            collection->readSettings( &group );
-        }
-    }
+    d->importConfiguration(config);
 }
 
+
+void KShortcutsEditor::importConfiguration( KConfigBase *config)
+{
+    d->importConfiguration(config);
+}
+
+
 void KShortcutsEditor::exportConfiguration( KConfig *config) const
 {
+    exportConfiguration(static_cast<KConfigBase*>(config));
+}
+
+
+void KShortcutsEditor::exportConfiguration( KConfigBase *config) const
+{
+    Q_ASSERT(config);
+    if (!config) return;
+
     if (d->actionTypes & KShortcutsEditor::GlobalAction) {
         QString groupName = "Global Shortcuts";
         KConfigGroup group( config, groupName );
@@ -202,6 +212,7 @@ void KShortcutsEditor::exportConfiguration( KConfig *config) const
     }
 }
 
+
 void KShortcutsEditor::writeConfiguration( KConfigGroup *config) const
 {
     foreach (KActionCollection* collection, d->actionCollections)
@@ -323,7 +334,7 @@ bool KShortcutsEditorPrivate::addAction(QAction *action, QTreeWidgetItem *hier[]
     // If the action name starts with unnamed- spit out a warning and ignore
     // it. That name will change at will and will break loading and writing
     QString actionName = action->objectName();
-    if (actionName.isEmpty() || actionName.startsWith("unnamed-")) {
+    if (actionName.isEmpty() || actionName.startsWith(QLatin1String("unnamed-"))) {
         kError() << "Skipping action without name " << action->text() << "," << actionName << "!";
         return false;
     }
@@ -343,7 +354,7 @@ bool KShortcutsEditorPrivate::addAction(QAction *action, QTreeWidgetItem *hier[]
 void KShortcutsEditorPrivate::allDefault()
 {
     for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
-        if (!(*it)->parent())
+        if (!(*it)->parent() || (*it)->type() != ActionItem)
             continue;
 
         KShortcutsEditorItem *item = static_cast<KShortcutsEditorItem *>(*it);
@@ -421,7 +432,7 @@ void KShortcutsEditorPrivate::changeKeyShortcut(KShortcutsEditorItem *item, uint
     }
 
     item->setKeySequence(column, capture);
-    q->keyChange();
+     q->keyChange();
     //force view update
     item->setText(column, capture.toString(QKeySequence::NativeText));
 }
@@ -490,6 +501,66 @@ void KShortcutsEditorPrivate::changeRockerGesture(KShortcutsEditorItem *item, co
 }
 
 
+void KShortcutsEditorPrivate::clearConfiguration()
+{
+    for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
+        if (!(*it)->parent())
+            continue;
+
+        KShortcutsEditorItem *item = static_cast<KShortcutsEditorItem *>(*it);
+
+        changeKeyShortcut(item, LocalPrimary,   QKeySequence());
+        changeKeyShortcut(item, LocalAlternate, QKeySequence());
+
+        changeKeyShortcut(item, GlobalPrimary,   QKeySequence());
+        changeKeyShortcut(item, GlobalAlternate, QKeySequence());
+
+        changeShapeGesture(item, KShapeGesture() );
+
+    }
+}
+
+
+void KShortcutsEditorPrivate::importConfiguration(KConfigBase *config)
+{
+    Q_ASSERT(config);
+    if (!config) return;
+
+    KConfigGroup globalShortcutsGroup(config, QLatin1String("Global Shortcuts"));
+    if ((actionTypes & KShortcutsEditor::GlobalAction) && globalShortcutsGroup.exists()) {
+
+        for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
+
+            if (!(*it)->parent())
+                continue;
+
+            KShortcutsEditorItem *item = static_cast<KShortcutsEditorItem *>(*it);
+
+            QString actionName = item->data(Id).toString();
+            KShortcut sc(globalShortcutsGroup.readEntry(actionName, QString()));
+            changeKeyShortcut(item, GlobalPrimary, sc.primary());
+        }
+    }
+
+    KConfigGroup localShortcutsGroup(config, QLatin1String("Shortcuts"));
+    if (actionTypes & ~KShortcutsEditor::GlobalAction) {
+
+        for (QTreeWidgetItemIterator it(ui.list); (*it); ++it) {
+
+            if (!(*it)->parent())
+                continue;
+
+            KShortcutsEditorItem *item = static_cast<KShortcutsEditorItem *>(*it);
+
+            QString actionName = item->data(Name).toString();
+            KShortcut sc(localShortcutsGroup.readEntry(actionName, QString()));
+            changeKeyShortcut(item, LocalPrimary, sc.primary());
+            changeKeyShortcut(item, LocalAlternate, sc.alternate());
+        }
+    }
+}
+
+
 bool KShortcutsEditorPrivate::stealShapeGesture(KShortcutsEditorItem *item, const KShapeGesture &gst)
 {
     QString title = i18n("Key Conflict");
@@ -596,10 +667,11 @@ void KShortcutsEditorPrivate::printShortcuts() const
         cell.firstCursorPosition().insertText(i18n("Description"));
         currow++;
 
-        for (int j = 0; j < item->childCount(); j++) {
-            QTreeWidgetItem* actionitem = item->child(j);
-            KShortcutsEditorItem* editoritem =
-                    dynamic_cast<KShortcutsEditorItem*>(actionitem);
+        for (QTreeWidgetItemIterator it(item); *it; ++it) {
+            if ((*it)->type() != ActionItem)
+                continue;
+
+            KShortcutsEditorItem* editoritem = static_cast<KShortcutsEditorItem*>(*it);
             table->insertRows(table->rows(),1);
             QVariant data = editoritem->data(Name,Qt::DisplayRole);
             table->cellAt(currow, 0).firstCursorPosition().insertText(data.toString());
