@@ -365,43 +365,6 @@ void KIconLoaderPrivate::drawOverlays(const KIconLoader *iconLoader, KIconLoader
     }
 }
 
-#define KICONLOADER_CHECKS
-#ifdef KICONLOADER_CHECKS
-// Keep a list of recently created and destroyed KIconLoader instances in order
-// to detect bugs like #68528.
-struct KIconLoaderDebug
-{
-    KIconLoaderDebug( KIconLoader* l, const QString& a )
-        : loader( l ), appname( a ), valid( true )
-        {}
-    KIconLoader* loader;
-    QString appname;
-    bool valid;
-    QString delete_bt;
-};
-
-static QList< KIconLoaderDebug > *kiconloaders;
-
-static void registerIconLoader( KIconLoader* iconloader, const QString& _appname )
-{
-    if( kiconloaders == NULL )
-        kiconloaders = new QList< KIconLoaderDebug>();
-    // check for the (very unlikely case) that new KIconLoader gets allocated
-    // at exactly same address like some previous one
-    for( QList< KIconLoaderDebug >::Iterator it = kiconloaders->begin();
-         it != kiconloaders->end();
-         )
-        {
-        if( (*it).loader == iconloader )
-            it = kiconloaders->erase( it );
-        else
-            ++it;
-        }
-    kiconloaders->append( KIconLoaderDebug( iconloader, _appname ));
-}
-
-#endif
-
 KIconLoader::KIconLoader(const QString& _appname, KStandardDirs *_dirs, QObject* parent)
     : QObject(parent)
 {
@@ -410,9 +373,6 @@ KIconLoader::KIconLoader(const QString& _appname, KStandardDirs *_dirs, QObject*
 
     connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)),
             this, SLOT(newIconLoader()));
-#ifdef KICONLOADER_CHECKS
-    registerIconLoader(this, _appname);
-#endif
     d->init( _appname, _dirs );
 }
 
@@ -424,9 +384,6 @@ KIconLoader::KIconLoader(const KComponentData &componentData, QObject* parent)
 
     connect(KGlobalSettings::self(), SIGNAL(iconChanged(int)),
             this, SLOT(newIconLoader()));
-#ifdef KICONLOADER_CHECKS
-    registerIconLoader(this, componentData.componentName());
-#endif
     d->init(componentData.componentName(), componentData.dirs());
 }
 
@@ -546,19 +503,6 @@ bool KIconLoaderPrivate::initIconThemes()
 
 KIconLoader::~KIconLoader()
 {
-#ifdef KICONLOADER_CHECKS
-    for( QList< KIconLoaderDebug >::Iterator it = kiconloaders->begin();
-         it != kiconloaders->end();
-         ++it )
-        {
-        if( (*it).loader == this )
-            {
-            (*it).valid = false;
-            (*it).delete_bt = kBacktrace();
-            break;
-            }
-        }
-#endif
     delete d;
 }
 
@@ -702,9 +646,11 @@ bool KIconLoader::extraDesktopThemesAdded() const
 
 QString KIconLoaderPrivate::removeIconExtension(const QString &name) const
 {
-    if (name.endsWith(".png") || name.endsWith(".xpm") || name.endsWith(".svg")) {
+    if (name.endsWith(QLatin1String(".png"))
+        || name.endsWith(QLatin1String(".xpm"))
+        || name.endsWith(QLatin1String(".svg"))) {
         return name.left(name.length() - 4);
-    } else if (name.endsWith(".svgz")) {
+    } else if (name.endsWith(QLatin1String(".svgz"))) {
         return name.left(name.length() - 5);
     }
 
@@ -834,12 +780,13 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
     const char * const ext[4] = { ".png", ".svgz", ".svg", ".xpm" };
 #endif
 
+    const bool genericFallback = name.endsWith(QLatin1String("-x-generic"));
+
     foreach(KIconThemeNode *themeNode, links)
     {
-        QStringList nameParts = name.split('-');
         QString currentName = name;
 
-        while (!nameParts.isEmpty())
+        while (!currentName.isEmpty())
         {
 
             //kDebug(264) << "Looking up" << currentName;
@@ -873,9 +820,18 @@ K3Icon KIconLoaderPrivate::findMatchingIcon(const QString& name, int size) const
                     return icon;
             }
 #endif
+            if (genericFallback)
+                // we already tested the base name
+                break;
 
-            nameParts.removeLast();
-            currentName = nameParts.join("-");
+            int rindex = currentName.lastIndexOf('-');
+            if (rindex < 0)
+                break;
+
+            currentName.truncate(rindex);
+
+            if (currentName.endsWith(QLatin1String("-x")))
+                currentName.chop(2);
         }
     }
     return icon;
@@ -906,12 +862,9 @@ QString KIconLoader::iconPath(const QString& _name, int group_or_size,
 
     if (_name.isEmpty()
 #ifdef Q_OS_WIN
-        || (_name.length() > 1 &&
-            (_name[0].isLetter() && _name[1] == QLatin1Char(':') ||
-             _name[0] == '/'     && _name[1] == '/' ||
-             _name[0] == '\\'    && _name[1] == '\\')))
+        || !QDir::isRelativePath(_name))
 #else
-        || _name[0] == '/')
+        || _name.at(0) == '/')
 #endif
     {
         // we have either an absolute path or nothing to work with
@@ -1005,17 +958,17 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
         return pix;
 
     // Special case for absolute path icons.
-    if (name.startsWith("favicons/"))
+    if (name.startsWith(QLatin1String("favicons/")))
     {
        favIconOverlay = true;
        name = KStandardDirs::locateLocal("cache", name+".png");
     }
 
     if (!name.isEmpty()
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN
         && !QDir::isRelativePath(name))
 #else
-        && name[0] == '/')
+        && name.at(0) == '/')
 #endif
     {
         absolutePath = true;
@@ -1028,7 +981,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
     {
         QString key;
         key.reserve(200);
-        key.append("$kicou_");
+        key.append(QLatin1String("$kicou_"));
         key.append(name).append('_').append(QString::number(size));
         key.append(overlays.join("_")); // krazy:exclude=doublequote_chars
 
@@ -1121,7 +1074,7 @@ QPixmap KIconLoader::loadIcon(const QString& _name, KIconLoader::Group group, in
 
     QString key;
     key.reserve(100);
-    key.append("$kico_");
+    key.append(QLatin1String("$kico_"));
     key.append(name).append('_').append(QString::number(size));
 
     QString overlayKey = overlays.join("_"); // krazy:exclude=doublequote_chars
@@ -1561,6 +1514,8 @@ QStringList KIconLoader::queryIcons(int group_or_size, KIconLoader::Context cont
 // used by KIconDialog to find out which contexts to offer in a combobox
 bool KIconLoader::hasContext(KIconLoader::Context context) const
 {
+    d->initIconThemes();
+
     foreach(KIconThemeNode *themeNode, d->links)
        if( themeNode->theme->hasContext( context ))
            return true;
@@ -1707,6 +1662,7 @@ void KIconLoader::newIconLoader()
     }
 
     reconfigure( objectName(), d->mpDirs );
+    emit iconLoaderSettingsChanged();
 }
 
 #include "kiconloader.moc"
