@@ -44,7 +44,6 @@
 #include <QtGui/QKeySequence>
 #include <QtGui/QPixmap>
 #include <QtGui/QPixmapCache>
-//#include <q3stylesheet.h> // no equivalent in Qt4
 #include <QApplication>
 #include <QtDBus/QtDBus>
 #include <QtGui/QStyleFactory>
@@ -149,13 +148,13 @@ class KGlobalSettings::Private
 {
     public:
         Private(KGlobalSettings *q)
-            : q(q)
+            : q(q), activated(false)
         {
         }
 
         void _k_slotNotifyChange(int, int);
 
-        void propagateSettings(SettingsCategory category);
+        void propagateQtSettings();
         void kdisplaySetPalette();
         void kdisplaySetStyle();
         void kdisplaySetFont();
@@ -180,6 +179,7 @@ class KGlobalSettings::Private
         static void rereadOtherSettings();
 
         KGlobalSettings *q;
+        bool activated;
 };
 
 KGlobalSettings* KGlobalSettings::self()
@@ -191,10 +191,6 @@ KGlobalSettings* KGlobalSettings::self()
 KGlobalSettings::KGlobalSettings()
     : QObject(0), d(new Private(this))
 {
-    d->kdisplaySetStyle();
-    d->kdisplaySetFont();
-    d->propagateSettings(SETTINGS_QT);
-
     QDBusConnection::sessionBus().connect( QString(), "/KGlobalSettings", "org.kde.KGlobalSettings",
                                            "notifyChange", this, SLOT(_k_slotNotifyChange(int,int)) );
 }
@@ -204,6 +200,17 @@ KGlobalSettings::~KGlobalSettings()
     delete d;
 }
 
+void KGlobalSettings::activate()
+{
+    if (!d->activated) {
+        d->activated = true;
+
+        d->kdisplaySetStyle(); // implies palette setup
+        d->kdisplaySetFont();
+        d->propagateQtSettings();
+    }
+}
+
 int KGlobalSettings::dndEventDelay()
 {
     KConfigGroup g( KGlobal::config(), "General" );
@@ -284,7 +291,7 @@ int KGlobalSettings::contextMenuKey ()
 
     s = shortCuts.at(0);
 
-    if ( s.startsWith( "default(" ) ) {
+    if ( s.startsWith( QLatin1String("default(") ) ) {
         s = s.mid( 8, s.length() - 9 );
     }
 
@@ -338,14 +345,14 @@ QColor KGlobalSettings::activeTextColor()
 int KGlobalSettings::contrast()
 {
     KConfigGroup g( KGlobal::config(), "KDE" );
-    return g.readEntry( "contrast", 7 );
+    return g.readEntry( "contrast", 5 );
 }
 
 qreal KGlobalSettings::contrastF(const KSharedConfigPtr &config)
 {
     if (config) {
         KConfigGroup g( config, "KDE" );
-        return 0.1 * g.readEntry( "contrast", 7 );
+        return 0.1 * g.readEntry( "contrast", 5 );
     }
     return 0.1 * (qreal)contrast();
 }
@@ -387,13 +394,13 @@ static const KFontData DefaultFontData[KGlobalSettingsData::FontTypesCount] =
     { GeneralId, "toolBarFont", DefaultMacFont, 11, -1, QFont::SansSerif },
     { GeneralId, "menuFont",    DefaultMacFont, 13, -1, QFont::SansSerif },
 #else
-    { GeneralId, "font",        DefaultFont, 10, -1, QFont::SansSerif },
-    { GeneralId, "fixed",       "Monospace", 10, -1, QFont::TypeWriter },
+    { GeneralId, "font",        DefaultFont, 9, -1, QFont::SansSerif },
+    { GeneralId, "fixed",       "Monospace", 9, -1, QFont::TypeWriter },
     { GeneralId, "toolBarFont", DefaultFont,  8, -1, QFont::SansSerif },
-    { GeneralId, "menuFont",    DefaultFont, 10, -1, QFont::SansSerif },
+    { GeneralId, "menuFont",    DefaultFont, 9, -1, QFont::SansSerif },
 #endif
-    { "WM",      "activeFont",           DefaultFont,  9, QFont::Bold, QFont::SansSerif },// inconsistency
-    { GeneralId, "taskbarFont",          DefaultFont, 10, -1, QFont::SansSerif },
+    { "WM",      "activeFont",           DefaultFont,  8, -1, QFont::SansSerif },
+    { GeneralId, "taskbarFont",          DefaultFont, 9, -1, QFont::SansSerif },
     { GeneralId, "smallestReadableFont", DefaultFont,  8, -1, QFont::SansSerif }
 };
 
@@ -453,7 +460,7 @@ QFont KGlobalSettingsData::largeFont( const QString& text )
 
     // Move a bunch of preferred fonts to the front.
     // most preferred last
-    static const char* PreferredFontNames[] =
+    static const char* const PreferredFontNames[] =
     {
         "Arial",
         "Sans Serif",
@@ -472,8 +479,10 @@ QFont KGlobalSettingsData::largeFont( const QString& text )
             fam.prepend(fontName);
     }
 
-    if (mLargeFont)
+    if (mLargeFont) {
         fam.prepend(mLargeFont->family());
+        delete mLargeFont;
+    }
 
     for(QStringList::ConstIterator it = fam.constBegin();
         it != fam.constEnd(); ++it)
@@ -521,6 +530,7 @@ void KGlobalSettingsData::dropFontSettingsCache()
         mFonts[i] = 0;
     }
     delete mLargeFont;
+    mLargeFont = 0;
 }
 
 KGlobalSettings::KMouseSettings& KGlobalSettingsData::mouseSettings()
@@ -594,7 +604,7 @@ QString KGlobalSettings::desktopPath()
     return path.isEmpty() ? QDir::homePath() : path;
 }
 
-// Autostart is not a XDG path, so we keep with old kdelibs code code
+// Autostart is not a XDG path, so we have our own code for it.
 QString KGlobalSettings::autostartPath()
 {
     QString s_autostartPath;
@@ -619,7 +629,7 @@ QString KGlobalSettings::downloadPath()
     // Qt 4.4.1 does not have DOWNLOAD, so we based on old code for now
     QString downloadPath = QDir::homePath();
 #ifndef Q_WS_WIN
-    QString xdgUserDirs = QDir::homePath() + QLatin1String( "/.config/user-dirs.dirs" );
+    const QString xdgUserDirs = KGlobal::dirs()->localxdgconfdir() + QLatin1String( "user-dirs.dirs" );
     if( QFile::exists( xdgUserDirs ) ) {
         KConfig xdgUserConf( xdgUserDirs, KConfig::SimpleConfig );
         KConfigGroup g( &xdgUserConf, "" );
@@ -732,6 +742,13 @@ bool KGlobalSettings::showIconsOnPushButtons()
                        KDE_DEFAULT_ICON_ON_PUSHBUTTON);
 }
 
+bool KGlobalSettings::naturalSorting()
+{
+    KConfigGroup g( KGlobal::config(), "KDE" );
+    return g.readEntry("NaturalSorting",
+                       KDE_DEFAULT_NATURAL_SORTING);
+}
+
 KGlobalSettings::GraphicEffects KGlobalSettings::graphicEffectsLevel()
 {
     // This variable stores whether _graphicEffects has the default value because it has not been
@@ -796,8 +813,10 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
 {
     switch(changeType) {
     case StyleChanged:
-        KGlobal::config()->reparseConfiguration();
-        kdisplaySetStyle();
+        if (activated) {
+            KGlobal::config()->reparseConfiguration();
+            kdisplaySetStyle();
+        }
         break;
 
     case ToolbarStyleChanged:
@@ -806,14 +825,18 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         break;
 
     case PaletteChanged:
-        KGlobal::config()->reparseConfiguration();
-        kdisplaySetPalette();
+        if (activated) {
+            KGlobal::config()->reparseConfiguration();
+            kdisplaySetPalette();
+        }
         break;
 
     case FontChanged:
         KGlobal::config()->reparseConfiguration();
         KGlobalSettingsData::self()->dropFontSettingsCache();
-        kdisplaySetFont();
+        if (activated) {
+            kdisplaySetFont();
+        }
         break;
 
     case SettingsChanged: {
@@ -823,7 +846,13 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         if (category == SETTINGS_MOUSE) {
             KGlobalSettingsData::self()->dropMouseSettingsCache();
         }
-        propagateSettings(category);
+        if (category == SETTINGS_QT) {
+            if (activated) {
+                propagateQtSettings();
+            }
+        } else {
+            emit q->settingsChanged(category);
+        }
         break;
     }
     case IconChanged:
@@ -842,6 +871,10 @@ void KGlobalSettings::Private::_k_slotNotifyChange(int changeType, int arg)
         emit q->blockShortcuts(arg); // see kwin
         break;
 
+    case NaturalSortingChanged:
+        emit q->naturalSortingChanged();
+        break;
+
     default:
         kWarning(101) << "Unknown type of change in KGlobalSettings::slotNotifyChange: " << changeType;
     }
@@ -862,7 +895,7 @@ void KGlobalSettings::Private::applyGUIStyle()
         if (styleStr.isEmpty() ||
                 // check whether we already use the correct style to return then
                 // (workaround for Qt misbehavior to avoid double style initialization)
-                0 == (styleStr + QLatin1String("Style")).compare(currentStyleName, Qt::CaseInsensitive) ||
+                0 == (QString(styleStr + QLatin1String("Style"))).compare(currentStyleName, Qt::CaseInsensitive) ||
                 0 == styleStr.compare(currentStyleName, Qt::CaseInsensitive)) {
             return;
         }
@@ -880,7 +913,7 @@ void KGlobalSettings::Private::applyGUIStyle()
             sp = QStyleFactory::create( QStyleFactory::keys().first() );
         qApp->setStyle(sp);
     } else if (0 != kde_overrideStyle.compare(currentStyleName, Qt::CaseInsensitive) &&
-            0 != (kde_overrideStyle + QLatin1String("Style")).compare(currentStyleName, Qt::CaseInsensitive)) {
+            0 != (QString(kde_overrideStyle + QLatin1String("Style"))).compare(currentStyleName, Qt::CaseInsensitive)) {
         qApp->setStyle(kde_overrideStyle);
     }
     emit q->kdisplayStyleChanged();
@@ -935,7 +968,7 @@ void KGlobalSettings::Private::kdisplaySetPalette()
     if (cg.readEntry("nopaletteChange", false))
         return;
 
-    if (qobject_cast<KApplication *>(qApp) && qApp->type() == QApplication::GuiClient) {
+    if (qApp->type() == QApplication::GuiClient) {
         QApplication::setPalette( q->createApplicationPalette() );
     }
     emit q->kdisplayPaletteChanged();
@@ -945,7 +978,7 @@ void KGlobalSettings::Private::kdisplaySetPalette()
 
 void KGlobalSettings::Private::kdisplaySetFont()
 {
-    if (qobject_cast<KApplication *>(qApp) && qApp->type() == QApplication::GuiClient) {
+    if (qApp->type() == QApplication::GuiClient) {
         KGlobalSettingsData* data = KGlobalSettingsData::self();
 
         QApplication::setFont( data->font(KGlobalSettingsData::GeneralFont) );
@@ -954,15 +987,6 @@ void KGlobalSettings::Private::kdisplaySetFont()
         QApplication::setFont( menuFont, "QMenu" );
         QApplication::setFont( menuFont, "KPopupTitle" );
         QApplication::setFont( data->font(KGlobalSettingsData::ToolbarFont), "QToolBar" );
-
-#if 0
-        // "patch" standard QStyleSheet to follow our fonts
-        Q3StyleSheet* sheet = Q3StyleSheet::defaultSheet();
-        const QFont fixedFont = data->font( KGlobalSettingsData::FixedFont );
-        sheet->item( QLatin1String("pre"))->setFontFamily(fixedFont.family() );
-        sheet->item( QLatin1String("code"))->setFontFamily(fixedFont.family() );
-        sheet->item( QLatin1String("tt"))->setFontFamily(fixedFont.family() );
-#endif
     }
     emit q->kdisplayFontChanged();
     emit q->appearanceChanged();
@@ -971,7 +995,7 @@ void KGlobalSettings::Private::kdisplaySetFont()
 
 void KGlobalSettings::Private::kdisplaySetStyle()
 {
-    if (qobject_cast<KApplication *>(qApp) && qApp->type() == QApplication::GuiClient) {
+    if (qApp->type() == QApplication::GuiClient) {
         applyGUIStyle();
 
         // Reread palette from config file.
@@ -1026,7 +1050,7 @@ void KGlobalSettings::Private::applyCursorTheme()
 }
 
 
-void KGlobalSettings::Private::propagateSettings(SettingsCategory arg)
+void KGlobalSettings::Private::propagateQtSettings()
 {
     KConfigGroup cg( KGlobal::config(), "KDE" );
 
@@ -1045,7 +1069,8 @@ void KGlobalSettings::Private::propagateSettings(SettingsCategory arg)
     num = cg.readEntry("WheelScrollLines", QApplication::wheelScrollLines());
     QApplication::setWheelScrollLines(num);
 
-    emit q->settingsChanged(arg);
+    // KDE5: this seems fairly pointless
+    emit q->settingsChanged(SETTINGS_QT);
 }
 
 #include "kglobalsettings.moc"
