@@ -23,6 +23,8 @@
 #include "kglobalaccel.h"
 #include "kglobalaccel_p.h"
 
+#include <memory>
+
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusMetaType>
 #ifdef Q_WS_X11
@@ -38,22 +40,77 @@
 #include "kaction_p.h"
 #include "kmessagebox.h"
 #include "kshortcut.h"
+#include "kglobalaccel_component_interface.h"
+#include "kglobalaccel_interface.h"
+
+static org::kde::kglobalaccel::Component *getComponent(const QString &componentUnique)
+    {
+    // Connect to the kglobalaccel daemon
+    org::kde::KGlobalAccel kglobalaccel(
+            "org.kde.kglobalaccel",
+            "/kglobalaccel",
+            QDBusConnection::sessionBus());
+    if (!kglobalaccel.isValid()) {
+        kDebug() << "Failed to connect to the kglobalaccel daemon" << QDBusConnection::sessionBus().lastError();
+        return NULL;
+    }
+
+    // Get the path for our component. We have to do that because
+    // componentUnique is probably not a valid dbus object path
+    QDBusReply<QDBusObjectPath> reply = kglobalaccel.getComponent(componentUnique);
+    if (!reply.isValid()) {
+
+        if (reply.error().name() == "org.kde.kglobalaccel.NoSuchComponent") {
+            // No problem. The component doesn't exists. That's normal
+            return NULL;
+        }
+
+        // An unknown error.
+        kDebug() << "Failed to get dbus path for component " << componentUnique << reply.error();
+        return NULL;
+    }
+
+    // Now get the component
+    org::kde::kglobalaccel::Component *component = new org::kde::kglobalaccel::Component(
+        "org.kde.kglobalaccel",
+        reply.value().path(),
+        QDBusConnection::sessionBus());
+
+    // No component no cleaning
+    if (!component->isValid()) {
+        kDebug() << "Failed to get component" << componentUnique << QDBusConnection::sessionBus().lastError();
+        return NULL;
+    }
+
+    return component;
+}
+
 
 
 KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
-     : isUsingForeignComponentName(false),
-       enabled(true),
-       iface("org.kde.kded", "/modules/kdedglobalaccel", QDBusConnection::sessionBus())
+     :  isUsingForeignComponentName(false),
+        enabled(true),
+        iface("org.kde.kglobalaccel", "/kglobalaccel", QDBusConnection::sessionBus())
 {
-    // Make sure kded is running
+    // Make sure kded is running. The iface declaration above somehow
+    // works anyway.
     QDBusConnectionInterface* bus = QDBusConnection::sessionBus().interface();
-    if (!bus->isServiceRegistered("org.kde.kded")) {
-        KToolInvocation::klauncher(); // this calls startKdeinit
+    if (!bus->isServiceRegistered("org.kde.kglobalaccel")) {
+        QString error;
+        int ret = KToolInvocation::startServiceByDesktopPath(
+                "kglobalaccel.desktop",
+                QStringList(),
+                &error);
+
+        if (ret > 0) {
+            kError() << "Couldn't start kglobalaccel from kglobalaccel.desktop: " << error << endl;
+        }
     }
     QObject::connect(bus, SIGNAL(serviceOwnerChanged(QString, QString, QString)),
                      q, SLOT(_k_serviceOwnerChanged(QString, QString, QString)));
 }
 
+
 void KGlobalAccelPrivate::readComponentData(const KComponentData &componentData)
 {
     Q_ASSERT(!componentData.componentName().isEmpty());
@@ -96,11 +153,32 @@ void KGlobalAccel::activateGlobalShortcutContext(
         const QString &contextFriendly,
         const KComponentData &component)
 {
-    // TODO provide contextFriendly
+    Q_UNUSED(contextFriendly);
+    // TODO: provide contextFriendly
     self()->d->iface.activateGlobalShortcutContext(component.aboutData()->programName(), contextUnique);
 }
 
 
+// static
+bool KGlobalAccel::cleanComponent(const QString &componentUnique)
+{
+    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
+    if (!component.get()) return false;
+
+    return component->cleanUp();
+}
+
+
+// static
+bool KGlobalAccel::isComponentActive(const QString &componentUnique)
+{
+    std::auto_ptr<org::kde::kglobalaccel::Component> component(getComponent(componentUnique));
+    if (!component.get()) return false;
+
+    return component->isActive();
+}
+
+
 bool KGlobalAccel::isEnabled() const
 {
     return d->enabled;
@@ -175,7 +253,7 @@ void KGlobalAccelPrivate::remove(KAction *action, Removal removal)
         // internal registry. That happened above.
         if (!action->property("isConfigurationAction").toBool()) {
             // If it's a session shortcut unregister it.
-            action->objectName().startsWith("_k_session:")
+            action->objectName().startsWith(QLatin1String("_k_session:"))
                 ? iface.unRegister(actionId)
                 : iface.setInactive(actionId);
         }
@@ -205,7 +283,7 @@ void KGlobalAccelPrivate::updateGlobalShortcut(KAction *action, uint flags)
             || action->property("isConfigurationAction").toBool();
         uint activeSetterFlags = setterFlags;
 
-        // setPresent tells kdedglobalaccel that the shortcut is active
+        // setPresent tells kglobalaccel that the shortcut is active
         if (!isConfigurationAction) {
             activeSetterFlags |= SetPresent;
         }
@@ -232,7 +310,7 @@ void KGlobalAccelPrivate::updateGlobalShortcut(KAction *action, uint flags)
             iface.setForeignShortcut(actionId, result);
         }
         if (scResult != activeShortcut) {
-            // If kdedglobalaccel returned a shortcut that differs from the one we
+            // If kglobalaccel returned a shortcut that differs from the one we
             // sent, use that one. There must have been clashes or some other problem.
             action->d->setActiveGlobalShortcutNoEnable(scResult);
         }
@@ -348,7 +426,7 @@ void KGlobalAccelPrivate::_k_serviceOwnerChanged(const QString &name, const QStr
                                                  const QString &newOwner)
 {
     Q_UNUSED(oldOwner);
-    if (name == QLatin1String("org.kde.kded") && !newOwner.isEmpty()) {
+    if (name == QLatin1String("org.kde.kglobalaccel") && !newOwner.isEmpty()) {
         // kded was restarted (what? you mean it crashes sometimes?)
         reRegisterAll();
     }
@@ -467,7 +545,7 @@ bool KGlobalAccel::promptStealShortcutSystemwide(
 //static
 void KGlobalAccel::stealShortcutSystemwide(const QKeySequence &seq)
 {
-    //get the shortcut, remove seq, and set the new shorctut
+    //get the shortcut, remove seq, and set the new shortcut
     const QStringList actionId = self()->d->iface.action(seq[0]);
     if (actionId.size() < 4) // not a global shortcut
         return;
