@@ -2,6 +2,8 @@
  * This file is part of the KDE Libraries
  * Copyright (C) 2000 Timo Hummel <timo.hummel@sap.com>
  *                    Tom Braun <braunt@fh-konstanz.de>
+ * Copyright 2009 KDE e.V.
+ *   By Adriaan de Groot <groot@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -60,6 +62,16 @@
 #include <X11/Xlib.h>
 #endif
 
+#ifdef Q_OS_SOLARIS
+// Solaris has built-in, thread-safe, async-signal-safe, mechanisms
+// to walk the stack in the case of a crash, as well as (optionally)
+// to demangle C++ symbol names. In the case of a crash, dump a stack
+// trace to stderr before starting drKonqui (because what drKonqui is
+// going to do is -- through a complicated process -- print the
+// exact same information, but less reliably).
+#include <ucontext.h>
+#endif
+
 static KCrash::HandlerType s_emergencySaveFunction = 0;
 static KCrash::HandlerType s_crashHandler = 0;
 static char *s_appName = 0;
@@ -169,6 +181,10 @@ KCrash::setCrashHandler (HandlerType handler)
   signal (SIGSEGV, handler);
   sigaddset(&mask, SIGSEGV);
 #endif
+#ifdef SIGBUS
+  signal (SIGBUS, handler);
+  sigaddset(&mask, SIGBUS);
+#endif
 #ifdef SIGFPE
   signal (SIGFPE, handler);
   sigaddset(&mask, SIGFPE);
@@ -219,6 +235,10 @@ KCrash::defaultCrashHandler (int sig)
   signal(SIGALRM, SIG_DFL);
   alarm(3); // Kill me... (in case we deadlock in malloc)
 
+#ifdef Q_OS_SOLARIS
+  (void) printstack(2 /* stderr, assuming it's still open. */);
+#endif
+
   if (crashRecursionCounter < 2) {
     if (s_emergencySaveFunction) {
       s_emergencySaveFunction (sig);
@@ -259,7 +279,7 @@ KCrash::defaultCrashHandler (int sig)
             return;
         }
 
-          const char * argv[24]; // don't forget to update this
+          const char * argv[25]; // don't forget to update this
           int i = 0;
 
           // argument 0 has to be drkonqi
@@ -330,6 +350,9 @@ KCrash::defaultCrashHandler (int sig)
           if ( s_flags & SaferDialog )
             argv[i++] = "--safer";
 
+          if ((s_flags & AutoRestart) && s_autoRestartCommand)
+            argv[i++] = "--restarted"; //tell drkonqi if the app has been restarted
+
           // NULL terminated list
           argv[i] = NULL;
 
@@ -433,15 +456,13 @@ void KCrash::startDirectly( const char* argv[], int )
   default:
     alarm(0); // Seems we made it....
     // wait for child to exit
-    waitpid(pid, NULL, 0);
+    while(waitpid(-1, NULL, 0) != pid) {}
     _exit(253);
   }
 }
 
 // From now on this code is copy&pasted from kinit/wrapper.c :
 
-extern char **environ;
-
 static char *getDisplay()
 {
    const char *display;
