@@ -34,21 +34,13 @@
 #include <windows.h>
 #include <windowsx.h>
 
-//functions to register us as taskmanager
-typedef bool (WINAPI *PtrSetTaskmanWindow)(HWND);
-typedef bool (WINAPI *PtrRegisterShellHookWindow)(HWND);
-typedef bool (WINAPI *PtrDeregisterShellHookWindow)(HWND);
-
 
+//function to register us as taskmanager
 #define RSH_UNREGISTER  0
 #define RSH_REGISTER    1
 #define RSH_TASKMGR     3
 typedef bool (WINAPI *PtrRegisterShellHook)(HWND hWnd, DWORD method);
 
-
-static PtrSetTaskmanWindow pSetTaskmanWindow = 0;
-static PtrRegisterShellHookWindow pRegisterShellHookWindow = 0;
-static PtrDeregisterShellHookWindow pDeregisterShellHookWindow = 0;
 static PtrRegisterShellHook pRegisterShellHook = 0;
 static int WM_SHELLHOOK = -1;
 
@@ -77,11 +69,11 @@ class KWindowSystemPrivate : public QWidget
     friend class KWindowSystem;
     public:
         KWindowSystemPrivate ( int what );
-        ~KWindowSystemPrivate();  
-        
+        ~KWindowSystemPrivate();
+
         static bool CALLBACK EnumWindProc (WId hwnd, LPARAM lparam);
         static void readWindowInfo  (WId wid  , InternalWindowInfo *winfo);
-        
+
         void windowAdded        (WId wid);
         void windowRemoved      (WId wid);
         void windowActivated    (WId wid);
@@ -90,12 +82,13 @@ class KWindowSystemPrivate : public QWidget
         void windowStateChanged (WId wid);
         void reloadStackList    ( );
         void activate           ( );
-        
-        
+
+
     protected:
         bool winEvent ( MSG * message, long * result );
-        
+
     private:
+	    bool activated; 
         int what;
         WId fakeHwnd;
         QList<WId> stackingOrder;
@@ -142,7 +135,7 @@ static HICON QPixmap2HIcon(const QPixmap &pix)
 static QPixmap HIcon2QPixmap( HICON hIcon )
 {
     ICONINFO ii;
-    if( GetIconInfo( hIcon, &ii ) == NULL )
+    if( GetIconInfo( hIcon, &ii ) == 0 )
         return QPixmap();
 
     QPixmap pix  = QPixmap::fromWinHBITMAP( ii.hbmColor );
@@ -154,7 +147,7 @@ static QPixmap HIcon2QPixmap( HICON hIcon )
     return pix;
 }
 
-KWindowSystemPrivate::KWindowSystemPrivate(int what) : QWidget(0)
+KWindowSystemPrivate::KWindowSystemPrivate(int what) : QWidget(0),activated(false)
 {
     //i think there is no difference in windows we always load everything
     what = KWindowSystem::INFO_WINDOWS;
@@ -163,59 +156,69 @@ KWindowSystemPrivate::KWindowSystemPrivate(int what) : QWidget(0)
 
 void KWindowSystemPrivate::activate ( )
 {
+	//prevent us from doing the same over and over again
+	if(activated)
+		return;
+	activated = true;
+	
     //resolve winapi stuff
-    if(!pSetTaskmanWindow) pSetTaskmanWindow = (PtrSetTaskmanWindow)QLibrary::resolve("user32","SetTaskmanWindow");
-    if(!pRegisterShellHookWindow) pRegisterShellHookWindow = (PtrRegisterShellHookWindow)QLibrary::resolve("user32","RegisterShellHookWindow");
-    if(!pDeregisterShellHookWindow) pDeregisterShellHookWindow = (PtrDeregisterShellHookWindow)QLibrary::resolve("user32","DeregisterShellHookWindow");
     if(!pRegisterShellHook) pRegisterShellHook = (PtrRegisterShellHook)QLibrary::resolve("shell32",(LPCSTR)0xb5);
-    
+
     //get the id for the shellhook message
-    if(WM_SHELLHOOK==-1) WM_SHELLHOOK  = RegisterWindowMessage(TEXT("SHELLHOOK"));
-    
-    bool shellHookRegistered = false;
-    if(pRegisterShellHook) {
-        kDebug()<<"use RegisterShellHook";
-        shellHookRegistered = pRegisterShellHook(winId(),RSH_REGISTER);
-        if(!shellHookRegistered)
-            shellHookRegistered = pRegisterShellHook(winId(),RSH_TASKMGR);
-    } else {     
-        kDebug()<<"use RegisterShellHookWindow";
-        //i'm not sure if i have to do this, and what happens if some other process uses KWindowSystem
-        //if(pSetTaskmanWindow)
-        //    pSetTaskmanWindow(winId());
-        
-        if(pRegisterShellHookWindow)
-            shellHookRegistered = pRegisterShellHookWindow(winId());
+    if(WM_SHELLHOOK==-1) {
+        WM_SHELLHOOK = RegisterWindowMessage(TEXT("SHELLHOOK"));
+        //kDebug() << "WM_SHELLHOOK:" << WM_SHELLHOOK << winId();
     }
-    
+
+    bool shellHookRegistered = false;
+    if(pRegisterShellHook)
+        shellHookRegistered = pRegisterShellHook(winId(),RSH_TASKMGR);
+ 
     if(!shellHookRegistered)
         //use a timer and poll the windows ?
-         kDebug()<<"Could not create shellhook to receive WindowManager Events";
-            
+          kDebug() << "Could not create shellhook to receive WindowManager Events";
+
     //fetch window infos
     reloadStackList();
 }
 
 KWindowSystemPrivate::~KWindowSystemPrivate()
 {
-    if(pRegisterShellHook){
-        pRegisterShellHook(winId(),RSH_UNREGISTER);
-    }else{
-        if(pDeregisterShellHookWindow)
-            pDeregisterShellHookWindow(winId());
-    }
+    if(pRegisterShellHook)
+        pRegisterShellHook(winId(),RSH_UNREGISTER);		
 }
 
 /**
  *the callback procedure for the invisible ShellHook window
  */
-bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )  
+bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )
 {
+    /* 
+        check winuser.h for the following codes
+        HSHELL_WINDOWCREATED        1
+        HSHELL_WINDOWDESTROYED      2
+        HSHELL_ACTIVATESHELLWINDOW  3
+        HSHELL_WINDOWACTIVATED      4
+        HSHELL_GETMINRECT           5
+        HSHELL_RUDEAPPACTIVATED     32768 + 4 = 32772
+        HSHELL_REDRAW               6
+        HSHELL_FLASH                32768 + 6 = 32774
+        HSHELL_TASKMAN              7
+        HSHELL_LANGUAGE             8
+        HSHELL_SYSMENU              9
+        HSHELL_ENDTASK              10
+        HSHELL_ACCESSIBILITYSTATE   11
+        HSHELL_APPCOMMAND           12
+        HSHELL_WINDOWREPLACED       13
+        HSHELL_WINDOWREPLACING      14
+       */
     if (message->message == WM_SHELLHOOK) {
+//         kDebug() << "what has happened?:" << message->wParam << message->message;
+
         switch(message->wParam) {
           case HSHELL_WINDOWCREATED:
             KWindowSystem::s_d_func()->windowAdded(reinterpret_cast<WId>(message->lParam));
-            break;          
+            break;
           case HSHELL_WINDOWDESTROYED:
             KWindowSystem::s_d_func()->windowRemoved(reinterpret_cast<WId>(message->lParam));
             break;
@@ -223,15 +226,15 @@ bool KWindowSystemPrivate::winEvent ( MSG * message, long * result )
           case HSHELL_RUDEAPPACTIVATED:
             KWindowSystem::s_d_func()->windowActivated(reinterpret_cast<WId>(message->lParam));
             break;
-          case HSHELL_REDRAW: //the caption has changed 
+          case HSHELL_GETMINRECT:
+            KWindowSystem::s_d_func()->windowStateChanged(reinterpret_cast<WId>(message->lParam));
+            break;
+          case HSHELL_REDRAW: //the caption has changed
             KWindowSystem::s_d_func()->windowRedraw(reinterpret_cast<WId>(message->lParam));
             break;
           case HSHELL_FLASH:
             KWindowSystem::s_d_func()->windowFlash(reinterpret_cast<WId>(message->lParam));
             break;
-          case HSHELL_GETMINRECT:
-            KWindowSystem::s_d_func()->windowStateChanged(reinterpret_cast<WId>(message->lParam));
-            break;
         }
     }
     return QWidget::winEvent(message,result);
@@ -243,20 +246,20 @@ bool CALLBACK KWindowSystemPrivate::EnumWindProc(WId hWnd, LPARAM lparam)
     GetWindowTextA(hWnd, windowText.data(), windowText.size());
 	DWORD ex_style = GetWindowExStyle(hWnd);
     KWindowSystemPrivate *p = KWindowSystem::s_d_func();
-    
+
     QString add;
-    if( !QString(windowText).trimmed().isEmpty() && IsWindowVisible( hWnd ) && !(ex_style&WS_EX_TOOLWINDOW) 
+    if( !QString(windowText).trimmed().isEmpty() && IsWindowVisible( hWnd ) && !(ex_style&WS_EX_TOOLWINDOW)
        && !GetParent(hWnd) && !GetWindow(hWnd,GW_OWNER) && !p->winInfos.contains(hWnd) ) {
-       
-        kDebug()<<"Adding window to windowList " << add + QString(windowText).trimmed();
-        
+
+//        kDebug()<<"Adding window to windowList " << add + QString(windowText).trimmed();
+
         InternalWindowInfo winfo;
         KWindowSystemPrivate::readWindowInfo(hWnd,&winfo);
-        
+
         p->stackingOrder.append(hWnd);
-        p->winInfos.insert(hWnd,winfo);  
+        p->winInfos.insert(hWnd,winfo);
     }
-    return true; 
+    return true;
 }
 
 void KWindowSystemPrivate::readWindowInfo ( WId hWnd , InternalWindowInfo *winfo)
@@ -272,7 +275,7 @@ void KWindowSystemPrivate::readWindowInfo ( WId hWnd , InternalWindowInfo *winfo
     if(!hSmallIcon) hSmallIcon = (HICON)GetClassLong(hWnd, GCL_HICON);
     if(!hSmallIcon) hSmallIcon = (HICON)SendMessage(hWnd, WM_QUERYDRAGICON, 0, 0);
     if(hSmallIcon)  smallIcon  = HIcon2QPixmap(hSmallIcon);
-    
+
     QPixmap bigIcon;
     HICON hBigIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_BIG, 0);
     //if(!hBigIcon) hBigIcon = (HICON)SendMessage(hWnd, WM_GETICON, ICON_SMALL2, 0);
@@ -281,22 +284,25 @@ void KWindowSystemPrivate::readWindowInfo ( WId hWnd , InternalWindowInfo *winfo
     if(!hBigIcon) hBigIcon = (HICON)GetClassLong(hWnd, GCL_HICONSM);
     if(!hBigIcon) hBigIcon = (HICON)SendMessage(hWnd, WM_QUERYDRAGICON, 0, 0);
     if(hBigIcon)  bigIcon  = HIcon2QPixmap(hBigIcon);
-       
+
     winfo->bigIcon    = bigIcon;
     winfo->smallIcon  = smallIcon;
-    winfo->windowName = QString(windowText).trimmed();   
+    winfo->windowName = QString(windowText).trimmed();
 }
 
 
 void KWindowSystemPrivate::windowAdded     (WId wid)
 {
+//     kDebug() << "window added!";
     KWindowSystem::s_d_func()->reloadStackList();
     emit KWindowSystem::self()->windowAdded(wid);
+    emit KWindowSystem::self()->activeWindowChanged(wid);
     emit KWindowSystem::self()->stackingOrderChanged();
 }
 
 void KWindowSystemPrivate::windowRemoved   (WId wid)
 {
+//     kDebug() << "window removed!";
     KWindowSystem::s_d_func()->reloadStackList();
     emit KWindowSystem::self()->windowRemoved(wid);
     emit KWindowSystem::self()->stackingOrderChanged();
@@ -304,6 +310,7 @@ void KWindowSystemPrivate::windowRemoved   (WId wid)
 
 void KWindowSystemPrivate::windowActivated (WId wid)
 {
+//     kDebug() << "window activated!";
     KWindowSystem::s_d_func()->reloadStackList();
     emit KWindowSystem::self()->activeWindowChanged(wid);
     emit KWindowSystem::self()->stackingOrderChanged();
@@ -366,6 +373,11 @@ void KWindowSystem::init(int what)
 
 }
 
+bool KWindowSystem::allowedActionsSupported()
+{
+    return false;
+}
+
 int KWindowSystem::currentDesktop()
 {
     return 1;
@@ -378,53 +390,54 @@ int KWindowSystem::numberOfDesktops()
 
 void KWindowSystem::setMainWindow( QWidget* subwindow, WId mainwindow )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     SetForegroundWindow(subwindow->winId());
 }
 
 void KWindowSystem::setCurrentDesktop( int desktop )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     kDebug() << "KWindowSystem::setCurrentDesktop( int desktop ) isn't yet implemented!";
     //TODO
 }
 
 void KWindowSystem::setOnAllDesktops( WId win, bool b )
 {
-    KWindowSystem::init(INFO_WINDOWS);
      kDebug() << "KWindowSystem::setOnAllDesktops( WId win, bool b ) isn't yet implemented!";
      //TODO
 }
 
 void KWindowSystem::setOnDesktop( WId win, int desktop )
 {
-    KWindowSystem::init(INFO_WINDOWS);
      //TODO
      kDebug() << "KWindowSystem::setOnDesktop( WId win, int desktop ) isn't yet implemented!";
 }
 
 WId KWindowSystem::activeWindow()
 {
-    KWindowSystem::init(INFO_WINDOWS);
     return GetActiveWindow();
 }
 
 void KWindowSystem::activateWindow( WId win, long )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     SetActiveWindow( win );
 }
 
 void KWindowSystem::forceActiveWindow( WId win, long time )
 {
-    KWindowSystem::init(INFO_WINDOWS);
-    BringWindowToTop( win );
+    // FIXME restoring a hidden window doesn't work: the window contents just appear white.
+    // But the mouse cursor still acts as if the widgets were there (e.g. button clicking works),
+    // which indicates the issue is at the window/backingstore level.
+    // This is probably a side effect of bypassing Qt's internal window state handling.
+    if ( IsIconic( win ) /*|| !IsWindowVisible( win ) */) {
+        // Do not activate the window as we restore it,
+        // otherwise the window appears see-through (contents not updated).
+        ShowWindow( win, SW_SHOWNOACTIVATE );
+    }
+    // Puts the window in front and activates it.
     SetForegroundWindow( win );
 }
 
 void KWindowSystem::demandAttention( WId win, bool set )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     FLASHWINFO fi;
     fi.cbSize = sizeof( FLASHWINFO );
     fi.hwnd = win;
@@ -439,7 +452,7 @@ void KWindowSystem::demandAttention( WId win, bool set )
 QPixmap KWindowSystem::icon( WId win, int width, int height, bool scale )
 {
     KWindowSystem::init(INFO_WINDOWS);
-    
+
     QPixmap pm;
     if(KWindowSystem::s_d_func()->winInfos.contains(win)){
         if( width < 24 || height < 24 )
@@ -454,7 +467,7 @@ QPixmap KWindowSystem::icon( WId win, int width, int height, bool scale )
             size = ICON_SMALL;
         HICON hIcon = (HICON)SendMessage( win, WM_GETICON, size, 0);
         pm = HIcon2QPixmap( hIcon );
-    } 
+    }
     if( scale )
         pm = pm.scaled( width, height );
     return pm;
@@ -469,24 +482,23 @@ void KWindowSystem::setIcons( WId win, const QPixmap& icon, const QPixmap& miniI
 {
     KWindowSystem::init(INFO_WINDOWS);
     KWindowSystemPrivate* s_d = s_d_func();
-    
+
     if(s_d->winInfos.contains(win)){
         // is this safe enough or do i have to refresh() the window infos
         s_d->winInfos[win].smallIcon = miniIcon;
         s_d->winInfos[win].bigIcon   = icon;
     }
-    
+
     HICON hIconBig = QPixmap2HIcon(icon);
     HICON hIconSmall = QPixmap2HIcon(miniIcon);
 
     hIconBig = (HICON)SendMessage( win, WM_SETICON, ICON_BIG,   (LPARAM)hIconBig );
     hIconSmall = (HICON)SendMessage( win, WM_SETICON, ICON_SMALL, (LPARAM)hIconSmall );
-    
+
 }
 
 void KWindowSystem::setState( WId win, unsigned long state )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     bool got = false;
     if (state & NET::SkipTaskbar) {
         got = true;
@@ -501,51 +513,65 @@ void KWindowSystem::setState( WId win, unsigned long state )
         got = true;
         SetWindowPos(win, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
     }
+    if(state & NET::Max){
+        got = true;
+        ShowWindow( win, SW_MAXIMIZE );
+    }
     if (!got)
         kDebug() << "KWindowSystem::setState( WId win, unsigned long state ) isn't yet implemented for the state you requested!";
 }
 
 void KWindowSystem::clearState( WId win, unsigned long state )
 {
-    KWindowSystem::init(INFO_WINDOWS);
-    //TODO
-    kDebug() << "KWindowSystem::clearState( WId win, unsigned long state ) isn't yet implemented!";
+    bool got = false;
+
+    if (state & NET::SkipTaskbar) {
+        got = true;
+        LONG_PTR lp = GetWindowLongPtr(win, GWL_EXSTYLE);
+        SetWindowLongPtr(win, GWL_EXSTYLE, lp & ~WS_EX_TOOLWINDOW);
+    }
+    if (state & NET::KeepAbove) {
+        got = true;
+        //lets hope this remove the topmost
+        SetWindowPos(win, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
+    }
+    if(state & NET::Max){
+        got = true;
+        ShowWindow( win, SW_RESTORE );
+    }
+    if (!got)
+        kDebug() << "KWindowSystem::clearState( WId win, unsigned long state ) isn't yet implemented!";
 }
 
 void KWindowSystem::minimizeWindow( WId win, bool animation)
 {
     Q_UNUSED( animation );
-    KWindowSystem::init(INFO_WINDOWS);    
     ShowWindow( win, SW_MINIMIZE );
 }
 
 void KWindowSystem::unminimizeWindow( WId win, bool animation )
 {
     Q_UNUSED( animation );
-    KWindowSystem::init(INFO_WINDOWS);    
     ShowWindow( win, SW_RESTORE );
 }
 
 void KWindowSystem::raiseWindow( WId win )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     SetWindowPos( win, HWND_TOP, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE ); // mhhh?
 }
 
 void KWindowSystem::lowerWindow( WId win )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     SetWindowPos( win, HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOSIZE ); // mhhh?
 }
 
 bool KWindowSystem::compositingActive()
 {
-    return false;
+    return true;
 }
 
 QRect KWindowSystem::workArea( int desktop )
 {
-    KWindowSystem::init(INFO_WINDOWS);
     return s_deskWidget->availableGeometry( desktop );
 }
 
@@ -553,7 +579,6 @@ QRect KWindowSystem::workArea( const QList<WId>& exclude, int desktop )
 {
     //TODO
     kDebug() << "QRect KWindowSystem::workArea( const QList<WId>& exclude, int desktop ) isn't yet implemented!";
-    KWindowSystem::init(INFO_WINDOWS);    
     return QRect();
 }
 
@@ -564,7 +589,7 @@ QString KWindowSystem::desktopName( int desktop )
 
 void KWindowSystem::setDesktopName( int desktop, const QString& name )
 {
-     kDebug() << "KWindowSystem::setDesktopName( int desktop, const QString& name ) isn't yet implemented!";  
+     kDebug() << "KWindowSystem::setDesktopName( int desktop, const QString& name ) isn't yet implemented!";
     //TODO
 }
 
@@ -631,20 +656,19 @@ void KWindowSystem::doNotManage( const QString& title )
 
 QList<WId> KWindowSystem::stackingOrder()
 {
-  KWindowSystem::init(INFO_WINDOWS);  
+  KWindowSystem::init(INFO_WINDOWS);
   return KWindowSystem::s_d_func()->stackingOrder;
 }
 
 const QList<WId>& KWindowSystem::windows()
 {
-  KWindowSystem::init(INFO_WINDOWS);  
+  KWindowSystem::init(INFO_WINDOWS);
   return KWindowSystem::s_d_func()->stackingOrder;
 }
 
 void KWindowSystem::setType( WId win, NET::WindowType windowType )
 {
  //TODO
- KWindowSystem::init(INFO_WINDOWS); 
  kDebug() << "setType( WId win, NET::WindowType windowType ) isn't yet implemented!";
 }
 
@@ -660,4 +684,9 @@ bool KWindowSystem::hasWId(WId w)
     return KWindowSystem::s_d_func()->winInfos.contains(w);
 }
 
+void KWindowSystem::allowExternalProcessWindowActivation( int pid )
+{
+    AllowSetForegroundWindow( pid == -1 ? ASFW_ANY : pid );
+}
+
 #include "kwindowsystem.moc"
