@@ -118,7 +118,8 @@ public:
        displays, the screen number may be explicitly defined. If this argument is
        omitted, the default screen will be used.
 
-       @param doActivate true to activate the window
+       @param doActivate true to call activate() to do an initial data read/update
+       of the query information.
     **/
     NETRootInfo(Display *display, const unsigned long properties[], int properties_size,
                 int screen = -1, bool doActivate = true);
@@ -179,6 +180,39 @@ public:
     **/
     int screenNumber() const;
 
+    /**
+      Sets the given property if on is true, and clears the property otherwise.
+      In WindowManager mode this function updates _NET_SUPPORTED.
+      In Client mode this function does nothing.
+
+      @since 4.4
+     **/
+    void setSupported( NET::Property property, bool on = true );
+
+    /**
+      @overload
+      @since 4.4
+     **/
+    void setSupported( NET::Property2 property, bool on = true );
+
+    /**
+      @overload
+      @since 4.4
+     **/
+    void setSupported( NET::WindowType property, bool on = true );
+
+    /**
+      @overload
+      @since 4.4
+     **/
+    void setSupported( NET::State property, bool on = true );
+
+    /**
+      @overload
+      @since 4.4
+     **/
+    void setSupported( NET::Action property, bool on = true );
+
     /**
        Returns true if the given property is supported by the window
        manager. Note that for Client mode, NET::Supported needs
@@ -1109,6 +1143,34 @@ public:
     **/
     void setFrameExtents(NETStrut strut);
 
+    /**
+       Returns the frame decoration strut, i.e. the width of the decoration borders.
+
+       @since 4.3
+    **/
+    NETStrut frameExtents() const;
+
+    /**
+       Sets the window frame overlap strut, i.e. how far the window frame extends
+       behind the client area on each side.
+
+       Set the strut values to -1 if you want the window frame to cover the whole
+       client area.
+
+       The default values are 0.
+
+       @since 4.4
+    **/
+    void setFrameOverlap(NETStrut strut);
+
+    /**
+       Returns the frame overlap strut, i.e. how far the window frame extends
+       behind the client area on each side.
+
+       @since 4.4
+    **/ 
+    NETStrut frameOverlap() const;
+
     /**
        Returns an icon.  If width and height are passed, the icon returned will be
        the closest it can find (the next biggest).  If width and height are omitted,
@@ -1121,6 +1183,13 @@ public:
        @return the icon
     **/
     NETIcon icon(int width = -1, int height = -1) const;
+    
+    /**
+      Returns a list of provided icon sizes. Each size is pair width,height, terminated
+      with pair 0,0.
+      @since 4.3
+    **/
+    const int* iconSizes() const;
 
     /**
      * Sets user timestamp @p time on the window (property _NET_WM_USER_TIME).
