@@ -2,8 +2,9 @@
  * This file is part of the DOM implementation for KDE.
  *
  * Copyright (C) 1999-2003 Lars Knoll (knoll@kde.org)
- *           (C) 2004-2007 Apple Computer, Inc.
+ *           (C) 2004-2008 Apple Computer, Inc.
  *           (C) 2005 Allan Sandfeld Jensen (kde@carewolf.com)
+ *           (C) 2009 Germain Garand (germain@ebooksfrance.org)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -102,7 +103,7 @@ static DOMString quoteString(const DOMString &string)
     QString s = string.string();
     s.replace('\\', "\\\\");
     s.replace('\'', "\\'");
-    return '\'' + s + '\'';
+    return QString('\'' + s + '\'');
 }
 
 // Quotes the string if it needs quoting.
@@ -587,7 +588,8 @@ static void initShorthandMap(QHash<int, PropertyLonghand>& shorthandMap)
         CSS_PROP_BACKGROUND_POSITION_X,
         CSS_PROP_BACKGROUND_POSITION_Y,
         CSS_PROP_BACKGROUND_REPEAT,
-        CSS_PROP__KHTML_BACKGROUND_SIZE
+        CSS_PROP__KHTML_BACKGROUND_SIZE,
+        CSS_PROP_BACKGROUND_SIZE
     };
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_BACKGROUND, backgroundProperties);
 
@@ -607,12 +609,26 @@ static void initShorthandMap(QHash<int, PropertyLonghand>& shorthandMap)
     SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_OVERFLOW, overflowProperties);
 
     static const int borderRadiusProperties[] = {
+        CSS_PROP_BORDER_TOP_RIGHT_RADIUS,
+        CSS_PROP_BORDER_TOP_LEFT_RADIUS,
+        CSS_PROP_BORDER_BOTTOM_LEFT_RADIUS,
+        CSS_PROP_BORDER_BOTTOM_RIGHT_RADIUS
+    };
+    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_BORDER_RADIUS, borderRadiusProperties);
+    static const int prefixedBorderRadiusProperties[] = {
         CSS_PROP__KHTML_BORDER_TOP_RIGHT_RADIUS,
         CSS_PROP__KHTML_BORDER_TOP_LEFT_RADIUS,
         CSS_PROP__KHTML_BORDER_BOTTOM_LEFT_RADIUS,
         CSS_PROP__KHTML_BORDER_BOTTOM_RIGHT_RADIUS
     };
-    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP__KHTML_BORDER_RADIUS, borderRadiusProperties);
+    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP__KHTML_BORDER_RADIUS, prefixedBorderRadiusProperties);
+                                                
+    static const int markerProperties[] = {
+        CSS_PROP_MARKER_START, 
+        CSS_PROP_MARKER_MID,
+        CSS_PROP_MARKER_END
+    };
+    SET_SHORTHAND_MAP_ENTRY(shorthandMap, CSS_PROP_MARKER, markerProperties);
 
     #undef SET_SHORTHAND_MAP_ENTRY
 }
@@ -866,6 +882,26 @@ bool CSSStyleDeclarationImpl::parseString( const DOMString &/*string*/, bool )
     // ###
 }
 
+// --------------------------------------------------------------------------------------
+
+void CSSInlineStyleDeclarationImpl::setChanged()
+{
+    if (m_node)
+        m_node->setNeedsStyleAttributeUpdate();
+    CSSStyleDeclarationImpl::setChanged();
+}
+
+void CSSInlineStyleDeclarationImpl::updateFromAttribute(const DOMString &value)
+{
+    if(!m_lstValues) {
+	m_lstValues = new QList<CSSProperty*>;
+    } else {
+        clear();
+    }
+    CSSParser parser( strictParsing );
+    parser.parseDeclaration( this, value );
+    CSSStyleDeclarationImpl::setChanged();
+}
 
 // --------------------------------------------------------------------------------------
 
@@ -913,7 +949,13 @@ DOM::DOMString CSSValueListImpl::cssText() const
     DOMString result = "";
 
     for (QListIterator<CSSValueImpl*> iterator(m_values); iterator.hasNext();) {
-	result += iterator.next()->cssText();
+        if (!result.isEmpty()) {
+            if (m_separator == Comma)
+                result += ", ";
+            else if (m_separator == Space)
+                result += " ";
+        }
+        result += iterator.next()->cssText();
     }
 
     return result;
@@ -1211,7 +1253,9 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 	    text = getValueName(m_value.ident);
 	    break;
 	case CSSPrimitiveValue::CSS_ATTR:
-	    // ###
+            text = "attr(";
+            text += DOMString( m_value.string );
+            text += ")";
 	    break;
 	case CSSPrimitiveValue::CSS_COUNTER:
             text = "counter(";
@@ -1234,14 +1278,14 @@ DOM::DOMString CSSPrimitiveValueImpl::cssText() const
 		if (m_value.rgbcolor == khtml::transparentColor)
 		    text = "transparent";
 		else
-		    text = "rgba(" + QString::number(qRed  (m_value.rgbcolor)) + ", "
+                    text = QString("rgba(" + QString::number(qRed  (m_value.rgbcolor)) + ", "
 				   + QString::number(qGreen(m_value.rgbcolor)) + ", "
 				   + QString::number(qBlue (m_value.rgbcolor)) + ", "
-				   + QString::number(qAlpha(m_value.rgbcolor)/255.0) + ")";
+                                   + QString::number(qAlpha(m_value.rgbcolor)/255.0) + ")");
 	    } else {
-                text = "rgb(" + QString::number(qRed  (m_value.rgbcolor)) + ", "
+                text = QString("rgb(" + QString::number(qRed  (m_value.rgbcolor)) + ", "
                               + QString::number(qGreen(m_value.rgbcolor)) + ", "
-                              + QString::number(qBlue (m_value.rgbcolor)) + ")";
+                              + QString::number(qBlue (m_value.rgbcolor)) + ")");
 	    }
 	    break;
         case CSSPrimitiveValue::CSS_PAIR:
@@ -1357,15 +1401,21 @@ FontFamilyValueImpl::FontFamilyValueImpl( const QString &string)
 : CSSPrimitiveValueImpl( DOMString(string), CSSPrimitiveValue::CSS_STRING)
 {
     static const QRegExp parenReg(" \\(.*\\)$");
-    static const QRegExp braceReg(" \\[.*\\]$");
+//  static const QRegExp braceReg(" \\[.*\\]$");
 
     parsedFontName = string;
+
     // a language tag is often added in braces at the end. Remove it.
     parsedFontName.replace(parenReg, QString());
+
+#if 0
+    // cannot use such early checks against the font database anymore,
+    // as the font subsystem might not contain the requested font yet
+    // (case of downloadable font faces)
+
     // remove [Xft] qualifiers
     parsedFontName.replace(braceReg, QString());
 
-#ifndef APPLE_CHANGES
     const QString &available = KHTMLSettings::availableFamilies();
 
     parsedFontName = parsedFontName.toLower();
@@ -1389,8 +1439,7 @@ FontFamilyValueImpl::FontFamilyValueImpl( const QString &string)
        assert( p != -1 ); // available is supposed to start and end with ,
        parsedFontName = available.mid( pos, p - pos);
        // kDebug(0) << "going for '" << parsedFontName << "'";
-    } else
-        parsedFontName.clear();
+    }
 
 #endif // !APPLE_CHANGES
 }
@@ -1461,7 +1510,7 @@ QuotesValueImpl::QuotesValueImpl()
 
 DOMString QuotesValueImpl::cssText() const
 {
-    return "\"" + data.join("\" \"") + "\"";
+    return QString("\"" + data.join("\" \"") + "\"");
 }
 
 void QuotesValueImpl::addLevel(const QString& open, const QString& close)
@@ -1546,3 +1595,50 @@ DOMString CSSProperty::cssText() const
 {
     return getPropertyName(m_id) + DOMString(": ") + m_value->cssText() + (m_important ? DOMString(" !important") : DOMString()) + DOMString("; ");
 }
+
+// -------------------------------------------------------------------------
+
+#if 0
+    // ENABLE(SVG_FONTS)
+bool CSSFontFaceSrcValueImpl::isSVGFontFaceSrc() const
+{
+    return !strcasecmp(m_format, "svg");
+}
+#endif
+
+bool CSSFontFaceSrcValueImpl::isSupportedFormat() const
+{
+    // Normally we would just check the format, but in order to avoid conflicts with the old WinIE style of font-face,
+    // we will also check to see if the URL ends with .eot.  If so, we'll go ahead and assume that we shouldn't load it.
+    if (m_format.isEmpty()) {
+        // Check for .eot.
+        if (m_resource.endsWith(".eot") || m_resource.endsWith(".EOT"))
+            return false;
+        return true;
+    }
+
+    return !strcasecmp(m_format, "truetype") || !strcasecmp(m_format, "opentype")
+#if 0
+    //ENABLE(SVG_FONTS)
+           || isSVGFontFaceSrc()
+#endif
+           ;
+}
+
+DOMString CSSFontFaceSrcValueImpl::cssText() const
+{
+    DOMString result;
+    if (isLocal())
+        result += "local(";
+    else
+        result += "url(";
+    result += m_resource;
+    result += ")";
+    if (!m_format.isEmpty()) {
+        result += " format(";
+        result += m_format;
+        result += ")";
+    }
+    return result;
+}
+
