@@ -48,6 +48,7 @@
 #include "kjs_events.h"
 #include "kjs_views.h"
 #include "kjs_window.h"
+#include "xmlhttprequest.h"
 #include <kjs/PropertyNameArray.h>
 #include <dom/dom_exception.h>
 #include <dom/html_document.h>
@@ -1364,14 +1365,23 @@ JSValue* DOMElementProtoFunc::callAsFunction(ExecState *exec, JSObject *thisObj,
       return jsBoolean(element.hasAttributeNS(args[0]->toString(exec).domString(),args[1]->toString(exec).domString()));
     case DOMElement::GetElementsByClassName: // HTML 5
       return getDOMNodeList(exec, element.getElementsByClassName(args[0]->toString(exec).domString()));
-    case DOMElement::Focus:
-      element.focus();
-      return jsUndefined();
-    case DOMElement::Blur:
-      element.blur();
-      return jsUndefined();
     default:
-      return jsUndefined();
+
+      // Make sure our layout is up to date before we call these
+      DOM::DocumentImpl* docimpl = element.document();
+      if (docimpl) {
+          docimpl->updateLayout();
+      }
+      switch(id) {
+        case DOMElement::Focus:
+          element.focus();
+          return jsUndefined();
+        case DOMElement::Blur:
+          element.blur();
+          return jsUndefined();
+        default:
+          return jsUndefined();
+      }
   }
 }
 
@@ -1567,14 +1577,15 @@ bool DOMNamedNodeMap::getOwnPropertySlot(ExecState *exec, const Identifier& prop
     return true;
   }
 
+  // See if it's an item name get
+  DOM::NodeImpl* attr = impl()->getNamedItem(propertyName.ustring().domString());
+  if (attr)
+    return getImmediateValueSlot(this, getDOMNode(exec, attr), slot);
+
   //May be it's an index?
   if (getIndexSlot(this, *m_impl, propertyName, slot))
     return true;
 
-  // Could also be a name... A local name, that is.
-//  if (
-//  slot.setCustom(this, nameGetter)
-
   return DOMObject::getOwnPropertySlot(exec, propertyName, slot);
 }
 
@@ -1744,6 +1755,16 @@ static DOMObject* create##name##Wrapper(ExecState* exec, DOM::NodeImpl* n) \
 CREATE_WRAPPER_FUNCTION(Audio)
 CREATE_WRAPPER_FUNCTION(Video)
 
+JSValue* KJS::getEventTarget(ExecState* exec, DOM::EventTargetImpl* t)
+{
+    if (!t)
+        return jsNull();
+    if (t->eventTargetType() == EventTargetImpl::DOM_NODE)
+        return getDOMNode(exec, static_cast<DOM::NodeImpl*>(t));
+    else
+        return static_cast<XMLHttpRequest*>(t); //static
+}
+
 JSValue* KJS::getDOMNode(ExecState *exec, DOM::NodeImpl* n)
 {
   DOMObject *ret = 0;
