@@ -49,7 +49,7 @@ using KIO::NetAccess;
 
 using namespace KJS;
 using namespace DOM;
-// 
+//
 ////////////////////// XMLHttpRequest Object ////////////////////////
 
 /* Source for XMLHttpRequestProtoTable.
@@ -269,12 +269,11 @@ static bool isValidFieldValue(const QString& name)
 static bool canSetRequestHeader(const QString& name)
 {
     static QSet<CaseInsensitiveString> forbiddenHeaders;
-    
+
     if (forbiddenHeaders.isEmpty()) {
 	static const char* hdrs[] = {
 	    "accept-charset",
 	    "accept-encoding",
-	    "authorization",
 	    "content-length",
 	    "connect",
 	    "copy",
@@ -299,12 +298,12 @@ static bool canSetRequestHeader(const QString& name)
 	    "transfer-encoding",
 	    "unlock",
 	    "upgrade",
-	    "via" 
+	    "via"
 	};
 	for (size_t i = 0; i < sizeof(hdrs)/sizeof(char*); ++i)
 	    forbiddenHeaders.insert(CaseInsensitiveString(hdrs[i]));
     }
-    
+
     return !forbiddenHeaders.contains(name);
 }
 
@@ -322,14 +321,19 @@ XMLHttpRequest::XMLHttpRequest(ExecState *exec, DOM::DocumentImpl* d)
     createdDocument(false),
     aborted(false)
 {
+  ref(); // we're a GC point, so refcount pin.
   setPrototype(XMLHttpRequestProto::self(exec));
 }
 
 XMLHttpRequest::~XMLHttpRequest()
 {
-  if (onLoadListener) 
+  if (job && method.toUpper() != QLatin1String("POST")) {
+        job->kill();
+        job = 0;
+  }
+  if (onLoadListener)
       onLoadListener->deref();
-  if (onReadyStateChangeListener) 
+  if (onReadyStateChangeListener)
       onReadyStateChangeListener->deref();
   delete qObject;
   qObject = 0;
@@ -339,6 +343,17 @@ XMLHttpRequest::~XMLHttpRequest()
 
 void XMLHttpRequest::changeState(XMLHttpRequestState newState)
 {
+  // Other engines cancel transfer if the controlling document doesn't
+  // exist anymore. Match that, though being paranoid about post
+  // (And don't emit any events w/o a doc, if we're kept alive otherwise).
+  if (!doc) {
+    if (job && method.toUpper() != QLatin1String("POST")) {
+      job->kill();
+      job = 0;
+    }
+    return;
+  }
+
   if (m_state != newState) {
     m_state = newState;
     ProtectedPtr<JSObject> ref(this);
@@ -346,13 +361,24 @@ void XMLHttpRequest::changeState(XMLHttpRequestState newState)
     if (onReadyStateChangeListener != 0 && doc->view() && doc->view()->part()) {
       DOM::Event ev = doc->view()->part()->document().createEvent("HTMLEvents");
       ev.initEvent("readystatechange", true, true);
+      ev.handle()->setTarget(this);
+      ev.handle()->setCurrentTarget(this);
       onReadyStateChangeListener->handleEvent(ev);
+
+      // Make sure the event doesn't point to us, since it can't prevent
+      // us from being collecte.
+      ev.handle()->setTarget(0);
+      ev.handle()->setCurrentTarget(0);
     }
 
     if (m_state == XHRS_Loaded && onLoadListener != 0 && doc->view() && doc->view()->part()) {
       DOM::Event ev = doc->view()->part()->document().createEvent("HTMLEvents");
       ev.initEvent("load", true, true);
+      ev.handle()->setTarget(this);
+      ev.handle()->setCurrentTarget(this);
       onLoadListener->handleEvent(ev);
+      ev.handle()->setTarget(0);
+      ev.handle()->setCurrentTarget(0);
     }
   }
 }
@@ -380,7 +406,7 @@ bool XMLHttpRequest::urlMatchesDocumentDomain(const KUrl& _url) const
   return false;
 }
 
-void XMLHttpRequest::open(const QString& _method, const KUrl& _url, bool _async)
+void XMLHttpRequest::open(const QString& _method, const KUrl& _url, bool _async, int& ec)
 {
   abort();
   aborted = false;
@@ -393,6 +419,7 @@ void XMLHttpRequest::open(const QString& _method, const KUrl& _url, bool _async)
   responseXML = 0;
 
   if (!urlMatchesDocumentDomain(_url)) {
+    ec = DOMException::SECURITY_ERR;
     return;
   }
 
@@ -412,17 +439,18 @@ void XMLHttpRequest::send(const QString& _body, int& ec)
       return;
   }
 
-  if (method == "post") {
-    QString protocol = url.protocol().toLower();
+  const QString protocol = url.protocol().toLower();
+  // Abandon the request when the protocol is other than "http",
+  // instead of blindly doing a KIO::get on other protocols like file:/.
+  if (!protocol.startsWith(QLatin1String("http")) &&
+      !protocol.startsWith(QLatin1String("webdav")))
+  {
+    ec = DOMException::INVALID_ACCESS_ERR;
+    abort();
+    return;
+  }
 
-    // Abondon the request when the protocol is other than "http",
-    // instead of blindly changing it to a "get" request.
-    if (!protocol.startsWith(QLatin1String("http")) &&
-        !protocol.startsWith(QLatin1String("webdav")))
-    {
-      abort();
-      return;
-    }
+  if (method == "post") {
 
     // FIXME: determine post encoding correctly by looking in headers
     // for charset.
@@ -560,7 +588,7 @@ void XMLHttpRequest::setRequestHeader(const QString& _name, const QString& _valu
   // the webDAV headers such as PROPFIND etc...
   if (name == "get"  || name == "post") {
     KUrl reqURL (doc->URL(), value.trimmed());
-    open(name, reqURL, async);
+    open(name, reqURL, async, ec);
     return;
   }
 
@@ -580,7 +608,7 @@ JSValue *XMLHttpRequest::getAllResponseHeaders(int& ec) const
       ec = DOMException::INVALID_STATE_ERR;
       return jsString("");
   }
-  
+
   // ### test error flag, return jsNull
 
   if (responseHeaders.isEmpty()) {
@@ -729,6 +757,15 @@ void XMLHttpRequest::slotRedirection(KIO::Job*, const KUrl& url)
   }
 }
 
+static QString encodingFromContentType(const QString& type)
+{
+    QString encoding;
+    int index = type.indexOf(';');
+    if (index > -1)
+        encoding = type.mid( index+1 ).remove(QRegExp("charset[ ]*=[ ]*", Qt::CaseInsensitive)).trimmed();
+    return encoding;
+}
+
 #ifdef APPLE_CHANGES
 void XMLHttpRequest::slotData( KIO::Job*, const char *data, int len )
 #else
@@ -756,20 +793,22 @@ void XMLHttpRequest::slotData(KIO::Job*, const QByteArray &_data)
 #endif
 
   if ( decoder == NULL ) {
-    int pos = responseHeaders.indexOf(QLatin1String("content-type:"), 0, Qt::CaseInsensitive);
+    if (!m_mimeTypeOverride.isEmpty())
+        encoding = encodingFromContentType(m_mimeTypeOverride);
 
-    if ( pos > -1 ) {
-      pos += 13;
-      int index = responseHeaders.indexOf('\n', pos);
-      QString type = responseHeaders.mid(pos, (index-pos));
-      index = type.indexOf(';');
-      if (index > -1)
-        encoding = type.mid( index+1 ).remove(QRegExp("charset[ ]*=[ ]*", Qt::CaseInsensitive)).trimmed();
+    if (encoding.isEmpty()) {
+      int pos = responseHeaders.indexOf(QLatin1String("content-type:"), 0, Qt::CaseInsensitive);
+      if ( pos > -1 ) {
+        pos += 13;
+        int index = responseHeaders.indexOf('\n', pos);
+        QString type = responseHeaders.mid(pos, (index-pos));
+        encoding = encodingFromContentType(type);
+      }
     }
 
     decoder = new KEncodingDetector;
 
-    if (!encoding.isNull())
+    if (!encoding.isEmpty())
       decoder->setEncoding(encoding.toLatin1().constData(), KEncodingDetector::EncodingFromHTTPHeader);
     else
       decoder->setEncoding("UTF-8", KEncodingDetector::DefaultEncoding);
@@ -797,6 +836,11 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
 
   XMLHttpRequest *request = static_cast<XMLHttpRequest *>(thisObj);
 
+  if (!request->doc) {
+    setDOMException(exec, DOMException::INVALID_STATE_ERR);
+    return jsUndefined();
+  }
+
   int ec = 0;
 
   switch (id) {
@@ -809,7 +853,7 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
       setDOMException(exec, ec);
       return ret;
     }
-  case XMLHttpRequest::GetResponseHeader: 
+  case XMLHttpRequest::GetResponseHeader:
     {
       if (args.size() < 1)
           return throwError(exec, SyntaxError, "Not enough arguments");
@@ -841,8 +885,8 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
 	url.setPass(args[4]->toString(exec).qstring());
       }
 
-      request->open(method, url, async);
-
+      request->open(method, url, async, ec);
+      setDOMException(exec, ec);
       return jsUndefined();
     }
   case XMLHttpRequest::Send:
@@ -852,11 +896,11 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
         DOM::NodeImpl* docNode = toNode(args[0]);
         if (docNode && docNode->isDocumentNode()) {
           DOM::DocumentImpl *doc = static_cast<DOM::DocumentImpl *>(docNode);
-          
+
           try {
             body = doc->toString().string();
             // FIXME: also need to set content type, including encoding!
-  
+
           } catch(DOM::DOMException&) {
             return throwError(exec, GeneralError, "Exception serializing document");
           }
@@ -866,7 +910,7 @@ JSValue *XMLHttpRequestProtoFunc::callAsFunction(ExecState *exec, JSObject *this
       }
 
       request->send(body, ec);
-      setDOMException(exec, ec);  
+      setDOMException(exec, ec);
       return jsUndefined();
     }
   case XMLHttpRequest::SetRequestHeader:
