@@ -114,34 +114,6 @@ DOMString HTMLElementImpl::namespaceURI() const
         DOMString(XHTML_NAMESPACE) : DOMString();
 }
 
-
-DOMString HTMLElementImpl::localName() const
-{
-    // We only have a localName if we were created by createElementNS(), in which
-    // case we are an XHTML element. This also means we have a lowercase name.
-    if (!m_htmlCompat) // XHTML == not HTMLCompat
-    {
-        return LocalName::fromId(id()).toString();
-    }
-    // createElement() always returns elements with a null localName.
-    else
-	return DOMString();
-}
-
-DOMString HTMLElementImpl::tagName() const
-{
-    DOMString tn = LocalName::fromId(id()).toString();
-
-    if ( m_htmlCompat )
-        tn = tn.upper();
-
-    DOMString prefix = m_prefix.toString();
-    if (!prefix.isEmpty())
-        return prefix + ":" + tn;
-
-    return tn;
-}
-
 void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
 {
     DOMString indexstring;
@@ -184,11 +156,7 @@ void HTMLElementImpl::parseAttribute(AttributeImpl *attr)
         setContentEditable(attr);
         break;
     case ATTR_STYLE:
-        if (inlineStyleDecls())
-	    inlineStyleDecls()->clear();
-	else
-	    createInlineDecl();
-        inlineStyleDecls()->setProperty(attr->value());
+        getInlineStyleDecls()->updateFromAttribute(attr->value());
         setChanged();
         break;
     case ATTR_TABINDEX:
