@@ -353,7 +353,7 @@ void HTMLTokenizer::parseSpecial(TokenizerString &src)
     while ( !src.isEmpty() ) {
         checkScriptBuffer();
         unsigned char ch = src->toLatin1();
-        if ( !scriptCodeResync && !brokenComments && !textarea && !xmp && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QString::fromRawData( scriptCode+scriptCodeSize-3, 3 ) == "<!-" ) {
+        if ( !scriptCodeResync && !brokenComments && !title && !textarea && !xmp && ch == '-' && scriptCodeSize >= 3 && !src.escaped() && QString::fromRawData( scriptCode+scriptCodeSize-3, 3 ) == "<!-" ) {
             comment = true;
             scriptCode[ scriptCodeSize++ ] = ch;
             ++src;
@@ -1038,7 +1038,7 @@ void HTMLTokenizer::parseEntity(TokenizerString &src, QChar *&dest, bool start)
 #ifdef TOKEN_DEBUG
                 kDebug( 6036 ) << "unknown entity!";
 #endif
-                checkBuffer(10);
+                checkBuffer(11);
                 // ignore the sequence, add it to the buffer as plaintext
                 *dest++ = '&';
                 for(unsigned int i = 0; i < cBufferPos; i++)
@@ -1168,7 +1168,7 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                 // Accept empty xml tags like <br/>
                 if(len > 1 && ptr[len-1] == '/' ) {
                     ptr[--len] = '\0';
-                    // if its like <br/> and not like <input/ value=foo>, take it as flat
+                    // if it is like <br/> and not like <input/ value=foo>, take it as flat
                     if (*src == '>')
                         currToken.flat = true;
                 }
@@ -1176,7 +1176,6 @@ void HTMLTokenizer::parseTag(TokenizerString &src)
                 uint tagID = 0;
                 if (!tagID) {
                     DOMString tagName(ptr);
-                    DocumentImpl *doc = parser->docPtr();
                     if (Element::khtmlValidQualifiedName(tagName)) {
                         safeLocalName = LocalName::fromString(tagName, IDS_NormalizeLower);
                         tagID = safeLocalName.id();
