@@ -193,33 +193,35 @@ void KHTMLPartBrowserExtension::cut()
 {
     if ( m_extensionProxy )
     {
-        callExtensionProxyMethod( "cut()" );
+        callExtensionProxyMethod( "cut" );
         return;
     }
 
     if ( !m_editableFormWidget )
         return;
 
-    if ( m_editableFormWidget->inherits( "QLineEdit" ) )
-        static_cast<QLineEdit *>( &(*m_editableFormWidget) )->cut();
-    else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-        static_cast<QTextEdit *>( &(*m_editableFormWidget) )->cut();
+    QLineEdit* lineEdit = qobject_cast<QLineEdit *>( m_editableFormWidget );
+    if ( lineEdit && !lineEdit->isReadOnly() )
+        lineEdit->cut();
+    QTextEdit* textEdit = qobject_cast<QTextEdit *>( m_editableFormWidget );
+    if ( textEdit && !textEdit->isReadOnly() )
+        textEdit->cut();
 }
 
 void KHTMLPartBrowserExtension::copy()
 {
     if ( m_extensionProxy )
     {
-        callExtensionProxyMethod( "copy()" );
+        callExtensionProxyMethod( "copy" );
         return;
     }
 
-    kDebug( 6050 ) << "************! KHTMLPartBrowserExtension::copy()";
     if ( !m_editableFormWidget )
     {
         // get selected text and paste to the clipboard
         QString text = m_part->selectedText();
         text.replace( QChar( 0xa0 ), ' ' );
+        //kDebug(6050) << text;
 
         QClipboard *cb = QApplication::clipboard();
         disconnect( cb, SIGNAL( selectionChanged() ), m_part, SLOT( slotClearSelection() ) );
@@ -249,10 +251,12 @@ void KHTMLPartBrowserExtension::copy()
     }
     else
     {
-        if ( m_editableFormWidget->inherits( "QLineEdit" ) )
-            static_cast<QLineEdit *>( &(*m_editableFormWidget) )->copy();
-        else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-            static_cast<QTextEdit *>( &(*m_editableFormWidget) )->copy();
+        QLineEdit* lineEdit = qobject_cast<QLineEdit *>( m_editableFormWidget );
+        if ( lineEdit )
+            lineEdit->copy();
+        QTextEdit* textEdit = qobject_cast<QTextEdit *>( m_editableFormWidget );
+        if ( textEdit )
+            textEdit->copy();
     }
 }
 
@@ -261,7 +265,7 @@ void KHTMLPartBrowserExtension::searchProvider()
     // action name is of form "previewProvider[<searchproviderprefix>:]"
     const QString searchProviderPrefix = QString( sender()->objectName() ).mid( 14 );
 
-    const QString text = m_part->simplifiedSelectedText();
+    const QString text = KHTMLPopupGUIClient::selectedTextAsOneLine(m_part); // #219815
     KUriFilterData data;
     QStringList list;
     data.setData( searchProviderPrefix + text );
@@ -285,17 +289,19 @@ void KHTMLPartBrowserExtension::paste()
 {
     if ( m_extensionProxy )
     {
-        callExtensionProxyMethod( "paste()" );
+        callExtensionProxyMethod( "paste" );
         return;
     }
 
     if ( !m_editableFormWidget )
         return;
 
-    if ( m_editableFormWidget->inherits( "QLineEdit" ) )
-        static_cast<QLineEdit *>( &(*m_editableFormWidget) )->paste();
-    else if ( m_editableFormWidget->inherits( "QTextEdit" ) )
-        static_cast<QTextEdit *>( &(*m_editableFormWidget) )->paste();
+    QLineEdit* lineEdit = qobject_cast<QLineEdit *>( m_editableFormWidget );
+    if ( lineEdit && !lineEdit->isReadOnly() )
+        lineEdit->paste();
+    QTextEdit* textEdit = qobject_cast<QTextEdit *>( m_editableFormWidget );
+    if ( textEdit && !textEdit->isReadOnly() )
+        textEdit->paste();
 }
 
 void KHTMLPartBrowserExtension::callExtensionProxyMethod( const char *method )
@@ -303,10 +309,6 @@ void KHTMLPartBrowserExtension::callExtensionProxyMethod( const char *method )
     if ( !m_extensionProxy )
         return;
 
-    int slot = m_extensionProxy->metaObject()->indexOfSlot( method );
-    if ( slot == -1 )
-        return;
-
     QMetaObject::invokeMethod(m_extensionProxy, method, Qt::DirectConnection);
 }
 
@@ -428,7 +430,7 @@ KHTMLPopupGUIClient::KHTMLPopupGUIClient( KHTMLPart *khtml, const KUrl &url )
 
         addSearchActions(editActions);
 
-        QString selectedTextURL = selectedTextAsOneLine();
+        QString selectedTextURL = selectedTextAsOneLine(d->m_khtml);
         if ( selectedTextURL.contains("://") && KUrl(selectedTextURL).isValid() ) {
             if (selectedTextURL.length() > 18) {
                 selectedTextURL.truncate(15);
@@ -648,7 +650,9 @@ void KHTMLPopupGUIClient::addSearchActions(QList<QAction *>& editActions)
     }
 
     // default search provider
-    KService::Ptr service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(defaultEngine));
+    KService::Ptr service;
+    if( !defaultEngine.isEmpty())
+        service = KService::serviceByDesktopPath(QString("searchproviders/%1.desktop").arg(defaultEngine));
 
     // search provider icon
     KIcon icon;
@@ -715,9 +719,9 @@ void KHTMLPopupGUIClient::addSearchActions(QList<QAction *>& editActions)
     }
 }
 
-QString KHTMLPopupGUIClient::selectedTextAsOneLine() const
+QString KHTMLPopupGUIClient::selectedTextAsOneLine(KHTMLPart* part)
 {
-    QString text = d->m_khtml->simplifiedSelectedText();
+    QString text = part->simplifiedSelectedText();
     // in addition to what simplifiedSelectedText does,
     // remove linefeeds and any whitespace surrounding it (#113177),
     // to get it all in a single line.
@@ -730,7 +734,7 @@ void KHTMLPopupGUIClient::openSelection()
     KParts::BrowserArguments browserArgs;
     browserArgs.frameName = "_blank";
 
-    emit d->m_khtml->browserExtension()->openUrlRequest(selectedTextAsOneLine(), KParts::OpenUrlArguments(), browserArgs);
+    emit d->m_khtml->browserExtension()->openUrlRequest(selectedTextAsOneLine(d->m_khtml), KParts::OpenUrlArguments(), browserArgs);
 }
 
 KParts::BrowserExtension::ActionGroupMap KHTMLPopupGUIClient::actionGroups() const
@@ -881,8 +885,9 @@ void KHTMLPopupGUIClient::slotFrameInWindow()
 {
   KParts::OpenUrlArguments args = d->m_khtml->arguments();
   args.metaData()["referrer"] = d->m_khtml->pageReferrer();
-  args.metaData()["forcenewwindow"] = "true";
-  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args );
+  KParts::BrowserArguments browserArgs( d->m_khtml->browserExtension()->browserArguments() );
+  browserArgs.setForcesNewWindow(true);
+  emit d->m_khtml->browserExtension()->createNewWindow( d->m_khtml->url(), args, browserArgs );
 }
 
 void KHTMLPopupGUIClient::slotFrameInTop()
@@ -912,8 +917,8 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
   QString name = QLatin1String( "index.html" );
   if ( !suggestedFilename.isEmpty() )
     name = suggestedFilename;
-  else if ( !url.fileName().isEmpty() )
-    name = url.fileName();
+  else if ( !url.fileName(KUrl::ObeyTrailingSlash).isEmpty() )
+    name = url.fileName(KUrl::ObeyTrailingSlash);
 
   KUrl destURL;
   int query;
@@ -922,7 +927,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget *parent, const QString &caption,
     destURL = KFileDialog::getSaveUrl( name, filter, parent, caption );
       if( destURL.isLocalFile() )
       {
-        QFileInfo info( destURL.path() );
+        QFileInfo info( destURL.toLocalFile() );
         if( info.exists() ) {
           // TODO: use KIO::RenameDlg (shows more information)
           query = KMessageBox::warningContinueCancel( parent, i18n( "A file named \"%1\" already exists. " "Are you sure you want to overwrite it?" ,  info.fileName() ), i18n( "Overwrite File?" ), KGuiItem(i18n( "Overwrite" )) );
@@ -945,7 +950,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget* parent, const KUrl &url, const KUrl
         {
             if (destURL.isLocalFile())
             {
-                KSaveFile destFile(destURL.path());
+                KSaveFile destFile(destURL.toLocalFile());
                 if (destFile.open())
                 {
                     QDataStream stream ( &destFile );
@@ -1007,11 +1012,7 @@ void KHTMLPopupGUIClient::saveURL( QWidget* parent, const KUrl &url, const KUrl
 
           if ( downloadViaKIO )
           {
-              KIO::Job *job = KIO::file_copy( url, destURL, -1, KIO::Overwrite );
-              job->setMetaData(metadata);
-              job->addMetaData("MaxCacheSize", "0"); // Don't store in http cache.
-              job->addMetaData("cache", "cache"); // Use entry from cache if available.
-              job->uiDelegate()->setAutoErrorHandlingEnabled( true );
+              KParts::BrowserRun::saveUrlUsingKIO(url, destURL, parent, metadata);
           }
         } //end if(!saved)
     }
