@@ -46,6 +46,7 @@
 #include "seed.h"
 #include <imload/image.h>
 #include <imload/imagepainter.h>
+#include <kfilterdev.h>
 
 #include <assert.h>
 
@@ -325,14 +326,14 @@ QString CachedCSSStyleSheet::checkCharset(const QByteArray& buffer ) const
     // @charset has to be first or directly after BOM.
     // CSS 2.1 says @charset should win over BOM, but since more browsers support BOM
     // than @charset, we default to that.
-    const char* d = (const char*) buffer.data();
+    const char* d = buffer.data();
     if (strncmp(d, "@charset \"",10) == 0)
     {
         // the string until "; is the charset name
         const char *p = strchr(d+10, '"');
         if (p == 0) return m_charset;
         QString charset = QString::fromAscii(d+10, p-(d+10));
-	return charset;
+        return charset;
     }
     return m_charset;
 }
@@ -623,7 +624,7 @@ QPixmap CachedImage::pixmap( ) const
         ImagePainter pi(i);
         pi.paint(0, 0, &paint);
         paint.end();
-        return QPixmap::fromImage( im );
+        return QPixmap::fromImage( im, Qt::NoOpaqueDetection );
     } else {
         QPixmap pm(w, h);
         if (i->hasAlpha())
@@ -997,6 +998,56 @@ void CachedSound::error( int /*err*/, const char* /*text*/ )
     checkNotify();
 }
 
+// -------------------------------------------------------------------------------------------
+
+CachedFont::CachedFont(DocLoader* dl, const DOMString &url, KIO::CacheControl _cachePolicy, const char*)
+    : CachedObject(url, Font, _cachePolicy, 0)
+{
+    setAccept( QLatin1String("*/*") );
+    Cache::loader()->load(dl, this, true /*highPriority*/);
+    m_loading = true;
+}
+
+void CachedFont::ref(CachedObjectClient *c)
+{
+    CachedObject::ref(c);
+
+    if(!m_loading) c->notifyFinished(this);
+}
+
+void CachedFont::data( QBuffer &buffer, bool eof )
+{
+    if(!eof) return;
+    buffer.close();
+    m_font = buffer.buffer();
+
+    // some fonts are compressed.
+    QIODevice* dev = KFilterDev::device(&buffer, mimetype(), false /*autoDeleteInDevice*/);
+    if (dev && dev->open( QIODevice::ReadOnly )) {
+        m_font = dev->readAll();
+        delete dev;
+    }
+
+    setSize(m_font.size());
+
+    m_loading = false;
+    checkNotify();
+}
+
+void CachedFont::checkNotify()
+{
+    if(m_loading) return;
+
+    for (QHashIterator<CachedObjectClient*,CachedObjectClient*> it( m_clients ); it.hasNext();)
+        it.next().value()->notifyFinished(this);
+}
+
+void CachedFont::error( int /*err*/, const char* /*text*/ )
+{
+    m_loading = false;
+    checkNotify();
+}
+
 // ------------------------------------------------------------------------------------------
 
 Request::Request(DocLoader* dl, CachedObject *_object, bool _incremental)
@@ -1061,10 +1112,7 @@ void DocLoader::insertCachedObject( CachedObject* o ) const
 bool DocLoader::needReload(CachedObject *existing, const QString& fullURL)
 {
     bool reload = false;
-    if (m_cachePolicy == KIO::CC_Verify ||    
-        // During a softReload, we favour using cached images
-        // over forcibly re-downloading them.
-        (existing->type() == CachedObject::Image && m_part->browserExtension()->browserArguments().softReload))
+    if (m_cachePolicy == KIO::CC_Verify)
     {
        if (!m_reloadedURLs.contains(fullURL))
        {
@@ -1104,7 +1152,7 @@ void DocLoader::registerPreload(CachedObject* resource)
     fprintf(stderr, "PRELOADING %s\n", resource->url().string().toLatin1().data());
 #endif
 }
- 
+
 void DocLoader::clearPreloads()
 {
     printPreloadStats();
@@ -1136,7 +1184,7 @@ void DocLoader::printPreloadStats()
             fprintf(stderr,"HIT COMPLETE PRELOAD %s\n", res->url().string().toLatin1().data());
         else if (res->preloadResult() == CachedObject::PreloadReferencedWhileLoading)
             fprintf(stderr,"HIT LOADING PRELOAD %s\n", res->url().string().toLatin1().data());
-        
+
         if (res->type() == CachedObject::Script) {
             scripts++;
             if (res->preloadResult() < CachedObject::PreloadReferencedWhileLoading)
@@ -1164,7 +1212,7 @@ void DocLoader::printPreloadStats()
     KUrl fullURL (m_doc->completeURL( url.string() )); \
     if ( !fullURL.isValid() || \
          ( m_part && m_part->onlyLocalReferences() && fullURL.protocol() != "file" && fullURL.protocol() != "data") || \
-         doRedirectCheck && (m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL))) \
+         ( doRedirectCheck && (m_doc && !KAuthorized::authorizeUrlAction("redirect", m_doc->URL(), fullURL)))) \
          return 0L;
 
 CachedImage *DocLoader::requestImage( const DOM::DOMString &url)
@@ -1205,9 +1253,16 @@ CachedScript *DocLoader::requestScript( const DOM::DOMString &url, const QString
 }
 
 CachedSound *DocLoader::requestSound( const DOM::DOMString &url )
+{
+      DOCLOADER_SECCHECK(true);
+      CachedSound* s = Cache::requestObject<CachedSound, CachedObject::Sound>( this, fullURL, 0 );
+      return s;
+}
+
+CachedFont *DocLoader::requestFont( const DOM::DOMString &url )
 {
     DOCLOADER_SECCHECK(true);
-    CachedSound* s = Cache::requestObject<CachedSound, CachedObject::Sound>( this, fullURL, 0 );
+    CachedFont* s = Cache::requestObject<CachedFont, CachedObject::Font>( this, fullURL, 0 );
     return s;
 }
 
@@ -1370,12 +1425,11 @@ void Loader::servePendingRequests()
 
 void Loader::slotMimetype( KIO::Job *j, const QString& s )
 {
-    KIO::TransferJob* job = static_cast<KIO::TransferJob*>(j);
     Request *r = m_requestsLoading.value( j );
     if (!r)
         return;
     CachedObject *o = r->object;
-    
+
     // Mozilla plain ignores any  mimetype that doesn't have / in it, and handles it as "",
     // including when being picky about mimetypes. Match that for better compatibility with broken servers.
     if (s.contains('/'))
@@ -1413,7 +1467,7 @@ void Loader::slotFinished( KJob* job )
       r->object->setExpireDate( expireDate );
 
       if ( r->object->type() == CachedObject::Image ) {
-          QString fn = j->queryMetaData("content-disposition");
+          QString fn = j->queryMetaData("content-disposition-filename");
           static_cast<CachedImage*>( r->object )->setSuggestedFilename(fn);
 #ifdef IMAGE_TITLES
           static_cast<CachedImage*>( r->object )->setSuggestedTitle(fn);
@@ -1728,6 +1782,7 @@ void Cache::statistics()
     int scripts = 0;
     int stylesheets = 0;
     int sound = 0;
+    int fonts = 0;
     foreach (CachedObject* o, *cache)
     {
         switch(o->type()) {
@@ -1751,6 +1806,9 @@ void Cache::statistics()
         case CachedObject::Sound:
             sound++;
             break;
+        case CachedObject::Font:
+            fonts++;
+            break;
         }
         size += o->size();
     }
@@ -1763,6 +1821,7 @@ void Cache::statistics()
     kDebug( 6060 ) << "Number of cached scripts: " << scripts;
     kDebug( 6060 ) << "Number of cached stylesheets: " << stylesheets;
     kDebug( 6060 ) << "Number of cached sounds: " << sound;
+    kDebug( 6060 ) << "Number of cached fonts: " << fonts;
     kDebug( 6060 ) << "pixmaps:   allocated space approx. " << size << " kB";
     kDebug( 6060 ) << "movies :   allocated space approx. " << msize/1024 << " kB";
     kDebug( 6060 ) << "--------------------------------------------------------------------";
