@@ -147,7 +147,7 @@ public:
     // called from lineWidth, to position the floats added in the last line.
     void positionNewFloats();
     void clearFloats();
-    int getClearDelta(RenderObject *child);
+    int getClearDelta(RenderObject *child, int yPos);
     virtual void markAllDescendantsWithFloatsForLayout(RenderObject* floatToRemove = 0);
 
     // FIXME: containsFloats() should not return true if the floating objects list
@@ -186,7 +186,7 @@ public:
 
     bool isPointInScrollbar(int x, int y, int tx, int ty);
 
-    virtual DOM::Position positionForCoordinates(int x, int y);
+    virtual RenderPosition positionForCoordinates(int x, int y);
 
     virtual void calcMinMaxWidth();
     void calcInlineMinMaxWidth();
@@ -196,8 +196,8 @@ public:
 
     virtual int getBaselineOfFirstLineBox();
 
-    RootInlineBox* firstRootBox() { return static_cast<RootInlineBox*>(m_firstLineBox); }
-    RootInlineBox* lastRootBox() { return static_cast<RootInlineBox*>(m_lastLineBox); }
+    RootInlineBox* firstRootBox() const { return static_cast<RootInlineBox*>(m_firstLineBox); }
+    RootInlineBox* lastRootBox() const { return static_cast<RootInlineBox*>(m_lastLineBox); }
 
     virtual InlineFlowBox* getFirstLineBox();
 
@@ -213,8 +213,8 @@ protected:
     void newLine();
 
 private:
-    DOM::Position positionForBox(InlineBox *box, bool start=true) const;
-    DOM::Position positionForRenderer(RenderObject *renderer, bool start=true) const;
+    RenderPosition positionForBox(InlineBox *box, bool start=true) const;
+    RenderPosition positionForRenderer(RenderObject *renderer, bool start=true) const;
 
 protected:
     struct FloatingObject {
@@ -355,8 +355,8 @@ protected:
     RenderObject* handlePositionedChild(RenderObject* child, const MarginInfo& marginInfo, bool& handled);
     RenderObject* handleCompactChild(RenderObject* child, CompactInfo& compactInfo, const MarginInfo& marginInfo, bool& handled);
     RenderObject* handleRunInChild(RenderObject* child, bool& handled);
-    void collapseMargins(RenderObject* child, MarginInfo& marginInfo, int yPosEstimate);
-    void clearFloatsIfNeeded(RenderObject* child, MarginInfo& marginInfo, int oldTopPosMargin, int oldTopNegMargin);
+    int collapseMargins(RenderObject* child, MarginInfo& marginInfo, int yPos);
+    int clearFloatsIfNeeded(RenderObject* child, MarginInfo& marginInfo, int oldTopPosMargin, int oldTopNegMargin, int yPos);
     void adjustSizeForCompactIfNeeded(RenderObject* child, CompactInfo& compactInfo);
     void insertCompactIfNeeded(RenderObject* child, CompactInfo& compactInfo);
     int estimateVerticalPosition(RenderObject* child, const MarginInfo& info);
