@@ -383,8 +383,6 @@ bool RenderImage::nodeAtPoint(NodeInfo& info, int _x, int _y, int _tx, int _ty,
     if (inside && element()) {
         int tx = _tx + m_x;
         int ty = _ty + m_y;
-        if (isRelPositioned())
-            relativePositionOffset(tx, ty);
 
         HTMLImageElementImpl* i = element()->id() == ID_IMG ? static_cast<HTMLImageElementImpl*>(element()) : 0;
         HTMLMapElementImpl* map;
@@ -501,7 +499,7 @@ short RenderImage::calcReplacedWidth() const
     else
         width = calcAspectRatioWidth();
     int minW = calcReplacedWidthUsing(MinWidth);
-    int maxW = style()->maxWidth().value() == UNDEFINED ? width : calcReplacedWidthUsing(MaxWidth);
+    int maxW = style()->maxWidth().isUndefined() ? width : calcReplacedWidthUsing(MaxWidth);
 
     if (width > maxW)
         width = maxW;
@@ -521,7 +519,7 @@ int RenderImage::calcReplacedHeight() const
         height = calcAspectRatioHeight();
 
     int minH = calcReplacedHeightUsing(MinHeight);
-    int maxH = style()->maxHeight().value() == UNDEFINED ? height : calcReplacedHeightUsing(MaxHeight);
+    int maxH = style()->maxHeight().isUndefined() ? height : calcReplacedHeightUsing(MaxHeight);
 
     if (height > maxH)
         height = maxH;
