@@ -26,6 +26,7 @@
 #include "render_layer.h"
 #include "render_canvas.h"
 #include "render_line.h"
+#include "rendering/render_position.h"
 
 #include "render_arena.h"
 
@@ -146,11 +147,11 @@ unsigned long RenderReplaced::caretMaxRenderedOffset() const
     return 1;
 }
 
-Position RenderReplaced::positionForCoordinates(int _x, int _y)
+RenderPosition RenderReplaced::positionForCoordinates(int _x, int _y)
 {
     InlineBox *box = placeHolderBox();
     if (!box)
-        return Position(element(), 0);
+        return RenderPosition(element(), 0);
   
     RootInlineBox *root = box->root();
   
@@ -161,15 +162,15 @@ Position RenderReplaced::positionForCoordinates(int _x, int _y)
     int bottom = root->nextRootBox() ? absy + root->nextRootBox()->topOverflow() : absy + root->bottomOverflow();
   
     if (_y < top)
-        return Position(element(), caretMinOffset()); // coordinates are above
+        return RenderPosition(element(), caretMinOffset()); // coordinates are above
         
     if (_y >= bottom)
-        return Position(element(), caretMaxOffset()); // coordinates are below
+        return RenderPosition(element(), caretMaxOffset()); // coordinates are below
         
     if (element()) {
         if (_x <= absx + xPos() + (width() / 2))
-            return Position(element(), 0);
-        return Position(element(), 1);
+            return RenderPosition(element(), 0);
+        return RenderPosition(element(), 1);
     }
   
     return RenderBox::positionForCoordinates(_x, _y);
@@ -191,7 +192,7 @@ RenderWidget::RenderWidget(DOM::NodeImpl* node)
     m_needsMask = false;
     m_ownsWidget = true;
 
-    // this is no real reference counting, its just there
+    // this is no real reference counting, it is just there
     // to make sure that we're not deleted while we're recursed
     // in an eventFilter of the widget
     ref();
@@ -251,7 +252,7 @@ public:
 void  RenderWidget::resizeWidget( int w, int h )
 {
     // ugly hack to limit the maximum size of the widget ( as X11 has problems if
-    // its bigger )
+    // it is bigger )
     h = qMin( h, 3072 );
     w = qMin( w, 2000 );
 
@@ -479,7 +480,7 @@ void RenderWidget::updateFromElement()
         }
 
         // Border:
-        if (shouldPaintBorder())
+        if (shouldPaintBorder() || (!shouldPaintBackgroundOrBorder() && canHaveBorder()))
         {
             if (QFrame* frame = qobject_cast<QFrame*>(m_widget))
                 frame->setFrameShape(QFrame::NoFrame);
@@ -692,14 +693,9 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
             pp.fillRect(r, Qt::transparent);
         }
         d = pm;
+    } else {
+        p->end();
     }
-    // Qt 4.4 regression #1: 
-    // can't let a painter active on the view as Qt thinks it is opened on the *pixmap*
-    // and prints "paint device can only be painted by one painter at a time" warnings.
-    //
-    // Testcase: paintEvent(...) { QPainter p(this); aChildWidget->render( aPixmapTarget, ...); }
-    //
-    p->end();
 
     setInPaintEventFlag( widget, false );
 
@@ -707,7 +703,7 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
 
     setInPaintEventFlag( widget );
 
-//    if (!buffered) {
+    if (!buffered) {
         p->begin(x);
         p->setWorldTransform(t);
         p->setWindow(w);
@@ -720,8 +716,7 @@ static void copyWidget(const QRect& r, QPainter *p, QWidget *widget, int tx, int
             p->setOpacity(op);
         p->setPen(pen);
         p->setBrush(brush);
-//    } else {
-    if (buffered) {
+    } else {
         // transfer results
         QPoint off(r.x(), r.y());
         p->drawPixmap(thePoint+off, static_cast<QPixmap&>(*d), r);
@@ -882,7 +877,10 @@ void RenderWidget::EventPropagator::sendEvent(QEvent *e) {
         keyPressEvent(static_cast<QKeyEvent *>(e));
         break;
     case QEvent::KeyRelease:
-        keyReleaseEvent(static_cast<QKeyEvent *>(e));
+        if (qobject_cast<QLineEdit*>(this))
+            event(e);
+        else
+            keyReleaseEvent(static_cast<QKeyEvent *>(e));
         break;
     case QEvent::FocusIn:
         focusInEvent(static_cast<QFocusEvent *>(e));
@@ -1003,6 +1001,20 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         QPoint p(me.clientX() - absx + m_view->contentsX(),
                  me.clientY() - absy + m_view->contentsY());
 
+        if (ev.id() == EventImpl::MOUSEMOVE_EVENT && view()->mouseEventsTarget()) {
+            // mitigate the problem that mouse moves outside of the widget rect
+            // interrupt the selection process in textarea (#156574)
+            // e.g. when extending a selection upward to make a textarea scroll
+            // while selecting its content.
+            // We don't emulate properly what Qt does it seems,tough I verified
+            // the move event is properly forwarded to the textarea and has
+            // appropriate coordinates. Might be Enter/Leave issue. ### FIXME
+            // In the meantime, clamping the event to the widget rect
+            // will at least prevent the selection to be lost.
+            p.setX(qMin(qMax(0,p.x()),m_widget->width()));
+            p.setY(qMin(qMax(0,p.y()),m_widget->height()));
+        }
+
         QWidget* target = 0;
         target = m_widget->childAt(p);
 
@@ -1033,6 +1045,8 @@ bool RenderWidget::handleEvent(const DOM::EventImpl& ev)
         }
 
         if (target && ev.id() == EventImpl::MOUSEMOVE_EVENT) {
+            // ### is this one still necessary? it doubles every mouse event...
+            // I'd reckon it's no longer needed since Harri made the event propagation bubble
             QMouseEvent evt(QEvent::MouseMove, p, Qt::NoButton,
                             QApplication::mouseButtons(), QApplication::keyboardModifiers());
             QApplication::sendEvent(target, &evt);
