@@ -349,87 +349,87 @@ namespace WebCore {
 
         void init()
         {
-            aTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_A), DOM::emptyPrefixName);
-            altGlyphTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ALTGLYPH), DOM::emptyPrefixName);
-            altGlyphDefTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ALTGLYPHDEF), DOM::emptyPrefixName);
-            altGlyphItemTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ALTGLYPHITEM), DOM::emptyPrefixName);
-            animateTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ANIMATE), DOM::emptyPrefixName);
-            animateColorTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ANIMATECOLOR), DOM::emptyPrefixName);
-            animateMotionTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ANIMATEMOTION), DOM::emptyPrefixName);
-            animateTransformTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ANIMATETRANSFORM), DOM::emptyPrefixName);
-            setTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_SET), DOM::emptyPrefixName);
-            circleTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_CIRCLE), DOM::emptyPrefixName);
-            clipPathTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_CLIPPATH), DOM::emptyPrefixName);
-            color_profileTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_COLOR_PROFILE), DOM::emptyPrefixName);
-            cursorTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_CURSOR), DOM::emptyPrefixName);
-            definition_srcTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_DEFINITION_SRC), DOM::emptyPrefixName);
-            defsTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_DEFS), DOM::emptyPrefixName);
-            descTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_DESC), DOM::emptyPrefixName);
-            ellipseTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_ELLIPSE), DOM::emptyPrefixName);
-            feBlendTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEBLEND), DOM::emptyPrefixName);
-            feColorMatrixTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FECOLORMATRIX), DOM::emptyPrefixName);
-            feComponentTransferTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FECOMPONENTTRANSFER), DOM::emptyPrefixName);
-            feCompositeTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FECOMPOSITE), DOM::emptyPrefixName);
-            feConvolveMatrixTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FECONVOLVEMATRIX), DOM::emptyPrefixName);
-            feDiffuseLightingTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEDIFFUSELIGHTING), DOM::emptyPrefixName);
-            feDisplacementMapTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEDISPLACEMENTMAP), DOM::emptyPrefixName);
-            feDistantLightTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEDISTANTLIGHT), DOM::emptyPrefixName);
-            feFloodTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEFLOOD), DOM::emptyPrefixName);
-            feFuncATag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEFUNCA), DOM::emptyPrefixName);
-            feFuncBTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEFUNCB), DOM::emptyPrefixName);
-            feFuncGTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEFUNCG), DOM::emptyPrefixName);
-            feFuncRTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEFUNCR), DOM::emptyPrefixName);
-            feGaussianBlurTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEGAUSSIANBLUR), DOM::emptyPrefixName);
-            feImageTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEIMAGE), DOM::emptyPrefixName);
-            feMergeTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEMERGE), DOM::emptyPrefixName);
-            feMergeNodeTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEMERGENODE), DOM::emptyPrefixName);
-            feMorphologyTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEMORPHOLOGY), DOM::emptyPrefixName);
-            feOffsetTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEOFFSET), DOM::emptyPrefixName);
-            fePointLightTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FEPOINTLIGHT), DOM::emptyPrefixName);
-            feSpecularLightingTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FESPECULARLIGHTING), DOM::emptyPrefixName);
-            feSpotLightTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FESPOTLIGHT), DOM::emptyPrefixName);
-            feTileTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FETILE), DOM::emptyPrefixName);
-            feTurbulenceTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FETURBULENCE), DOM::emptyPrefixName);
-            filterTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FILTER), DOM::emptyPrefixName);
-            fontTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT), DOM::emptyPrefixName);
-            font_faceTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT_FACE), DOM::emptyPrefixName);
-            font_face_formatTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT_FACE_FORMAT), DOM::emptyPrefixName);
-            font_face_nameTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT_FACE_NAME), DOM::emptyPrefixName);
-            font_face_srcTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT_FACE_SRC), DOM::emptyPrefixName);
-            font_face_uriTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FONT_FACE_URI), DOM::emptyPrefixName);
-            foreignObjectTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_FOREIGNOBJECT), DOM::emptyPrefixName);
-            gTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_G), DOM::emptyPrefixName);
-            glyphTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_GLYPH), DOM::emptyPrefixName);
-            glyphRefTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_GLYPHREF), DOM::emptyPrefixName);
-            hkernTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_HKERN), DOM::emptyPrefixName);
-            imageTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_IMAGE), DOM::emptyPrefixName);
-            lineTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_LINE), DOM::emptyPrefixName);
-            linearGradientTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_LINEARGRADIENT), DOM::emptyPrefixName);
-            markerTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_MARKER), DOM::emptyPrefixName);
-            maskTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_MASK), DOM::emptyPrefixName);
-            metadataTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_METADATA), DOM::emptyPrefixName);
-            missing_glyphTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_MISSING_GLYPH), DOM::emptyPrefixName);
-            mpathTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_MPATH), DOM::emptyPrefixName);
-            pathTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_PATH), DOM::emptyPrefixName);
-            patternTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_PATTERN), DOM::emptyPrefixName);
-            polygonTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_POLYGON), DOM::emptyPrefixName);
-            polylineTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_POLYLINE), DOM::emptyPrefixName);
-            radialGradientTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_RADIALGRADIENT), DOM::emptyPrefixName);
-            rectTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_RECT), DOM::emptyPrefixName);
-            scriptTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_SCRIPT), DOM::emptyPrefixName);
-            stopTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_STOP), DOM::emptyPrefixName);
-            styleTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_STYLE), DOM::emptyPrefixName);
-            svgTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_SVG), DOM::emptyPrefixName);
-            switchTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_SWITCH), DOM::emptyPrefixName);
-            symbolTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_SYMBOL), DOM::emptyPrefixName);
-            textTag = DOM::QualifiedName(makeId(DOM::svgNamespace, localNamePart(ATTR_TEXT)), DOM::emptyPrefixName);
-            textPathTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_TEXTPATH), DOM::emptyPrefixName);
-            titleTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_TITLE), DOM::emptyPrefixName);
-            trefTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_TREF), DOM::emptyPrefixName);
-            tspanTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_TSPAN), DOM::emptyPrefixName);
-            useTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_USE), DOM::emptyPrefixName);
-            viewTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_VIEW), DOM::emptyPrefixName);
-            vkernTag = DOM::QualifiedName(makeId(DOM::svgNamespace, ID_VKERN), DOM::emptyPrefixName);
+            aTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_A), DOM::emptyPrefixName);
+            altGlyphTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ALTGLYPH), DOM::emptyPrefixName);
+            altGlyphDefTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ALTGLYPHDEF), DOM::emptyPrefixName);
+            altGlyphItemTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ALTGLYPHITEM), DOM::emptyPrefixName);
+            animateTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ANIMATE), DOM::emptyPrefixName);
+            animateColorTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ANIMATECOLOR), DOM::emptyPrefixName);
+            animateMotionTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ANIMATEMOTION), DOM::emptyPrefixName);
+            animateTransformTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ANIMATETRANSFORM), DOM::emptyPrefixName);
+            setTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_SET), DOM::emptyPrefixName);
+            circleTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_CIRCLE), DOM::emptyPrefixName);
+            clipPathTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_CLIPPATH), DOM::emptyPrefixName);
+            color_profileTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_COLOR_PROFILE), DOM::emptyPrefixName);
+            cursorTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_CURSOR), DOM::emptyPrefixName);
+            definition_srcTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_DEFINITION_SRC), DOM::emptyPrefixName);
+            defsTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_DEFS), DOM::emptyPrefixName);
+            descTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_DESC), DOM::emptyPrefixName);
+            ellipseTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_ELLIPSE), DOM::emptyPrefixName);
+            feBlendTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEBLEND), DOM::emptyPrefixName);
+            feColorMatrixTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FECOLORMATRIX), DOM::emptyPrefixName);
+            feComponentTransferTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FECOMPONENTTRANSFER), DOM::emptyPrefixName);
+            feCompositeTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FECOMPOSITE), DOM::emptyPrefixName);
+            feConvolveMatrixTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FECONVOLVEMATRIX), DOM::emptyPrefixName);
+            feDiffuseLightingTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEDIFFUSELIGHTING), DOM::emptyPrefixName);
+            feDisplacementMapTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEDISPLACEMENTMAP), DOM::emptyPrefixName);
+            feDistantLightTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEDISTANTLIGHT), DOM::emptyPrefixName);
+            feFloodTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEFLOOD), DOM::emptyPrefixName);
+            feFuncATag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEFUNCA), DOM::emptyPrefixName);
+            feFuncBTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEFUNCB), DOM::emptyPrefixName);
+            feFuncGTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEFUNCG), DOM::emptyPrefixName);
+            feFuncRTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEFUNCR), DOM::emptyPrefixName);
+            feGaussianBlurTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEGAUSSIANBLUR), DOM::emptyPrefixName);
+            feImageTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEIMAGE), DOM::emptyPrefixName);
+            feMergeTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEMERGE), DOM::emptyPrefixName);
+            feMergeNodeTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEMERGENODE), DOM::emptyPrefixName);
+            feMorphologyTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEMORPHOLOGY), DOM::emptyPrefixName);
+            feOffsetTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEOFFSET), DOM::emptyPrefixName);
+            fePointLightTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FEPOINTLIGHT), DOM::emptyPrefixName);
+            feSpecularLightingTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FESPECULARLIGHTING), DOM::emptyPrefixName);
+            feSpotLightTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FESPOTLIGHT), DOM::emptyPrefixName);
+            feTileTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FETILE), DOM::emptyPrefixName);
+            feTurbulenceTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FETURBULENCE), DOM::emptyPrefixName);
+            filterTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FILTER), DOM::emptyPrefixName);
+            fontTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT), DOM::emptyPrefixName);
+            font_faceTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT_FACE), DOM::emptyPrefixName);
+            font_face_formatTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT_FACE_FORMAT), DOM::emptyPrefixName);
+            font_face_nameTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT_FACE_NAME), DOM::emptyPrefixName);
+            font_face_srcTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT_FACE_SRC), DOM::emptyPrefixName);
+            font_face_uriTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FONT_FACE_URI), DOM::emptyPrefixName);
+            foreignObjectTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_FOREIGNOBJECT), DOM::emptyPrefixName);
+            gTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_G), DOM::emptyPrefixName);
+            glyphTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_GLYPH), DOM::emptyPrefixName);
+            glyphRefTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_GLYPHREF), DOM::emptyPrefixName);
+            hkernTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_HKERN), DOM::emptyPrefixName);
+            imageTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_IMAGE), DOM::emptyPrefixName);
+            lineTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_LINE), DOM::emptyPrefixName);
+            linearGradientTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_LINEARGRADIENT), DOM::emptyPrefixName);
+            markerTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_MARKER), DOM::emptyPrefixName);
+            maskTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_MASK), DOM::emptyPrefixName);
+            metadataTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_METADATA), DOM::emptyPrefixName);
+            missing_glyphTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_MISSING_GLYPH), DOM::emptyPrefixName);
+            mpathTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_MPATH), DOM::emptyPrefixName);
+            pathTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_PATH), DOM::emptyPrefixName);
+            patternTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_PATTERN), DOM::emptyPrefixName);
+            polygonTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_POLYGON), DOM::emptyPrefixName);
+            polylineTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_POLYLINE), DOM::emptyPrefixName);
+            radialGradientTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_RADIALGRADIENT), DOM::emptyPrefixName);
+            rectTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_RECT), DOM::emptyPrefixName);
+            scriptTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_SCRIPT), DOM::emptyPrefixName);
+            stopTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_STOP), DOM::emptyPrefixName);
+            styleTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_STYLE), DOM::emptyPrefixName);
+            svgTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_SVG), DOM::emptyPrefixName);
+            switchTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_SWITCH), DOM::emptyPrefixName);
+            symbolTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_SYMBOL), DOM::emptyPrefixName);
+            textTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, DOM::localNamePart(ATTR_TEXT)), DOM::emptyPrefixName);
+            textPathTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_TEXTPATH), DOM::emptyPrefixName);
+            titleTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_TITLE), DOM::emptyPrefixName);
+            trefTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_TREF), DOM::emptyPrefixName);
+            tspanTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_TSPAN), DOM::emptyPrefixName);
+            useTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_USE), DOM::emptyPrefixName);
+            viewTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_VIEW), DOM::emptyPrefixName);
+            vkernTag = DOM::QualifiedName(DOM::makeId(DOM::svgNamespace, ID_VKERN), DOM::emptyPrefixName);
             accent_heightAttr = DOM::QualifiedName(ATTR_ACCENT_HEIGHT, DOM::emptyPrefixName);
             accumulateAttr = DOM::QualifiedName(ATTR_ACCUMULATE, DOM::emptyPrefixName);
             additiveAttr = DOM::QualifiedName(ATTR_ADDITIVE, DOM::emptyPrefixName);
@@ -678,13 +678,13 @@ namespace WebCore {
             zAttr = DOM::QualifiedName(ATTR_Z, DOM::emptyPrefixName);
             zoomAndPanAttr = DOM::QualifiedName(ATTR_ZOOMANDPAN, DOM::emptyPrefixName);
             WebCore::HTMLNames::classAttr = DOM::QualifiedName(ATTR_CLASS, DOM::emptyPrefixName);
-            WebCore::XLinkNames::actuateAttr = DOM::QualifiedName(ATTR_ACTUATE, DOM::emptyPrefixName);
-            WebCore::XLinkNames::arcroleAttr = DOM::QualifiedName(ATTR_ARCROLE, DOM::emptyPrefixName);
-            WebCore::XLinkNames::hrefAttr = DOM::QualifiedName(ATTR_HREF, DOM::emptyPrefixName);
-            WebCore::XLinkNames::roleAttr = DOM::QualifiedName(ATTR_ROLE, DOM::emptyPrefixName);
-            WebCore::XLinkNames::showAttr = DOM::QualifiedName(ATTR_SHOW, DOM::emptyPrefixName);
-            WebCore::XLinkNames::titleAttr = DOM::QualifiedName(ATTR_TITLE, DOM::emptyPrefixName);
-            WebCore::XLinkNames::typeAttr = DOM::QualifiedName(ATTR_TYPE, DOM::emptyPrefixName);
+            WebCore::XLinkNames::actuateAttr = DOM::QualifiedName(ATTR_XLINK_ACTUATE, DOM::emptyPrefixName);
+            WebCore::XLinkNames::arcroleAttr = DOM::QualifiedName(ATTR_XLINK_ARCROLE, DOM::emptyPrefixName);
+            WebCore::XLinkNames::hrefAttr = DOM::QualifiedName(ATTR_XLINK_HREF, DOM::emptyPrefixName);
+            WebCore::XLinkNames::roleAttr = DOM::QualifiedName(ATTR_XLINK_ROLE, DOM::emptyPrefixName);
+            WebCore::XLinkNames::showAttr = DOM::QualifiedName(ATTR_XLINK_SHOW, DOM::emptyPrefixName);
+            WebCore::XLinkNames::titleAttr = DOM::QualifiedName(ATTR_XLINK_TITLE, DOM::emptyPrefixName);
+            WebCore::XLinkNames::typeAttr = DOM::QualifiedName(ATTR_XLINK_TYPE, DOM::emptyPrefixName);
 
         }
     }
