@@ -26,13 +26,13 @@
 // Dump SVGElementInstance object tree - useful to debug instanceRoot problems
 // #define DUMP_INSTANCE_TREE
 
-// Dump the deep-expanded shadow tree (where the renderes are built from)
+// Dump the deep-expanded shadow tree (where the renderers are built from)
 // #define DUMP_SHADOW_TREE
 
 #if ENABLE(SVG)
 #include "SVGUseElement.h"
 
-#include "CSSStyleSelector.h"
+#include "css/cssstyleselector.h"
 /*#include "CString.h"*/
 #include "Document.h"
 /*#include "Event.h"
@@ -176,6 +176,7 @@ void SVGUseElement::recalcStyle(StyleChange change)
     // as the shadow tree root element has no (direct) parent node. Yes, shadow trees are tricky.
     if (change >= Inherit || m_shadowTreeRootElement->changed()) {
         RenderStyle* newStyle = document()->styleSelector()->styleForElement(m_shadowTreeRootElement.get());
+        newStyle->ref();
         StyleChange ch = m_shadowTreeRootElement->diff((m_shadowTreeRootElement->renderer() ? m_shadowTreeRootElement->renderer()->style() : 0)/*renderStyle()*/, newStyle);
         if (ch == Detach) {
             ASSERT(m_shadowTreeRootElement->attached());
@@ -183,13 +184,13 @@ void SVGUseElement::recalcStyle(StyleChange change)
             attachShadowTree();
 
             // attach recalulates the style for all children. No need to do it twice.
-            m_shadowTreeRootElement->setChanged(/*NoStyleChange*/);
+            m_shadowTreeRootElement->setChanged(false);
             m_shadowTreeRootElement->setHasChangedChild(false);
-            /*newStyle->deref(document()->renderArena());*/
+            newStyle->deref();
             return;
         }
 
-        /*newStyle->deref(document()->renderArena());*/
+        newStyle->deref();
     }
 
     // Only change==Detach needs special treatment, for anything else recalcStyle() works.
@@ -385,10 +386,10 @@ void SVGUseElement::attach()
 
 void SVGUseElement::detach()
 {
-    SVGStyledTransformableElement::detach();
-
     if (m_shadowTreeRootElement)
         m_shadowTreeRootElement->detach();
+    
+    SVGStyledTransformableElement::detach();
 }
 
 static bool isDirectReference(Node* n)
@@ -407,6 +408,9 @@ Path SVGUseElement::toClipPath() const
     if (!m_shadowTreeRootElement)
         const_cast<SVGUseElement*>(this)->buildPendingResource();
 
+    if (!m_shadowTreeRootElement)
+        return Path();
+
     Node* n = m_shadowTreeRootElement->firstChild();
     if (n->isSVGElement() && static_cast<SVGElement*>(n)->isStyledTransformable()) {
         if (!isDirectReference(n))
@@ -713,6 +717,7 @@ void SVGUseElement::attachShadowTree()
     // Inspired by RenderTextControl::createSubtreeIfNeeded(). 
     if (renderer()->childAllowed()/*canHaveChildren()*/ && childShouldCreateRenderer(m_shadowTreeRootElement.get())) {
         RenderStyle* style = m_shadowTreeRootElement->styleForRenderer(renderer());
+        style->ref();
 
         if (m_shadowTreeRootElement->rendererIsNeeded(style)) {
             m_shadowTreeRootElement->setRenderer(m_shadowTreeRootElement->createRenderer(document()->renderArena(), style));
@@ -723,7 +728,7 @@ void SVGUseElement::attachShadowTree()
             }
         }
 
-        /*style->deref(document()->renderArena());*/
+        style->deref();
 
         // This will take care of attaching all shadow tree child nodes.
         for (Node* child = m_shadowTreeRootElement->firstChild(); child; child = child->nextSibling())
