@@ -83,7 +83,6 @@
 #include <html/HTMLAudioElement.h>
 #include <html/HTMLVideoElement.h>
 #include <html/HTMLSourceElement.h>
-#include <editing/htmlediting.h>
 #include <editing/jsediting.h>
 
 // SVG (WebCore)
@@ -409,6 +408,7 @@ DocumentImpl::DocumentImpl(DOMImplementationImpl *_implementation, KHTMLView *v)
     m_docChanged = false;
     m_elemSheet = 0;
     m_tokenizer = 0;
+    m_decoder = 0;
     m_doctype = 0;
     m_implementation = _implementation;
     m_implementation->ref();
@@ -675,32 +675,19 @@ NodeImpl *DocumentImpl::importNode(NodeImpl *importedNode, bool deep, int &excep
 	NamedAttrMapImpl *otherMap = static_cast<ElementImpl *>(importedNode)->attributes(true);
 
 	ElementImpl *tempElementImpl;
-	if (!importedNode->localName().isNull())
-	    tempElementImpl = createElementNS(otherElem->namespaceURI(),otherElem->nodeName());
-	else
-	    tempElementImpl = createElement(otherElem->nodeName());
+	tempElementImpl = createElementNS(otherElem->namespaceURI(),otherElem->nodeName());
+	tempElementImpl->setHTMLCompat(htmlMode() != XHtml && otherElem->htmlCompat());
 	result = tempElementImpl;
 
-
 	if(otherMap) {
 	    for(unsigned i = 0; i < otherMap->length(); i++)
 	    {
 		AttrImpl *otherAttr = otherMap->attributeAt(i).createAttr(otherElem,otherElem->docPtr());
 
-		if (!otherAttr->localName().isNull()) {
-		    // attr was created via createElementNS()
-		    tempElementImpl->setAttributeNS(otherAttr->namespaceURI(),
+		tempElementImpl->setAttributeNS(otherAttr->namespaceURI(),
 						    otherAttr->name(),
 						    otherAttr->nodeValue(),
-						  exceptioncode);
-		}
-		else {
-		    // attr was created via createElement()
-		    tempElementImpl->setAttribute(otherAttr->id(),
-                    emptyPrefixName, true /*nsAware*/,
-                                                  otherAttr->nodeValue(),
-						  exceptioncode);
-		}
+						    exceptioncode);
 
 		if(exceptioncode != 0)
 		    break; // ### properly cleanup here
@@ -1592,7 +1579,9 @@ void DocumentImpl::updateSelection()
     else {
         RenderObject *startRenderer = s.start().node() ? s.start().node()->renderer() : 0;
         RenderObject *endRenderer = s.end().node() ? s.end().node()->renderer() : 0;
-        static_cast<RenderCanvas*>(m_render)->setSelection(startRenderer, s.start().offset(), endRenderer, s.end().offset());
+        RenderPosition renderedStart = RenderPosition::fromDOMPosition(s.start());
+        RenderPosition renderedEnd   = RenderPosition::fromDOMPosition(s.end());
+        static_cast<RenderCanvas*>(m_render)->setSelection(startRenderer, renderedStart.renderedOffset(), endRenderer, renderedEnd.renderedOffset());
     }
 }
 
@@ -1687,7 +1676,6 @@ void DocumentImpl::write( const QString &text )
         if (m_view)
             m_view->part()->resetFromScript();
         m_tokenizer->setAutoClose();
-        write(QLatin1String("<html>"));
     }
     m_tokenizer->write(text, false);
 }
@@ -1704,6 +1692,11 @@ void DocumentImpl::finishParsing (  )
         m_tokenizer->finish();
 }
 
+QString DocumentImpl::completeURL(const QString& url) const
+{
+    return KUrl(baseURL(),url /*,m_decoderMibEnum*/).url();
+}
+
 void DocumentImpl::setUserStyleSheet( const QString& sheet )
 {
     if ( m_usersheet != sheet ) {
@@ -2339,7 +2332,7 @@ void DocumentImpl::rebuildStyleSheetList(bool force)
                 title = title.replace('&',  "&&");
                 if ( !m_availableSheets.contains( title ) )
                     m_availableSheets.append( title );
-                title = QString();
+                title.clear();
             }
 
             if (sheet) {
@@ -2459,8 +2452,11 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
 
     if (m_focusNode != newFocusNode) {
         NodeImpl *oldFocusNode = m_focusNode;
-        // Set focus on the new node
-        m_focusNode = newFocusNode;
+
+        // We are blurring, so m_focusNode ATM is 0; this is observable to the
+        // event handlers.
+        m_focusNode = 0;
+        
         // Remove focus from the existing focus node (if any)
         if (oldFocusNode) {
             if (oldFocusNode->active())
@@ -2478,7 +2474,8 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
             oldFocusNode->dispatchUIEvent(EventImpl::DOMFOCUSOUT_EVENT);
 
             if ((oldFocusNode == this) && oldFocusNode->hasOneRef()) {
-                oldFocusNode->deref(); // deletes this
+                oldFocusNode->deref(); // may delete this, if there are not kids keeping it alive...
+                                       // so we better not add any.
                 return;
             }
 	    else {
@@ -2486,7 +2483,11 @@ void DocumentImpl::setFocusNode(NodeImpl *newFocusNode)
             }
         }
 
-        if (m_focusNode) {
+        // It's possible that one of the blur, etc. handlers has already set focus.
+        // in that case, we don't want to override it.
+        if (!m_focusNode && newFocusNode) {
+            // Set focus on the new node
+            m_focusNode = newFocusNode;
             m_focusNode->ref();
             m_focusNode->dispatchHTMLEvent(EventImpl::FOCUS_EVENT,false,false);
             if (m_focusNode != newFocusNode) return;
@@ -2954,6 +2955,11 @@ void DOM::DocumentImpl::releaseCachedNodeListInfo(NodeListImpl::Cache* entry)
     entry->deref();
 }
 
+bool DOM::DocumentImpl::isSVGDocument() const
+{
+    return (documentElement()->id() == WebCore::SVGNames::svgTag.id()); 
+}
+
 const WebCore::SVGDocumentExtensions* DOM::DocumentImpl::svgExtensions()
 {
     return m_svgExtensions;
@@ -2979,6 +2985,7 @@ JSEditor *DocumentImpl::jsEditor()
 
 bool DocumentImpl::execCommand(const DOMString &command, bool userInterface, const DOMString &value)
 {
+    kDebug() << "[execute command]" << command << userInterface << value << endl;
     return jsEditor()->execCommand(jsEditor()->commandImp(command), userInterface, value);
 }
 
@@ -2999,6 +3006,7 @@ bool DocumentImpl::queryCommandState(const DOMString &command)
 
 bool DocumentImpl::queryCommandSupported(const DOMString &command)
 {
+    kDebug() << "[query command supported]" << command << endl;
     return jsEditor()->queryCommandSupported(jsEditor()->commandImp(command));
 }
 
