@@ -70,7 +70,6 @@ NodeImpl::NodeImpl(DocumentImpl *doc)
       m_previous(0),
       m_next(0),
       m_render(0),
-      m_tabIndex( 0 ),
       m_hasId( false ),
       m_attached(false),
       m_closed(false),
@@ -79,14 +78,16 @@ NodeImpl::NodeImpl(DocumentImpl *doc)
       m_changedAscendentAttribute( false ),
       m_inDocument( false ),
       m_hasAnchor( false ),
-      m_elementHasRareData( false ),
       m_hovered( false ),
       m_focused( false ),
       m_active( false ),
       m_implicit( false ),
       m_htmlCompat( false ),
       m_hasClass( false ),
-      m_hasCombinedStyle( false )
+      m_hasCombinedStyle( false ),
+      m_hasHoverDependency( false ),
+      m_elementHasRareData( false ),
+      m_needsStyleAttributeUpdate( false )
 {
 }
 
@@ -315,17 +316,18 @@ NodeImpl *NodeImpl::addChild(NodeImpl *)
   return 0;
 }
 
-void NodeImpl::getCaret(int offset, bool override, int &_x, int &_y, int &width, int &height)
+void NodeImpl::getCaret(int /*offset*/, bool override, int &_x, int &_y, int &width, int &height)
 {
     if (m_render) {
         RenderObject *r = 0;
-	long r_ofs;
+	long r_ofs = 0;
 	bool outside, outsideEnd;
 #if 0
 kDebug(6200) << "getCaret: node " << this << " " << nodeName().string() << " offset: " << offset;
 #endif
 //         mapDOMPosToRenderPos(this, offset, r, r_ofs, outside, outsideEnd);
 	outside = false;
+	outsideEnd = false;
 #if 0
 kDebug(6200) << "getCaret: r " << r << " " << (r?r->renderName():QString()) << " r_ofs: " << r_ofs << " outside " << outside << " outsideEnd " << outsideEnd;
 #endif
@@ -340,7 +342,7 @@ kDebug(6200) << "getCaret: r " << r << " " << (r?r->renderName():QString()) << "
 
 bool NodeImpl::isContentEditable() const
 {
-    return m_parent ? m_parent->isContentEditable() : false;
+    return parentNode() ? parentNode()->isContentEditable() : false;
 }
 
 QRect NodeImpl::getRect() const
@@ -670,11 +672,13 @@ void NodeImpl::dispatchSubtreeModifiedEvent()
     if (!document()->hasListenerType(DocumentImpl::DOMSUBTREEMODIFIED_LISTENER))
         return;
     int exceptioncode = 0;
+    ref();
     MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMSUBTREEMODIFIED_EVENT,true,
                                                          false,0,DOMString(),DOMString(),DOMString(),0);
     evt->ref();
     dispatchEvent(evt,exceptioncode,true);
     evt->deref();
+    derefOnly();
 }
 
 bool NodeImpl::dispatchKeyEvent(QKeyEvent *key, bool keypress)
@@ -1203,12 +1207,12 @@ bool NodeImpl::inSameContainingBlockFlowElement(NodeImpl *n)
     return n ? enclosingBlockFlowElement() == n->enclosingBlockFlowElement() : false;
 }
 
-Position NodeImpl::positionForCoordinates(int x, int y) const
+RenderPosition NodeImpl::positionForCoordinates(int x, int y) const
 {
     if (renderer())
         return renderer()->positionForCoordinates(x, y);
 
-    return Position(const_cast<NodeImpl *>(this), 0);
+    return RenderPosition();
 }
 
 bool NodeImpl::isPointInsideSelection(int x, int y, const Selection &sel) const
@@ -1216,15 +1220,15 @@ bool NodeImpl::isPointInsideSelection(int x, int y, const Selection &sel) const
     if (sel.state() != Selection::RANGE)
         return false;
 
-    Position pos(positionForCoordinates(x, y));
+    RenderPosition pos(positionForCoordinates(x, y));
     if (pos.isEmpty())
         return false;
 
     NodeImpl *n = sel.start().node();
     while (n) {
         if (n == pos.node()) {
-            if ((n == sel.start().node() && pos.offset() < sel.start().offset()) ||
-                (n == sel.end().node() && pos.offset() > sel.end().offset())) {
+            if ((n == sel.start().node() && pos.domOffset() < sel.start().offset()) ||
+                (n == sel.end().node() && pos.domOffset() > sel.end().offset())) {
                 return false;
             }
             return true;
@@ -1987,10 +1991,14 @@ NodeImpl *NodeBaseImpl::childNode(unsigned long index)
 void NodeBaseImpl::dispatchChildInsertedEvents( NodeImpl *child, int &exceptioncode )
 {
     if (document()->hasListenerType(DocumentImpl::DOMNODEINSERTED_LISTENER)) {
+        // We need to weak-guard ourselves since 'this' may be a fresh node, so
+        // we don't want the mutation event to delete it.
+        ref();
         MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEINSERTED_EVENT,true,false,this,DOMString(),DOMString(),DOMString(),0);
         evt->ref();
         child->dispatchEvent(evt,exceptioncode,true);
         evt->deref();
+        derefOnly();
         if (exceptioncode)
             return;
     }
@@ -2005,10 +2013,12 @@ void NodeBaseImpl::dispatchChildInsertedEvents( NodeImpl *child, int &exceptionc
             c->insertedIntoDocument();
 
             if (hasInsertedListeners) {
+                ref();
                 MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEINSERTEDINTODOCUMENT_EVENT,false,false,0,DOMString(),DOMString(),DOMString(),0);
                 evt->ref();
                 c->dispatchEvent(evt,exceptioncode,true);
                 evt->deref();
+                derefOnly();
                 if (exceptioncode)
                     return;
             }
@@ -2021,10 +2031,12 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
     // Dispatch pre-removal mutation events
     document()->notifyBeforeNodeRemoval(child); // ### use events instead
     if (document()->hasListenerType(DocumentImpl::DOMNODEREMOVED_LISTENER)) {
+        ref();
         MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEREMOVED_EVENT,true,false,this,DOMString(),DOMString(),DOMString(),0);
         evt->ref();
         child->dispatchEvent(evt,exceptioncode,true);
         evt->deref();
+        derefOnly();
         if (exceptioncode)
             return;
     }
@@ -2038,10 +2050,12 @@ void NodeBaseImpl::dispatchChildRemovalEvents( NodeImpl *child, int &exceptionco
     if (p->nodeType() == Node::DOCUMENT_NODE) {
         for (NodeImpl *c = child; c; c = c->traverseNextNode(child)) {
             if (hasRemovalListeners) {
+                ref();
                 MutationEventImpl* const evt = new MutationEventImpl(EventImpl::DOMNODEREMOVEDFROMDOCUMENT_EVENT,false,false,0,DOMString(),DOMString(),DOMString(),0);
                 evt->ref();
                 c->dispatchEvent(evt,exceptioncode,true);
                 evt->deref();
+                derefOnly();
                 if (exceptioncode)
                     return;
             }
