@@ -45,6 +45,7 @@ namespace khtml {
     class RenderStyle;
     class RenderObject;
     class RenderArena;
+    class RenderPosition;
 }
 
 namespace DOM {
@@ -55,7 +56,6 @@ class DocumentImpl;
 class ElementImpl;
 class RegisteredEventListener;
 class EventImpl;
-class Position;
 class Selection;
 
 struct RegisteredListenerList {
@@ -81,7 +81,18 @@ private:
     bool isHTMLEventListener(EventListener* listener);
 };
 
-class NodeImpl : public khtml::TreeShared<NodeImpl>
+class EventTargetImpl : public khtml::TreeShared<EventTargetImpl>
+{
+public:
+    enum Type {
+        DOM_NODE,
+        XML_HTTP_REQUEST
+    };
+
+    virtual Type eventTargetType() const = 0;
+};
+
+class NodeImpl : public EventTargetImpl
 {
     friend class DocumentImpl;
 public:
@@ -91,12 +102,17 @@ public:
     //stuff for WebCore DOM & SVG
     virtual bool hasTagName(const QualifiedName& /*name*/) const { return false; }
 
+    // EventTarget
+    virtual Type eventTargetType() const { return DOM_NODE; }
+    // covariant override
+    NodeImpl* parent() const { return parentNode(); }
+
     // DOM methods & attributes for Node
     virtual DOMString nodeName() const;
     virtual DOMString nodeValue() const;
     virtual void setNodeValue( const DOMString &_nodeValue, int &exceptioncode );
     virtual unsigned short nodeType() const;
-    NodeImpl *parentNode() const { return m_parent; }
+    NodeImpl *parentNode() const { return static_cast<NodeImpl*>(m_parent); }
     NodeImpl *previousSibling() const { return m_previous; }
     NodeImpl *nextSibling() const { return m_next; }
     virtual NodeListImpl *childNodes();
@@ -185,7 +201,7 @@ public:
     bool inSameRootEditableElement(NodeImpl *);
     bool inSameContainingBlockFlowElement(NodeImpl *);
 
-    Position positionForCoordinates(int x, int y) const;
+    khtml::RenderPosition positionForCoordinates(int x, int y) const;
     bool isPointInsideSelection(int x, int y, const Selection &) const;
 
     // used by the parser. Doesn't do as many error checkings as
@@ -209,12 +225,6 @@ public:
     //                 qualified element name.
     virtual Id id() const { return 0; }
 
-    enum IdType {
-        AttributeId,
-        ElementId,
-        NamespaceId
-    };
-
     enum MouseEventType {
         MousePress,
         MouseRelease,
@@ -268,6 +278,9 @@ public:
     void setHasChangedChild( bool b = true ) { m_hasChangedChild = b; }
     void setInDocument(bool b=true) { m_inDocument = b; }
     void setHTMLCompat(bool b) { m_htmlCompat = b; }
+    bool hasHoverDependency() { return m_hasHoverDependency; }
+    void setHasHoverDependency(bool b = true) { m_hasHoverDependency = b; }
+    void setNeedsStyleAttributeUpdate(bool b = true) { m_needsStyleAttributeUpdate = b; }
     virtual void setFocus(bool b=true) { m_focused = b; }
     virtual void setActive(bool b=true) { m_active = b; }
     virtual void setHovered(bool b=true) { m_hovered = b; }
@@ -278,9 +291,8 @@ public:
     // for descending restyle when ID or CLASS changes
     bool changedAscendentAttribute() const { return m_changedAscendentAttribute; }
     void setChangedAscendentAttribute(bool b) { m_changedAscendentAttribute = b; }
-
-    short tabIndex() const { return m_tabIndex; }
-    void setTabIndex(short _tabIndex) { m_tabIndex = _tabIndex; }
+ 
+    virtual short tabIndex() const { return 0; }
 
     virtual bool isFocusable() const { return false; }
     virtual bool isMouseFocusable() const { return isFocusable(); }
@@ -443,7 +455,7 @@ public:
     virtual void restoreState(const QString &state);
 
     // -----------------------------------------------------------------------------
-    // Notification of document stucture changes
+    // Notification of document structure changes
 
     /**
      * Notifies the node that it has been inserted into the document. This is called during document parsing, and also
@@ -508,8 +520,6 @@ protected:
     khtml::RenderObject *m_render;
     RegisteredListenerList m_regdListeners;
 
-    signed m_tabIndex : 15; // ### needs one bit more
-
     bool m_hasId : 1;
     bool m_attached : 1;
     bool m_closed : 1;
@@ -518,7 +528,6 @@ protected:
     bool m_changedAscendentAttribute : 1;
     bool m_inDocument : 1;
     bool m_hasAnchor : 1;
-    bool m_elementHasRareData : 1;
 
     bool m_hovered : 1;
     bool m_focused : 1;
@@ -527,7 +536,12 @@ protected:
     bool m_htmlCompat : 1; // true if element was created in HTML compat mode
     bool m_hasClass : 1;   // true if element has a class property, as relevant to CSS
     bool m_hasCombinedStyle : 1; // true if element has inline styles and presentational styles
-    bool m_unused : 1; // free bit
+    bool m_hasHoverDependency : 1; // true if element has hover dependency on itself
+
+    bool m_elementHasRareData : 1;
+    mutable bool m_needsStyleAttributeUpdate : 1; // true if |style| attribute is out of sync (i.e. CSSOM modified our inline styles)
+
+    // 14 bits left
 };
 
 // this is the full Node Implementation with parents and children.
