@@ -22,9 +22,11 @@
 #define QT_NO_CAST_FROM_ASCII
 
 #include <config.h>
+#include <config-kdeinit.h>
 
 #include <sys/types.h>
 #include <sys/time.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -33,6 +35,7 @@
 #include <sys/select.h>		// Needed on some systems.
 #endif
 
+#include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include "proctitle.h"
@@ -51,6 +54,7 @@
 #include <QtCore/QRegExp>
 #include <QtGui/QFont>
 #include <kcomponentdata.h>
+#include <kdemacros.h>
 #include <kstandarddirs.h>
 #include <kglobalsettings.h>
 #include <kglobal.h>
@@ -83,12 +87,6 @@
 #include <kstartupinfo.h>
 #endif
 
-#if KDE_IS_VERSION( 3, 90, 0 )
-#ifdef __GNUC__
-#warning Check if Linux OOM-killer still sucks and if yes, forwardport revision 579164 and following fixes.
-#endif
-#endif
-
 // #define SKIP_PROCTITLE 1
 
 extern char **environ;
@@ -102,7 +100,6 @@ static Display *X11_startup_notify_display = 0;
 static KComponentData *s_instance = 0;
 #define MAX_SOCK_FILE 255
 static char sock_file[MAX_SOCK_FILE];
-//static char sock_file_old[MAX_SOCK_FILE];
 
 #ifdef Q_WS_X11
 #define DISPLAY "DISPLAY"
@@ -124,14 +121,12 @@ static struct {
   int deadpipe[2]; /* pipe used to detect dead children */
   int initpipe[2];
   int wrapper; /* socket for wrapper communication */
-  int wrapper_old; /* old socket for wrapper communication */
   int accepted_fd; /* socket accepted and that must be closed in the child process */
   char result;
   int exit_status;
   pid_t fork;
   pid_t launcher_pid;
   pid_t kded_pid;
-  pid_t my_pid;
   int n;
   char **argv;
   int (*func)(int, char *[]);
@@ -165,6 +160,10 @@ extern "C" KParts::Plugin* _kinit_init_kparts() { return new KParts::Plugin(); }
 #include <kio/authinfo.h>
 extern "C" KIO::AuthInfo* _kioslave_init_kio() { return new KIO::AuthInfo(); }
 
+#ifdef KDEINIT_OOM_PROTECT
+static int oom_pipe = -1;
+#endif
+
 /*
  * Clean up the file descriptor table by closing all file descriptors
  * that are still open.
@@ -174,8 +173,17 @@ extern "C" KIO::AuthInfo* _kioslave_init_kio() { return new KIO::AuthInfo(); }
  */
 static void cleanup_fds()
 {
-    for (int fd = 3; fd < FD_SETSIZE; ++fd)
-       close(fd);
+    int maxfd = FD_SETSIZE;
+    struct rlimit rl;
+    if (getrlimit(RLIMIT_NOFILE, &rl) == 0)
+        maxfd = rl.rlim_max;
+    for (int fd = 3; fd < maxfd; ++fd)
+    {
+#ifdef KDEINIT_OOM_PROTECT
+       if( fd != oom_pipe )
+#endif
+          close(fd);
+    }
 }
 
 /*
@@ -214,21 +222,16 @@ static void close_fds()
       d.initpipe[1] = -1;
    }
 
-   if (d.launcher_pid)
+   if (d.launcher[0] != -1)
    {
       close(d.launcher[0]);
-      d.launcher_pid = 0;
+      d.launcher[0] = -1;
    }
    if (d.wrapper != -1)
    {
       close(d.wrapper);
       d.wrapper = -1;
    }
-   if (d.wrapper_old != -1)
-   {
-      close(d.wrapper_old);
-      d.wrapper_old = -1;
-   }
    if (d.accepted_fd != -1)
    {
       close(d.accepted_fd);
@@ -306,15 +309,11 @@ static void setup_tty( const char* tty )
     }
     if( dup2( fd, STDOUT_FILENO ) < 0 )
     {
-        perror( "kdeinit4: could not dup2() tty" );
-        close( fd );
-        return;
+        perror( "kdeinit4: could not dup2() stdout tty" );
     }
     if( dup2( fd, STDERR_FILENO ) < 0 )
     {
-        perror( "kdeinit4: could not dup2() tty" );
-        close( fd );
-        return;
+        perror( "kdeinit4: could not dup2() stderr tty" );
     }
     close( fd );
 }
@@ -428,55 +427,93 @@ QByteArray execpath_avoid_loops( const QByteArray& exec, int envc, const char* e
      return QFile::encodeName(execpath);
 }
 
+#ifdef KDEINIT_OOM_PROTECT
+static void oom_protect_sighandler( int ) {
+}
+
+static void reset_oom_protect() {
+   if( oom_pipe <= 0 )
+      return;
+   struct sigaction act, oldact;
+   act.sa_handler = oom_protect_sighandler;
+   act.sa_flags = 0;
+   sigemptyset( &act.sa_mask );
+   sigaction( SIGUSR1, &act, &oldact );
+   sigset_t sigs, oldsigs;
+   sigemptyset( &sigs );
+   sigaddset( &sigs, SIGUSR1 );
+   sigprocmask( SIG_BLOCK, &sigs, &oldsigs );
+   pid_t pid = getpid();
+   if( write( oom_pipe, &pid, sizeof( pid_t )) > 0 ) {
+      sigsuspend( &oldsigs ); // wait for the signal to come
+   } else {
+#ifndef NDEBUG
+      fprintf( stderr, "Failed to reset OOM protection: %d\n", pid );
+#endif
+   }
+   sigprocmask( SIG_SETMASK, &oldsigs, NULL );
+   sigaction( SIGUSR1, &oldact, NULL );
+   close( oom_pipe );
+   oom_pipe = -1;
+}
+#else
+static void reset_oom_protect() {
+}
+#endif
+
 static pid_t launch(int argc, const char *_name, const char *args,
                     const char *cwd=0, int envc=0, const char *envs=0,
                     bool reset_env = false,
                     const char *tty=0, bool avoid_loops = false,
-                    const char* startup_id_str = "0" )
+                    const char* startup_id_str = "0" ) // krazy:exclude=doublequote_chars
 {
-  int starting_klauncher = 0;
   QString lib;
   QByteArray name;
   QByteArray exec;
 
-  if (strcmp(_name, "klauncher") == 0) {
-     /* klauncher is launched in a special way:
-      * It has a communication socket on LAUNCHER_FD
-      */
-     if (0 > socketpair(AF_UNIX, SOCK_STREAM, 0, d.launcher))
-     {
-        perror("kdeinit4: socketpair() failed!\n");
-        exit(255);
-     }
-     starting_klauncher = 1;
-  }
-
     QString libpath;
     QByteArray execpath;
     if (_name[0] != '/') {
         name = _name;
         lib = QFile::decodeName(name);
         exec = name;
-        libpath = KLibLoader::findLibrary(lib, *s_instance);
+        libpath = KLibLoader::findLibrary( QLatin1String("libkdeinit4_") + lib, *s_instance);
+        if( libpath.isEmpty())
+            libpath = KLibLoader::findLibrary(lib, *s_instance);
         execpath = execpath_avoid_loops(exec, envc, envs, avoid_loops);
     } else {
         name = _name;
         lib = QFile::decodeName(name);
         name = name.mid(name.lastIndexOf('/') + 1);
         exec = _name;
-        if (lib.endsWith(QLatin1String(".la")))
+        if (lib.endsWith(QLatin1String(".so")))
             libpath = lib;
-        else
+        else {
+            // try to match an absolute path to an executable binary (either in bin/ or in libexec/)
+            // to a kdeinit module in the same prefix
+            if( lib.contains( QLatin1String( "/lib" KDELIBSUFF "/kde4/libexec/" ))) {
+                libpath = QString( lib ).replace( QLatin1String( "/lib" KDELIBSUFF "/kde4/libexec/" ),
+                    QLatin1String("/lib" KDELIBSUFF "/libkdeinit4_")) + QLatin1String(".so");
+            } else if( lib.contains( QLatin1String( "/bin/" ))) {
+                libpath = QString( lib ).replace( QLatin1String( "/bin/" ),
+                    QLatin1String("/lib" KDELIBSUFF "/libkdeinit4_")) + QLatin1String(".so");
+            }
+            // Don't confuse the user with "Could not load libkdeinit4_foo.so" if it doesn't exist
+            if (!QFile::exists(libpath)) {
+                libpath.clear();
+            }
             execpath = exec;
+        }
     }
-    fprintf(stderr,"kdeinit4: preparing to launch %s\n", execpath.constData());
+    fprintf(stderr,"kdeinit4: preparing to launch %s\n", libpath.isEmpty()
+        ? execpath.constData() : libpath.toUtf8().constData());
     if (!args) {
         argc = 1;
     }
 
   if (0 > pipe(d.fd))
   {
-     perror("kdeinit4: pipe() failed!\n");
+     perror("kdeinit4: pipe() failed");
      d.result = 3;
      d.errorMsg = i18n("Unable to start new process.\n"
                        "The system may have reached the maximum number of open files possible or the maximum number of open files that you are allowed to use has been reached.").toUtf8();
@@ -495,7 +532,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
   d.fork = fork();
   switch(d.fork) {
   case -1:
-     perror("kdeinit4: fork() failed!\n");
+     perror("kdeinit4: fork() failed");
      d.result = 3;
      d.errorMsg = i18n("Unable to create new process.\n"
                        "The system may have reached the maximum number of processes possible or the maximum number of processes that you are allowed to use has been reached.").toUtf8();
@@ -508,25 +545,16 @@ static pid_t launch(int argc, const char *_name, const char *args,
      /** Child **/
      close(d.fd[0]);
      close_fds();
-     if (starting_klauncher)
-     {
-        if (d.fd[1] == LAUNCHER_FD)
-        {
-          d.fd[1] = dup(d.fd[1]); // Evacuate from LAUNCHER_FD
-        }
-        if (d.launcher[1] != LAUNCHER_FD)
-        {
-          dup2( d.launcher[1], LAUNCHER_FD); // Make sure the socket has fd LAUNCHER_FD
-          close( d.launcher[1] );
-        }
-        close( d.launcher[0] );
-     }
+     reset_oom_protect();
 
-     if (cwd && *cwd)
-        chdir(cwd);
-     else {
+     // Try to chdir, either to the requested directory or to the user's document path by default.
+     // We ignore errors - if you write a desktop file with Exec=foo and Path=/doesnotexist,
+     // we still want to execute `foo` even if the chdir() failed.
+     if (cwd && *cwd) {
+         (void)chdir(cwd);
+     } else {
          const QByteArray docPath = QFile::encodeName(KGlobalSettings::documentPath());
-         chdir(docPath.constData());
+         (void)chdir(docPath.constData());
      }
 
      if( reset_env ) // KWRAPPER/SHELL
@@ -603,7 +631,7 @@ static pid_t launch(int argc, const char *_name, const char *args,
      }
 
 
-     if ( !qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty() && !starting_klauncher)
+     if ( !qgetenv("KDE_IS_PRELINKED").isEmpty() && !execpath.isEmpty())
          libpath.truncate(0);
 
      QLibrary l(libpath);
@@ -693,11 +721,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
   default:
      /** Parent **/
      close(d.fd[1]);
-     if (starting_klauncher)
-     {
-        close(d.launcher[1]);
-        d.launcher_pid = d.fork;
-     }
      bool exec = false;
      for(;;)
      {
@@ -738,16 +761,15 @@ static pid_t launch(int argc, const char *_name, const char *args,
              continue;
           }
        }
-       if (exec)
-       {
-          d.result = 0;
-          break;
-       }
        if (d.n == 0)
        {
-          fprintf(stderr,"kdeinit4: (%s %s) Pipe closed unexpectedly", name.constData(), execpath.constData());
-          perror("kdeinit4: Pipe closed unexpectedly");
-          d.result = 1; // Error
+          if (exec) {
+             d.result = 0;
+          } else {
+             fprintf(stderr,"kdeinit4: (%s %s) Pipe closed unexpectedly", name.constData(), execpath.constData());
+             perror("kdeinit4: Pipe closed unexpectedly");
+             d.result = 1; // Error
+          }
           break;
        }
        perror("kdeinit4: Error reading from pipe");
@@ -755,11 +777,6 @@ static pid_t launch(int argc, const char *_name, const char *args,
        break;
      }
      close(d.fd[0]);
-     if (starting_klauncher && (d.result == 0))
-     {
-        // klauncher launched successfully
-        d.launcher_pid = d.fork;
-     }
   }
 #ifdef Q_WS_X11
   if( !startup_id.none())
@@ -773,6 +790,8 @@ static pid_t launch(int argc, const char *_name, const char *args,
   return d.fork;
 }
 
+extern "C" {
+
 static void sig_child_handler(int)
 {
    /*
@@ -787,6 +806,8 @@ static void sig_child_handler(int)
    write(d.deadpipe[1], &c, 1);
 }
 
+}
+
 static void init_signals()
 {
   struct sigaction act;
@@ -794,20 +815,20 @@ static void init_signals()
 
   if (pipe(d.deadpipe) != 0)
   {
-     perror("kdeinit4: Aborting. Can not create pipe: ");
+     perror("kdeinit4: Aborting. Can not create pipe");
      exit(255);
   }
 
   options = fcntl(d.deadpipe[0], F_GETFL);
   if (options == -1)
   {
-     perror("kdeinit4: Aborting. Can not make pipe non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make pipe non-blocking");
      exit(255);
   }
 
   if (fcntl(d.deadpipe[0], F_SETFL, options | O_NONBLOCK) == -1)
   {
-     perror("kdeinit4: Aborting. Can not make pipe non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make pipe non-blocking");
      exit(255);
   }
 
@@ -841,7 +862,6 @@ static void init_signals()
 static void init_kdeinit_socket()
 {
   struct sockaddr_un sa;
-  //struct sockaddr_un sa_old;
   kde_socklen_t socklen;
   long options;
   const QByteArray home_dir = qgetenv("HOME");
@@ -851,7 +871,10 @@ static void init_kdeinit_socket()
      fprintf(stderr, "kdeinit4: Aborting. $HOME not set!");
      exit(255);
   }
-  chdir(home_dir);
+  if (chdir(home_dir) != 0) {
+     fprintf(stderr, "kdeinit4: Aborting. Couldn't enter '%s'!", home_dir.constData());
+     exit(255);
+  }
 
   {
      QByteArray path = home_dir;
@@ -900,7 +923,7 @@ static void init_kdeinit_socket()
      s = socket(PF_UNIX, SOCK_STREAM, 0);
      if (s < 0)
      {
-        perror("socket() failed: ");
+        perror("socket() failed");
         exit(255);
      }
      server.sun_family = AF_UNIX;
@@ -921,34 +944,26 @@ static void init_kdeinit_socket()
 
   /** Delete any stale socket file (and symlink) **/
   unlink(sock_file);
-//  unlink(sock_file_old);
 
   /** create socket **/
   d.wrapper = socket(PF_UNIX, SOCK_STREAM, 0);
   if (d.wrapper < 0)
   {
-     perror("kdeinit4: Aborting. socket() failed: ");
+     perror("kdeinit4: Aborting. socket() failed");
      exit(255);
   }
 
   options = fcntl(d.wrapper, F_GETFL);
   if (options == -1)
   {
-     perror("kdeinit4: Aborting. Can not make socket non-blocking: ");
+     perror("kdeinit4: Aborting. Can not make socket non-blocking");
      close(d.wrapper);
      exit(255);
   }
 
   if (fcntl(d.wrapper, F_SETFL, options | O_NONBLOCK) == -1)
   {
-     perror("kdeinit4: Aborting. Can not make socket non-blocking: ");
-     close(d.wrapper);
-     exit(255);
-  }
-
-  if (fcntl(d.wrapper, F_SETFD, FD_CLOEXEC) == -1)
-  {
-     perror("kdeinit4: Aborting. Can not make socket close-on-execute: ");
+     perror("kdeinit4: Aborting. Can not make socket non-blocking");
      close(d.wrapper);
      exit(255);
   }
@@ -962,7 +977,7 @@ static void init_kdeinit_socket()
       if(bind(d.wrapper, (struct sockaddr *)&sa, socklen) != 0)
       {
           if (max_tries == 0) {
-	      perror("kdeinit4: Aborting. bind() failed: ");
+	      perror("kdeinit4: Aborting. bind() failed");
 	      fprintf(stderr, "Could not bind to socket '%s'\n", sock_file);
 	      close(d.wrapper);
 	      exit(255);
@@ -975,7 +990,7 @@ static void init_kdeinit_socket()
   /** set permissions **/
   if (chmod(sock_file, 0600) != 0)
   {
-     perror("kdeinit4: Aborting. Can not set permissions on socket: ");
+     perror("kdeinit4: Aborting. Can not set permissions on socket");
      fprintf(stderr, "Wrong permissions of socket '%s'\n", sock_file);
      unlink(sock_file);
      close(d.wrapper);
@@ -984,85 +999,11 @@ static void init_kdeinit_socket()
 
   if(listen(d.wrapper, SOMAXCONN) < 0)
   {
-     perror("kdeinit4: Aborting. listen() failed: ");
+     perror("kdeinit4: Aborting. listen() failed");
      unlink(sock_file);
      close(d.wrapper);
      exit(255);
   }
-
-#if 0
-  /** create compatibility socket **/
-  d.wrapper_old = socket(PF_UNIX, SOCK_STREAM, 0);
-  if (d.wrapper_old < 0)
-  {
-     // perror("kdeinit4: Aborting. socket() failed: ");
-     return;
-  }
-
-  options = fcntl(d.wrapper_old, F_GETFL);
-  if (options == -1)
-  {
-     // perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
-     close(d.wrapper_old);
-     d.wrapper_old = -1;
-     return;
-  }
-
-  if (fcntl(d.wrapper_old, F_SETFL, options | O_NONBLOCK) == -1)
-  {
-     // perror("kdeinit4: Aborting. Can't make socket non-blocking: ");
-     close(d.wrapper_old);
-     d.wrapper_old = -1;
-     return;
-  }
-
-  if (fcntl(d.wrapper, F_SETFD, FD_CLOEXEC) == -1)
-  {
-     //perror("kdeinit4: Aborting. Can't make socket close-on-execute: ");
-     close(d.wrapper);
-     d.wrapper_old = -1;
-     return;
-  }
-
-  max_tries = 10;
-  while (1) {
-      /** bind it **/
-      socklen = sizeof(sa_old);
-      memset(&sa_old, 0, socklen);
-      sa_old.sun_family = AF_UNIX;
-      strcpy(sa_old.sun_path, sock_file_old);
-      if(bind(d.wrapper_old, (struct sockaddr *)&sa_old, socklen) != 0)
-      {
-          if (max_tries == 0) {
-	      // perror("kdeinit4: Aborting. bind() failed: ");
-	      fprintf(stderr, "Could not bind to socket '%s'\n", sock_file_old);
-	      close(d.wrapper_old);
-	      d.wrapper_old = -1;
-	      return;
-	  }
-	  max_tries--;
-      } else
-          break;
-  }
-
-  /** set permissions **/
-  if (chmod(sock_file_old, 0600) != 0)
-  {
-     fprintf(stderr, "Wrong permissions of socket '%s'\n", sock_file);
-     unlink(sock_file_old);
-     close(d.wrapper_old);
-     d.wrapper_old = -1;
-     return;
-  }
-
-  if(listen(d.wrapper_old, SOMAXCONN) < 0)
-  {
-     // perror("kdeinit4: Aborting. listen() failed: ");
-     unlink(sock_file_old);
-     close(d.wrapper_old);
-     d.wrapper_old = -1;
-  }
-#endif
 }
 
 /*
@@ -1089,6 +1030,23 @@ static int read_socket(int sock, char *buffer, int len)
   return 0;
 }
 
+static void start_klauncher()
+{
+    if (socketpair(AF_UNIX, SOCK_STREAM, 0, d.launcher) < 0) {
+        perror("kdeinit4: socketpair() failed");
+        exit(255);
+    }
+    char args[32];
+    strcpy(args, "--fd=");
+    sprintf(args + 5, "%d", d.launcher[1]);
+    d.launcher_pid = launch( 2, "klauncher", args );
+    close(d.launcher[1]);
+#ifndef NDEBUG
+    fprintf(stderr, "kdeinit4: Launched KLauncher, pid = %ld, result = %d\n",
+                    (long) d.launcher_pid, d.result);
+#endif
+}
+
 static void launcher_died()
 {
    if (!d.launcher_ok)
@@ -1115,33 +1073,19 @@ static void launcher_died()
    close(d.launcher[0]);
    d.launcher[0] = -1;
 
-   pid_t pid = launch( 1, "klauncher", 0 );
-#ifndef NDEBUG
-   fprintf(stderr, "kdeinit4: Relaunching KLauncher, pid = %ld result = %d\n", (long) pid, d.result);
-#endif
+   start_klauncher();
 }
 
-static void handle_launcher_request(int sock = -1)
+static bool handle_launcher_request(int sock, const char *who)
 {
-   bool launcher = false;
-   if (sock < 0)
-   {
-       sock = d.launcher[0];
-       launcher = true;
-   }
-   else
-   {
-       d.accepted_fd = sock;
-   }
+   (void)who; // for NDEBUG
 
    klauncher_header request_header;
    char *request_data = 0L;
    int result = read_socket(sock, (char *) &request_header, sizeof(request_header));
    if (result != 0)
    {
-      if (launcher)
-         launcher_died();
-      return;
+      return false;
    }
 
    if ( request_header.arg_length != 0 )
@@ -1151,10 +1095,8 @@ static void handle_launcher_request(int sock = -1)
        result = read_socket(sock, request_data, request_header.arg_length);
        if (result != 0)
        {
-           if (launcher)
-               launcher_died();
            free(request_data);
-           return;
+           return false;
        }
    }
 
@@ -1183,15 +1125,12 @@ static void handle_launcher_request(int sock = -1)
       const char *envs = 0;
       const char *tty = 0;
       int avoid_loops = 0;
-      const char *startup_id_str = "0";
+      const char *startup_id_str = "0"; // krazy:exclude=doublequote_chars
 
 #ifndef NDEBUG
      fprintf(stderr, "kdeinit4: Got %s '%s' from %s.\n",
-        (request_header.cmd == LAUNCHER_EXEC ? "EXEC" :
-        (request_header.cmd == LAUNCHER_EXT_EXEC ? "EXT_EXEC" :
-        (request_header.cmd == LAUNCHER_EXEC_NEW ? "EXEC_NEW" :
-        (request_header.cmd == LAUNCHER_SHELL ? "SHELL" : "KWRAPPER" )))),
-         name, launcher ? "launcher" : "socket" );
+             commandToString(request_header.cmd),
+             name, who);
 #endif
 
       const char *arg_n = args;
@@ -1252,7 +1191,7 @@ static void handle_launcher_request(int sock = -1)
 #endif
        free(request_data);
        d.debug_wait = false;
-       return;
+       return true; // sure?
      }
 
       // support for the old a bit broken way of setting DISPLAY for multihead
@@ -1288,13 +1227,6 @@ static void handle_launcher_request(int sock = -1)
          child->sock = dup(sock);
          child->next = children;
          children = child;
-
-         /* check child hasn't terminated already */
-         if (kill(pid, 0))
-         {
-            children = child->next;
-            free(child);
-         }
       }
       else
       {
@@ -1316,10 +1248,7 @@ static void handle_launcher_request(int sock = -1)
       env_value = env_name + strlen(env_name) + 1;
 
 #ifndef NDEBUG
-      if (launcher)
-         fprintf(stderr, "kdeinit4: Got SETENV '%s=%s' from klauncher.\n", env_name, env_value);
-      else
-         fprintf(stderr, "kdeinit4: Got SETENV '%s=%s' from socket.\n", env_name, env_value);
+      fprintf(stderr, "kdeinit4: Got SETENV '%s=%s' from %s.\n", env_name, env_value, who);
 #endif
 
       if ( request_header.arg_length !=
@@ -1329,7 +1258,7 @@ static void handle_launcher_request(int sock = -1)
          fprintf(stderr, "kdeinit4: SETENV request has invalid format.\n");
 #endif
          free(request_data);
-         return;
+         return true; // sure?
       }
       setenv( env_name, env_value, 1);
    }
@@ -1345,12 +1274,24 @@ static void handle_launcher_request(int sock = -1)
    else if (request_header.cmd == LAUNCHER_TERMINATE_KDEINIT)
    {
 #ifndef NDEBUG
-       fprintf(stderr,"kdeinit4: Killing kdeinit/klauncher.\n");
+      fprintf(stderr,"kdeinit4: Got termination request (PID %ld).\n", (long) getpid());
 #endif
-       if (d.launcher_pid)
-          kill(d.launcher_pid, SIGTERM);
-       if (d.my_pid)
-          kill(d.my_pid, SIGTERM);
+      if (d.launcher_pid) {
+         kill(d.launcher_pid, SIGTERM);
+         d.launcher_pid = 0;
+         close(d.launcher[0]);
+         d.launcher[0] = -1;
+      }
+      unlink(sock_file);
+      if (children) {
+         close(d.wrapper);
+         d.wrapper = -1;
+#ifndef NDEBUG
+         fprintf(stderr,"kdeinit4: Closed sockets, but not exiting until all children terminate.\n");
+#endif
+      } else {
+         raise(SIGTERM);
+      }
    }
    else if (request_header.cmd == LAUNCHER_DEBUG_WAIT)
    {
@@ -1361,14 +1302,15 @@ static void handle_launcher_request(int sock = -1)
    }
    if (request_data)
        free(request_data);
+   return true;
 }
 
 static void handle_requests(pid_t waitForPid)
 {
-   int max_sock = d.wrapper;
-   if (d.wrapper_old > max_sock)
-      max_sock = d.wrapper_old;
-   if (d.launcher_pid && (d.launcher[0] > max_sock))
+   int max_sock = d.deadpipe[0];
+   if (d.wrapper > max_sock)
+      max_sock = d.wrapper;
+   if (d.launcher[0] > max_sock)
       max_sock = d.launcher[0];
 #ifdef Q_WS_X11
    if (X11fd > max_sock)
@@ -1406,77 +1348,71 @@ static void handle_requests(pid_t waitForPid)
            else if( WIFSIGNALED( exit_status ))
                exit_status = 128 + WTERMSIG( exit_status );
            child_died(exit_pid, exit_status);
+
+           if (d.wrapper < 0 && !children) {
+#ifndef NDEBUG
+               fprintf(stderr, "kdeinit4: Last child terminated, exiting (PID %ld).\n",
+                               (long) getpid());
+#endif
+               raise(SIGTERM);
+           }
         }
       }
       while( exit_pid > 0);
 
-      /* Set up the next loop */
-      d.accepted_fd = -1;
-
       FD_ZERO(&rd_set);
       FD_ZERO(&wr_set);
       FD_ZERO(&e_set);
 
-      if (d.launcher_pid)
-      {
+      if (d.launcher[0] >= 0)
          FD_SET(d.launcher[0], &rd_set);
-      }
-      FD_SET(d.wrapper, &rd_set);
-      if (d.wrapper_old != -1)
-      {
-         FD_SET(d.wrapper_old, &rd_set);
-      }
+      if (d.wrapper >= 0)
+         FD_SET(d.wrapper, &rd_set);
       FD_SET(d.deadpipe[0], &rd_set);
 #ifdef Q_WS_X11
       if(X11fd >= 0) FD_SET(X11fd, &rd_set);
 #endif
 
       result = select(max_sock, &rd_set, &wr_set, &e_set, 0);
+      if (result < 0) {
+          if (errno == EINTR || errno == EAGAIN)
+              continue;
+          perror("kdeinit4: Aborting. select() failed");
+          return;
+      }
 
       /* Handle wrapper request */
-      if ((result > 0) && (FD_ISSET(d.wrapper, &rd_set)))
+      if (d.wrapper >= 0 && FD_ISSET(d.wrapper, &rd_set))
       {
          struct sockaddr_un client;
          kde_socklen_t sClient = sizeof(client);
          int sock = accept(d.wrapper, (struct sockaddr *)&client, &sClient);
          if (sock >= 0)
          {
-            handle_launcher_request(sock);
-            close(sock);
-         }
-      }
-      if ((result > 0) && d.wrapper_old != -1 && (FD_ISSET(d.wrapper_old, &rd_set)))
-      {
-         struct sockaddr_un client;
-         kde_socklen_t sClient = sizeof(client);
-         int sock = accept(d.wrapper_old, (struct sockaddr *)&client, &sClient);
-         if (sock >= 0)
-         {
-            handle_launcher_request(sock);
+            d.accepted_fd = sock;
+            handle_launcher_request(sock, "wrapper");
             close(sock);
+            d.accepted_fd = -1;
          }
       }
 
       /* Handle launcher request */
-      if ((result > 0) && (d.launcher_pid) && (FD_ISSET(d.launcher[0], &rd_set)))
+      if (d.launcher[0] >= 0 && FD_ISSET(d.launcher[0], &rd_set))
       {
-         handle_launcher_request();
+         if (!handle_launcher_request(d.launcher[0], "launcher"))
+             launcher_died();
          if (waitForPid == d.launcher_pid)
             return;
       }
 
 #ifdef Q_WS_X11
       /* Look for incoming X11 events */
-      if((result > 0) && (X11fd >= 0))
-      {
-        if(FD_ISSET(X11fd,&rd_set))
-        {
+      if(X11fd >= 0 && FD_ISSET(X11fd,&rd_set)) {
           if (X11display != 0) {
 	    XEvent event_return;
 	    while (XPending(X11display))
 	      XNextEvent(X11display, &event_return);
 	  }
-        }
       }
 #endif
    }
@@ -1541,21 +1477,6 @@ static void kdeinit_library_path()
    if((i = display.lastIndexOf('.')) > display.lastIndexOf(':') && i >= 0)
      display.truncate(i);
 
-   // Was kdeinit-display initially,
-   // then at some point in KDE-3.x it became kdeinit_display_with_underscores
-   // And for KDE4 it became kdeinit4_display_with_underscores, to avoid messing up the kde3 kdeinit.
-   // Compat code needed just in case we need it later for something else.
-#if 0
-   QByteArray socketName = QFile::encodeName(locateLocal("socket", QString("kdeinit-%1").arg(QLatin1String(display)), *s_instance));
-   if (socketName.length() >= MAX_SOCK_FILE)
-   {
-     fprintf(stderr, "kdeinit4: Aborting. Socket name will be too long:\n");
-     fprintf(stderr, "         '%s'\n", socketName.data());
-     exit(255);
-   }
-   strcpy(sock_file_old, socketName.data());
-#endif
-
    display.replace(':','_');
 #ifdef __APPLE__
    display.replace('/','_');
@@ -1584,13 +1505,6 @@ int kdeinit_xio_errhandler( Display *disp )
       /** Delete any stale socket file **/
       unlink(sock_file);
     }
-#if 0
-    if (sock_file_old[0])
-    {
-      /** Delete any stale socket file **/
-      unlink(sock_file_old);
-    }
-#endif
 
     // Don't kill our children in suicide mode, they may still be in use
     if (d.suicide)
@@ -1681,43 +1595,15 @@ static int initXconnection()
 }
 #endif
 
-#ifdef __KCC
-/* One of my horrible hacks.  KCC includes in each "main" function a call
-   to _main(), which is provided by the C++ runtime system.  It is
-   responsible for calling constructors for some static objects.  That must
-   be done only once, so _main() is guarded against multiple calls.
-   For unknown reasons the designers of KAI's libKCC decided it would be
-   a good idea to actually abort() when it's called multiple times, instead
-   of ignoring further calls.  This breaks our mechanism of KLM's, because
-   most KLM's have a main() function which is called from us.
-   The "solution" is to simply define our own _main(), which ignores multiple
-   calls, which is easy, and which does the same work as KAI'c _main(),
-   which is difficult.  Currently (KAI 4.0f) it only calls __call_ctors(void)
-   (a C++ function), but if that changes we need to change our's too.
-   (matz) */
-/*
- Those 'unknown reasons' are C++ standard forbidding recursive calls to main()
- or any means that would possibly allow that (e.g. taking address of main()).
- The correct solution is not using main() as entry point for kdeinit modules,
- but only kdemain().
-*/
-extern "C" void _main(void);
-extern "C" void __call_ctors__Fv(void);
-static int main_called = 0;
-void _main(void)
-{
-  if (main_called)
-    return;
-  main_called = 1;
-  __call_ctors__Fv ();
-}
-#endif
+extern "C" {
 
 static void secondary_child_handler(int)
 {
    waitpid(-1, 0, WNOHANG);
 }
 
+}
+
 int main(int argc, char **argv, char **envp)
 {
     setlocale (LC_ALL, "");
@@ -1756,6 +1642,10 @@ int main(int argc, char **argv, char **envp)
 	 printf("KDE: %s\n", KDE_VERSION_STRING);
 	 exit(0);
       }
+#ifdef KDEINIT_OOM_PROTECT
+      if (strcmp(safe_argv[i], "--oom-pipe") == 0 && i+1<argc)
+         oom_pipe = atol(argv[i+1]);
+#endif
       if (strcmp(safe_argv[i], "--help") == 0)
       {
         printf("Usage: kdeinit4 [options]\n");
@@ -1776,11 +1666,18 @@ int main(int argc, char **argv, char **envp)
 
    cleanup_fds();
 
+   // Redirect stdout to stderr. We have no reason to use stdout anyway.
+   // This minimizes our impact on commands used in pipes.
+   (void)dup2(2, 1);
+
    if (do_fork) {
 #ifdef Q_WS_MACX
       mac_fork_and_reexec_self();
 #else
-      pipe(d.initpipe);
+      if (pipe(d.initpipe) != 0) {
+          perror("kdeinit4: pipe failed");
+          return 1;
+      }
 
       // Fork here and let parent process exit.
       // Parent process may only exit after all required services have been
@@ -1804,13 +1701,6 @@ int main(int argc, char **argv, char **envp)
 #endif
    }
 
-   /** We start a few processes (including klauncher and kded) before
-    * we close our pipe, so make sure it doesn't leak */
-   if (d.initpipe[1] != -1)
-      fcntl(d.initpipe[1], F_SETFD, FD_CLOEXEC);
-
-   d.my_pid = getpid();
-
    /** Make process group leader (for shutting down children later) **/
    if(keep_running)
       setsid();
@@ -1837,7 +1727,6 @@ int main(int argc, char **argv, char **envp)
    d.launcher_pid = 0;
    d.kded_pid = 0;
    d.wrapper = -1;
-   d.wrapper_old = -1;
    d.accepted_fd = -1;
    d.debug_wait = false;
    d.launcher_ok = false;
@@ -1858,11 +1747,15 @@ int main(int argc, char **argv, char **envp)
 #ifdef Q_WS_X11
    if (!d.suicide && qgetenv("KDE_IS_PRELINKED").isEmpty())
    {
-       QString konq = KStandardDirs::locate("lib", QLatin1String("libkonq.so.5"), *s_instance);
+#ifdef __KDE_HAVE_GCC_VISIBILITY
+       QString extra = KStandardDirs::locate("lib", QLatin1String("libplasma.so.3"), *s_instance);
+#else
+       QString extra;
+#endif
        // can't use KLibLoader here as it would unload the library
        // again
-       if (!konq.isEmpty()) {
-           QLibrary l(konq);
+       if (!extra.isEmpty()) {
+           QLibrary l(extra);
            l.setLoadHints(QLibrary::ExportExternalSymbolsHint);
            l.load();
        }
@@ -1870,11 +1763,8 @@ int main(int argc, char **argv, char **envp)
 #endif
    if (launch_klauncher)
    {
-      pid = launch( 1, "klauncher", 0 );
-#ifndef NDEBUG
-      fprintf(stderr, "kdeinit4: Launched KLauncher, pid = %ld result = %d\n", (long) pid, d.result);
-#endif
-      handle_requests(pid); // Wait for klauncher to be ready
+      start_klauncher();
+      handle_requests(d.launcher_pid); // Wait for klauncher to be ready
    }
 
 #ifdef Q_WS_X11
@@ -1913,7 +1803,11 @@ int main(int argc, char **argv, char **envp)
 #endif
          handle_requests(pid);
       }
-      else if (safe_argv[i][0] == '-')
+      else if (safe_argv[i][0] == '-'
+#ifdef KDEINIT_OOM_PROTECT
+          || isdigit(safe_argv[i][0])
+#endif
+          )
       {
          // Ignore
       }
