@@ -87,7 +87,6 @@ KACLEditWidget::KACLEditWidget( QWidget *parent )
 {
     QHBoxLayout *hbox = new QHBoxLayout( this );
     hbox->setMargin( 0 );
-    hbox->setSpacing(  KDialog::spacingHint() );
     d->m_listView = new KACLListView(this);
     hbox->addWidget(d->m_listView);
     connect(d->m_listView->selectionModel(),
@@ -96,7 +95,6 @@ KACLEditWidget::KACLEditWidget( QWidget *parent )
             SLOT(_k_slotUpdateButtons()));
     QVBoxLayout *vbox = new QVBoxLayout();
     hbox->addLayout( vbox );
-    vbox->setSpacing(  KDialog::spacingHint() );
     d->m_AddBtn = new QPushButton(i18n("Add Entry..."), this);
     vbox->addWidget(d->m_AddBtn);
     d->m_AddBtn->setObjectName(QLatin1String("add_entry_button"));
@@ -184,9 +182,9 @@ QString KACLListViewItem::key() const
 {
     QString key;
     if ( !isDefault )
-        key = "A";
+        key = 'A';
     else
-        key = "B";
+        key = 'B';
     switch ( type )
     {
         case KACLListView::User:
@@ -439,10 +437,8 @@ EditACLEntryDialog::EditACLEntryDialog( KACLListView *listView, KACLListViewItem
     setMainWidget( page );
     QVBoxLayout *mainLayout = new QVBoxLayout( page );
     mainLayout->setMargin( 0 );
-    mainLayout->setSpacing( spacingHint() );
     QGroupBox *gb = new QGroupBox( i18n("Entry Type"), page );
     QVBoxLayout *gbLayout = new QVBoxLayout( gb );
-    gbLayout->setSpacing( spacingHint() );
 
     m_buttonGroup = new QButtonGroup( page );
 
@@ -496,10 +492,10 @@ EditACLEntryDialog::EditACLEntryDialog( KACLListView *listView, KACLListViewItem
     mainLayout->addWidget( m_widgetStack );
 
     KHBox *usersBox = new KHBox( m_widgetStack );
-    m_widgetStack->insertWidget( KACLListView::NamedUser,usersBox );
+    m_widgetStack->addWidget( usersBox );
 
     KHBox *groupsBox = new KHBox( m_widgetStack );
-    m_widgetStack->insertWidget( KACLListView::NamedGroup,groupsBox );
+    m_widgetStack->addWidget( groupsBox );
 
     QLabel *usersLabel = new QLabel( i18n( "User: " ), usersBox );
     m_usersCombo = new KComboBox( usersBox );
@@ -608,11 +604,11 @@ void EditACLEntryDialog::slotSelectionChanged( QAbstractButton *button )
             break;
         case KACLListView::NamedUser:
             m_widgetStack->setEnabled( true );
-            m_widgetStack->setCurrentIndex( KACLListView::NamedUser );
+            m_widgetStack->setCurrentIndex( 0 /* User */ );
             break;
         case KACLListView::NamedGroup:
             m_widgetStack->setEnabled( true );
-            m_widgetStack->setCurrentIndex( KACLListView::NamedGroup );
+            m_widgetStack->setCurrentIndex( 1 /* Group */ );
             break;
         default:
             break;
@@ -686,7 +682,7 @@ QStringList KACLListView::allowedUsers( bool defaults, KACLListViewItem *allowed
     while ( *it ) {
         const KACLListViewItem *item = static_cast<const KACLListViewItem*>( *it );
         ++it;
-        if ( !item->type == NamedUser || item->isDefault != defaults ) continue;
+        if ( item->type != NamedUser || item->isDefault != defaults ) continue;
         if ( allowedItem && item == allowedItem && allowedItem->isDefault == defaults ) continue;
         allowedUsers.removeAll( item->qualifier );
     }
@@ -700,7 +696,7 @@ QStringList KACLListView::allowedGroups( bool defaults, KACLListViewItem *allowe
     while ( *it ) {
         const KACLListViewItem *item = static_cast<const KACLListViewItem*>( *it );
         ++it;
-        if ( !item->type == NamedGroup || item->isDefault != defaults ) continue;
+        if ( item->type != NamedGroup || item->isDefault != defaults ) continue;
         if ( allowedItem && item == allowedItem && allowedItem->isDefault == defaults ) continue;
         allowedGroups.removeAll( item->qualifier );
     }
