@@ -33,6 +33,7 @@
 #include <QtCore/QFileInfo>
 #include <QtGui/QProgressBar>
 #include <QtGui/QPainter>
+#include <QtGui/QScrollBar>
 
 
 /**
@@ -165,7 +166,9 @@ void KIconCanvas::KIconCanvasPrivate::_k_slotLoadFiles()
     q->setResizeMode(QListWidget::Fixed);
     QApplication::setOverrideCursor(Qt::WaitCursor);
 
-    // disable updates to not trigger paint events when adding child items
+    // disable updates to not trigger paint events when adding child items,
+    // but force an initial paint so that we do not get garbage
+    q->repaint();
     q->setUpdatesEnabled(false);
 
     // Cache these as we will call them frequently.
@@ -366,13 +369,13 @@ void KIconDialog::KIconDialogPrivate::init()
     top->addWidget(bgroup);
 
     QGridLayout *grid = new QGridLayout();
-    grid->setSpacing(KDialog::spacingHint());
-    bgroup->layout()->addItem(grid);
+    vbox->addLayout(grid);
 
     mpSystemIcons = new QRadioButton(i18n("S&ystem icons:"), bgroup);
     connect(mpSystemIcons, SIGNAL(clicked()), q, SLOT(_k_slotSystemIconClicked()));
     grid->addWidget(mpSystemIcons, 1, 0);
     mpCombo = new KComboBox(bgroup);
+    mpCombo->setMaxVisibleItems(12);
     connect(mpCombo, SIGNAL(activated(int)), q, SLOT(_k_slotContext(int)));
     grid->addWidget(mpCombo, 1, 1);
     mpOtherIcons = new QRadioButton(i18n("O&ther icons:"), bgroup);
@@ -387,7 +390,6 @@ void KIconDialog::KIconDialogPrivate::init()
     //
     QHBoxLayout *searchLayout = new QHBoxLayout();
     searchLayout->setMargin(0);
-    searchLayout->setSpacing(KDialog::spacingHint());
     top->addLayout(searchLayout);
 
     QLabel *searchLabel = new QLabel(i18n("&Search:"), main);
@@ -404,10 +406,20 @@ void KIconDialog::KIconDialogPrivate::init()
 
     mpCanvas = new KIconCanvas(main);
     connect(mpCanvas, SIGNAL(itemActivated(QListWidgetItem *)), q, SLOT(_k_slotAcceptIcons()));
-    mpCanvas->setMinimumSize(425, 125);
     top->addWidget(mpCanvas);
     searchLine->setListWidget(mpCanvas);
 
+    // Compute width of canvas with 4 icons displayed in a row
+    QStyleOption opt;
+    opt.initFrom(mpCanvas);
+    int width = 4 * mpCanvas->gridSize().width() + 1;
+    width += mpCanvas->verticalScrollBar()->sizeHint().width();
+    width += 2 * mpCanvas->frameWidth();
+    if (mpCanvas->style()->styleHint(QStyle::SH_ScrollView_FrameOnlyAroundContents, &opt, mpCanvas)) {
+        width += mpCanvas->style()->pixelMetric(QStyle::PM_ScrollView_ScrollBarSpacing, &opt, mpCanvas);
+    }
+    mpCanvas->setMinimumSize(width, 125);
+
     mpProgress = new QProgressBar(main);
     top->addWidget(mpProgress);
     connect(mpCanvas, SIGNAL(startLoading(int)), q, SLOT(_k_slotStartLoading(int)));
@@ -610,6 +622,7 @@ void KIconDialog::setCustomLocation( const QString& location )
 QString KIconDialog::openDialog()
 {
     d->showIcons();
+    d->searchLine->setFocus();
 
     if ( exec() == Accepted )
     {
@@ -629,6 +642,7 @@ void KIconDialog::showDialog()
 {
     setModal(false);
     d->showIcons();
+    d->searchLine->setFocus();
     show();
 }
 
