@@ -26,7 +26,6 @@
 #include <kglobal.h>
 #include <kiconloader.h>
 #include <kprotocolmanager.h>
-#include <kde_file.h>
 #include <kmimetype.h>
 #include <kdynamicjobtracker_p.h>
 
@@ -118,7 +117,7 @@ KIO_EXPORT QTime KIO::calculateRemaining( KIO::filesize_t totalSize, KIO::filesi
 KIO_EXPORT QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KIO::filesize_t size, bool showSize)
 {
     if ( files == 0 && dirs == 0 && items == 0 ) {
-        return i18n( "0 Items" );
+        return i18np( "%1 Item", "%1 Items", 0 );
     }
     
     QString summary;
@@ -135,7 +134,7 @@ KIO_EXPORT QString KIO::itemsSummaryString(uint items, uint files, uint dirs, KI
     }
     
     if ( items > dirs + files ) {
-        const QString itemsText = i18np( "1 Item", "%1 Items", items );
+        const QString itemsText = i18np( "%1 Item", "%1 Items", items );
         summary = summary.isEmpty() ? itemsText : i18nc( "items: folders, files (size)", "%1: %2", itemsText, summary );
     }
     
@@ -1077,7 +1076,7 @@ KIO_EXPORT QByteArray KIO::rawErrorDetail(int errorCode, const QString &errorTex
         "<li>Timeout for receiving a response: %2 seconds</li>"
         "<li>Timeout for accessing proxy servers: %3 seconds</li></ul>"
         "Please note that you can alter these timeout settings in the KDE "
-        "Control Center, by selecting Network -> Preferences." ,
+        "System Settings, by selecting Network Settings -> Connection Preferences." ,
           KProtocolManager::connectTimeout() ,
           KProtocolManager::responseTimeout() ,
           KProtocolManager::proxyConnectTimeout() );
@@ -1245,7 +1244,43 @@ KJobTrackerInterface *KIO::getJobTracker()
 }
 
 
+/***************************************************************
+ *
+ * KIO::MetaData
+ *
+ ***************************************************************/
+KIO::MetaData::MetaData(const QMap<QString,QVariant>& map)
+{
+  *this = map;
+}
 
+KIO::MetaData & KIO::MetaData::operator += ( const QMap<QString,QVariant> &metaData )
+{
+  QMapIterator<QString,QVariant> it (metaData);
+  
+  while(it.hasNext()) {
+     it.next();
+     insert(it.key(), it.value().toString());
+   }
 
+  return *this;
+}
 
+KIO::MetaData & KIO::MetaData::operator = ( const QMap<QString,QVariant> &metaData )
+{
+  clear();
+  return (*this += metaData);
+}
 
+QVariant KIO::MetaData::toVariant() const
+{
+  QMap<QString, QVariant> map;
+  QMapIterator <QString,QString> it (*this);
+ 
+ while (it.hasNext()) {
+   it.next();
+   map.insert(it.key(), it.value());
+ }
+
+ return QVariant(map);
+}
