@@ -95,27 +95,43 @@ KDirListerCache::~KDirListerCache()
 bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
                                bool _keep, bool _reload )
 {
-  // like this we don't have to worry about trailing slashes any further
   KUrl _url(_u);
   _url.cleanPath(); // kill consecutive slashes
 
-  if (!_url.host().isEmpty() && KProtocolInfo::protocolClass(_url.protocol()) == ":local") {
+  if (!_url.host().isEmpty() && KProtocolInfo::protocolClass(_url.protocol()) == ":local"
+      && _url.protocol() != "file") {
       // ":local" protocols ignore the hostname, so strip it out preventively - #160057
+      // kio_file is special cased since it does honor the hostname (by redirecting to e.g. smb)
       _url.setHost(QString());
       if (_keep == false)
           emit lister->redirection(_url);
   }
 
+  // like this we don't have to worry about trailing slashes any further
   _url.adjustPath(KUrl::RemoveTrailingSlash);
+
   const QString urlStr = _url.url();
 
+  QString resolved;
+  if (_url.isLocalFile()) {
+      // Resolve symlinks (#213799)
+      const QString local = _url.toLocalFile();
+      resolved = QFileInfo(local).canonicalFilePath();
+      if (local != resolved)
+          canonicalUrls[resolved].append(urlStr);
+      // TODO: remove entry from canonicalUrls again in forgetDirs
+      // Note: this is why we use a QStringList value in there rather than a QSet:
+      // we can just remove one entry and not have to worry about other dirlisters
+      // (the non-unicity of the stringlist gives us the refcounting, basically).
+  }
+
   if (!validUrl(lister, _url)) {
         kDebug(7004) << lister << "url=" << _url << "not a valid url";
         return false;
   }
 
 #ifdef DEBUG_CACHE
-  printDebug();
+    printDebug();
 #endif
   //kDebug(7004) << lister << "url=" << _url << "keep=" << _keep << "reload=" << _reload;
 
@@ -187,7 +203,7 @@ bool KDirListerCache::listDir( KDirLister *lister, const KUrl& _u,
                 kDebug(7004) << "Listing directory:" << _url;
             }
 
-            itemU = new DirItem(_url);
+            itemU = new DirItem(_url, resolved);
             itemsInUse.insert(urlStr, itemU);
 
 //        // we have a limit of MAX_JOBS_PER_LISTER concurrently running jobs
@@ -521,7 +537,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
             bool isManuallyMounted = false;
             bool containsManuallyMounted = false;
             if (isLocal) {
-                isManuallyMounted = manually_mounted( item->url.path(), possibleMountPoints );
+                isManuallyMounted = manually_mounted( item->url.toLocalFile(), possibleMountPoints );
                 if ( !isManuallyMounted ) {
                     // Look for a manually-mounted directory inside
                     // If there's one, we can't keep a watch either, FAM would prevent unmounting the CDROM
@@ -529,7 +545,7 @@ void KDirListerCache::forgetDirs( KDirLister *lister, const KUrl& _url, bool not
                     KFileItemList::const_iterator kit = item->lstItems.constBegin();
                     KFileItemList::const_iterator kend = item->lstItems.constEnd();
                     for ( ; kit != kend && !containsManuallyMounted; ++kit )
-                        if ( (*kit).isDir() && manually_mounted((*kit).url().path(), possibleMountPoints) )
+                        if ( (*kit).isDir() && manually_mounted((*kit).url().toLocalFile(), possibleMountPoints) )
                             containsManuallyMounted = true;
                 }
             }
@@ -608,7 +624,7 @@ void KDirListerCache::updateDirectory( const KUrl& _dir )
 
     if (!(listers.isEmpty() || killed)) {
         kWarning() << "The unexpected happened.";
-        kWarning() << "listers=" << listers;
+        kWarning() << "listers for" << _dir << "=" << listers;
         kWarning() << "job=" << job;
         Q_FOREACH(KDirLister *kdl, listers) {
             kDebug() << "lister" << kdl << "m_cachedItemsJob=" << kdl->d->m_cachedItemsJob;
@@ -721,43 +737,54 @@ KFileItem *KDirListerCache::findByUrl( const KDirLister *lister, const KUrl& _u
     KUrl url(_u);
     url.adjustPath(KUrl::RemoveTrailingSlash);
 
-    // Maybe _u is a directory itself? (see KDirModelTest::testChmodDirectory)
-    DirItem* dirItem = dirItemForUrl(url);
-    if (dirItem && !dirItem->rootItem.isNull() && dirItem->rootItem.url() == url) {
-        // If lister is set, check that it contains this dir
-        if (!lister || lister->d->lstDirs.contains(url))
-            return &dirItem->rootItem;
-    }
-
     KUrl parentDir(url);
     parentDir.setPath( parentDir.directory() );
 
-    // If lister is set, check that it contains this dir
-    if (lister && !lister->d->lstDirs.contains(parentDir))
-        return 0;
-
-    dirItem = dirItemForUrl(parentDir);
+    DirItem* dirItem = dirItemForUrl(parentDir);
     if (dirItem) {
-        KFileItemList::iterator it = dirItem->lstItems.begin();
-        const KFileItemList::iterator end = dirItem->lstItems.end();
-        for (; it != end ; ++it) {
-            if ((*it).url() == url) {
-                return &*it;
+        // If lister is set, check that it contains this dir
+        if (!lister || lister->d->lstDirs.contains(parentDir)) {
+            KFileItemList::iterator it = dirItem->lstItems.begin();
+            const KFileItemList::iterator end = dirItem->lstItems.end();
+            for (; it != end ; ++it) {
+                if ((*it).url() == url) {
+                    return &*it;
+                }
             }
         }
     }
 
+    // Maybe _u is a directory itself? (see KDirModelTest::testChmodDirectory)
+    // We check this last, though, we prefer returning a kfileitem with an actual
+    // name if possible (and we make it '.' for root items later).
+    dirItem = dirItemForUrl(url);
+    if (dirItem && !dirItem->rootItem.isNull() && dirItem->rootItem.url() == url) {
+        // If lister is set, check that it contains this dir
+        if (!lister || lister->d->lstDirs.contains(url)) {
+            return &dirItem->rootItem;
+        }
+    }
+
     return 0;
 }
 
-void KDirListerCache::slotFilesAdded( const QString &dir ) // from KDirNotify signals
+void KDirListerCache::slotFilesAdded( const QString &dir /*url*/ ) // from KDirNotify signals
 {
-  kDebug(7004) << dir;
-  updateDirectory( KUrl(dir) );
+    KUrl urlDir(dir);
+    kDebug(7004) << urlDir; // output urls, not qstrings, since they might contain a password
+    if (urlDir.isLocalFile()) {
+        Q_FOREACH(const QString& u, directoriesForCanonicalPath(urlDir.path())) {
+            updateDirectory(KUrl(u));
+        }
+    } else {
+        updateDirectory(urlDir);
+    }
 }
 
 void KDirListerCache::slotFilesRemoved( const QStringList &fileList ) // from KDirNotify signals
 {
+    // TODO: handling of symlinks-to-directories isn't done here,
+    // because I'm not sure how to do it and keep the performance ok...
     slotFilesRemoved(KUrl::List(fileList));
 }
 
@@ -827,11 +854,7 @@ void KDirListerCache::slotFilesChanged( const QStringList &fileList ) // from KD
             continue;
         }
         if (url.isLocalFile()) {
-            // we need to refresh the item, because e.g. the permissions can have changed.
-            aboutToRefreshItem(*fileitem);
-            KFileItem oldItem = *fileitem;
-            fileitem->refresh();
-            emitRefreshItem(oldItem, *fileitem);
+            pendingUpdates.insert(*it); // delegate the work to processPendingUpdates
         } else {
             pendingRemoteUpdates.insert(fileitem);
             // For remote files, we won't be able to figure out the new information,
@@ -848,6 +871,8 @@ void KDirListerCache::slotFilesChanged( const QStringList &fileList ) // from KD
         updateDirectory( *itdir );
     // ## TODO problems with current jobs listing/updating that dir
     // ( see kde-2.2.2's kdirlister )
+
+    processPendingUpdates();
 }
 
 void KDirListerCache::slotFileRenamed( const QString &_src, const QString &_dst ) // from KDirNotify signals
@@ -900,14 +925,16 @@ void KDirListerCache::slotFileRenamed( const QString &_src, const QString &_dst
     if (!oldItem.isLocalFile() && !oldItem.localPath().isEmpty()) { // it uses UDS_LOCAL_PATH? ouch, needs an update then
         slotFilesChanged( QStringList() << src.url() );
     } else {
-        aboutToRefreshItem( oldItem );
         if( nameOnly )
             fileitem->setName( dst.fileName() );
         else
             fileitem->setUrl( dst );
         fileitem->refreshMimeType();
         fileitem->determineMimeType();
-        emitRefreshItem( oldItem, *fileitem );
+        QSet<KDirLister*> listers = emitRefreshItem( oldItem, *fileitem );
+        Q_FOREACH(KDirLister * kdl, listers) {
+            kdl->d->emitItems();
+        }
     }
 
 #ifdef DEBUG_CACHE
@@ -915,31 +942,14 @@ void KDirListerCache::slotFileRenamed( const QString &_src, const QString &_dst
 #endif
 }
 
-void KDirListerCache::aboutToRefreshItem( const KFileItem& fileitem )
-{
-    // Look whether this item was shown in any view, i.e. held by any dirlister
-    KUrl parentDir( fileitem.url() );
-    parentDir.setPath( parentDir.directory() );
-    const QString parentDirURL = parentDir.url();
-
-    DirectoryDataHash::iterator dit = directoryData.find(parentDirURL);
-    if (dit == directoryData.end())
-        return;
-
-    foreach (KDirLister *kdl, (*dit).listersCurrentlyHolding)
-        kdl->d->aboutToRefreshItem( fileitem );
-
-    // Also look in listersCurrentlyListing, in case the user manages to rename during a listing
-    foreach (KDirLister *kdl, (*dit).listersCurrentlyListing)
-        kdl->d->aboutToRefreshItem( fileitem );
-}
-
-void KDirListerCache::emitRefreshItem(const KFileItem& oldItem, const KFileItem& fileitem )
+QSet<KDirLister*> KDirListerCache::emitRefreshItem(const KFileItem& oldItem, const KFileItem& fileitem)
 {
+    //kDebug(7004) << "old:" << oldItem.name() << oldItem.url()
+    //             << "new:" << fileitem.name() << fileitem.url();
     // Look whether this item was shown in any view, i.e. held by any dirlister
     KUrl parentDir( oldItem.url() );
     parentDir.setPath( parentDir.directory() );
-    QString parentDirURL = parentDir.url();
+    const QString parentDirURL = parentDir.url();
     DirectoryDataHash::iterator dit = directoryData.find(parentDirURL);
     QList<KDirLister *> listers;
     // Also look in listersCurrentlyListing, in case the user manages to rename during a listing
@@ -951,16 +961,33 @@ void KDirListerCache::emitRefreshItem(const KFileItem& oldItem, const KFileItem&
         if (dit != directoryData.end())
             listers += (*dit).listersCurrentlyHolding + (*dit).listersCurrentlyListing;
     }
+    QSet<KDirLister*> listersToRefresh;
     Q_FOREACH(KDirLister *kdl, listers) {
         // For a directory, look for dirlisters where it's the root item.
-        if (oldItem.isDir() && kdl->d->rootFileItem == oldItem) {
-            kdl->d->rootFileItem = fileitem;
-        }
         KUrl directoryUrl(oldItem.url());
-        directoryUrl.setPath(directoryUrl.directory());
-        kdl->d->addRefreshItem(directoryUrl, oldItem, fileitem);
-        kdl->d->emitItems();
+        if (oldItem.isDir() && kdl->d->rootFileItem == oldItem) {
+            const KFileItem oldRootItem = kdl->d->rootFileItem;
+            kdl->d->rootFileItem = fileitem;
+            kdl->d->addRefreshItem(directoryUrl, oldRootItem, fileitem);
+        } else {
+            directoryUrl.setPath(directoryUrl.directory());
+            kdl->d->addRefreshItem(directoryUrl, oldItem, fileitem);
+        }
+        listersToRefresh.insert(kdl);
     }
+    return listersToRefresh;
+}
+
+QStringList KDirListerCache::directoriesForCanonicalPath(const QString& dir) const
+{
+    QStringList dirs;
+    dirs << dir;
+    dirs << canonicalUrls.value(dir).toSet().toList(); /* make unique; there are faster ways, but this is really small anyway */
+
+    if (dirs.count() > 1)
+        kDebug() << dir << "known as" << dirs;
+
+    return dirs;
 }
 
 // private slots
@@ -972,32 +999,65 @@ void KDirListerCache::slotFileDirty( const QString& path )
     kDebug(7004) << path;
     // File or dir?
     KDE_struct_stat buff;
-    if ( KDE_stat( QFile::encodeName(path), &buff ) != 0 )
+    if ( KDE::stat( path, &buff ) != 0 )
         return; // error
     const bool isDir = S_ISDIR(buff.st_mode);
     KUrl url(path);
-
+    url.adjustPath(KUrl::RemoveTrailingSlash);
     if (isDir) {
-        // A dir: launch an update job if anyone cares about it
-        updateDirectory(url);
+        Q_FOREACH(const QString& dir, directoriesForCanonicalPath(url.path())) {
+            handleDirDirty(dir);
+        }
     } else {
-        // A file: do we know about it already?
-        KFileItem* existingItem = findByUrl(0, url);
-        if (!existingItem) {
-            // No - update the parent dir then
-            url.setPath(url.directory());
-            updateDirectory(url);
-        } else {
-            // A known file: delay updating it, FAM is flooding us with events
-            const QString urlStr = url.url(KUrl::RemoveTrailingSlash);
-            if (!pendingUpdates.contains(urlStr)) {
-                KUrl dir(url);
-                dir.setPath(dir.directory());
-                if (checkUpdate(dir.url())) {
-                    pendingUpdates.insert(urlStr);
-                    if (!pendingUpdateTimer.isActive())
-                        pendingUpdateTimer.start( 500 );
-                }
+        Q_FOREACH(const QString& dir, directoriesForCanonicalPath(url.directory())) {
+            KUrl aliasUrl(dir);
+            aliasUrl.addPath(url.fileName());
+            handleFileDirty(aliasUrl);
+        }
+    }
+}
+
+// Called by slotFileDirty
+void KDirListerCache::handleDirDirty(const KUrl& url)
+{
+    // A dir: launch an update job if anyone cares about it
+
+    // This also means we can forget about pending updates to individual files in that dir
+    const QString dirPath = url.toLocalFile(KUrl::AddTrailingSlash);
+    QMutableSetIterator<QString> pendingIt(pendingUpdates);
+    while (pendingIt.hasNext()) {
+        const QString updPath = pendingIt.next();
+        //kDebug(7004) << "had pending update" << updPath;
+        if (updPath.startsWith(dirPath) &&
+            updPath.indexOf('/', dirPath.length()) == -1) { // direct child item
+            kDebug(7004) << "forgetting about individual update to" << updPath;
+            pendingIt.remove();
+        }
+    }
+
+    updateDirectory(url);
+}
+
+// Called by slotFileDirty
+void KDirListerCache::handleFileDirty(const KUrl& url)
+{
+    // A file: do we know about it already?
+    KFileItem* existingItem = findByUrl(0, url);
+    if (!existingItem) {
+        // No - update the parent dir then
+        KUrl dir(url);
+        dir.setPath(url.directory());
+        updateDirectory(dir);
+    } else {
+        // A known file: delay updating it, FAM is flooding us with events
+        const QString filePath = url.toLocalFile();
+        if (!pendingUpdates.contains(filePath)) {
+            KUrl dir(url);
+            dir.setPath(dir.directory());
+            if (checkUpdate(dir.url())) {
+                pendingUpdates.insert(filePath);
+                if (!pendingUpdateTimer.isActive())
+                    pendingUpdateTimer.start(500);
             }
         }
     }
@@ -1005,18 +1065,23 @@ void KDirListerCache::slotFileDirty( const QString& path )
 
 void KDirListerCache::slotFileCreated( const QString& path ) // from KDirWatch
 {
-  kDebug(7004) << path;
-  // XXX: how to avoid a complete rescan here?
-  KUrl u( path );
-  u.setPath( u.directory() );
-  updateDirectory( u );
+    kDebug(7004) << path;
+    // XXX: how to avoid a complete rescan here?
+    // We'd need to stat that one file separately and refresh the item(s) for it.
+    KUrl fileUrl(path);
+    slotFilesAdded(fileUrl.directory());
 }
 
 void KDirListerCache::slotFileDeleted( const QString& path ) // from KDirWatch
 {
-  kDebug(7004) << path;
-  KUrl u( path );
-  slotFilesRemoved( QStringList() << u.url() );
+    kDebug(7004) << path;
+    KUrl u( path );
+    QStringList fileUrls;
+    Q_FOREACH(KUrl url, directoriesForCanonicalPath(u.directory())) {
+        url.addPath(u.fileName());
+        fileUrls << url.url();
+    }
+    slotFilesRemoved(fileUrls);
 }
 
 void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entries )
@@ -1067,6 +1132,7 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
             // twice, otherwise, so that both views manage to recognize the item.
             // 2) with kio_ftp we can only know that something is a symlink when
             // listing the parent, so prefer that item, which has more info.
+            // Note that it gives a funky name() to the root item, rather than "." ;)
             dir->rootItem = itemForUrl(url);
             if (dir->rootItem.isNull())
                 dir->rootItem = KFileItem( *it, url, delayedMimeTypes, true  );
@@ -1093,6 +1159,10 @@ void KDirListerCache::slotEntries( KIO::Job *job, const KIO::UDSEntryList &entri
 
 void KDirListerCache::slotResult( KJob *j )
 {
+#ifdef DEBUG_CACHE
+  printDebug();
+#endif
+
   Q_ASSERT( j );
   KIO::ListJob *job = static_cast<KIO::ListJob *>( j );
   runningListJobs.remove( job );
@@ -1102,9 +1172,6 @@ void KDirListerCache::slotResult( KJob *j )
   QString jobUrlStr = jobUrl.url();
 
   kDebug(7004) << "finished listing" << jobUrl;
-#ifdef DEBUG_CACHE
-  printDebug();
-#endif
 
   DirectoryDataHash::iterator dit = directoryData.find(jobUrlStr);
   Q_ASSERT(dit != directoryData.end());
@@ -1182,7 +1249,9 @@ void KDirListerCache::slotRedirection( KIO::Job *j, const KUrl& url )
     kDebug(7004) << oldUrl << "->" << newUrl;
 
 #ifdef DEBUG_CACHE
-    printDebug();
+    // Can't do that here. KDirListerCache::joburl() will use the new url already,
+    // while our data structures haven't been updated yet -> assert fail.
+    //printDebug();
 #endif
 
     // I don't think there can be dirItems that are children of oldUrl.
@@ -1353,6 +1422,7 @@ void KDirListerCache::renameDir( const KUrl &oldUrl, const KUrl &newUrl )
     //emitRedirections( oldUrl, url );
 
     QLinkedList<ItemInUseChange> itemsToChange;
+    QSet<KDirLister *> listers;
 
     // Look at all dirs being listed/shown
     QHash<QString, DirItem *>::iterator itu = itemsInUse.begin();
@@ -1382,7 +1452,6 @@ void KDirListerCache::renameDir( const KUrl &oldUrl, const KUrl &newUrl )
             for ( KFileItemList::iterator kit = dir->lstItems.begin(), kend = dir->lstItems.end();
                   kit != kend ; ++kit )
             {
-                aboutToRefreshItem(*kit);
                 const KFileItem oldItem = *kit;
 
                 const KUrl oldItemUrl ((*kit).url());
@@ -1393,12 +1462,16 @@ void KDirListerCache::renameDir( const KUrl &oldUrl, const KUrl &newUrl )
                 kDebug(7004) << "renaming" << oldItemUrl << "to" << newItemUrl;
                 (*kit).setUrl(newItemUrl);
 
-                emitRefreshItem(oldItem, *kit);
+                listers |= emitRefreshItem(oldItem, *kit);
             }
             emitRedirections( oldDirUrl, newDirUrl );
         }
     }
 
+    Q_FOREACH(KDirLister * kdl, listers) {
+        kdl->d->emitItems();
+    }
+
     // Do the changes to itemsInUse out of the loop to avoid messing up iterators,
     // and so that emitRefreshItem can find the stuff in the hash.
     foreach(const ItemInUseChange& i, itemsToChange) {
@@ -1583,8 +1656,6 @@ void KDirListerCache::slotUpdateResult( KJob * j )
                 if (inPendingRemoteUpdates) {
                     pendingRemoteUpdates.erase(pru_it);
                 }
-                foreach ( KDirLister *kdl, listers )
-                    kdl->d->aboutToRefreshItem( *tmp );
 
                 //kDebug(7004) << "file changed:" << tmp->name();
 
@@ -1669,7 +1740,7 @@ void KDirListerCache::deleteUnmarkedItems( const QList<KDirLister *>& listers, K
     while (kit.hasNext()) {
         const KFileItem& item = kit.next();
         if (!item.isMarked()) {
-            //kDebug() << "deleted:" << item.name() << &item;
+            //kDebug(7004) << "deleted:" << item.name() << &item;
             deletedItems.append(item);
             kit.remove();
         }
@@ -1765,19 +1836,22 @@ void KDirListerCache::deleteDir( const KUrl& dirUrl )
 // delayed updating of files, FAM is flooding us with events
 void KDirListerCache::processPendingUpdates()
 {
-    foreach(const QString& file, pendingUpdates) {
+    QSet<KDirLister *> listers;
+    foreach(const QString& file, pendingUpdates) { // always a local path
         kDebug(7004) << file;
         KUrl u(file);
         KFileItem *item = findByUrl( 0, u ); // search all items
         if ( item ) {
             // we need to refresh the item, because e.g. the permissions can have changed.
-            aboutToRefreshItem( *item );
             KFileItem oldItem = *item;
             item->refresh();
-            emitRefreshItem( oldItem, *item );
+            listers |= emitRefreshItem( oldItem, *item );
         }
     }
     pendingUpdates.clear();
+    Q_FOREACH(KDirLister * kdl, listers) {
+        kdl->d->emitItems();
+    }
 }
 
 #ifndef NDEBUG
@@ -1794,6 +1868,7 @@ void KDirListerCache::printDebug()
                      << QString("with %1 items.").arg(itu.value()->lstItems.count());
     }
 
+    QList<KDirLister*> listersWithoutJob;
     kDebug(7004) << "Directory data:";
     DirectoryDataHash::const_iterator dit = directoryData.constBegin();
     for ( ; dit != directoryData.constEnd(); ++dit )
@@ -1805,6 +1880,10 @@ void KDirListerCache::printDebug()
         foreach ( KDirLister* listit, (*dit).listersCurrentlyListing ) {
             if (listit->d->m_cachedItemsJob) {
                 kDebug(7004) << "  Lister" << listit << "has CachedItemsJob" << listit->d->m_cachedItemsJob;
+            } else if (KIO::ListJob* listJob = jobForUrl(dit.key())) {
+                kDebug(7004) << "  Lister" << listit << "has ListJob" << listJob;
+            } else {
+                listersWithoutJob.append(listit);
             }
         }
 
@@ -1827,6 +1906,11 @@ void KDirListerCache::printDebug()
                      << (!dirItem->rootItem.isNull() ? dirItem->rootItem.url().prettyUrl() : QString("NULL") )
                      << "with" << dirItem->lstItems.count() << "items.";
     }
+
+    // Abort on listers without jobs -after- showing the full dump. Easier debugging.
+    Q_FOREACH(KDirLister* listit, listersWithoutJob) {
+        kFatal() << "HUH? Lister" << listit << "is supposed to be listing, but has no job!";
+    }
 }
 #endif
 
@@ -2182,6 +2266,8 @@ void KDirLister::Private::addNewItem(const KUrl& directoryUrl, const KFileItem &
     if (!isItemVisible(item))
         return; // No reason to continue... bailing out here prevents a mimetype scan.
 
+    //kDebug(7004) << "in" << directoryUrl << "item:" << item.url();
+
   if ( m_parent->matchesMimeFilter( item ) )
   {
     if ( !lstNewItems )
@@ -2214,13 +2300,10 @@ void KDirLister::Private::addNewItems(const KUrl& directoryUrl, const KFileItemL
     addNewItem(directoryUrl, *kit);
 }
 
-void KDirLister::Private::aboutToRefreshItem( const KFileItem &item )
-{
-    refreshItemWasFiltered = !isItemVisible(item) || !m_parent->matchesMimeFilter(item);
-}
-
 void KDirLister::Private::addRefreshItem(const KUrl& directoryUrl, const KFileItem& oldItem, const KFileItem& item)
 {
+    const bool refreshItemWasFiltered = !isItemVisible(oldItem) ||
+                                        !m_parent->matchesMimeFilter(oldItem);
   if (isItemVisible(item) && m_parent->matchesMimeFilter(item)) {
     if ( refreshItemWasFiltered )
     {
