@@ -25,6 +25,7 @@
 #include <klocale.h>
 #include <kglobal.h>
 #include <kio/copyjob.h>
+#include <kio/fileundomanager.h>
 #include <kio/jobuidelegate.h>
 #include <kurl.h>
 #include <kdebug.h>
@@ -35,6 +36,10 @@
 #include <sys/types.h>
 #include <dirent.h>
 
+#ifdef Q_WS_WIN
+#include <windows.h>
+#endif
+
 class KDirModelNode;
 class KDirModelDirNode;
 
@@ -65,7 +70,7 @@ public:
     // linear search
     int rowNumber() const; // O(n)
     QIcon preview() const { return m_preview; }
-    void addPreview( const QPixmap& pix ) { m_preview.addPixmap(pix); }
+    void setPreview( const QPixmap& pix ) {  m_preview = QIcon(); m_preview.addPixmap(pix); }
     void setPreview( const QIcon& icn ) { m_preview = icn; }
 
 private:
@@ -361,6 +366,9 @@ void KDirModelPrivate::_k_slotNewItems(const KUrl& directoryUrl, const KFileItem
         kError(7008) << "Items emitted in directory" << directoryUrl
                      << "but that directory isn't in KDirModel!"
                      << "Root directory:" << urlForNode(m_rootNode);
+        Q_FOREACH(const KFileItem& item, items) {
+            kDebug() << "Item:" << item.url();
+        }
 #ifndef NDEBUG
         dump();
 #endif
@@ -475,8 +483,8 @@ void KDirModelPrivate::_k_slotDeleteItems(const KFileItemList& items)
             node = nodeForUrl(url);
             if (!node) {
                 kWarning(7008) << "No node found for item that was just removed:" << url;
+                continue;
             }
-            Q_ASSERT(node);
         }
         rowNumbers.setBit(node->rowNumber(), 1); // O(n)
         removeFromNodeHash(node, url);
@@ -689,9 +697,26 @@ QVariant KDirModel::data( const QModelIndex & index, int role ) const
                 if (count == ChildCountUnknown && item.isReadable()) {
                     const QString path = item.localPath();
                     if (!path.isEmpty()) {
-#if 0 // slow
-                        QDir dir(path);
-                        count = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System).count();
+//                        slow
+//                        QDir dir(path);
+//                        count = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot|QDir::System).count();
+#ifdef Q_WS_WIN
+                        QString s = path + QLatin1String( "\\*.*" );
+                        s.replace('/', '\\');
+                        count = 0;
+                        WIN32_FIND_DATA findData;
+                        HANDLE hFile = FindFirstFile( (LPWSTR)s.utf16(), &findData );
+                        if( hFile != INVALID_HANDLE_VALUE ) {
+                            do {
+                                if (!( findData.cFileName[0] == '.' &&
+                                       findData.cFileName[1] == '\0' ) &&
+                                    !( findData.cFileName[0] == '.' &&
+                                       findData.cFileName[1] == '.' &&
+                                       findData.cFileName[2] == '\0' ) )
+                                    ++count;
+                            } while( FindNextFile( hFile, &findData ) != 0 );
+                            FindClose( hFile );
+                        }
 #else
                         DIR* dir = ::opendir(QFile::encodeName(path));
                         if (dir) {
@@ -741,7 +766,8 @@ bool KDirModel::setData( const QModelIndex & index, const QVariant & value, int
             newurl.setPath(newurl.directory(KUrl::AppendTrailingSlash) + newName);
             KIO::Job * job = KIO::moveAs(item.url(), newurl, newurl.isLocalFile() ? KIO::HideProgressInfo : KIO::DefaultFlags);
             job->ui()->setAutoErrorHandlingEnabled(true);
-            // TODO undo handling
+            // undo handling
+            KIO::FileUndoManager::self()->recordJob( KIO::FileUndoManager::Rename, item.url(), newurl, job );
             return true;
         }
         break;
@@ -756,7 +782,7 @@ bool KDirModel::setData( const QModelIndex & index, const QVariant & value, int
                 const QIcon icon(qvariant_cast<QIcon>(value));
                 node->setPreview(icon);
             } else if (value.type() == QVariant::Pixmap) {
-                node->addPreview(qvariant_cast<QPixmap>(value));
+                node->setPreview(qvariant_cast<QPixmap>(value));
             }
             emit dataChanged(index, index);
             return true;
@@ -804,6 +830,10 @@ static bool lessThan(const KUrl &left, const KUrl &right)
     return left.url().compare(right.url()) < 0;
 }
 
+void KDirModel::requestSequenceIcon(const QModelIndex& index, int sequenceIndex) {
+    emit needSequenceIcon(index, sequenceIndex);
+}
+
 KUrl::List KDirModel::simplifiedUrlList(const KUrl::List &urls)
 {
     if (!urls.count()) {
