@@ -45,6 +45,7 @@
 #include <kcolorscheme.h>
 #include <kglobalsettings.h>
 #include <ktextedit.h>
+#include <kstringhandler.h>
 
 #include "delegateanimationhandler_p.h"
 
@@ -105,11 +106,14 @@ class KFileItemDelegate::Private
                            const QRect &textBoundingRect) const;
         KIO::AnimationState *animationState(const QStyleOptionViewItemV4 &option, const QModelIndex &index,
                                             const QAbstractItemView *view) const;
+        void restartAnimation(KIO::AnimationState* state);
         QPixmap applyHoverEffect(const QPixmap &icon) const;
         QPixmap transition(const QPixmap &from, const QPixmap &to, qreal amount) const;
         void initStyleOption(QStyleOptionViewItemV4 *option, const QModelIndex &index) const;
         void drawFocusRect(QPainter *painter, const QStyleOptionViewItemV4 &option, const QRect &rect) const;
 
+        void gotNewIcon(const QModelIndex& index);
+
     public:
         KFileItemDelegate::InformationList informationList;
         QColor shadowColor;
@@ -117,6 +121,7 @@ class KFileItemDelegate::Private
         qreal shadowBlur;
         QSize maximumSize;
         bool showToolTipWhenElided;
+        QTextOption::WrapMode wrapMode;
 
     private:
         KFileItemDelegate * const q;
@@ -129,7 +134,7 @@ class KFileItemDelegate::Private
 
 KFileItemDelegate::Private::Private(KFileItemDelegate *parent)
      : shadowColor(Qt::transparent), shadowOffset(1, 1), shadowBlur(2), maximumSize(0, 0),
-       showToolTipWhenElided(true), q(parent),
+       showToolTipWhenElided(true), wrapMode( QTextOption::WrapAtWordBoundaryOrAnywhere ), q(parent),
        animationHandler(new KIO::DelegateAnimationHandler(parent)), activeMargins(0)
 {
 }
@@ -417,8 +422,7 @@ void KFileItemDelegate::Private::setLayoutOptions(QTextLayout &layout, const QSt
     QTextOption textoption;
     textoption.setTextDirection(option.direction);
     textoption.setAlignment(QStyle::visualAlignment(option.direction, option.displayAlignment));
-    textoption.setWrapMode((option.features & QStyleOptionViewItemV2::WrapText) ?
-                           QTextOption::WrapAtWordBoundaryOrAnywhere : QTextOption::NoWrap);
+    textoption.setWrapMode((option.features & QStyleOptionViewItemV2::WrapText) ? wrapMode : QTextOption::NoWrap);
 
     layout.setFont(option.font);
     layout.setTextOption(textoption);
@@ -538,6 +542,13 @@ QPixmap KFileItemDelegate::Private::applyHoverEffect(const QPixmap &icon) const
     return icon;
 }
 
+void KFileItemDelegate::Private::gotNewIcon(const QModelIndex& index) {
+    animationHandler->gotNewIcon(index);
+}
+
+void KFileItemDelegate::Private::restartAnimation(KIO::AnimationState* state) {
+    animationHandler->restartAnimation(state);
+}
 
 KIO::AnimationState *KFileItemDelegate::Private::animationState(const QStyleOptionViewItemV4 &option,
                                                                 const QModelIndex &index,
@@ -561,7 +572,7 @@ QPixmap KFileItemDelegate::Private::transition(const QPixmap &from, const QPixma
     if (value == 0)
         return from;
 
-    if (value == 1)
+    if (value == 0xff)
         return to;
 
     QColor color;
@@ -895,8 +906,10 @@ QString KFileItemDelegate::Private::display(const QModelIndex &index) const
         {
             if (index.column() == KDirModel::Size)
                 return itemSize(index, fileItem(index));
-            else
-                return replaceNewlines(value.toString());
+            else {
+                const QString text = replaceNewlines(value.toString());
+                return KStringHandler::preProcessWrap(text);
+            }
         }
 
         case QVariant::Double:
@@ -992,6 +1005,23 @@ bool KFileItemDelegate::showToolTipWhenElided() const
     return d->showToolTipWhenElided;
 }
 
+void KFileItemDelegate::setWrapMode(QTextOption::WrapMode wrapMode)
+{
+    d->wrapMode = wrapMode;
+}
+
+QTextOption::WrapMode KFileItemDelegate::wrapMode() const
+{
+    return d->wrapMode;
+}
+
+QRect KFileItemDelegate::iconRect(const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+    QStyleOptionViewItemV4 opt(option);
+    d->initStyleOption(&opt, index);
+    return QRect(d->iconPosition(opt), opt.icon.actualSize(opt.decorationSize));
+}
+
 
 QIcon KFileItemDelegate::Private::decoration(const QStyleOptionViewItemV4 &option, const QModelIndex &index) const
 {
@@ -1127,6 +1157,11 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     QStyleOptionViewItemV4 opt(option);
     d->initStyleOption(&opt, index);
 
+    if (!(option.state & QStyle::State_Enabled))
+    {
+        opt.palette.setCurrentColorGroup(QPalette::Disabled);
+    }
+
     // Unset the mouse over bit if we're not drawing the first column
     if (index.column() > 0)
         opt.state &= ~QStyle::State_MouseOver;
@@ -1142,12 +1177,10 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     KIO::CachedRendering *cache = 0;
     qreal progress = ((opt.state & QStyle::State_MouseOver) &&
                 index.column() == KDirModel::Name) ? 1.0 : 0.0;
-
     if (state)
     {
         cache    = state->cachedRendering();
         progress = state->hoverProgress();
-
         // Clear the mouse over bit temporarily
         opt.state &= ~QStyle::State_MouseOver;
 
@@ -1156,15 +1189,39 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
         {
             if (cache->checkValidity(opt.state) && cache->regular.size() == opt.rect.size())
             {
-                const QPixmap pixmap = d->transition(cache->regular, cache->hover, progress);
+                QPixmap pixmap = d->transition(cache->regular, cache->hover, progress);
+
+                if (state->cachedRenderingFadeFrom() && state->fadeProgress() != 1.0)
+                {
+                    // Apply icon fading animation
+                    KIO::CachedRendering* fadeFromCache = state->cachedRenderingFadeFrom();
+                    const QPixmap fadeFromPixmap = d->transition(fadeFromCache->regular, fadeFromCache->hover, progress);
+
+                    pixmap = d->transition(fadeFromPixmap, pixmap, state->fadeProgress());
+                }
                 painter->drawPixmap(option.rect.topLeft(), pixmap);
                 return;
             }
 
+            if (!cache->checkValidity(opt.state))
+            {
+                if ((KGlobalSettings::graphicEffectsLevel() & KGlobalSettings::SimpleAnimationEffects))
+                {
+                    // Fade over from the old icon to the new one
+                    // Only start a new fade if the previous one is ready
+                    // Else we may start racing when checkValidity() always returns false
+                    if (state->fadeProgress() == 1)
+                        state->setCachedRenderingFadeFrom(state->takeCachedRendering());
+                }
+                d->gotNewIcon(index);
+            }
             // If it wasn't valid, delete it
             state->setCachedRendering(0);
-            delete cache;
-            cache = 0;
+        }
+        else
+        {
+            // The cache may have been discarded, but the animation handler still needs to know about new icons
+            d->gotNewIcon(index);
         }
     }
 
@@ -1198,9 +1255,9 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
     // We don't create a new cache for a fully hovered item, since we don't
     // know yet if a hover out animation will be run.
     // ========================================================================
-    if (state && progress < 1)
+    if (state && (state->hoverProgress() < 1 || state->fadeProgress() < 1))
     {
-        cache = new KIO::CachedRendering(opt.state, option.rect.size());
+        cache = new KIO::CachedRendering(opt.state, option.rect.size(), index);
 
         QPainter p;
         p.begin(&cache->regular);
@@ -1228,7 +1285,19 @@ void KFileItemDelegate::paint(QPainter *painter, const QStyleOptionViewItem &opt
 
         state->setCachedRendering(cache);
 
-        const QPixmap pixmap = d->transition(cache->regular, cache->hover, progress);
+        QPixmap pixmap = d->transition(cache->regular, cache->hover, progress);
+
+        if (state->cachedRenderingFadeFrom() && state->fadeProgress() == 0)
+        {
+            // Apply icon fading animation
+            KIO::CachedRendering* fadeFromCache = state->cachedRenderingFadeFrom();
+            const QPixmap fadeFromPixmap = d->transition(fadeFromCache->regular, fadeFromCache->hover, progress);
+
+            pixmap = d->transition(fadeFromPixmap, pixmap, state->fadeProgress());
+
+            d->restartAnimation(state);
+        }
+
         painter->drawPixmap(option.rect.topLeft(), pixmap);
         return;
     }
@@ -1266,6 +1335,7 @@ QWidget *KFileItemDelegate::createEditor(QWidget *parent, const QStyleOptionView
     edit->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     edit->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     edit->setAlignment(opt.displayAlignment);
+    edit->setEnabled(false); //Disable the text-edit to mark it as un-initialized
     return edit;
 }
 
@@ -1287,6 +1357,14 @@ void KFileItemDelegate::setEditorData(QWidget *editor, const QModelIndex &index)
     KTextEdit *textedit = qobject_cast<KTextEdit*>(editor);
     Q_ASSERT(textedit != 0);
 
+    //Do not update existing text that the user may already have edited.
+    //The models will call setEditorData(..) whenever the icon has changed,
+    //and this makes the editing work correctly despite that.
+    if(textedit->isEnabled()) {
+        return;
+    }
+    textedit->setEnabled(true); //Enable the text-edit to mark it as initialized
+
     const QVariant value = index.data(Qt::EditRole);
     const QString text = value.toString();
     textedit->insertPlainText(text);
@@ -1324,10 +1402,24 @@ void KFileItemDelegate::updateEditorGeometry(QWidget *editor, const QStyleOption
     QRect r = d->labelRectangle(opt);
 
     // Use the full available width for the editor when maximumSize is set
-    if (d->verticalLayout(option) && !d->maximumSize.isEmpty()) {
-        int diff = qMax(r.width(), d->maximumSize.width()) - r.width();
-        if (diff > 1)
-            r.adjust(-(diff / 2), 0, diff / 2, 0);
+    if (!d->maximumSize.isEmpty()) {
+        if (d->verticalLayout(option)) {
+            int diff = qMax(r.width(), d->maximumSize.width()) - r.width();
+            if (diff > 1) {
+                r.adjust(-(diff / 2), 0, diff / 2, 0);
+            }
+        }
+        else {
+            int diff = qMax(r.width(), d->maximumSize.width() - opt.decorationSize.width()) - r.width();
+            if (diff > 0) {
+                if (opt.decorationPosition == QStyleOptionViewItem::Left) {
+                    r.adjust(0, 0, diff, 0);
+                }
+                else {
+                    r.adjust(-diff, 0, 0, 0);
+                }
+            }
+        }
     }
 
     KTextEdit *textedit = qobject_cast<KTextEdit*>(editor);
