@@ -73,15 +73,14 @@ KTar::KTar( const QString& fileName, const QString & _mimetype )
 }
 
 KTar::KTar( QIODevice * dev )
-    : KArchive( dev ),d(new KTarPrivate(this))
+    : KArchive( dev ), d(new KTarPrivate(this))
 {
     Q_ASSERT( dev );
 }
 
-// Only called when the a filename was given
+// Only called when a filename was given
 bool KTar::createDevice( QIODevice::OpenMode mode )
 {
-    Q_UNUSED( mode );
     if ( d->mimetype.isEmpty() ) // Find out mimetype manually
     {
         if ( mode != QIODevice::WriteOnly && QFile::exists( fileName() ) )
@@ -102,6 +101,14 @@ bool KTar::createDevice( QIODevice::OpenMode mode )
         {
             d->mimetype = "application/x-bzip";
         }
+        else if ( d->mimetype == "application/x-lzma-compressed-tar" ) // that's a lzma compressed tar file, so ask for xz filter
+        {
+            d->mimetype = "application/x-lzma";
+        }
+        else if ( d->mimetype == "application/x-xz-compressed-tar" ) // that's a xz compressed tar file, so ask for xz filter
+        {
+            d->mimetype = "application/x-xz";
+        }
         else
         {
             // Something else. Check if it's not really gzip though (e.g. for old-style KOffice files)
@@ -116,6 +123,14 @@ bool KTar::createDevice( QIODevice::OpenMode mode )
                         d->mimetype = "application/x-gzip";
                     else if ( firstByte == 'B' && secondByte == 'Z' && thirdByte == 'h' )
                         d->mimetype = "application/x-bzip";
+                    else if ( firstByte == '\xFD' && secondByte == '7' && thirdByte == 'z' )
+                    {
+                        char fourthByte, fifthByte, sixthByte;
+                        if ( file.getChar(&fourthByte) && fourthByte == 'X' &&
+			     file.getChar(&fifthByte) && fifthByte == 'Z' &&
+			     file.getChar(&sixthByte) && sixthByte == 0x00 )
+                            d->mimetype = "application/x-xz";
+                    }
                     else if ( firstByte == 'P' && secondByte == 'K' && thirdByte == 3 )
                     {
                         char fourthByte;
@@ -128,7 +143,7 @@ bool KTar::createDevice( QIODevice::OpenMode mode )
         }
     }
 
-    if( d->mimetype == "application/x-tar" )
+    if (d->mimetype == "application/x-tar" || mode == QIODevice::WriteOnly)
     {
         return KArchive::createDevice( mode );
     }
@@ -348,7 +363,7 @@ bool KTar::openArchive( QIODevice::OpenMode mode ) {
     if ( !dev )
         return false;
 
-    // read dir infos
+    // read dir information
     char buffer[ 0x200 ];
     bool ende = false;
     do
@@ -443,8 +458,8 @@ bool KTar::openArchive( QIODevice::OpenMode mode ) {
                 }
 
                 // Skip contents + align bytes
-                int rest = size % 0x200;
-                int skip = size + (rest ? 0x200 - rest : 0);
+                qint64 rest = size % 0x200;
+                qint64 skip = size + (rest ? 0x200 - rest : 0);
                 //kDebug(7041) << "pos()=" << dev->pos() << "rest=" << rest << "skipping" << skip;
                 if (! dev->seek( dev->pos() + skip ) )
                     kWarning(7041) << "skipping" << skip << "failed";
@@ -483,10 +498,11 @@ bool KTar::openArchive( QIODevice::OpenMode mode ) {
 /*
  * Writes back the changes of the temporary file
  * to the original file.
- * Must only be called if in QIODevice::WriteOnly mode
+ * Must only be called if in write mode, not in read mode
  */
-bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName ) {
-    if ( ! tmpFile )
+bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName )
+{
+    if ( !tmpFile )
         return true;
 
     kDebug(7041) << "Write temporary file to compressed file";
@@ -494,7 +510,9 @@ bool KTar::KTarPrivate::writeBackTempFile( const QString & fileName ) {
 
     bool forced = false;
     if( "application/x-gzip" == mimetype
-        || "application/x-bzip" == mimetype)
+        || "application/x-bzip" == mimetype
+	|| "application/x-lzma" == mimetype
+	|| "application/x-xz" == mimetype)
         forced = true;
 
     // #### TODO this should use KSaveFile to avoid problems on disk full
@@ -537,7 +555,7 @@ bool KTar::closeArchive() {
     // If we are in write mode and had created
     // a temporary tar file, we have to write
     // back the changes to the original file
-    if( mode() == QIODevice::WriteOnly) {
+    if( mode() & QIODevice::WriteOnly) {
         ok = d->writeBackTempFile( fileName() );
         delete d->tmpFile;
         d->tmpFile = 0;
