@@ -17,26 +17,6 @@
    Boston, MA 02110-1301, USA.
 */
 
-/*
-	This class implements a kioslave to access ZIP files from KDE.
-    you can use it in QIODevice::ReadOnly or in QIODevice::WriteOnly mode, and it
-    behaves just as expected (i hope ;-) ).
-    It can also be used in QIODevice::ReadWrite mode, in this case one can
-    append files to an existing zip archive. when you append new files, which
-    are not yet in the zip, it works as expected, they are appended at the end.
-    when you append a file, which is already in the file, the reference to the
-    old file is dropped and the new one is added to the zip. but the
-    old data from the file itself is not deleted, it is still in the
-    zipfile. so when you want to have a small and garbagefree zipfile,
-    just read the contents of the appended zipfile and write it to a new one
-    in QIODevice::WriteOnly mode. especially take care of this, when you don't want
-    to leak information of how intermediate versions of files in the zip
-    were looking.
-    For more information on the zip fileformat go to
-    http://www.pkware.com/support/appnote.html .
-
-*/
-
 #include "kzip.h"
 #include "kfilterdev.h"
 #include "klimitediodevice.h"
@@ -327,7 +307,7 @@ public:
     // where new data can be appended. after openarchive it points to 0, when in
     // writeonly mode, or it points to the beginning of the central directory.
     // each call to writefile updates this value.
-    unsigned int            m_offset;
+    quint64                 m_offset;
 };
 
 KZip::KZip( const QString& fileName )
@@ -361,7 +341,7 @@ bool KZip::openArchive( QIODevice::OpenMode mode )
     // Check that it's a valid ZIP file
     // KArchive::open() opened the underlying device already.
 
-    uint offset = 0; // holds offset, where we read
+    quint64 offset = 0; // holds offset, where we read
     int n;
 
     // contains information gathered from the local file headers
@@ -906,8 +886,8 @@ bool KZip::closeArchive()
         buffer[ 26 ] = char(mysize >> 16);
         buffer[ 27 ] = char(mysize >> 24);
 
-        buffer[ 28 ] = char(it.value()->path().length()); // fileName length
-        buffer[ 29 ] = char(it.value()->path().length() >> 8);
+        buffer[ 28 ] = char(path.length()); // fileName length
+        buffer[ 29 ] = char(path.length() >> 8);
 
 	buffer[ 30 ] = char(extra_field_len);
 	buffer[ 31 ] = char(extra_field_len >> 8);
@@ -995,9 +975,15 @@ bool KZip::closeArchive()
     return true;
 }
 
-bool KZip::doWriteDir( const QString&, const QString&, const QString&,
-                       mode_t, time_t, time_t, time_t ) {
-        return true;
+bool KZip::doWriteDir( const QString &name, const QString &user, const QString &group,
+                       mode_t perm, time_t atime, time_t mtime, time_t ctime ) {
+    // Zip files have no explicit directories, they are implicitly created during extraction time
+    // when file entries have paths in them.
+    // However, to support empty directories, we must create a dummy file entry which ends with '/'.
+    QString dirName = name;
+    if (!name.endsWith("/"))
+        dirName = dirName.append('/');
+    return writeFile(dirName, user, group, 0, 0, perm, atime, mtime, ctime);
 }
 
 bool KZip::doPrepareWriting(const QString &name, const QString &user,
@@ -1189,9 +1175,10 @@ bool KZip::doFinishWriting( qint64 size )
     if ( d->m_extraField == ModificationTime )
         extra_field_len = 17;	// value also used in finishWriting()
 
+    const QByteArray encodedName = QFile::encodeName(d->m_currentFile->path());
     int csize = device()->pos() -
         d->m_currentFile->headerStart() - 30 -
-		d->m_currentFile->path().length() - extra_field_len;
+		encodedName.length() - extra_field_len;
     d->m_currentFile->setCompressedSize(csize);
     //kDebug(7040) << "usize: " << d->m_currentFile->size();
     //kDebug(7040) << "csize: " << d->m_currentFile->compressedSize();
