@@ -20,7 +20,6 @@
 #include "qtest_kde.h"
 
 #include "jobtest.h"
-#include <kprotocolmanager.h>
 
 #include <config.h>
 
@@ -54,11 +53,6 @@ QTEST_KDEMAIN( JobTest, NoGUI )
 
 // The code comes partly from kdebase/kioslave/trash/testtrash.cpp
 
-static QString homeTmpDir()
-{
-    return QFile::decodeName( getenv( "KDEHOME" ) ) + "/jobtest/";
-}
-
 static QString otherTmpDir()
 {
 #ifdef Q_WS_WIN
@@ -93,14 +87,9 @@ void JobTest::initTestCase()
 
     // Start with a clean base dir
     cleanupTestCase();
-    QDir dir; // TT: why not a static method?
-    if ( !QFile::exists( homeTmpDir() ) ) {
-        bool ok = dir.mkdir( homeTmpDir() );
-        if ( !ok )
-            kFatal() << "Couldn't create " << homeTmpDir();
-    }
+    homeTmpDir(); // create it
     if ( !QFile::exists( otherTmpDir() ) ) {
-        bool ok = dir.mkdir( otherTmpDir() );
+        bool ok = QDir().mkdir( otherTmpDir() );
         if ( !ok )
             kFatal() << "Couldn't create " << otherTmpDir();
     }
@@ -114,12 +103,10 @@ void JobTest::initTestCase()
     }
 #endif
 
+    qRegisterMetaType<KJob*>("KJob*");
     qRegisterMetaType<KIO::Job*>("KIO::Job*");
     qRegisterMetaType<KUrl>("KUrl");
     qRegisterMetaType<time_t>("time_t");
-
-    // Wrong ksycoca, if this is true in this branch...
-    QVERIFY(!KProtocolManager::canDeleteRecursive(KUrl("file:/")));
 }
 
 static void delDir(const QString& pathOrUrl) {
@@ -148,7 +135,7 @@ void JobTest::enterLoop()
     eventLoop.exec(QEventLoop::ExcludeUserInputEvents);
 }
 
-void JobTest::get()
+void JobTest::storedGet()
 {
     kDebug() ;
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -157,6 +144,8 @@ void JobTest::get()
     m_result = -1;
 
     KIO::StoredTransferJob* job = KIO::storedGet( u, KIO::NoReload, KIO::HideProgressInfo );
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*, unsigned long)));
+    QVERIFY(spyPercent.isValid());
     job->setUiDelegate( 0 );
     connect( job, SIGNAL( result( KJob* ) ),
             this, SLOT( slotGetResult( KJob* ) ) );
@@ -164,6 +153,7 @@ void JobTest::get()
     QCOMPARE( m_result, 0 ); // no error
     QCOMPARE( m_data, QByteArray("Hello\0world", 11) );
     QCOMPARE( m_data.size(), 11 );
+    QVERIFY(!spyPercent.isEmpty());
 }
 
 void JobTest::slotGetResult( KJob* job )
@@ -220,6 +210,27 @@ void JobTest::slotResult( KJob* job )
     emit exitLoop();
 }
 
+void JobTest::storedPut()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    KUrl u(filePath);
+    QByteArray putData = "This is the put data";
+    KIO::TransferJob* job = KIO::storedPut( putData, u, 0600, KIO::Overwrite | KIO::HideProgressInfo );
+    QSignalSpy spyPercent(job, SIGNAL(percent(KJob*, unsigned long)));
+    QVERIFY(spyPercent.isValid());
+    QDateTime mtime = QDateTime::currentDateTime().addSecs( -30 ); // 30 seconds ago
+    mtime.setTime_t(mtime.toTime_t()); // hack for losing the milliseconds
+    job->setModificationTime(mtime);
+    job->setUiDelegate( 0 );
+    QVERIFY(job->exec());
+    QFileInfo fileInfo(filePath);
+    QVERIFY(fileInfo.exists());
+    QCOMPARE(fileInfo.size(), (long long)putData.size());
+    QCOMPARE((int)fileInfo.permissions(), (int)(QFile::ReadOwner | QFile::WriteOwner | QFile::ReadUser | QFile::WriteUser ));
+    QCOMPARE(fileInfo.lastModified(), mtime);
+    QVERIFY(!spyPercent.isEmpty());
+}
+
 ////
 
 void JobTest::copyLocalFile( const QString& src, const QString& dest )
@@ -517,8 +528,7 @@ void JobTest::moveFileNoPermissions()
     // a direct rename(2) was used, or a full copy+del. In the first case
     // there is no destination file created, but in the second case the
     // destination file remains.
-    // In fact we assume /home is a separate partition, in this test, so:
-    QVERIFY( QFile::exists( dest ) );
+    //QVERIFY( QFile::exists( dest ) );
     QVERIFY( QFile::exists( src ) );
 #endif
 }
@@ -549,7 +559,7 @@ void JobTest::moveDirectoryNoPermissions()
     bool ok = KIO::NetAccess::synchronousRun( job, 0, 0, 0, &metaData );
     QVERIFY( !ok );
     QCOMPARE( KIO::NetAccess::lastError(), (int)KIO::ERR_ACCESS_DENIED );
-    QVERIFY( QFile::exists( dest ) ); // see moveFileNoPermissions
+    //QVERIFY( QFile::exists( dest ) ); // see moveFileNoPermissions
     QVERIFY( QFile::exists( src ) );
 #endif
 }
@@ -595,6 +605,23 @@ void JobTest::listRecursive()
     QCOMPARE( joinedNames.toLatin1(), ref_names );
 }
 
+void JobTest::listFile()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile( filePath );
+    KIO::ListJob* job = KIO::listDir(KUrl(filePath), KIO::HideProgressInfo);
+    job->setUiDelegate( 0 );
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), static_cast<int>(KIO::ERR_IS_FILE));
+
+    // And list something that doesn't exist
+    const QString path = homeTmpDir() + "fileFromHomeDoesNotExist";
+    job = KIO::listDir(KUrl(path), KIO::HideProgressInfo);
+    job->setUiDelegate( 0 );
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), static_cast<int>(KIO::ERR_DOES_NOT_EXIST));
+}
+
 void JobTest::killJob()
 {
     const QString src = homeTmpDir();
@@ -1092,7 +1119,7 @@ void JobTest::copyFileToSystem( bool resolve_local_urls )
 
 void JobTest::getInvalidUrl()
 {
-    KUrl url("file://\"\"");
+    KUrl url("http://strange<hostname>/");
     QVERIFY(!url.isValid());
 
     KIO::SimpleJob* job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
@@ -1277,6 +1304,26 @@ void JobTest::deleteManyFilesTogether()
     deleteManyFilesTogether(false);
 }
 
+void JobTest::rmdirEmpty()
+{
+    const QString dir = homeTmpDir() + "dir";
+    QDir().mkdir(dir);
+    QVERIFY(QFile::exists(dir));
+    KIO::Job* job = KIO::rmdir(dir);
+    QVERIFY(job->exec());
+    QVERIFY(!QFile::exists(dir));
+}
+
+void JobTest::rmdirNotEmpty()
+{
+    const QString dir = homeTmpDir() + "dir";
+    createTestDirectory(dir);
+    createTestDirectory(dir + "/subdir");
+    KIO::Job* job = KIO::rmdir(dir);
+    QVERIFY(!job->exec());
+    QVERIFY(QFile::exists(dir));
+}
+
 void JobTest::stat()
 {
 #if 1
@@ -1305,6 +1352,17 @@ void JobTest::stat()
 #endif
 }
 
+void JobTest::mostLocalUrl()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile( filePath );
+    KIO::StatJob* job = KIO::mostLocalUrl(filePath, KIO::HideProgressInfo);
+    QVERIFY(job);
+    bool ok = job->exec();
+    QVERIFY(ok);
+    QCOMPARE(job->mostLocalUrl().toLocalFile(), filePath);
+}
+
 void JobTest::mimeType()
 {
 #if 1
