@@ -111,6 +111,7 @@ void KDirModelTest::fillModel(bool reload, bool expectAllIndexes)
 {
     if (!m_dirModel)
         m_dirModel = new KDirModel;
+    m_dirModel->dirLister()->setAutoErrorHandlingEnabled(false, 0);
     const QString path = m_tempDir->name();
     KDirLister* dirLister = m_dirModel->dirLister();
     dirLister->openUrl(KUrl(path), reload ? KDirLister::Reload : KDirLister::NoFlags);
@@ -417,6 +418,12 @@ void KDirModelTest::testRenameFile()
     // check renaming happened
     QCOMPARE( m_dirModel->itemForIndex( m_secondFileIndex ).url().url(), newUrl.url() );
 
+    // check that KDirLister::cachedItemForUrl won't give a bad name if copying that item (#195385)
+    KFileItem cachedItem = KDirLister::cachedItemForUrl(newUrl);
+    Q_ASSERT(!cachedItem.isNull());
+    QCOMPARE(cachedItem.name(), QString("toplevelfile_2_renamed"));
+    QCOMPARE(cachedItem.entry().stringValue(KIO::UDSEntry::UDS_NAME), QString("toplevelfile_2_renamed"));
+
     // Put things back to normal
     job = KIO::rename(newUrl, url, KIO::HideProgressInfo);
     ok = job->exec();
@@ -560,12 +567,16 @@ void KDirModelTest::testRenameDirectory() // #172945, #174703, (and #180156)
 
 void KDirModelTest::testRenameDirectoryInCache() // #188807
 {
+    // Ensure the stuff is in cache.
+    fillModel(true);
+    const QString path = m_tempDir->name();
+    QVERIFY(!m_dirModel->dirLister()->findByUrl(path).isNull());
+
     // No more dirmodel nor dirlister.
     delete m_dirModel;
     m_dirModel = 0;
 
     // Now let's rename a directory that is in KDirListerCache
-    const QString path = m_tempDir->name();
     const KUrl url(path);
     KUrl newUrl(path);
     newUrl.adjustPath(KUrl::RemoveTrailingSlash);
@@ -580,7 +591,16 @@ void KDirModelTest::testRenameDirectoryInCache() // #188807
     ok = job->exec();
     QVERIFY(ok);
 
+    // KDirNotify emits FileRenamed for each rename() above, which in turn
+    // re-lists the directory. We need to wait for both signals to be emitted
+    // otherwise the dirlister will not be in the state we expect.
+    QTest::qWait(200);
+
     fillModel(true);
+
+    QVERIFY(m_dirIndex.isValid());
+    KFileItem rootItem = m_dirModel->dirLister()->findByUrl(path);
+    QVERIFY(!rootItem.isNull());
 }
 
 void KDirModelTest::testChmodDirectory() // #53397
@@ -607,6 +627,7 @@ void KDirModelTest::testChmodDirectory() // #53397
     // If we come here, then dataChanged() was emitted - all good.
     QCOMPARE(spyDataChanged.count(), 1);
     QModelIndex receivedIndex = spyDataChanged[0][0].value<QModelIndex>();
+    kDebug() << receivedIndex;
     QVERIFY(!receivedIndex.isValid());
 
     QCOMPARE(m_dirModel->itemForIndex(QModelIndex()).permissions(), newPerm);
@@ -640,6 +661,14 @@ void KDirModelTest::testExpandToUrl_data()
     QTest::newRow("subdir/subsubdir/testfile sync")
         << false << subsubdirfile << (QStringList()<<"subdir"<<subsubdir<<subsubdirfile);
 
+#ifndef Q_WS_WIN
+    // Expand a symlink to a directory (#219547)
+    const QString dirlink = m_tempDir->name() + "dirlink";
+    createTestSymlink(dirlink, "/");
+    QTest::newRow("dirlink")
+        << false << "dirlink/tmp" << (QStringList()<<"dirlink"<<"dirlink/tmp");
+#endif
+
     // Do a cold-cache test too, but nowadays it doesn't change anything anymore,
     // apart from testing different code paths inside KDirLister.
     QTest::newRow("subdir/subsubdir/testfile with reload")
@@ -921,8 +950,14 @@ void KDirModelTest::testSmb()
     KDirLister* dirLister = m_dirModel->dirLister();
     dirLister->openUrl(smbUrl, KDirLister::NoFlags);
     connect(dirLister, SIGNAL(completed()), this, SLOT(slotListingCompleted()));
+    connect(dirLister, SIGNAL(canceled()), this, SLOT(slotListingCompleted()));
+    QSignalSpy spyCanceled(dirLister, SIGNAL(canceled()));
     enterLoop(); // wait for completed signal
 
+    if (spyCanceled.count() > 0) {
+        QSKIP("smb:/ returns an error, probably no network available", SkipAll);
+    }
+
     QModelIndex index = m_dirModel->index(0, 0);
     if (index.isValid()) {
         QVERIFY(m_dirModel->canFetchMore(index));
@@ -963,7 +998,7 @@ void KDirModelTest::testDeleteFile()
                 &m_eventLoop, SLOT(quit()) );
 
     QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
-    Q_ASSERT(!fileIndex.isValid());
+    QVERIFY(!fileIndex.isValid());
 
     // Recreate the file, for consistency in the next tests
     // So the second part of this test is a "testCreateFile"
@@ -1103,6 +1138,8 @@ void KDirModelTest::testDeleteDirectory()
     connect( m_dirModel, SIGNAL(rowsRemoved(QModelIndex,int,int)),
              &m_eventLoop, SLOT(quit()) );
 
+    QSignalSpy spyDirWatchDeleted(KDirWatch::self(), SIGNAL(deleted(QString)));
+
     KIO::DeleteJob* job = KIO::del(url, KIO::HideProgressInfo);
     bool ok = job->exec();
     QVERIFY(ok);
@@ -1116,9 +1153,12 @@ void KDirModelTest::testDeleteDirectory()
                 &m_eventLoop, SLOT(quit()) );
 
     QModelIndex deletedDirIndex = m_dirModel->indexForUrl(path + "subdir/subsubdir");
-    Q_ASSERT(!deletedDirIndex.isValid());
+    QVERIFY(!deletedDirIndex.isValid());
     QModelIndex dirIndex = m_dirModel->indexForUrl(path + "subdir");
-    Q_ASSERT(dirIndex.isValid());
+    QVERIFY(dirIndex.isValid());
+
+    // TODO!!! Bug in KDirWatch?
+    // QCOMPARE(spyDirWatchDeleted.count(), 1);
 }
 
 void KDirModelTest::testDeleteCurrentDirectory()
@@ -1148,7 +1188,7 @@ void KDirModelTest::testDeleteCurrentDirectory()
                 &m_eventLoop, SLOT(quit()) );
 
     QModelIndex fileIndex = m_dirModel->indexForUrl(path + "toplevelfile_1");
-    Q_ASSERT(!fileIndex.isValid());
+    QVERIFY(!fileIndex.isValid());
 }
 
 // The old slow way. (this isn't QUrl's fault, I'm just using QUrl in order
