@@ -42,11 +42,24 @@
 #include <grp.h>
 #include <utime.h>
 #include <pwd.h>
+#include <stdlib.h>
 
 #if defined(HAVE_LIMITS_H)
 #include <limits.h>  // PATH_MAX
 #endif
 
+namespace KDEPrivate
+{
+
+struct CharArrayDeleter
+{
+    CharArrayDeleter(char *b) : buf(b) {}
+    ~CharArrayDeleter() { free(buf); }
+    char *buf;
+};
+
+}
+
 using namespace KIO;
 
 #define MAX_IPC_SIZE (1024*32)
@@ -333,6 +346,9 @@ void FileProtocol::listDir( const KUrl& url)
     //kDebug(7101) << "========= LIST " << url << "details=" << details << " =========";
     UDSEntry entry;
 
+#ifndef HAVE_DIRENT_D_TYPE
+    KDE_struct_stat st;
+#endif
     // Don't make this a QStringList. The locale file name we get here
     // should be passed intact to createUDSEntry to avoid problems with
     // files where QFile::encodeName(QFile::decodeName(a)) != a.
@@ -344,9 +360,20 @@ void FileProtocol::listDir( const KUrl& url)
         while ( ( ep = KDE_readdir( dp ) ) != 0 ) {
             entry.clear();
             entry.insert(KIO::UDSEntry::UDS_NAME, QFile::decodeName(ep->d_name));
+#ifdef HAVE_DIRENT_D_TYPE
             entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
                          (ep->d_type & DT_DIR) ? S_IFDIR : S_IFREG );
-            if (ep->d_type & DT_LNK) {
+            const bool isSymLink = (ep->d_type & DT_LNK);
+#else
+            // oops, no fast way, we need to stat (e.g. on Solaris)
+            if (KDE_lstat(ep->d_name, &st) == -1) {
+                continue; // how can stat fail?
+            }
+            entry.insert(KIO::UDSEntry::UDS_FILE_TYPE,
+                          (S_ISDIR(st.st_mode)) ? S_IFDIR : S_IFREG );
+            const bool isSymLink = S_ISLNK(st.st_mode);
+#endif
+            if (isSymLink) {
                 // for symlinks obey the UDSEntry contract and provide UDS_LINK_DEST
                 // even if we don't know the link dest (and DeleteJob doesn't care...)
                 entry.insert(KIO::UDSEntry::UDS_LINK_DEST, QLatin1String("Dummy Link Target"));
@@ -370,9 +397,14 @@ void FileProtocol::listDir( const KUrl& url)
            directories we keep as active directory. And
            as the slave runs in the background, it's hard
            to see for the user what the problem would be */
+#if !defined(PATH_MAX) && defined(__GLIBC__)
+        char *path_buffer = ::get_current_dir_name();
+        const KDEPrivate::CharArrayDeleter path_buffer_deleter(path_buffer);
+#else
         char path_buffer[PATH_MAX];
         path_buffer[0] = '\0';
         (void) getcwd(path_buffer, PATH_MAX - 1);
+#endif
         if ( chdir( _path.data() ) )  {
             if (errno == EACCES)
                 error(ERR_ACCESS_DENIED, path);
@@ -395,8 +427,14 @@ void FileProtocol::listDir( const KUrl& url)
 
         //kDebug(7101) << "============= COMPLETED LIST ============";
 
+#if !defined(PATH_MAX) && defined(__GLIBC__)
+        if (path_buffer)
+#else
         if (*path_buffer)
+#endif
+        {
             chdir(path_buffer);
+        }
     }
     finished();
 }
@@ -531,8 +569,10 @@ void FileProtocol::del(const KUrl& url, bool isfile)
        *****/
 
       kDebug( 7101 ) << "Deleting directory " << url.url();
-      if (!deleteRecursive(path))
-          return;
+      if (metaData(QLatin1String("recurse")) == QLatin1String("true")) {
+          if (!deleteRecursive(path))
+              return;
+      }
       if ( ::rmdir( _path.data() ) == -1 ) {
 	if ((errno == EACCES) || (errno == EPERM))
 	  error(KIO::ERR_ACCESS_DENIED, path);
