@@ -57,6 +57,7 @@
 #include <time.h>
 #endif
 
+#include <QtCore/QCoreApplication>
 #include <QtCore/QDir>
 #include <QtNetwork/QHostAddress>
 #include <QtNetwork/QTcpSocket>
@@ -142,6 +143,7 @@ using namespace KIO;
 
 extern "C" int KDE_EXPORT kdemain( int argc, char **argv )
 {
+  QCoreApplication app(argc, argv);
   KComponentData componentData( "kio_ftp", "kdelibs4" );
   ( void ) KGlobal::locale();
 
@@ -334,10 +336,7 @@ bool Ftp::ftpOpenConnection (LoginMode loginMode)
   m_initialPath.clear();
   m_currentPath.clear();
 
-  QString host = m_bUseProxy ? m_proxyURL.host() : m_host;
-  int port = m_bUseProxy ? m_proxyURL.port() : m_port;
-
-  if (!ftpOpenControlConnection(host, port) )
+  if (!ftpOpenControlConnection() )
     return false;          // error emitted by ftpOpenControlConnection
   infoMessage( i18n("Connected to host %1", m_host) );
 
@@ -359,6 +358,13 @@ bool Ftp::ftpOpenConnection (LoginMode loginMode)
  *
  * @return true on success.
  */
+bool Ftp::ftpOpenControlConnection()
+{
+  QString host = m_bUseProxy ? m_proxyURL.host() : m_host;
+  int port = m_bUseProxy ? m_proxyURL.port() : m_port;
+  return ftpOpenControlConnection(host, port);
+}
+
 bool Ftp::ftpOpenControlConnection( const QString &host, int port )
 {
   // implicitly close, then try to open a new connection ...
@@ -438,8 +444,13 @@ bool Ftp::ftpLogin()
   if ( m_port > 0 && m_port != DEFAULT_FTP_PORT )
       info.url.setPort( m_port );
   info.url.setUser( user );
+  if( user == FTP_LOGIN )
+      info.setExtraField("anonymous", true);
+  else
+      info.setExtraField("anonymous", false);
 
   QByteArray tempbuf;
+  QString lastServerResponse;
   int failedAuth = 0;
 
   do
@@ -457,7 +468,7 @@ bool Ftp::ftpLogin()
       {
         errorMsg = i18n("Message sent:\nLogin using username=%1 and "
                         "password=[hidden]\n\nServer replied:\n%2\n\n"
-                        , user, ftpResponse(0));
+                        , user, lastServerResponse);
       }
 
       if ( user != FTP_LOGIN )
@@ -468,7 +479,6 @@ bool Ftp::ftpLogin()
       info.commentLabel = i18n( "Site:" );
       info.comment = i18n("<b>%1</b>",  m_host );
       info.keepPassword = true; // Prompt the user for persistence as well.
-      info.readOnly = (!m_user.isEmpty() && m_user != FTP_LOGIN);
 
       bool disablePassDlg = config()->readEntry( "DisablePassDlg", false );
       if ( disablePassDlg || !openPasswordDialog( info, errorMsg ) )
@@ -478,8 +488,19 @@ bool Ftp::ftpLogin()
       }
       else
       {
-        user = info.username;
-        pass = info.password;
+		// User can decide go anonymous using checkbox
+        if( info.getExtraField( "anonymous" ).toBool() )
+        {
+	      user = FTP_LOGIN;
+	      pass = FTP_PASSWD;
+          m_user = FTP_LOGIN;
+          m_pass = FTP_PASSWD;
+		}
+		else
+		{
+		  user = info.username;
+          pass = info.password;
+	    }
       }
     }
 
@@ -504,7 +525,8 @@ bool Ftp::ftpLogin()
     // get back a "230" or "331".
     if ( !loggedIn && !needPass )
     {
-      kDebug(7102) << "Login failed: " << ftpResponse(0);
+      lastServerResponse = ftpResponse(0);
+      kDebug(7102) << "Login failed: " << lastServerResponse;
       ++failedAuth;
       continue;  // Well we failed, prompt the user please!!
     }
@@ -524,7 +546,16 @@ bool Ftp::ftpLogin()
         cacheAuthentication( info );
       failedAuth = -1;
     }
-
+    else
+    {
+        // some servers don't let you login anymore
+        // if you fail login once, so restart the connection here
+        lastServerResponse = ftpResponse(0);
+        if (!ftpOpenControlConnection())
+        {
+            return false;
+        }
+    }
   } while( ++failedAuth );
 
 
@@ -586,7 +617,7 @@ void Ftp::ftpAutoLoginMacro ()
 
   for(QStringList::const_iterator it = list.begin() ; it != list.end() ; ++it )
   {
-    if ( (*it).startsWith("init") )
+    if ( (*it).startsWith(QLatin1String("init")) )
     {
       const QStringList list2 = macro.split( '\\',QString::SkipEmptyParts);
       it = list2.begin();
@@ -596,7 +627,7 @@ void Ftp::ftpAutoLoginMacro ()
       {
         // TODO: Add support for arbitrary commands
         // besides simply changing directory!!
-        if ( (*it).startsWith( "cwd" ) )
+        if ( (*it).startsWith( QLatin1String("cwd") ) )
           ftpFolder( (*it).mid(4).trimmed(), false );
       }
 
@@ -1368,16 +1399,13 @@ void Ftp::listDir( const KUrl &url )
   {
     KUrl realURL;
     realURL.setProtocol( "ftp" );
-    if ( m_user != FTP_LOGIN )
-      realURL.setUser( m_user );
-    // We set the password, so that we don't ask for it if it was given
-    if ( m_pass != FTP_PASSWD )
-      realURL.setPass( m_pass );
+    realURL.setUser( m_user );
+    realURL.setPass( m_pass );
     realURL.setHost( m_host );
     if ( m_port > 0 && m_port != DEFAULT_FTP_PORT )
         realURL.setPort( m_port );
     if ( m_initialPath.isEmpty() )
-        m_initialPath = "/";
+        m_initialPath = '/';
     realURL.setPath( m_initialPath );
     kDebug(7102) << "REDIRECTION to " << realURL.prettyUrl();
     redirection( realURL );
@@ -1495,7 +1523,7 @@ bool Ftp::ftpReadDir(FtpEntry& de)
 
     // A special hack for "/dev". A listing may look like this:
     // crw-rw-rw-   1 root     root       1,   5 Jun 29  1997 zero
-    // So we just ignore the number in front of the ",". Ok, its a hack :-)
+    // So we just ignore the number in front of the ",". Ok, it is a hack :-)
     if ( strchr( p_size, ',' ) != 0L )
     {
       //kDebug(7102) << "Size contains a ',' -> reading size again (/dev hack)";
@@ -1610,6 +1638,7 @@ bool Ftp::ftpReadDir(FtpEntry& de)
       int currentMonth = tmptr->tm_mon;
       //kDebug(7102) << "Current time :" << asctime( tmptr );
       // Reset time fields
+      tmptr->tm_isdst = -1; // We do not anything about day saving time
       tmptr->tm_sec = 0;
       tmptr->tm_min = 0;
       tmptr->tm_hour = 0;
@@ -2207,8 +2236,7 @@ Ftp::StatusCode Ftp::ftpCopyPut(int& iError, int& iCopyFile, const QString &sCop
 {
   // check if source is ok ...
   KDE_struct_stat buff;
-  const QByteArray sSrc( QFile::encodeName(sCopyFile) );
-  bool bSrcExists = (KDE_stat( sSrc.data(), &buff ) != -1);
+  bool bSrcExists = (KDE::stat( sCopyFile, &buff ) != -1);
   if(bSrcExists)
   { if(S_ISDIR(buff.st_mode))
     {
@@ -2222,7 +2250,7 @@ Ftp::StatusCode Ftp::ftpCopyPut(int& iError, int& iCopyFile, const QString &sCop
     return statusClientError;
   }
 
-  iCopyFile = KDE_open( sSrc.data(), O_RDONLY );
+  iCopyFile = KDE::open( sCopyFile, O_RDONLY );
   if(iCopyFile == -1)
   {
     iError = ERR_CANNOT_OPEN_FOR_READING;
@@ -2244,8 +2272,7 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
 {
   // check if destination is ok ...
   KDE_struct_stat buff;
-  const QByteArray sDest( QFile::encodeName(sCopyFile) );
-  bool bDestExists = (KDE_stat( sDest.data(), &buff ) != -1);
+  const bool bDestExists = (KDE::stat( sCopyFile, &buff ) != -1);
   if(bDestExists)
   { if(S_ISDIR(buff.st_mode))
     {
@@ -2260,11 +2287,12 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
   }
 
   // do we have a ".part" file?
-  const QByteArray sPart = QFile::encodeName(sCopyFile + ".part");
+  const QString sPart = sCopyFile + QLatin1String(".part");
   bool bResume = false;
-  bool bPartExists = (KDE_stat( sPart.data(), &buff ) != -1);
-  bool bMarkPartial = config()->readEntry("MarkPartial", true);
-  if(bMarkPartial && bPartExists && buff.st_size > 0)
+  const bool bPartExists = (KDE::stat( sPart, &buff ) != -1);
+  const bool bMarkPartial = config()->readEntry("MarkPartial", true);
+  const QString dest = bMarkPartial ? sPart : sCopyFile;
+  if (bMarkPartial && bPartExists && buff.st_size > 0)
   { // must not be a folder! please fix a similar bug in kio_file!!
     if(S_ISDIR(buff.st_mode))
     {
@@ -2279,13 +2307,11 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
 #endif
   }
 
-  if(bPartExists && !bResume)                  // get rid of an unwanted ".part" file
-    remove(sPart.data());
+  if (bPartExists && !bResume)                  // get rid of an unwanted ".part" file
+    QFile::remove(sPart);
 
-  // JPF: in kio_file overwrite disables ".part" operations. I do not believe
-  // JPF: that this is a good behaviour!
-  if(bDestExists)                             // must delete for overwrite
-    remove(sDest.data());
+  if (bDestExists)                             // must delete for overwrite
+    QFile::remove(sCopyFile);
 
   // WABA: Make sure that we keep writing permissions ourselves,
   // otherwise we can be in for a surprise on NFS.
@@ -2297,9 +2323,8 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
 
   // open the output file ...
   KIO::fileoffset_t hCopyOffset = 0;
-  if(bResume)
-  {
-    iCopyFile = KDE_open( sPart.data(), O_RDWR );  // append if resuming
+  if (bResume) {
+    iCopyFile = KDE::open( sPart, O_RDWR );  // append if resuming
     hCopyOffset = KDE_lseek(iCopyFile, 0, SEEK_END);
     if(hCopyOffset < 0)
     {
@@ -2308,8 +2333,9 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
     }
     kDebug(7102) << "copy: resuming at " << hCopyOffset;
   }
-  else
-    iCopyFile = KDE_open(sPart.data(), O_CREAT | O_TRUNC | O_WRONLY, initialMode);
+  else {
+    iCopyFile = KDE::open(dest, O_CREAT | O_TRUNC | O_WRONLY, initialMode);
+  }
 
   if(iCopyFile == -1)
   {
@@ -2333,25 +2359,20 @@ Ftp::StatusCode Ftp::ftpCopyGet(int& iError, int& iCopyFile, const QString &sCop
   {
     if(iRes == statusSuccess)
     { // rename ".part" on success
-#ifdef Q_OS_WIN
-        if ( MoveFileExA( sPart.data(),
-                          sDest.data(),
-                          MOVEFILE_REPLACE_EXISTING|MOVEFILE_COPY_ALLOWED ) == 0 )
-#else
-      if ( KDE_rename( sPart.data(), sDest.data() ) )
-#endif
+      if ( KDE::rename( sPart, sCopyFile ) )
       {
-        kDebug(7102) << "copy: cannot rename " << sPart << " to " << sDest;
+        kDebug(7102) << "copy: cannot rename " << sPart << " to " << sCopyFile;
         iError = ERR_CANNOT_RENAME_PARTIAL;
         iRes = statusClientError;
       }
     }
-    else if(KDE_stat( sPart.data(), &buff ) == 0)
+    else if(KDE::stat( sPart, &buff ) == 0)
     { // should a very small ".part" be deleted?
       int size = config()->readEntry("MinimumKeepSize", DEFAULT_MINIMUM_KEEP_SIZE);
       if (buff.st_size <  size)
-        remove(sPart.data());
+        QFile::remove(sPart);
     }
   }
   return iRes;
 }
+
