@@ -64,6 +64,7 @@
 
 #include <kurl.h>
 #include <kdatetime.h>
+#include <ksystemtimezone.h>
 #include <kconfig.h>
 #include <kconfiggroup.h>
 #include <ksavefile.h>
@@ -88,16 +89,16 @@
 // latin1 regardless of their actual encoding.
 
 // L1 is used to indicate latin1 constants
-#define L1(x) QLatin1String(x)
+#define QL1S(x) QLatin1String(x)
 
 QString KCookieJar::adviceToStr(KCookieAdvice _advice)
 {
     switch( _advice )
     {
-    case KCookieAccept: return L1("Accept");
-    case KCookieReject: return L1("Reject");
-    case KCookieAsk: return L1("Ask");
-    default: return L1("Dunno");
+    case KCookieAccept: return QL1S("Accept");
+    case KCookieReject: return QL1S("Reject");
+    case KCookieAsk: return QL1S("Ask");
+    default: return QL1S("Dunno");
     }
 }
 
@@ -108,11 +109,11 @@ KCookieAdvice KCookieJar::strToAdvice(const QString &_str)
 
     QString advice = _str.toLower();
 
-    if (advice == QLatin1String("accept"))
+    if (advice == QL1S("accept"))
         return KCookieAccept;
-    else if (advice == QLatin1String("reject"))
+    else if (advice == QL1S("reject"))
         return KCookieReject;
-    else if (advice == QLatin1String("ask"))
+    else if (advice == QL1S("ask"))
         return KCookieAsk;
 
     return KCookieDunno;
@@ -129,7 +130,7 @@ KHttpCookie::KHttpCookie(const QString &_host,
                  const QString &_path,
                  const QString &_name,
                  const QString &_value,
-                 time_t _expireDate,
+                 qint64 _expireDate,
                  int _protocolVersion,
                  bool _secure,
                  bool _httpOnly,
@@ -150,8 +151,14 @@ KHttpCookie::KHttpCookie(const QString &_host,
 //
 // Checks if a cookie has been expired
 //
-bool    KHttpCookie::isExpired(time_t currentDate) const
+bool    KHttpCookie::isExpired(qint64 currentDate) const
 {
+    if (currentDate == -1) {
+        KDateTime epoch;
+        epoch.setTime_t(0);
+        currentDate = epoch.secsTo_long(KDateTime::currentUtcDateTime());
+    }
+
     return (mExpireDate != 0) && (mExpireDate < currentDate);
 }
 
@@ -172,9 +179,9 @@ QString KHttpCookie::cookieStr(bool useDOMFormat) const
     {
         result = mName + '=' + mValue;
         if (mExplicitPath)
-            result += L1("; $Path=\"") + mPath + L1("\"");
+            result += QL1S("; $Path=\"") + mPath + QL1S("\"");
         if (!mDomain.isEmpty())
-            result += L1("; $Domain=\"") + mDomain + L1("\"");
+            result += QL1S("; $Domain=\"") + mDomain + QL1S("\"");
     }
     return result;
 }
@@ -239,7 +246,7 @@ KCookieJar::KCookieJar()
     KConfig cfg( "khtml/domain_info", KConfig::NoGlobals, "data" );
     KConfigGroup group( &cfg, QString() );
     QStringList countries = group.readEntry( "twoLevelTLD", QStringList() );
-    foreach ( const QString& country, countries ) {
+    Q_FOREACH ( const QString& country, countries ) {
        m_twoLevelTLD.insert( country, 1 );
     }
 }
@@ -306,8 +313,8 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
     if (!parseUrl(_url, fqdn, path))
         return cookieStr;
 
-    bool secureRequest = _url.startsWith( L1("https://"), Qt::CaseInsensitive ) ||
-                         _url.startsWith( L1("webdavs://"), Qt::CaseInsensitive );
+    bool secureRequest = _url.startsWith( QL1S("https://"), Qt::CaseInsensitive ) ||
+                         _url.startsWith( QL1S("webdavs://"), Qt::CaseInsensitive );
 
     extractDomains(fqdn, domains);
 
@@ -327,7 +334,7 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
        }
        else
        {
-          QString key = (*it).isNull() ? L1("") : (*it);
+          QString key = (*it).isNull() ? QL1S("") : (*it);
           cookieList = m_cookieDomains.value(key);
           if (!cookieList)
              continue; // No cookies for this domain
@@ -361,7 +368,7 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
           }
 
           // Do not send expired cookies.
-          if ( cookie.isExpired (time(0)) )
+          if ( cookie.isExpired())
           {
              // Note there is no need to actually delete the cookie here
              // since the cookieserver will invoke ::saveCookieJar because
@@ -396,18 +403,18 @@ QString KCookieJar::findCookies(const QString &_url, bool useDOMFormat, long win
     Q_FOREACH(const KHttpCookie& cookie, allCookies) {
         if (useDOMFormat) {
             if (cookieCount > 0)
-                cookieStr += L1("; ");
+                cookieStr += QL1S("; ");
             cookieStr += cookie.cookieStr(true);
         } else {
             if (cookieCount == 0) {
-                cookieStr += L1("Cookie: ");
+                cookieStr += QL1S("Cookie: ");
                 if (protVersion > 0) {
                     QString version;
                     version.sprintf("$Version=%d; ", protVersion); // Without quotes
                     cookieStr += version;
                 }
             } else {
-                cookieStr += L1("; ");
+                cookieStr += QL1S("; ");
             }
             cookieStr += cookie.cookieStr(false);
         }
@@ -522,7 +529,7 @@ void KCookieJar::stripDomain(const QString &_fqdn, QString &_domain)
    else if ( domains.count() > 0 )
       _domain = domains[0];
    else
-      _domain = L1("");
+      _domain = QL1S("");
 }
 
 QString KCookieJar::stripDomain(const KHttpCookie& cookie)
@@ -540,14 +547,14 @@ bool KCookieJar::parseUrl(const QString &_url,
                           QString &_path)
 {
     KUrl kurl(_url);
-    if (!kurl.isValid())
+    if (!kurl.isValid() || kurl.protocol().isEmpty())
        return false;
 
     _fqdn = kurl.host().toLower();
     if (kurl.port() > 0)
     {
-       if (((kurl.protocol() == L1("http")) && (kurl.port() != 80)) ||
-           ((kurl.protocol() == L1("https")) && (kurl.port() != 443)))
+       if (((kurl.protocol() == QL1S("http")) && (kurl.port() != 80)) ||
+           ((kurl.protocol() == QL1S("https")) && (kurl.port() != 443)))
        {
           // It's <port>:<host> so that the sorting works as expected
           _fqdn = QString::fromLatin1("%1:%2").arg(kurl.port()).arg(_fqdn);
@@ -564,9 +571,9 @@ bool KCookieJar::parseUrl(const QString &_url,
 
     _path = kurl.path();
     if (_path.isEmpty())
-       _path = L1("/");
+       _path = QL1S("/");
 
-    QRegExp exp(L1("[\\\\/]\\.\\.[\\\\/]"));
+    QRegExp exp(QL1S("[\\\\/]\\.\\.[\\\\/]"));
     // Weird path, cookie stealing attempt?
     if (exp.indexIn(_path) != -1)
        return false; // Deny everything!!
@@ -574,11 +581,14 @@ bool KCookieJar::parseUrl(const QString &_url,
     return true;
 }
 
+// not static because it uses m_twoLevelTLD
 void KCookieJar::extractDomains(const QString &_fqdn,
                                 QStringList &_domains) const
 {
-    if (_fqdn.isEmpty()) // localhost...
+    if (_fqdn.isEmpty()) {
+        _domains.append( QL1S("localhost") );
         return;
+    }
 
     // Return numeric IPv6 addresses as is...
     if (_fqdn[0] == '[')
@@ -627,7 +637,7 @@ void KCookieJar::extractDomains(const QString &_fqdn,
               break;
        }
 
-       QString domain = partList.join(L1("."));
+       QString domain = partList.join(QL1S("."));
        _domains.append(domain);
        _domains.append('.' + domain);
        partList.erase(partList.begin()); // Remove part
@@ -639,30 +649,6 @@ void KCookieJar::extractDomains(const QString &_fqdn,
     _domains.prepend( _fqdn );
 }
 
-/*
-   Changes dates in from the following format
-
-      Wed Sep 12 07:00:00 2007 GMT
-   to
-      Wed Sep 12 2007 07:00:00 GMT
-
-   to allow KDateTime::fromString to properly parse expiration date formats
-   used in cookies by some servers such as amazon.com. See BR# 145244.
-*/
-static QString fixupDateTime(const QString& date)
-{
-  QStringList list = date.split(' ');
-  const int index = list.indexOf(QRegExp("[0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}"));
-
-  if (index > -1 && (index+1) < list.count())
-  {
-    list.insert(index+1, list.takeAt(index));
-    return list.join(" ");
-  }
-
-  return date;
-}
-
 //
 // This function parses cookie_headers and returns a linked list of
 // KHttpCookie objects for all cookies found in cookie_headers.
@@ -695,6 +681,9 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
     if (i > 0)
        defaultPath = path.left(i);
 
+    KDateTime epoch;
+    epoch.setTime_t(0);
+
     //  The hard stuff :)
     for(;;)
     {
@@ -712,14 +701,14 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             // Default domain = ""
             // Default path according to rfc2109
 
-            KHttpCookie cookie(fqdn, L1(""), defaultPath, Name, Value);
+            KHttpCookie cookie(fqdn, QL1S(""), defaultPath, Name, Value);
             if (windowId)
                cookie.mWindowIds.append(windowId);
             cookie.mCrossDomain = crossDomain;
 
             // Insert cookie in chain
             cookieList.append(cookie);
-            lastCookie = cookieList.end(); --lastCookie;
+            lastCookie = cookieList.end(); --lastCookie;            
         }
         else if (strncasecmp(cookieStr, "Set-Cookie2:", 12) == 0)
         {
@@ -730,7 +719,7 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
             // Default domain = ""
             // Default path according to rfc2965
 
-            KHttpCookie cookie(fqdn, L1(""), defaultPath, Name, Value);
+            KHttpCookie cookie(fqdn, QL1S(""), defaultPath, Name, Value);
             if (windowId)
                cookie.mWindowIds.append(windowId);
             cookie.mCrossDomain = crossDomain;
@@ -782,21 +771,35 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 if (max_age == 0)
                     lastCookie->mExpireDate = 1;
                 else
-                    lastCookie->mExpireDate = time(0)+max_age;
+                    lastCookie->mExpireDate = epoch.secsTo_long(KDateTime::currentUtcDateTime().addSecs(max_age));
             }
             else if (cName == "expires")
             {
-                // Parse brain-dead netscape cookie-format
-                lastCookie->mExpireDate = KDateTime::fromString(Value, KDateTime::RFCDate).toTime_t();
+                KTimeZones *zones = KSystemTimeZones::timeZones();
+                // Check for the most common cookie expire date format: Thu, 01-Jan-1970 00:00:00 GMT
+                KDateTime dt = KDateTime::fromString(Value, QL1S("%:A,%t%d-%:B-%Y%t%H:%M:%S%t%Z"), zones);
+                if (!dt.isValid()) {
+                    // Check for a variation of the above format: Thu, 01 Jan 1970 00:00:00 GMT
+                    dt = KDateTime::fromString(Value, QL1S("%:A,%t%d%t%:B%t%Y%t%H:%M:%S%t%Z"), zones);
+                    if (!dt.isValid()) {
+                        // Check for incorrect formats (amazon.com): Thu Jan 01 1970 00:00:00 GMT
+                        dt = KDateTime::fromString(Value, QL1S("%:A%t%:B%t%d%t%Y%t%H:%M:%S%t%Z"), zones);
+                        if (!dt.isValid()) {
+                            // Check for a variation of the above format: Thu Jan 01 00:00:00 1970 GMT (BR# 145244)
+                            dt = KDateTime::fromString(Value, QL1S("%:A%t%:B%t%d%t%H:%M:%S%t%Y%t%Z"), zones);
+                            if (!dt.isValid()) {
+                                // Finally we try the RFC date formats as last resort
+                                dt = KDateTime::fromString(Value, KDateTime::RFCDate);
+                            }
+                        }
+                    }
+                }
 
-                // Workaround for servers that send the expiration date in
-                // 'Wed Sep 12 07:00:00 2007 GMT' format. See BR# 145244.
-                if (lastCookie->mExpireDate == -1)
-                  lastCookie->mExpireDate = KDateTime::fromString(fixupDateTime(Value), KDateTime::RFCDate).toTime_t();
-
-                // We encode parse error/invalid as 0, but KDateTime likes -1, so convert
-                if (lastCookie->mExpireDate == -1)
-                  lastCookie->mExpireDate = 0;
+                if (dt.isValid()) {
+                    lastCookie->mExpireDate = epoch.secsTo_long(dt);
+                    if (lastCookie->mExpireDate == 0)
+                        lastCookie->mExpireDate = 1;
+                }
             }
             else if (cName == "path")
             {
@@ -811,12 +814,12 @@ KHttpCookieList KCookieJar::makeCookies(const QString &_url,
                 lastCookie->mProtocolVersion = Value.toInt();
             }
             else if ((cName == "secure") ||
-                     (cName.isEmpty() && Value.toLower() == L1("secure")))
+                     (cName.isEmpty() && Value.toLower() == QL1S("secure")))
             {
                 lastCookie->mSecure = true;
             }
             else if ((cName == "httponly") ||
-                     (cName.isEmpty() && Value.toLower() == L1("httponly")))
+                     (cName.isEmpty() && Value.toLower() == QL1S("httponly")))
             {
                 lastCookie->mHttpOnly = true;
             }
@@ -958,7 +961,7 @@ void KCookieJar::addCookie(KHttpCookie &cookie)
 
     // Add the cookie to the cookie list
     // The cookie list is sorted 'longest path first'
-    if (!cookie.isExpired(time(0)))
+    if (!cookie.isExpired())
     {
 #ifdef MAX_COOKIE_LIMIT
         if (cookieList->count() >= MAX_COOKIES_PER_HOST)
@@ -1126,9 +1129,9 @@ KHttpCookieList *KCookieJar::getCookieList(const QString & _domain,
     QString domain;
 
     if (_domain.isEmpty())
-        stripDomain( _fqdn, domain );
+        stripDomain(_fqdn, domain);
     else
-        domain = _domain;
+        stripDomain (_domain, domain);
 
     return m_cookieDomains.value(domain);
 }
@@ -1243,8 +1246,6 @@ bool KCookieJar::saveCookies(const QString &_filename)
 
     QTextStream ts(&saveFile);
 
-    time_t curTime = time(0);
-
     ts << "# KDE Cookie File v2\n#\n";
 
     QString s;
@@ -1263,7 +1264,7 @@ bool KCookieJar::saveCookies(const QString &_filename)
         QMutableListIterator<KHttpCookie> cookieIterator(*cookieList);
         while (cookieIterator.hasNext()) {
             const KHttpCookie& cookie = cookieIterator.next();
-            if (cookie.isExpired(curTime)) {
+            if (cookie.isExpired()) {
                 // Delete expired cookies
                 cookieIterator.remove();
             } else if (cookie.expireDate() != 0 && !m_ignoreCookieExpirationDate) {
@@ -1273,16 +1274,16 @@ bool KCookieJar::saveCookies(const QString &_filename)
                     ts << '[' << domain.toLocal8Bit().data() << "]\n";
                 }
                 // Store persistent cookies
-                QString path = L1("\"");
+                QString path = QL1S("\"");
                 path += cookie.path();
                 path += '"';
-                QString domain = L1("\"");
+                QString domain = QL1S("\"");
                 domain += cookie.domain();
                 domain += '"';
                 // TODO: replace with direct QTextStream output ?
-                s.sprintf("%-20s %-20s %-12s %10lu  %3d %-20s %-4i %s\n",
+                s.sprintf("%-20s %-20s %-12s %10lld  %3d %-20s %-4i %s\n",
                         cookie.host().toLatin1().constData(), domain.toLatin1().constData(),
-                        path.toLatin1().constData(), (unsigned long) cookie.expireDate(),
+                        path.toLatin1().constData(), cookie.expireDate(),
                         cookie.protocolVersion(),
                         cookie.name().isEmpty() ? cookie.value().toLatin1().constData() : cookie.name().toLatin1().constData(),
                         (cookie.isSecure() ? 1 : 0) + (cookie.isHttpOnly() ? 2 : 0) +
@@ -1339,7 +1340,7 @@ bool KCookieJar::loadCookies(const QString &_filename)
         return false;
     }
 
-    time_t curTime = time(0);
+    qint64 curTime = time(0);
 
     char *buffer = new char[READ_BUFFER_SIZE];
 
@@ -1375,7 +1376,7 @@ bool KCookieJar::loadCookies(const QString &_filename)
             const QString path = QString::fromLatin1( parseField(line) );
             const QString expStr = QString::fromLatin1( parseField(line) );
             if (expStr.isEmpty()) continue;
-            const int expDate = expStr.toInt();
+            const qint64 expDate = expStr.toLongLong();
             const QString verStr = QString::fromLatin1( parseField(line) );
             if (verStr.isEmpty()) continue;
             int protVer  = verStr.toInt();
@@ -1484,6 +1485,7 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse )
 
     KConfigGroup policyGroup(_config,"Cookie Policy");
     const QStringList domainSettings = policyGroup.readEntry("CookieDomainAdvice", QStringList());
+    // Warning: those default values are duplicated in the kcm (kio/kcookiespolicies.cpp)
     m_rejectCrossDomainCookies = policyGroup.readEntry("RejectCrossDomainCookies", true);
     m_autoAcceptSessionCookies = policyGroup.readEntry("AcceptSessionCookies", true);
     m_ignoreCookieExpirationDate = policyGroup.readEntry("IgnoreExpirationDate", false);
@@ -1493,7 +1495,7 @@ void KCookieJar::loadConfig(KConfig *_config, bool reparse )
     // Reset current domain settings first.
     //  (must make a copy because setDomainAdvice() might delete the domain from m_domainList inside the for loop)
     const QStringList domains = m_domainList;
-    foreach( const QString &domain, domains )
+    Q_FOREACH( const QString &domain, domains )
     {
          setDomainAdvice(domain, KCookieDunno);
     }
