@@ -31,11 +31,15 @@
 #include <kservice.h>
 #include <kstandarddirs.h>
 
+#include "authorizationmanager.h"
 #include "datacontainer.h"
 #include "package.h"
 #include "service.h"
 #include "scripting/dataenginescript.h"
 
+#include "private/authorizationmanager_p.h"
+#include "private/dataengineservice_p.h"
+#include "private/remotedataengine_p.h"
 #include "private/service_p.h"
 
 namespace Plasma
@@ -45,14 +49,12 @@ DataEngine::DataEngine(QObject *parent, KService::Ptr service)
     : QObject(parent),
       d(new DataEnginePrivate(this, service))
 {
-    connect(d->updateTimer, SIGNAL(timeout()), this, SLOT(scheduleSourcesUpdated()));
 }
 
 DataEngine::DataEngine(QObject *parent, const QVariantList &args)
     : QObject(parent),
       d(new DataEnginePrivate(this, KService::serviceByStorageId(args.count() > 0 ? args[0].toString() : QString())))
 {
-    connect(d->updateTimer, SIGNAL(timeout()), this, SLOT(scheduleSourcesUpdated()));
 }
 
 DataEngine::~DataEngine()
@@ -73,10 +75,13 @@ QStringList DataEngine::sources() const
 Service *DataEngine::serviceForSource(const QString &source)
 {
     if (d->script) {
-        return d->script->serviceForSource(source);
-    } else {
-        return new NullService(source, this);
+        Service * s = d->script->serviceForSource(source);
+        if (s) {
+            return s;
+        }
     }
+
+    return new NullService(source, this);
 }
 
 void DataEngine::connectSource(const QString &source, QObject *visualization,
@@ -92,6 +97,9 @@ void DataEngine::connectSource(const QString &source, QObject *visualization,
         // source was prexisting and they don't request delayed updates
         // (we want to do an immediate update in that case so they don't
         // have to wait for the first time out)
+        if (newSource && !s->data().isEmpty()) {
+            newSource = false;
+        }
         d->connectSource(s, visualization, pollingInterval, intervalAlignment,
                          !newSource || pollingInterval > 0);
         //kDebug() << " ==> source connected";
@@ -129,8 +137,9 @@ DataEngine::Data DataEngine::query(const QString &source) const
         return DataEngine::Data();
     } else if (!newSource && d->minPollingInterval >= 0 &&
                s->timeSinceLastUpdate() >= uint(d->minPollingInterval)) {
-        if (const_cast<DataEngine*>(this)->updateSourceEvent(source)) {
-            d->queueUpdate();
+        DataEngine *unconstThis = const_cast<DataEngine*>(this);
+        if (unconstThis->updateSourceEvent(source)) {
+            unconstThis->scheduleSourcesUpdated();
         }
     }
 
@@ -142,6 +151,7 @@ DataEngine::Data DataEngine::query(const QString &source) const
 void DataEngine::init()
 {
     if (d->script) {
+        d->setupScriptSupport();
         d->script->init();
     } else {
         // kDebug() << "called";
@@ -189,7 +199,7 @@ void DataEngine::setData(const QString &source, const QString &key, const QVaria
         emit sourceAdded(source);
     }
 
-    d->queueUpdate();
+    scheduleSourcesUpdated();
 }
 
 void DataEngine::setData(const QString &source, const Data &data)
@@ -211,7 +221,7 @@ void DataEngine::setData(const QString &source, const Data &data)
         emit sourceAdded(source);
     }
 
-    d->queueUpdate();
+    scheduleSourcesUpdated();
 }
 
 void DataEngine::removeAllData(const QString &source)
@@ -219,7 +229,7 @@ void DataEngine::removeAllData(const QString &source)
     DataContainer *s = d->source(source, false);
     if (s) {
         s->removeAllData();
-        d->queueUpdate();
+        scheduleSourcesUpdated();
     }
 }
 
@@ -228,7 +238,7 @@ void DataEngine::removeData(const QString &source, const QString &key)
     DataContainer *s = d->source(source, false);
     if (s) {
         s->setData(key, QVariant());
-        d->queueUpdate();
+        scheduleSourcesUpdated();
     }
 }
 
@@ -243,7 +253,7 @@ void DataEngine::addSource(DataContainer *source)
                      this, SLOT(internalUpdateSource(DataContainer*)));
     d->sources.insert(source->objectName(), source);
     emit sourceAdded(source->objectName());
-    d->queueUpdate();
+    scheduleSourcesUpdated();
 }
 
 void DataEngine::setMaxSourceCount(uint limit)
@@ -359,27 +369,35 @@ DataEngine::SourceDict DataEngine::containerDict() const
 
 void DataEngine::timerEvent(QTimerEvent *event)
 {
-    if (event->timerId() != d->updateTimerId) {
-        kDebug() << "bzzzt";
-        return;
+    //kDebug();
+    if (event->timerId() == d->updateTimerId) {
+        // if the freq update is less than 0, don't bother
+        if (d->minPollingInterval < 0) {
+            //kDebug() << "uh oh.. no polling allowed!";
+            return;
+        }
+
+        // minPollingInterval
+        if (d->updateTimestamp.elapsed() < d->minPollingInterval) {
+            //kDebug() << "hey now.. slow down!";
+            return;
+        }
+
+        d->updateTimestamp.restart();
+        d->updateTimerId = 0;
+        updateAllSources();
+    } else if (event->timerId() == d->checkSourcesTimerId) {
+        killTimer(d->checkSourcesTimerId);
+        d->checkSourcesTimerId = 0;
+
+        QHashIterator<QString, Plasma::DataContainer*> it(d->sources);
+        while (it.hasNext()) {
+            it.next();
+            it.value()->checkForUpdate();
+        }
+    } else {
+        QObject::timerEvent(event);
     }
-
-    event->accept();
-
-    // if the freq update is less than 0, don't bother
-    if (d->minPollingInterval < 0) {
-        //kDebug() << "uh oh.. no polling allowed!";
-        return;
-    }
-
-    // minPollingInterval
-    if (d->updateTimestamp.elapsed() < d->minPollingInterval) {
-        //kDebug() << "hey now.. slow down!";
-        return;
-    }
-
-    d->updateTimestamp.restart();
-    updateAllSources();
 }
 
 void DataEngine::updateAllSources()
@@ -394,6 +412,13 @@ void DataEngine::updateAllSources()
     scheduleSourcesUpdated();
 }
 
+void DataEngine::forceImmediateUpdateOfAllVisualizations()
+{
+    foreach (DataContainer *source, d->sources) {
+        source->forceImmediateUpdate();
+    }
+}
+
 void DataEngine::setIcon(const QString &icon)
 {
     d->icon = icon;
@@ -413,6 +438,38 @@ QString DataEngine::pluginName() const
     return d->dataEngineDescription.pluginName();
 }
 
+void DataEnginePrivate::publish(AnnouncementMethods methods, const QString &name)
+{
+    if (!publishedService) {
+        publishedService = new DataEngineService(q);
+    }
+
+    //QString resourceName =
+    //i18nc("%1 is the name of a dataengine, %2 the name of the machine that engine is published
+//on",
+          //"%1 dataengine on %2", name(), AuthorizationManager::self()->d->myCredentials.name());
+    kDebug() << "name: " << name;
+    publishedService->d->publish(methods, name);
+}
+
+void DataEnginePrivate::unpublish(const QString &name)
+{
+    Q_UNUSED(name)
+
+    if (publishedService) {
+        publishedService->d->unpublish();
+    }
+}
+
+bool DataEnginePrivate::isPublished() const
+{
+    if (publishedService) {
+        return publishedService->d->isPublished();
+    } else {
+        return false;
+    }
+}
+
 const Package *DataEngine::package() const
 {
     return d->package;
@@ -420,11 +477,11 @@ const Package *DataEngine::package() const
 
 void DataEngine::scheduleSourcesUpdated()
 {
-    QHashIterator<QString, Plasma::DataContainer*> it(d->sources);
-    while (it.hasNext()) {
-        it.next();
-        it.value()->checkForUpdate();
+    if (d->checkSourcesTimerId) {
+        return;
     }
+
+    d->checkSourcesTimerId = startTimer(0);
 }
 
 QString DataEngine::name() const
@@ -443,15 +500,15 @@ DataEnginePrivate::DataEnginePrivate(DataEngine *e, KService::Ptr service)
     : q(e),
       dataEngineDescription(service),
       refCount(-1), // first ref
+      checkSourcesTimerId(0),
       updateTimerId(0),
       minPollingInterval(-1),
       limit(0),
       valid(true),
       script(0),
-      package(0)
+      package(0),
+      publishedService(0)
 {
-    updateTimer = new QTimer(q);
-    updateTimer->setSingleShot(true);
     updateTimestamp.start();
 
     if (!service) {
@@ -512,7 +569,7 @@ void DataEnginePrivate::internalUpdateSource(DataContainer *source)
 
     if (q->updateSourceEvent(source->objectName())) {
         //kDebug() << "queuing an update";
-        queueUpdate();
+        q->scheduleSourcesUpdated();
     }/* else {
         kDebug() << "no update";
     }*/
@@ -578,6 +635,11 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
                                       bool immediateCall)
 {
     //kDebug() << "connect source called" << s->objectName() << "with interval" << pollingInterval;
+
+    //FIXME: at the moment a remote dataengine can only poll, a push mechanism will be needed instead
+    if (pollingInterval == 0 && qobject_cast<RemoteDataEngine *>(q)) {
+        pollingInterval = 5000;
+    }
     if (pollingInterval > 0) {
         // never more frequently than allowed, never more than 20 times per second
         uint min = qMax(50, minPollingInterval); // for qMax below
@@ -591,7 +653,8 @@ void DataEnginePrivate::connectSource(DataContainer *s, QObject *visualization,
         // we don't want to do an immediate call if we are simply
         // reconnecting
         //kDebug() << "immediate call requested, we have:" << s->visualizationIsConnected(visualization);
-        immediateCall = !s->visualizationIsConnected(visualization);
+        immediateCall = !s->data().isEmpty() &&
+                        !s->visualizationIsConnected(visualization);
     }
 
     s->connectVisualization(visualization, pollingInterval, align);
@@ -643,12 +706,25 @@ void DataEnginePrivate::trimQueue()
     }
 }
 
-void DataEnginePrivate::queueUpdate()
+// put all setup routines for script here. at this point we can assume that
+// package exists and that we have a script engine
+void DataEnginePrivate::setupScriptSupport()
 {
-    if (updateTimer->isActive()) {
+    if (!package) {
         return;
     }
-    updateTimer->start(0);
+
+    kDebug() << "setting up script support, package is in" << package->path()
+             << "which is a" << package->structure()->type() << "package"
+             << ", main script is" << package->filePath("mainscript");
+
+    QString translationsPath = package->filePath("translations");
+    if (!translationsPath.isEmpty()) {
+        //FIXME: we should _probably_ use a KComponentData to segregate the applets
+        //       from each other; but I want to get the basics working first :)
+        KGlobal::dirs()->addResourceDir("locale", translationsPath);
+        KGlobal::locale()->insertCatalog(package->metadata().pluginName());
+    }
 }
 
 }
