@@ -19,10 +19,16 @@
 *******************************************************************************/
 
 #include "package.h"
+#include "config-plasma.h"
 
 #include <QDir>
 #include <QFile>
 #include <QRegExp>
+#include <QtNetwork/QHostInfo>
+
+#ifdef QCA2_FOUND
+#include <QtCrypto>
+#endif
 
 #include <karchive.h>
 #include <kcomponentdata.h>
@@ -31,60 +37,34 @@
 #include <kio/deletejob.h>
 #include <kio/jobclasses.h>
 #include <kio/job.h>
+#include <kmimetype.h>
 #include <kplugininfo.h>
 #include <kstandarddirs.h>
+#include <ktar.h>
 #include <ktempdir.h>
 #include <ktemporaryfile.h>
 #include <kzip.h>
 #include <kdebug.h>
 
+#include "authorizationmanager.h"
 #include "packagemetadata.h"
+#include "private/authorizationmanager_p.h"
+#include "private/package_p.h"
+#include "private/plasmoidservice_p.h"
+#include "private/service_p.h"
 
 namespace Plasma
 {
 
-class PackagePrivate
-{
-public:
-    PackagePrivate(const PackageStructure::Ptr st, const QString &p)
-        : structure(st),
-          basePath(p)
-    {
-        QDir dir(basePath);
-        basePath = dir.canonicalPath();
-        valid = QFile::exists(basePath);
-
-        if (valid) {
-            QFileInfo info(basePath);
-            if (info.isDir()) {
-                basePath.append(QDir::separator());
-            }
-            kDebug() << "basePath is" << basePath;
-        } else {
-            kDebug() << p << "invalid, basePath is" << basePath;
-        }
-    }
-
-    ~PackagePrivate()
-    {
-    }
-
-    PackageStructure::Ptr structure;
-    QString basePath;
-    bool valid;
-};
-
 Package::Package(const QString &packageRoot, const QString &package,
                  PackageStructure::Ptr structure)
     : d(new PackagePrivate(structure, packageRoot + '/' + package))
 {
-    structure->setPath(d->basePath);
 }
 
 Package::Package(const QString &packagePath, PackageStructure::Ptr structure)
     : d(new PackagePrivate(structure, packagePath))
 {
-    structure->setPath(d->basePath);
 }
 
 Package::~Package()
@@ -99,7 +79,7 @@ bool Package::isValid() const
     }
 
     foreach (const char *dir, d->structure->requiredDirectories()) {
-        if (!QFile::exists(d->basePath + d->structure->contentsPrefix() + d->structure->path(dir))) {
+        if (!QFile::exists(d->structure->path() + d->structure->contentsPrefix() + d->structure->path(dir))) {
             kWarning() << "Could not find required directory" << dir;
             d->valid = false;
             return false;
@@ -107,9 +87,9 @@ bool Package::isValid() const
     }
 
     foreach (const char *file, d->structure->requiredFiles()) {
-        if (!QFile::exists(d->basePath + d->structure->contentsPrefix() + d->structure->path(file))) {
+        if (!QFile::exists(d->structure->path() + d->structure->contentsPrefix() + d->structure->path(file))) {
             kWarning() << "Could not find required file" << file << ", look in"
-                       << d->basePath + d->structure->contentsPrefix() + d->structure->path(file) << endl;
+                       << d->structure->path() + d->structure->contentsPrefix() + d->structure->path(file) << endl;
             d->valid = false;
             return false;
         }
@@ -125,25 +105,33 @@ QString Package::filePath(const char *fileType, const QString &filename) const
         return QString();
     }
 
-    QString path = d->structure->path(fileType);
+    QString path;
 
-    if (path.isEmpty()) {
-        kDebug() << "no matching path came of it";
-        return QString();
+    if (qstrlen(fileType) != 0) {
+        path = d->structure->path(fileType);
+
+        if (path.isEmpty()) {
+            kDebug() << "no matching path came of it, while looking for" << fileType << filename;
+            return QString();
+        }
     }
 
-    path.prepend(d->basePath + d->structure->contentsPrefix());
+    path.prepend(d->structure->path() + d->structure->contentsPrefix());
 
     if (!filename.isEmpty()) {
         path.append("/").append(filename);
     }
 
     if (QFile::exists(path)) {
+        if (d->structure->allowExternalPaths()) {
+            return path;
+        }
+
         // ensure that we don't return files outside of our base path
         // due to symlink or ../ games
         QDir dir(path);
         QString canonicalized = dir.canonicalPath() + QDir::separator();
-        if (canonicalized.startsWith(d->basePath)) {
+        if (canonicalized.startsWith(d->structure->path())) {
             return path;
         }
     }
@@ -163,23 +151,7 @@ QStringList Package::entryList(const char *fileType) const
         return QStringList();
     }
 
-    QString path = d->structure->path(fileType);
-    if (path.isEmpty()) {
-        return QStringList();
-    }
-
-    QDir dir(d->basePath + d->structure->contentsPrefix() + path);
-
-    if (dir.exists()) {
-        // ensure that we don't return files outside of our base path
-        // due to symlink or ../ games
-        QString canonicalized = dir.canonicalPath();
-        if (canonicalized.startsWith(d->basePath)) {
-            return dir.entryList(QDir::Files | QDir::Readable);
-        }
-    }
-
-    return QStringList();
+    return d->structure->entryList(fileType);
 }
 
 PackageMetadata Package::metadata() const
@@ -187,9 +159,15 @@ PackageMetadata Package::metadata() const
     return d->structure->metadata();
 }
 
+void Package::setPath(const QString &path)
+{
+    d->structure->setPath(path);
+    d->valid = !d->structure->path().isEmpty();
+}
+
 const QString Package::path() const
 {
-    return d->basePath;
+    return d->structure->path();
 }
 
 const PackageStructure::Ptr Package::structure() const
@@ -197,6 +175,99 @@ const PackageStructure::Ptr Package::structure() const
     return d->structure;
 }
 
+#ifdef QCA2_FOUND
+void PackagePrivate::updateHash(const QString &basePath, const QString &subPath, const QDir &dir, QCA::Hash &hash)
+{
+    // hash is calculated as a function of:
+    // * files ordered alphabetically by name, with each file's:
+    //      * path relative to the content root
+    //      * file data
+    // * directories ordered alphabetically by name, with each dir's:
+    //      * path relative to the content root
+    //      * file listing (recursing)
+    // symlinks (in both the file and dir case) are handled by adding
+    // the name of the symlink itself and the abs path of what it points to
+
+    const QDir::SortFlags sorting = QDir::Name | QDir::IgnoreCase;
+    const QDir::Filters filters = QDir::Hidden | QDir::System | QDir::NoDotAndDotDot;
+    foreach (const QString &file, dir.entryList(QDir::Files | filters, sorting)) {
+        if (!subPath.isEmpty()) {
+            hash.update(subPath.toUtf8());
+        }
+
+        hash.update(file.toUtf8());
+
+        QFileInfo info(dir.path() + '/' + file);
+        if (info.isSymLink()) {
+            hash.update(info.symLinkTarget().toUtf8());
+        } else {
+            QFile f(info.filePath());
+            if (f.open(QIODevice::ReadOnly)) {
+                while (!f.atEnd()) {
+                    hash.update(f.read(1024));
+                }
+            } else {
+                kWarning() << "could not add" << f.fileName() << "to the hash; file could not be opened for reading. "
+                           << "permissions fail?" << info.permissions() << info.isFile();
+            }
+        }
+    }
+
+    foreach (const QString &subDirPath, dir.entryList(QDir::Dirs | filters, sorting)) {
+        const QString relativePath = subPath + subDirPath + '/';
+        hash.update(relativePath.toUtf8());
+
+        QDir subDir(dir.path());
+        subDir.cd(subDirPath);
+
+        if (subDir.path() != subDir.canonicalPath()) {
+            hash.update(subDir.canonicalPath().toUtf8());
+        } else {
+            updateHash(basePath, relativePath, subDir, hash);
+        }
+    }
+}
+#endif
+
+QString Package::contentsHash() const
+{
+#ifdef QCA2_FOUND
+    if (!QCA::isSupported("sha1")) {
+        kWarning() << "can not create hash for" << path() << "due to no SHA1 support in QCA2";
+        return QString();
+    }
+
+    const QString basePath = d->structure->path() + d->structure->contentsPrefix();
+    QDir dir(basePath);
+
+    if (!dir.exists()) {
+        return QString();
+    }
+
+    QCA::Hash hash("sha1");
+    QString metadataPath = d->structure->path() + "metadata.desktop";
+    if (QFile::exists(metadataPath)) {
+        QFile f(metadataPath);
+        if (f.open(QIODevice::ReadOnly)) {
+            while (!f.atEnd()) {
+                hash.update(f.read(1024));
+            }
+        } else {
+            kWarning() << "could not add" << f.fileName() << "to the hash; file could not be opened for reading.";
+        }
+    } else {
+        kWarning() << "no metadata at" << metadataPath;
+    }
+
+    d->updateHash(basePath, QString(), dir, hash);
+    return QCA::arrayToHex(hash.final().toByteArray());
+#else
+    // no QCA2!
+    kWarning() << "can not create hash for" << path() << "due to no cryptographic support (QCA2)";
+    return QString();
+#endif
+}
+
 //TODO: provide a version of this that allows one to ask for certain types of packages, etc?
 //      should we be using KService here instead/as well?
 QStringList Package::listInstalled(const QString &packageRoot) // static
@@ -274,28 +345,44 @@ bool Package::installPackage(const QString &package,
             path.append('/');
         }
     } else {
-        KZip archive(package);
-        if (!archive.open(QIODevice::ReadOnly)) {
+        KArchive *archive = 0;
+        KMimeType::Ptr mimetype = KMimeType::findByPath(package);
+
+        if (mimetype->is("application/zip")) {
+            archive = new KZip(package);
+        } else if (mimetype->is("application/x-compressed-tar") ||
+                   mimetype->is("application/x-tar") || mimetype->is("application/x-bzip-compressed-tar")) {
+            archive = new KTar(package);
+        } else {
+            kWarning() << "Could not open package file, unsupported archive format:" << package << mimetype->name();
+            return false;
+        }
+
+        if (!archive->open(QIODevice::ReadOnly)) {
             kWarning() << "Could not open package file:" << package;
+	    delete archive;
             return false;
         }
 
         archivedPackage = true;
-        const KArchiveDirectory *source = archive.directory();
-        const KArchiveEntry *metadata = source->entry("metadata.desktop");
-
-        if (!metadata) {
-            kWarning() << "No metadata file in package" << package;
-            return false;
-        }
-
         path = tempdir.name();
+
+        const KArchiveDirectory *source = archive->directory();
         source->copyTo(path);
+
+        QStringList entries = source->entries();
+        if (entries.count() == 1) {
+            const KArchiveEntry *entry = source->entry(entries[0]);
+            if (entry->isDirectory()) {
+                path.append(entry->name()).append("/");
+            }
+        }
+	delete archive;
     }
 
     QString metadataPath = path + "metadata.desktop";
     if (!QFile::exists(metadataPath)) {
-        kWarning() << "No metadata file in package" << package;
+        kWarning() << "No metadata file in package" << package << metadataPath;
         return false;
     }
 
@@ -343,31 +430,35 @@ bool Package::installPackage(const QString &package,
         tempdir.setAutoRemove(false);
     }
 
-    // and now we register it as a service =)
-    QString metaPath = targetName + "/metadata.desktop";
-    KDesktopFile df(metaPath);
-    KConfigGroup cg = df.desktopGroup();
+    if (!servicePrefix.isEmpty()) {
+        // and now we register it as a service =)
+        QString metaPath = targetName + "/metadata.desktop";
+        KDesktopFile df(metaPath);
+        KConfigGroup cg = df.desktopGroup();
 
-    // Q: should not installing it as a service disqualify it?
-    // Q: i don't think so since KServiceTypeTrader may not be
-    // used by the installing app in any case, and the
-    // package is properly installed - aseigo
+        // Q: should not installing it as a service disqualify it?
+        // Q: i don't think so since KServiceTypeTrader may not be
+        // used by the installing app in any case, and the
+        // package is properly installed - aseigo
 
-    //TODO: reduce code duplication with registerPackage below
+        //TODO: reduce code duplication with registerPackage below
 
-    QString serviceName = servicePrefix + meta.pluginName();
+        QString serviceName = servicePrefix + meta.pluginName();
 
-    QString service = KStandardDirs::locateLocal("services", serviceName + ".desktop");
-    KIO::FileCopyJob *job = KIO::file_copy(metaPath, service, -1, KIO::HideProgressInfo);
-    if (job->exec()) {
-        // the icon in the installed file needs to point to the icon in the
-        // installation dir!
-        QString iconPath = targetName + '/' + cg.readEntry("Icon");
-        QFile icon(iconPath);
-        if (icon.exists()) {
-            KDesktopFile df(service);
-            KConfigGroup cg = df.desktopGroup();
-            cg.writeEntry("Icon", iconPath);
+        QString service = KStandardDirs::locateLocal("services", serviceName + ".desktop");
+        KIO::FileCopyJob *job = KIO::file_copy(metaPath, service, -1, KIO::HideProgressInfo);
+        if (job->exec()) {
+            // the icon in the installed file needs to point to the icon in the
+            // installation dir!
+            QString iconPath = targetName + '/' + cg.readEntry("Icon");
+            QFile icon(iconPath);
+            if (icon.exists()) {
+                KDesktopFile df(service);
+                KConfigGroup cg = df.desktopGroup();
+                cg.writeEntry("Icon", iconPath);
+            }
+        } else {
+            kWarning() << "Could not register package as service (this is not necessarily fatal):" << serviceName << " : " << job->errorString();
         }
     }
 
@@ -471,4 +562,45 @@ bool Package::createPackage(const PackageMetadata &metadata,
     return true;
 }
 
+PackagePrivate::PackagePrivate(const PackageStructure::Ptr st, const QString &p)
+        : structure(st),
+          service(0)
+{
+    structure->setPath(p);
+    valid = !structure->path().isEmpty();
+}
+
+PackagePrivate::~PackagePrivate()
+{
+}
+
+void PackagePrivate::publish(AnnouncementMethods methods)
+{
+    if (!service) {
+        service = new PlasmoidService(structure->path());
+    }
+
+    QString resourceName =
+    i18nc("%1 is the name of a plasmoid, %2 the name of the machine that plasmoid is published on",
+          "%1 on %2", structure->metadata().name(), QHostInfo::localHostName());
+    kDebug() << "publishing package under name " << resourceName;
+    service->d->publish(methods, resourceName, structure->metadata());
+}
+
+void PackagePrivate::unpublish()
+{
+    if (service) {
+        service->d->unpublish();
+    }
+}
+
+bool PackagePrivate::isPublished() const
+{
+    if (service) {
+        return service->d->isPublished();
+    } else {
+        return false;
+    }
+}
+
 } // Namespace
