@@ -23,7 +23,6 @@
 #include <QMatrix>
 #include <QPainter>
 #include <QSharedData>
-#include <QTimer>
 
 #include <kcolorscheme.h>
 #include <kconfiggroup.h>
@@ -68,11 +67,13 @@ class SvgPrivate
     public:
         SvgPrivate(Svg *svg)
             : q(svg),
-              saveTimer(0),
               renderer(0),
+              lastModified(0),
               multipleImages(false),
               themed(false),
-              applyColors(false)
+              applyColors(false),
+              cacheRendering(true),
+              themeFailed(false)
         {
         }
 
@@ -98,11 +99,11 @@ class SvgPrivate
         QString cachePath(const QString &path, const QSize &size)
         {
              return QString("%3_%2_%1_").arg(int(size.height()))
-                                       .arg(int(size.width()))
-                                       .arg(path);
+                                        .arg(int(size.width()))
+                                        .arg(path);
         }
 
-        bool setImagePath(const QString &imagePath, Svg *q)
+        bool setImagePath(const QString &imagePath)
         {
             bool isThemed = !QDir::isAbsolutePath(imagePath);
 
@@ -118,47 +119,70 @@ class SvgPrivate
             bool updateNeeded = true; //!path.isEmpty() || !themePath.isEmpty();
 
             if (themed) {
-                QObject::disconnect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
+                QObject::disconnect(actualTheme(), SIGNAL(themeChanged()),
                                     q, SLOT(themeChanged()));
-                QObject::disconnect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()),
-                                    q, SLOT(colorsChanged()));
             }
 
             themed = isThemed;
             path.clear();
             themePath.clear();
+            localRectCache.clear();
 
             if (themed) {
                 themePath = imagePath;
-                QObject::connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()),
-                                 q, SLOT(themeChanged()));
-
-                // check if svg wants colorscheme applied
-                checkApplyColorHint();
-                if (applyColors && !Theme::defaultTheme()->colorScheme()) {
-                    QObject::connect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()),
-                                     q, SLOT(colorsChanged()));
-                }
-
-                QRectF rect;
-                bool found = Theme::defaultTheme()->findInRectsCache(path, "_Natural", rect);
-
-                if (found && !rect.isValid()) {
-                    createRenderer();
-                    naturalSize = renderer->defaultSize();
-                    Theme::defaultTheme()->insertIntoRectsCache(path, "_Natural", QRectF(QPointF(0,0), naturalSize));
-                } else {
-                    naturalSize = rect.size();
-                }
+                themeFailed = false;
+                QObject::connect(actualTheme(), SIGNAL(themeChanged()), q, SLOT(themeChanged()));
             } else if (QFile::exists(imagePath)) {
                 path = imagePath;
             } else {
                 kDebug() << "file '" << path << "' does not exist!";
             }
 
+            // check if svg wants colorscheme applied
+            QObject::disconnect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()),
+                                q, SLOT(colorsChanged()));
+
+            checkApplyColorHint();
+            if (applyColors && !actualTheme()->colorScheme()) {
+                QObject::connect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()),
+                                 q, SLOT(colorsChanged()));
+            }
+
+            // also images with absolute path needs to have a natural size initialized,
+            // even if looks a bit weird using Theme to store non-themed stuff
+            if (themed || QFile::exists(imagePath)) {
+                QRectF rect;
+                bool found = actualTheme()->findInRectsCache(path, "_Natural", rect);
+
+                if (!found) {
+                    createRenderer();
+                    naturalSize = renderer->defaultSize();
+                    //kDebug() << "natural size for" << path << "from renderer is" << naturalSize;
+                    actualTheme()->insertIntoRectsCache(path, "_Natural", QRectF(QPointF(0,0), naturalSize));
+                } else {
+                    naturalSize = rect.size();
+                    //kDebug() << "natural size for" << path << "from cache is" << naturalSize;
+                }
+            }
+
+            if (!themed) {
+                QFile f(imagePath);
+                QFileInfo info(f);
+                lastModified = info.lastModified().toTime_t();
+            }
+
             return updateNeeded;
         }
 
+        Theme *actualTheme()
+        {
+            if (!theme) {
+                theme = Plasma::Theme::defaultTheme();
+            }
+
+            return theme.data();
+        }
+
         QPixmap findInCache(const QString &elementId, const QSizeF &s = QSizeF())
         {
             QSize size;
@@ -180,16 +204,16 @@ class SvgPrivate
 
             //kDebug() << "id is " << id;
 
-            Theme *theme = Theme::defaultTheme();
             QPixmap p;
-
-            if (theme->findInCache(id, p)) {
-                //kDebug() << "found cached version of " << id << p.size();
-                return p;
-            } else {
-                //kDebug() << "didn't find cached version of " << id << ", so re-rendering";
+            if (cacheRendering) {
+                if (actualTheme()->findInCache(id, p, lastModified)) {
+                    //kDebug() << "found cached version of " << id << p.size();
+                    return p;
+                }
             }
 
+            //kDebug() << "didn't find cached version of " << id << ", so re-rendering";
+
             //kDebug() << "size for " << elementId << " is " << s;
             // we have to re-render this puppy
 
@@ -210,31 +234,17 @@ class SvgPrivate
             // Apply current color scheme if the svg asks for it
             if (applyColors) {
                 QImage itmp = p.toImage();
-                KIconEffect::colorize(itmp, theme->color(Theme::BackgroundColor), 1.0);
+                KIconEffect::colorize(itmp, actualTheme()->color(Theme::BackgroundColor), 1.0);
                 p = p.fromImage(itmp);
             }
 
-            if (!itemsToSave.contains(id)) {
-                itemsToSave.insert(id, p);
-                saveTimer->start(300);
+            if (cacheRendering) {
+                actualTheme()->insertIntoCache(id, p, QString::number((qint64)q, 16)+elementId);
             }
 
             return p;
         }
 
-        void scheduledCacheUpdate()
-        {
-            QHash<QString, QPixmap>::const_iterator i = itemsToSave.constBegin();
-
-            while (i != itemsToSave.constEnd()) {
-                //kDebug()<<"Saving item to cache: "<<i.key();
-                Theme::defaultTheme()->insertIntoCache(i.key(), i.value());
-                ++i;
-            }
-
-            itemsToSave.clear();
-        }
-
         void createRenderer()
         {
             if (renderer) {
@@ -242,7 +252,7 @@ class SvgPrivate
             }
 
             //kDebug() << kBacktrace();
-            if (themed && path.isEmpty()) {
+            if (themed && path.isEmpty() && !themeFailed) {
                 Applet *applet = qobject_cast<Applet*>(q->parent());
                 if (applet && applet->package()) {
                     path = applet->package()->filePath("images", themePath + ".svg");
@@ -253,7 +263,11 @@ class SvgPrivate
                 }
 
                 if (path.isEmpty()) {
-                    path = Plasma::Theme::defaultTheme()->imagePath(themePath);
+                    path = actualTheme()->imagePath(themePath);
+                    themeFailed = path.isEmpty();
+                    if (themeFailed) {
+                        kWarning() << "No image path found for" << themePath;
+                    }
                 }
             }
 
@@ -267,7 +281,12 @@ class SvgPrivate
                 //kDebug() << "gots us an existing one!";
                 renderer = it.value();
             } else {
-                renderer = new SharedSvgRenderer(path);
+                if (path.isEmpty()) {
+                    renderer = new SharedSvgRenderer();
+                } else {
+                    renderer = new SharedSvgRenderer(path);
+                }
+
                 s_renderers[path] = renderer;
             }
 
@@ -281,7 +300,10 @@ class SvgPrivate
             if (renderer && renderer.count() == 2) {
                 // this and the cache reference it
                 s_renderers.erase(s_renderers.find(path));
-                Plasma::Theme::defaultTheme()->releaseRectsCache(path);
+
+                if (theme) {
+                    theme.data()->releaseRectsCache(path);
+                }
             }
 
             renderer = 0;
@@ -291,7 +313,16 @@ class SvgPrivate
         QRectF elementRect(const QString &elementId)
         {
             if (themed && path.isEmpty()) {
-                path = Plasma::Theme::defaultTheme()->imagePath(themePath);
+                if (themeFailed) {
+                    return QRectF();
+                }
+
+                path = actualTheme()->imagePath(themePath);
+                themeFailed = path.isEmpty();
+
+                if (themeFailed) {
+                    return QRectF();
+                }
             }
 
             QString id = cacheId(elementId);
@@ -300,7 +331,7 @@ class SvgPrivate
             }
 
             QRectF rect;
-            bool found = Theme::defaultTheme()->findInRectsCache(path, id, rect);
+            bool found = actualTheme()->findInRectsCache(path, id, rect);
 
             if (found) {
                 localRectCache.insert(id, rect);
@@ -316,13 +347,14 @@ class SvgPrivate
             QRectF elementRect = renderer->elementExists(elementId) ?
                                  renderer->boundsOnElement(elementId) : QRectF();
             naturalSize = renderer->defaultSize();
+            //kDebug() << "natural size for" << path << "is" << naturalSize;
             qreal dx = size.width() / naturalSize.width();
             qreal dy = size.height() / naturalSize.height();
 
             elementRect = QRectF(elementRect.x() * dx, elementRect.y() * dy,
                                  elementRect.width() * dx, elementRect.height() * dy);
-            Theme::defaultTheme()->insertIntoRectsCache(path, cacheId(elementId), elementRect);
 
+            actualTheme()->insertIntoRectsCache(path, cacheId(elementId), elementRect);
             return elementRect;
         }
 
@@ -334,37 +366,15 @@ class SvgPrivate
 
         void checkApplyColorHint()
         {
-            QRectF elementRect;
-            bool found = Theme::defaultTheme()->findInRectsCache(themePath, cacheId("hint-apply-color-scheme"), elementRect);
-
-            if (found) {
-                applyColors = elementRect.isValid();
-            } else {
-                findAndCacheElementRect("hint-apply-color-scheme");
-                applyColors = renderer->elementExists("hint-apply-color-scheme");
-            }
+            applyColors = elementRect("hint-apply-color-scheme").isValid();
         }
 
         void themeChanged()
         {
-            if (!themed) {
-                return;
-            }
-
-            QString newPath = Theme::defaultTheme()->imagePath(themePath);
-
-            if (path == newPath) {
-                return;
-            }
-
-            path = newPath;
-            //delete d->renderer; we're a KSharedPtr
-            eraseRenderer();
-
             // check if new theme svg wants colorscheme applied
             bool wasApplyColors = applyColors;
             checkApplyColorHint();
-            if (applyColors && !Theme::defaultTheme()->colorScheme()) {
+            if (applyColors && actualTheme()->colorScheme()) {
                 if (!wasApplyColors) {
                     QObject::connect(KGlobalSettings::self(), SIGNAL(kdisplayPaletteChanged()),
                                      q, SLOT(colorsChanged()));
@@ -374,9 +384,14 @@ class SvgPrivate
                                     q, SLOT(colorsChanged()));
             }
 
-            localRectCache.clear();
-            itemsToSave.clear();
-            saveTimer->stop();
+            if (!themed) {
+                return;
+            }
+
+            QString currentPath = themePath;
+            themePath.clear();
+            eraseRenderer();
+            setImagePath(currentPath);
 
             //kDebug() << themePath << ">>>>>>>>>>>>>>>>>> theme changed";
             emit q->repaintNeeded();
@@ -389,24 +404,26 @@ class SvgPrivate
             }
 
             eraseRenderer();
-            itemsToSave.clear();
-            saveTimer->stop();
+            //kDebug() << "repaint needed from colorsChanged";
             emit q->repaintNeeded();
         }
 
-        Svg *q;
         static QHash<QString, SharedSvgRenderer::Ptr> s_renderers;
+
+        Svg *q;
+        QWeakPointer<Theme> theme;
         QHash<QString, QRectF> localRectCache;
-        QHash<QString, QPixmap> itemsToSave;
-        QTimer *saveTimer;
         SharedSvgRenderer::Ptr renderer;
         QString themePath;
         QString path;
         QSizeF size;
         QSizeF naturalSize;
-        bool multipleImages;
-        bool themed;
-        bool applyColors;
+        unsigned int lastModified;
+        bool multipleImages : 1;
+        bool themed : 1;
+        bool applyColors : 1;
+        bool cacheRendering : 1;
+        bool themeFailed : 1;
 };
 
 QHash<QString, SharedSvgRenderer::Ptr> SvgPrivate::s_renderers;
@@ -415,9 +432,6 @@ Svg::Svg(QObject *parent)
     : QObject(parent),
       d(new SvgPrivate(this))
 {
-    d->saveTimer = new QTimer(this);
-    d->saveTimer->setSingleShot(true);
-    connect(d->saveTimer, SIGNAL(timeout()), this, SLOT(scheduledCacheUpdate()));
 }
 
 Svg::~Svg()
@@ -515,21 +529,7 @@ bool Svg::hasElement(const QString &elementId) const
         return false;
     }
 
-    QString id = d->cacheId(elementId);
-    if (d->localRectCache.contains(id)) {
-        return d->localRectCache.value(id).isValid();
-    }
-
-    QRectF elementRect;
-    bool found = Theme::defaultTheme()->findInRectsCache(d->path, id, elementRect);
-
-    if (found) {
-        d->localRectCache.insert(id, elementRect);
-        return elementRect.isValid();
-    } else {
-//        kDebug() << "** ** *** !!!!!!!! *** ** ** creating renderer due to hasElement miss" << d->path << elementId;
-        return d->findAndCacheElementRect(elementId).isValid();
-    }
+    return d->elementRect(elementId).isValid();
 }
 
 QString Svg::elementAtPoint(const QPoint &point) const
@@ -572,8 +572,9 @@ bool Svg::containsMultipleImages() const
 
 void Svg::setImagePath(const QString &svgFilePath)
 {
-    d->setImagePath(svgFilePath, this);
     d->eraseRenderer();
+    d->setImagePath(svgFilePath);
+    //kDebug() << "repaintNeeded";
     emit repaintNeeded();
 }
 
@@ -582,6 +583,35 @@ QString Svg::imagePath() const
    return d->themed ? d->themePath : d->path;
 }
 
+void Svg::setUsingRenderingCache(bool useCache)
+{
+    d->cacheRendering = useCache;
+}
+
+bool Svg::isUsingRenderingCache() const
+{
+    return d->cacheRendering;
+}
+
+void Svg::setTheme(Plasma::Theme *theme)
+{
+    if (d->theme) {
+        disconnect(d->theme.data(), 0, this, 0);
+    }
+
+    d->theme = theme;
+    if (!imagePath().isEmpty()) {
+        QString path = imagePath();
+        d->themePath.clear();
+        setImagePath(path);
+    }
+}
+
+Theme *Svg::theme() const
+{
+    return d->theme ? d->theme.data() : Theme::defaultTheme();
+}
+
 } // Plasma namespace
 
 #include "svg.moc"
