@@ -231,7 +231,7 @@ void SolidHwTest::testDeviceInterfaces()
     Solid::Device cpu("/org/kde/solid/fakehw/acpi_CPU0");
 
     Solid::DeviceInterface *iface = cpu.asDeviceInterface(Solid::DeviceInterface::Processor);
-    Solid::Processor *processor = cpu.as<Solid::Processor>();
+    Solid::DeviceInterface *processor = cpu.as<Solid::Processor>();
 
     QVERIFY(cpu.isDeviceInterface(Solid::DeviceInterface::Processor));
     QVERIFY(iface!=0);
@@ -347,18 +347,24 @@ void SolidHwTest::testPredicate()
     Solid::Predicate p8 = Solid::Predicate::fromString("AudioInterface.deviceType == 'AudioInput|AudioOutput'");
     Solid::Predicate p9 = Solid::Predicate::fromString("AudioInterface.deviceType == 'AudioInput'");
     Solid::Predicate p10 = Solid::Predicate::fromString("AudioInterface.deviceType  & 'AudioInput'");
+    Solid::Predicate p11 = Solid::Predicate::fromString("AudioInterface.deviceType  & 'foobar'");
     QVERIFY(!p8.matches(dev));
     QVERIFY(!p9.matches(dev));
     QVERIFY(!p10.matches(dev));
+    QVERIFY(!p11.matches(dev));
     dev = Solid::Device("/org/kde/solid/fakehw/pci_8086_266e_oss_pcm_0");
     QVERIFY(p8.matches(dev));
     QVERIFY(!p9.matches(dev));
     QVERIFY(p10.matches(dev));
+    QVERIFY(!p11.matches(dev));
 
     QString str_pred = "[[Processor.maxSpeed == 3201 AND Processor.canChangeFrequency == false] OR StorageVolume.mountPoint == '/media/blup']";
     // Since str_pred is canonicalized, fromString().toString() should be invariant
     QCOMPARE(Solid::Predicate::fromString(str_pred).toString(), str_pred);
 
+    // Invalid predicate
+    str_pred = "[StorageVolume.ignored == false AND OpticalDisc.isBlank == true AND OpticalDisc.discType & 'CdRecordable|CdRewritable']";
+    QVERIFY(!Solid::Predicate::fromString(str_pred).isValid());
 
     QString parentUdi = "/org/kde/solid/fakehw/storage_model_solid_reader";
     Solid::DeviceInterface::Type ifaceType = Solid::DeviceInterface::Unknown;
